/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_Condition.qpp

    Qore Programming Language

    Copyright (C) 2005 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Condition.h"
#include "qore/intern/QC_AbstractSmartLock.h"
#include <qore/ReferenceHolder.h>

#include <cerrno>
/* Qore class Qore::Thread::Condition */

qore_classid_t CID_CONDITION;
QoreClass* QC_CONDITION;

// nothing Condition::broadcast(){}
static QoreValue Condition_broadcast(QoreObject* self, Condition* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 110 "QC_Condition.qpp"
    if (c->broadcast()) {
        xsink->raiseException("CONDITION-BROADCAST-ERROR", q_strerror(errno));
    }
    return QoreValue();
}

// Condition::constructor() {}
static void Condition_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 59 "QC_Condition.qpp"
self->setPrivate(CID_CONDITION, new Condition);
}

// Condition::copy() {}
static void Condition_copy(QoreObject* self, QoreObject* old, Condition* c, ExceptionSink* xsink) {
# 70 "QC_Condition.qpp"
self->setPrivate(CID_CONDITION, new Condition);
}

// nothing Condition::signal(){}
static QoreValue Condition_signal(QoreObject* self, Condition* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 89 "QC_Condition.qpp"
    if (c->signal()) {
        xsink->raiseException("CONDITION-SIGNAL-ERROR", q_strerror(errno));
    }
    return QoreValue();
}

// int Condition::wait(AbstractSmartLock lock, timeout timeout_ms = 0){}
static QoreValue Condition_wait_C17AbstractSmartLockVt(QoreObject* self, Condition* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(lock, AbstractSmartLock, args, 0, CID_ABSTRACTSMARTLOCK, "Condition::wait()", "AbstractSmartLock", xsink);
    if (*xsink)
        return 0;
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 146 "QC_Condition.qpp"
    ReferenceHolder<AbstractSmartLock> holder(lock, xsink);

    int rc = timeout_ms > 0
        ? c->wait(lock, timeout_ms, xsink)
        : c->wait(lock, xsink);

    //printd(5, "Condition::wait() m: %s (%p) timeout: %d rc: %d\n", lock->getName(), lock, timeout_ms, rc);

    if (rc && rc != ETIMEDOUT && !*xsink) {
        xsink->raiseErrnoException("CONDITION-WAIT-ERROR", rc, "unknown system error code returned from " \
            "Condition::wait(lock: %s, timeout: %d): rc: %d", lock->getName(), timeout_ms, rc);
    }
    return rc;
}

// int Condition::wait_count(AbstractSmartLock lock){}
static QoreValue Condition_wait_count_C17AbstractSmartLock(QoreObject* self, Condition* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(lock, AbstractSmartLock, args, 0, CID_ABSTRACTSMARTLOCK, "Condition::wait_count()", "AbstractSmartLock", xsink);
    if (*xsink)
        return 0;
# 174 "QC_Condition.qpp"
    ReferenceHolder<AbstractSmartLock> holder(lock, xsink);

    return c->wait_count(lock);
}

DLLLOCAL void preinitConditionClass() {
    QC_CONDITION = new QoreBuiltinClass("Condition", "::Qore::Thread::Condition", QDOM_THREAD_CLASS);
    CID_CONDITION = QC_CONDITION->getID();
    QC_CONDITION->setSystem();
}

DLLLOCAL QoreClass* initConditionClass(QoreNamespace& ns) {
    if (!QC_CONDITION)
        preinitConditionClass();

    // nothing Condition::broadcast(){}
    QC_CONDITION->addMethod("broadcast", (q_method_n_t)Condition_broadcast, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // Condition::constructor() {}
    QC_CONDITION->addConstructor(Condition_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Condition::copy() {}
    QC_CONDITION->setCopy((q_copy_t)Condition_copy);

    // nothing Condition::signal(){}
    QC_CONDITION->addMethod("signal", (q_method_n_t)Condition_signal, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int Condition::wait(AbstractSmartLock lock, timeout timeout_ms = 0){}
    QC_CONDITION->addMethod("wait", (q_method_n_t)Condition_wait_C17AbstractSmartLockVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, QC_ABSTRACTSMARTLOCK->getTypeInfo(), QORE_PARAM_NO_ARG, "lock", timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms");

    // int Condition::wait_count(AbstractSmartLock lock){}
    QC_CONDITION->addMethod("wait_count", (q_method_n_t)Condition_wait_count_C17AbstractSmartLock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, QC_ABSTRACTSMARTLOCK->getTypeInfo(), QORE_PARAM_NO_ARG, "lock");

    return QC_CONDITION;
}
