/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_FileLineIterator.qpp FileLineIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <cerrno>

#include "qore/Qore.h"
#include "qore/intern/FileLineIterator.h"
#include "qore/intern/qore_qf_private.h"
/* Qore class Qore::FileLineIterator */

qore_classid_t CID_FILELINEITERATOR;
QoreClass* QC_FILELINEITERATOR;

// FileLineIterator::constructor(string path, *string encoding, *string eol, bool trim = True, *bool nonblocking_open) {}
static void FileLineIterator_constructor_VsNsNsVbNb(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
    const QoreStringNode* eol = get_param_value(args, 2).get<const QoreStringNode>();
    bool trim = HARD_QORE_VALUE_BOOL(args, 3);
    bool nonblocking_open = HARD_QORE_VALUE_BOOL(args, 4);
# 78 "QC_FileLineIterator.qpp"
if (eol && eol->empty())
        eol = 0;
    int flags = nonblocking_open ? O_NONBLOCK : 0;

    SimpleRefHolder<FileLineIterator> fli(new FileLineIterator(xsink, path, encoding ? QEM.findCreate(encoding) : QCS_DEFAULT, eol, trim, flags));
    if (*xsink)
        return;

    if (fli->isTty() && runtime_check_parse_option(PO_NO_TERMINAL_IO)) {
        xsink->raiseException("ILLEGAL-EXPRESSION", "FileLineIterator::constructor() cannot be called with a TTY target when 'no-terminal-io' is set");
        return;
    }

    self->setPrivate(CID_FILELINEITERATOR, fli.release());
}

// FileLineIterator::copy() {}
static void FileLineIterator_copy(QoreObject* self, QoreObject* old, FileLineIterator* i, ExceptionSink* xsink) {
# 103 "QC_FileLineIterator.qpp"
SimpleRefHolder<FileLineIterator> fli(new FileLineIterator(xsink, *i));

    if (fli->isTty() && runtime_check_parse_option(PO_NO_TERMINAL_IO)) {
        xsink->raiseException("ILLEGAL-EXPRESSION", "FileLineIterator::copy() cannot be called with a TTY target when 'no-terminal-io' is set");
        return;
    }

    if (!*xsink)
        self->setPrivate(CID_FILELINEITERATOR, fli.release());
}

// string FileLineIterator::getEncoding(){}
static QoreValue FileLineIterator_getEncoding(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 209 "QC_FileLineIterator.qpp"
    return new QoreStringNode(i->getEncoding()->getCode());
}

// string FileLineIterator::getFileName(){}
static QoreValue FileLineIterator_getFileName(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 233 "QC_FileLineIterator.qpp"
    return i->getFileName()->stringRefSelf();
}

// string FileLineIterator::getLine(){}
static QoreValue FileLineIterator_getLine(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 170 "QC_FileLineIterator.qpp"
    return i->checkValid(xsink) ? 0 : i->getValue();
}

// string FileLineIterator::getValue(){}
static QoreValue FileLineIterator_getValue(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 151 "QC_FileLineIterator.qpp"
    return i->checkValid(xsink) ? 0 : i->getValue();
}

// hash<StatInfo> FileLineIterator::hstat(){}
static QoreValue FileLineIterator_hstat(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 262 "QC_FileLineIterator.qpp"
    return i->hstat(xsink);
}

// int FileLineIterator::index(){}
static QoreValue FileLineIterator_index(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 197 "QC_FileLineIterator.qpp"
    return i->index();
}

// bool FileLineIterator::isTty(){}
static QoreValue FileLineIterator_isTty(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 221 "QC_FileLineIterator.qpp"
    return i->isTty();
}

// bool FileLineIterator::next(){}
static QoreValue FileLineIterator_next(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 130 "QC_FileLineIterator.qpp"
    if (i->check(xsink))
        return false;
    return i->next(xsink);
}

// nothing FileLineIterator::reset(){}
static QoreValue FileLineIterator_reset(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 276 "QC_FileLineIterator.qpp"
    if (!i->check(xsink))
        i->reset(xsink);
    return QoreValue();
}

// list<auto> FileLineIterator::stat(){}
static QoreValue FileLineIterator_stat(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 247 "QC_FileLineIterator.qpp"
    return i->stat(xsink);
}

// bool FileLineIterator::valid(){}
static QoreValue FileLineIterator_valid(QoreObject* self, FileLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 183 "QC_FileLineIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitFileLineIteratorClass() {
    QC_FILELINEITERATOR = new QoreBuiltinClass("FileLineIterator", "::Qore::FileLineIterator", QDOM_FILESYSTEM);
    CID_FILELINEITERATOR = QC_FILELINEITERATOR->getID();
    QC_FILELINEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initFileLineIteratorClass(QoreNamespace& ns) {
    if (!QC_FILELINEITERATOR)
        preinitFileLineIteratorClass();

    // set parent class
    assert(QC_ABSTRACTLINEITERATOR);
    QC_FILELINEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTLINEITERATOR);

    // FileLineIterator::constructor(string path, *string encoding, *string eol, bool trim = True, *bool nonblocking_open) {}
    QC_FILELINEITERATOR->addConstructor(FileLineIterator_constructor_VsNsNsVbNb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 5, stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "eol", boolTypeInfo, QoreSimpleValue().assign(true), "trim", boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "nonblocking_open");

    // FileLineIterator::copy() {}
    QC_FILELINEITERATOR->setCopy((q_copy_t)FileLineIterator_copy);

    // string FileLineIterator::getEncoding(){}
    QC_FILELINEITERATOR->addMethod("getEncoding", (q_method_n_t)FileLineIterator_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string FileLineIterator::getFileName(){}
    QC_FILELINEITERATOR->addMethod("getFileName", (q_method_n_t)FileLineIterator_getFileName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string FileLineIterator::getLine(){}
    QC_FILELINEITERATOR->addMethod("getLine", (q_method_n_t)FileLineIterator_getLine, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string FileLineIterator::getValue(){}
    QC_FILELINEITERATOR->addMethod("getValue", (q_method_n_t)FileLineIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // hash<StatInfo> FileLineIterator::hstat(){}
    QC_FILELINEITERATOR->addMethod("hstat", (q_method_n_t)FileLineIterator_hstat, Public, QCF_CONSTANT, QDOM_DEFAULT, hashdeclStatInfo->getTypeInfo(false));

    // int FileLineIterator::index(){}
    QC_FILELINEITERATOR->addMethod("index", (q_method_n_t)FileLineIterator_index, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool FileLineIterator::isTty(){}
    QC_FILELINEITERATOR->addMethod("isTty", (q_method_n_t)FileLineIterator_isTty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool FileLineIterator::next(){}
    QC_FILELINEITERATOR->addMethod("next", (q_method_n_t)FileLineIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing FileLineIterator::reset(){}
    QC_FILELINEITERATOR->addMethod("reset", (q_method_n_t)FileLineIterator_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // list<auto> FileLineIterator::stat(){}
    QC_FILELINEITERATOR->addMethod("stat", (q_method_n_t)FileLineIterator_stat, Public, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo);

    // bool FileLineIterator::valid(){}
    QC_FILELINEITERATOR->addMethod("valid", (q_method_n_t)FileLineIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_FILELINEITERATOR;
}
