/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_SSLCertificate.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_SSLCertificate.h"
#include "qore/intern/QC_Serializable.h"

static QoreHashNode* SSLCertificate_serializer(const QoreObject& self, const QoreSSLCertificate& cert,
        QoreSerializationContext& context, ExceptionSink* xsink) {
    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);
    // we serialize with DER format as it's approx 2/3 the size of a PEM string
    SimpleRefHolder<BinaryNode> der(cert.getDER(xsink));
    if (*xsink) {
        return nullptr;
    }
    rv->setKeyValue("der", der.release(), xsink);
    return *xsink ? nullptr : rv.release();
}

static void SSLCertificate_deserializer(QoreObject& self, const QoreHashNode* sdata,
        QoreDeserializationContext& context, ExceptionSink* xsink) {
    if (sdata) {
        // we serialize with DER format as it's approx 2/3 the size of a PEM string
        QoreValue val = sdata->getKeyValue("der");
        if (val) {
            if (val.getType() != NT_BINARY) {
                xsink->raiseException("DESERIALIZATION-ERROR", "SSLCertificate serialization key 'der' has type "
                    "'%s'; expecting 'binary'", val.getTypeName());
                return;
            }
            const BinaryNode* der = val.get<const BinaryNode>();
            SimpleRefHolder<QoreSSLCertificate> qcert(new QoreSSLCertificate(der, xsink));
            if (!*xsink) {
                self.setPrivate(CID_SSLCERTIFICATE, qcert.release());
            }
            return;
        }
    }

    xsink->raiseException("DESERIALIZATION-ERROR", "missing serialization data for the SSLCertificate class");
}
/* Qore class Qore::SSLCertificate */

qore_classid_t CID_SSLCERTIFICATE;
QoreClass* QC_SSLCERTIFICATE;

// SSLCertificate::constructor(string pem) {}
static void SSLCertificate_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pem = HARD_QORE_VALUE_STRING(args, 0);
# 97 "QC_SSLCertificate.qpp"
// create assuming a certificate in PEM format
    SimpleRefHolder<QoreSSLCertificate> qc(new QoreSSLCertificate(pem, xsink));
    if (!*xsink) {
        self->setPrivate(CID_SSLCERTIFICATE, qc.release());
    }
}

// SSLCertificate::constructor(binary der) {}
static void SSLCertificate_constructor_Vy(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* der = HARD_QORE_VALUE_BINARY(args, 0);
# 115 "QC_SSLCertificate.qpp"
SimpleRefHolder<QoreSSLCertificate> qc(new QoreSSLCertificate(der, xsink));
    if (!*xsink) {
        self->setPrivate(CID_SSLCERTIFICATE, qc.release());
    }
}

// SSLCertificate::copy() {}
static void SSLCertificate_copy(QoreObject* self, QoreObject* old, QoreSSLCertificate* s, ExceptionSink* xsink) {
# 126 "QC_SSLCertificate.qpp"
SimpleRefHolder<BinaryNode> der(s->getDER(xsink));
    if (*xsink) {
        return;
    }
    SimpleRefHolder<QoreSSLCertificate> qcert(new QoreSSLCertificate(*der, xsink));
    if (!*xsink) {
        self->setPrivate(CID_SSLCERTIFICATE, qcert.release());
    }
}

// binary SSLCertificate::getDER(){}
static QoreValue SSLCertificate_getDER(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 163 "QC_SSLCertificate.qpp"
    return s->getDER(xsink);
}

// hash<auto> SSLCertificate::getInfo(){}
static QoreValue SSLCertificate_getInfo(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 185 "QC_SSLCertificate.qpp"
    return s->getInfo();
}

// hash<auto> SSLCertificate::getIssuerHash(){}
static QoreValue SSLCertificate_getIssuerHash(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 257 "QC_SSLCertificate.qpp"
    return s->getIssuerHash();
}

// date SSLCertificate::getNotAfterDate(){}
static QoreValue SSLCertificate_getNotAfterDate(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 317 "QC_SSLCertificate.qpp"
    return s->getNotAfterDate();
}

// date SSLCertificate::getNotBeforeDate(){}
static QoreValue SSLCertificate_getNotBeforeDate(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 305 "QC_SSLCertificate.qpp"
    return s->getNotBeforeDate();
}

// string SSLCertificate::getPEM(){}
static QoreValue SSLCertificate_getPEM(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 147 "QC_SSLCertificate.qpp"
    return s->getPEM(xsink);
}

// *binary SSLCertificate::getPublicKey(){}
static QoreValue SSLCertificate_getPublicKey(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 221 "QC_SSLCertificate.qpp"
    return s->getPublicKey();
}

// string SSLCertificate::getPublicKeyAlgorithm(){}
static QoreValue SSLCertificate_getPublicKeyAlgorithm(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 233 "QC_SSLCertificate.qpp"
    return s->getPublicKeyAlgorithm();
}

// hash<auto> SSLCertificate::getPurposeHash(){}
static QoreValue SSLCertificate_getPurposeHash(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 293 "QC_SSLCertificate.qpp"
    return s->getPurposeHash();
}

// int SSLCertificate::getSerialNumber(){}
static QoreValue SSLCertificate_getSerialNumber(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 269 "QC_SSLCertificate.qpp"
    return s->getSerialNumber();
}

// binary SSLCertificate::getSignature(){}
static QoreValue SSLCertificate_getSignature(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 197 "QC_SSLCertificate.qpp"
    return s->getSignature();
}

// string SSLCertificate::getSignatureType(){}
static QoreValue SSLCertificate_getSignatureType(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 209 "QC_SSLCertificate.qpp"
    return s->getSignatureType();
}

// hash<auto> SSLCertificate::getSubjectHash(){}
static QoreValue SSLCertificate_getSubjectHash(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 245 "QC_SSLCertificate.qpp"
    return s->getSubjectHash();
}

// int SSLCertificate::getVersion(){}
static QoreValue SSLCertificate_getVersion(QoreObject* self, QoreSSLCertificate* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 281 "QC_SSLCertificate.qpp"
    return s->getVersion();
}

DLLLOCAL void preinitSSLCertificateClass() {
    QC_SSLCERTIFICATE = new QoreBuiltinClass("SSLCertificate", "::Qore::SSLCertificate", QDOM_DEFAULT);
    CID_SSLCERTIFICATE = QC_SSLCERTIFICATE->getID();
    QC_SSLCERTIFICATE->setSystem();
}

DLLLOCAL QoreClass* initSSLCertificateClass(QoreNamespace& ns) {
    if (!QC_SSLCERTIFICATE)
        preinitSSLCertificateClass();

    // set serializer function
    QC_SSLCERTIFICATE->setSerializer(reinterpret_cast<q_serializer_t>(SSLCertificate_serializer));

    // set deserializer function
    QC_SSLCERTIFICATE->setDeserializer(SSLCertificate_deserializer);

    // set parent class
    assert(QC_SERIALIZABLE);
    QC_SSLCERTIFICATE->addBuiltinVirtualBaseClass(QC_SERIALIZABLE);

    // SSLCertificate::constructor(string pem) {}
    QC_SSLCERTIFICATE->addConstructor(SSLCertificate_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "pem");

    // SSLCertificate::constructor(binary der) {}
    QC_SSLCERTIFICATE->addConstructor(SSLCertificate_constructor_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "der");

    // SSLCertificate::copy() {}
    QC_SSLCERTIFICATE->setCopy((q_copy_t)SSLCertificate_copy);

    // binary SSLCertificate::getDER(){}
    QC_SSLCERTIFICATE->addMethod("getDER", (q_method_n_t)SSLCertificate_getDER, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo);

    // hash<auto> SSLCertificate::getInfo(){}
    QC_SSLCERTIFICATE->addMethod("getInfo", (q_method_n_t)SSLCertificate_getInfo, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // hash<auto> SSLCertificate::getIssuerHash(){}
    QC_SSLCERTIFICATE->addMethod("getIssuerHash", (q_method_n_t)SSLCertificate_getIssuerHash, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // date SSLCertificate::getNotAfterDate(){}
    QC_SSLCERTIFICATE->addMethod("getNotAfterDate", (q_method_n_t)SSLCertificate_getNotAfterDate, Public, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);

    // date SSLCertificate::getNotBeforeDate(){}
    QC_SSLCERTIFICATE->addMethod("getNotBeforeDate", (q_method_n_t)SSLCertificate_getNotBeforeDate, Public, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);

    // string SSLCertificate::getPEM(){}
    QC_SSLCERTIFICATE->addMethod("getPEM", (q_method_n_t)SSLCertificate_getPEM, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // *binary SSLCertificate::getPublicKey(){}
    QC_SSLCERTIFICATE->addMethod("getPublicKey", (q_method_n_t)SSLCertificate_getPublicKey, Public, QCF_CONSTANT, QDOM_DEFAULT, binaryOrNothingTypeInfo);

    // string SSLCertificate::getPublicKeyAlgorithm(){}
    QC_SSLCERTIFICATE->addMethod("getPublicKeyAlgorithm", (q_method_n_t)SSLCertificate_getPublicKeyAlgorithm, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // hash<auto> SSLCertificate::getPurposeHash(){}
    QC_SSLCERTIFICATE->addMethod("getPurposeHash", (q_method_n_t)SSLCertificate_getPurposeHash, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // int SSLCertificate::getSerialNumber(){}
    QC_SSLCERTIFICATE->addMethod("getSerialNumber", (q_method_n_t)SSLCertificate_getSerialNumber, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // binary SSLCertificate::getSignature(){}
    QC_SSLCERTIFICATE->addMethod("getSignature", (q_method_n_t)SSLCertificate_getSignature, Public, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo);

    // string SSLCertificate::getSignatureType(){}
    QC_SSLCERTIFICATE->addMethod("getSignatureType", (q_method_n_t)SSLCertificate_getSignatureType, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // hash<auto> SSLCertificate::getSubjectHash(){}
    QC_SSLCERTIFICATE->addMethod("getSubjectHash", (q_method_n_t)SSLCertificate_getSubjectHash, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // int SSLCertificate::getVersion(){}
    QC_SSLCERTIFICATE->addMethod("getVersion", (q_method_n_t)SSLCertificate_getVersion, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_SSLCERTIFICATE;
}
