/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_Serializable.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Serializable.h"
#include "qore/intern/BinaryOutputStream.h"
#include "qore/intern/BinaryInputStream.h"
#include "qore/intern/StringInputStream.h"
#include "qore/intern/QC_TimeZone.h"

static QoreHashNode* Serializable_serializer(const QoreObject& self, const TimeZoneData& z,
        QoreSerializationContext& context, ExceptionSink* xsink) {
    return nullptr;
}

static void Serializable_deserializer(QoreObject& self, const QoreHashNode* sdata,
        QoreDeserializationContext& context, ExceptionSink* xsink) {
    if (sdata) {
        xsink->raiseException("DESERIALIZATION-ERROR", "Deserialization data provided for the 'Serializable' class "
            "where none was expected");
        return;
    }
    self.setPrivate(CID_SERIALIZABLE, new QoreSerializable);
}
TypedHashDecl* init_hashdecl_SerializationInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("SerializationInfo", "::Qore::SerializationInfo");
    hd->addMember("_data", autoTypeInfo, QoreValue());
    hd->addMember("_index", qore_get_complex_hash_or_nothing_type(autoHashTypeInfo), QoreValue());
    hd->addMember("_modules", qore_get_complex_list_or_nothing_type(stringTypeInfo), QoreValue());
    hd->addMember("_sdata", qore_get_complex_hash_or_nothing_type(autoHashTypeInfo), QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_ObjectSerializationInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("ObjectSerializationInfo", "::Qore::ObjectSerializationInfo");
    hd->addMember("_class", stringTypeInfo, QoreValue());
    hd->addMember("_class_data", qore_get_complex_hash_or_nothing_type(qore_get_complex_hash_type(autoTypeInfo)), QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_IndexedObjectSerializationInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("IndexedObjectSerializationInfo", "::Qore::IndexedObjectSerializationInfo");
    hd->addMember("_index", stringTypeInfo, QoreValue());
    hd->addMember("_weak", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_HashSerializationInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("HashSerializationInfo", "::Qore::HashSerializationInfo");
    hd->addMember("_hash", stringTypeInfo, QoreValue());
    hd->addMember("_members", qore_get_complex_hash_or_nothing_type(autoTypeInfo), QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_ListSerializationInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("ListSerializationInfo", "::Qore::ListSerializationInfo");
    hd->addMember("_elements", autoListOrNothingTypeInfo, QoreValue());
    hd->addMember("_list", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::Serializable */

qore_classid_t CID_SERIALIZABLE;
QoreClass* QC_SERIALIZABLE;

// Serializable::constructor() {}
static void Serializable_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 173 "QC_Serializable.qpp"
self->setPrivate(CID_SERIALIZABLE, new QoreSerializable);
}

// Serializable::copy() {}
static void Serializable_copy(QoreObject* self, QoreObject* old, QoreSerializable* s, ExceptionSink* xsink) {
# 180 "QC_Serializable.qpp"
self->setPrivate(CID_SERIALIZABLE, new QoreSerializable);
}

// private:internal nothing Serializable::deserializeMembers(hash<auto> members){}
static QoreValue Serializable_deserializeMembers_C10hash_auto_(QoreObject* self, QoreSerializable* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 337 "QC_Serializable.qpp"
    
    return QoreValue();
}

// nothing Serializable::serialize(OutputStream stream, *int flags){}
static QoreValue Serializable_serialize_C12OutputStreamNi(QoreObject* self, QoreSerializable* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(stream, OutputStream, args, 0, CID_OUTPUTSTREAM, "Serializable::serialize()", "OutputStream", xsink);
    if (*xsink)
        return 0;
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 223 "QC_Serializable.qpp"
    ReferenceHolder<OutputStream> holder(stream, xsink);
    QoreSerializable::serialize(*self, *stream, flags, xsink);
    return QoreValue();
}

// binary Serializable::serialize(*int flags){}
static QoreValue Serializable_serialize_Ni(QoreObject* self, QoreSerializable* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 flags = HARD_QORE_VALUE_INT(args, 0);
# 246 "QC_Serializable.qpp"
    ReferenceHolder<BinaryOutputStream> stream(new BinaryOutputStream, xsink);
    QoreSerializable::serialize(*self, **stream, flags, xsink);
    return *xsink ? QoreValue() : stream->getData(xsink);
}

// private:internal *hash<auto> Serializable::serializeMembers(*hash<auto> members){}
static QoreValue Serializable_serializeMembers_C11_hash_auto_(QoreObject* self, QoreSerializable* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 295 "QC_Serializable.qpp"
    
    return QoreValue();
}

// hash<SerializationInfo> Serializable::serializeToData(*int flags){}
static QoreValue Serializable_serializeToData_Ni(QoreObject* self, QoreSerializable* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 flags = HARD_QORE_VALUE_INT(args, 0);
# 202 "QC_Serializable.qpp"
    return s->serializeToData(self, flags, xsink);
}

// static auto Serializable::deserialize(InputStream stream, *int flags){}
static QoreValue static_Serializable_deserialize_C11InputStreamNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(stream, InputStream, args, 0, CID_INPUTSTREAM, "Serializable::deserialize()", "InputStream", xsink);
    if (*xsink)
        return 0;
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 392 "QC_Serializable.qpp"
ReferenceHolder<InputStream> holder(stream, xsink);
    return QoreSerializable::deserialize(xsink, *stream, flags);
}

// static auto Serializable::deserialize(binary bin, *int flags){}
static QoreValue static_Serializable_deserialize_VyNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 410 "QC_Serializable.qpp"
ReferenceHolder<BinaryInputStream> stream(new BinaryInputStream(bin), xsink);
    return QoreSerializable::deserialize(xsink, **stream, flags);
}

// static auto Serializable::deserialize(string bin, *int flags){}
static QoreValue static_Serializable_deserialize_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* bin = HARD_QORE_VALUE_STRING(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 431 "QC_Serializable.qpp"
ReferenceHolder<StringInputStream> stream(new StringInputStream(bin), xsink);
    return QoreSerializable::deserialize(xsink, **stream, flags);
}

// static auto Serializable::deserialize(hash<SerializationInfo> data, *int flags){}
static QoreValue static_Serializable_deserialize_C23hash_SerializationInfo_Ni(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* data = HARD_QORE_VALUE_HASH(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 450 "QC_Serializable.qpp"
return QoreSerializable::deserialize(xsink, *data, flags);
}

// static hash<SerializationInfo> Serializable::deserializeToData(InputStream stream, *int flags){}
static QoreValue static_Serializable_deserializeToData_C11InputStreamNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(stream, InputStream, args, 0, CID_INPUTSTREAM, "Serializable::deserializeToData()", "InputStream", xsink);
    if (*xsink)
        return 0;
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 354 "QC_Serializable.qpp"
ReferenceHolder<InputStream> holder(stream, xsink);
    return QoreSerializable::deserializeToData(xsink, *stream, flags);
}

// static hash<SerializationInfo> Serializable::deserializeToData(binary bin, *int flags){}
static QoreValue static_Serializable_deserializeToData_VyNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 373 "QC_Serializable.qpp"
ReferenceHolder<BinaryInputStream> stream(new BinaryInputStream(bin), xsink);
    return QoreSerializable::deserializeToData(xsink, **stream, flags);
}

// static nothing Serializable::serialize(auto val, OutputStream stream, *int flags){}
static QoreValue static_Serializable_serialize_VAC12OutputStreamNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    HARD_QORE_VALUE_OBJ_DATA(stream, OutputStream, args, 1, CID_OUTPUTSTREAM, "Serializable::serialize()", "OutputStream", xsink);
    if (*xsink)
        return 0;
    int64 flags = HARD_QORE_VALUE_INT(args, 2);
# 495 "QC_Serializable.qpp"
ReferenceHolder<OutputStream> holder(stream, xsink);
    QoreSerializable::serialize(val, *stream, flags, xsink);
    return QoreValue();
}

// static binary Serializable::serialize(auto val, *int flags){}
static QoreValue static_Serializable_serialize_VANi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 519 "QC_Serializable.qpp"
ReferenceHolder<BinaryOutputStream> stream(new BinaryOutputStream, xsink);
    QoreSerializable::serialize(val, **stream, flags, xsink);
    return *xsink ? QoreValue() : stream->getData(xsink);
}

// static hash<SerializationInfo> Serializable::serializeToData(auto val, *int flags){}
static QoreValue static_Serializable_serializeToData_VANi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
# 473 "QC_Serializable.qpp"
return QoreSerializable::serializeToData(val, flags, xsink);
}

DLLLOCAL void preinitSerializableClass() {
    QC_SERIALIZABLE = new QoreBuiltinClass("Serializable", "::Qore::Serializable", QDOM_DEFAULT);
    CID_SERIALIZABLE = QC_SERIALIZABLE->getID();
    QC_SERIALIZABLE->setSystem();
}

DLLLOCAL QoreClass* initSerializableClass(QoreNamespace& ns) {
    if (!QC_SERIALIZABLE)
        preinitSerializableClass();

    // set serializer function
    QC_SERIALIZABLE->setSerializer(reinterpret_cast<q_serializer_t>(Serializable_serializer));

    // set deserializer function
    QC_SERIALIZABLE->setDeserializer(Serializable_deserializer);

    // Serializable::constructor() {}
    QC_SERIALIZABLE->addConstructor(Serializable_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Serializable::copy() {}
    QC_SERIALIZABLE->setCopy((q_copy_t)Serializable_copy);

    // private:internal nothing Serializable::deserializeMembers(hash<auto> members){}
    QC_SERIALIZABLE->addMethod("deserializeMembers", (q_method_n_t)Serializable_deserializeMembers_C10hash_auto_, Internal, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "members");

    // nothing Serializable::serialize(OutputStream stream, *int flags){}
    QC_SERIALIZABLE->addMethod("serialize", (q_method_n_t)Serializable_serialize_C12OutputStreamNi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "stream", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // binary Serializable::serialize(*int flags){}
    QC_SERIALIZABLE->addMethod("serialize", (q_method_n_t)Serializable_serialize_Ni, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 1, bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // private:internal *hash<auto> Serializable::serializeMembers(*hash<auto> members){}
    QC_SERIALIZABLE->addMethod("serializeMembers", (q_method_n_t)Serializable_serializeMembers_C11_hash_auto_, Internal, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashOrNothingTypeInfo, 1, autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "members");

    // hash<SerializationInfo> Serializable::serializeToData(*int flags){}
    QC_SERIALIZABLE->addMethod("serializeToData", (q_method_n_t)Serializable_serializeToData_Ni, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSerializationInfo->getTypeInfo(false), 1, bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static auto Serializable::deserialize(InputStream stream, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("deserialize", (q_func_n_t)static_Serializable_deserialize_C11InputStreamNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "stream", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static auto Serializable::deserialize(binary bin, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("deserialize", (q_func_n_t)static_Serializable_deserialize_VyNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static auto Serializable::deserialize(string bin, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("deserialize", (q_func_n_t)static_Serializable_deserialize_VsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "bin", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static auto Serializable::deserialize(hash<SerializationInfo> data, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("deserialize", (q_func_n_t)static_Serializable_deserialize_C23hash_SerializationInfo_Ni, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, hashdeclSerializationInfo->getTypeInfo(false), QORE_PARAM_NO_ARG, "data", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static hash<SerializationInfo> Serializable::deserializeToData(InputStream stream, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("deserializeToData", (q_func_n_t)static_Serializable_deserializeToData_C11InputStreamNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSerializationInfo->getTypeInfo(false), 2, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "stream", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static hash<SerializationInfo> Serializable::deserializeToData(binary bin, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("deserializeToData", (q_func_n_t)static_Serializable_deserializeToData_VyNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSerializationInfo->getTypeInfo(false), 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static nothing Serializable::serialize(auto val, OutputStream stream, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("serialize", (q_func_n_t)static_Serializable_serialize_VAC12OutputStreamNi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, autoTypeInfo, QORE_PARAM_NO_ARG, "val", QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "stream", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static binary Serializable::serialize(auto val, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("serialize", (q_func_n_t)static_Serializable_serialize_VANi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, autoTypeInfo, QORE_PARAM_NO_ARG, "val", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");

    // static hash<SerializationInfo> Serializable::serializeToData(auto val, *int flags){}
    QC_SERIALIZABLE->addStaticMethod("serializeToData", (q_func_n_t)static_Serializable_serializeToData_VANi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSerializationInfo->getTypeInfo(false), 2, autoTypeInfo, QORE_PARAM_NO_ARG, "val", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "flags");
    ns.addConstant("QSF_INCLUDE_STATIC_CLASS_VARS", ((int64)QSF_INCLUDE_STATIC_CLASS_VARS));
# 64 "QC_Serializable.qpp"
    ns.addConstant("QDF_MERGE_STATIC_CLASS_VARS", ((int64)QDF_MERGE_STATIC_CLASS_VARS));
    ns.addConstant("QDF_STATIC_CLASS_VARS", ((int64)QDF_STATIC_CLASS_VARS));

    return QC_SERIALIZABLE;
}
