/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_Socket.qpp

    Socket class definition

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore/Qore.h"
#include "qore/intern/QC_Socket.h"
#include "qore/intern/QC_SocketPollOperation.h"
#include "qore/intern/QC_Queue.h"
#include "qore/intern/ssl_constants.h"
#include "qore/intern/qore_socket_private.h"
#include "qore/QoreSSLCertificate.h"
#include "qore/QoreSSLPrivateKey.h"

#include <cerrno>
#include <cstring>

static void hash_set_int_key(QoreHashNode& h, int k, const char* str) {
    char buf[15];
    sprintf(buf, "%d", k);
    h.setKeyValue(buf, new QoreStringNode(str), nullptr);
}

static QoreHashNode* get_network_address_family_map() {
    QoreHashNode* rv = new QoreHashNode(autoTypeInfo);

    hash_set_int_key(*rv, AF_INET, "AF_INET");
    hash_set_int_key(*rv, AF_INET6, "AF_INET6");
    hash_set_int_key(*rv, AF_UNIX, "AF_UNIX");
    hash_set_int_key(*rv, AF_UNSPEC, "AF_UNSPEC");

    return rv;
}

static int64 checkOpenResult(int rc, const char *method_name, ExceptionSink *xsink) {
    if (*xsink)
        return 0;
    if (rc == -2) {
        xsink->raiseException("SOCKET-NOT-OPEN", "socket must be open before Socket::%s() call", method_name);
        return 0;
    }
    return rc;
}

static int check_response_code(int status_code, ExceptionSink* xsink) {
    if (status_code < 100 || status_code >= 600) {
        xsink->raiseException("SOCKET-SENDHTTPRESPONSE-STATUS-ERROR", "expecting valid HTTP status code " \
            "between 100 and 599 as first parameter of Socket::sendHTTPResponse() call, got value %d instead",
            status_code);
        return -1;
    }
    return 0;
}
TypedHashDecl* init_hashdecl_SseMessageInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("SseMessageInfo", "::Qore::SseMessageInfo");
    hd->addMember("comment", stringTypeInfo, QoreValue());
    hd->addMember("data", autoTypeInfo, QoreValue());
    hd->addMember("event", stringTypeInfo, QoreValue());
    hd->addMember("id", autoTypeInfo, QoreValue());
    hd->addMember("retry", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_SocketPollInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("SocketPollInfo", "::Qore::SocketPollInfo");
    hd->addMember("events", bigIntTypeInfo, QoreValue());
    hd->addMember("socket", QC_ABSTRACTPOLLABLEIOOBJECT->getTypeInfo(), QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::Socket */

#include "qore/intern/QoreClassIntern.h"
qore_classid_t CID_SOCKET;
QoreClass* QC_SOCKET;

// Socket Socket::accept(){}
static QoreValue Socket_accept(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1211 "QC_Socket.qpp"
    QoreSocketObject *n = s->accept(-1, xsink);
    if (!n) {
        assert(*xsink);
        return QoreValue();
    }

    assert(n);

    // ensure that a socket object is returned (and not a subclass)
    QoreObject *ns = new QoreObject(QC_SOCKET, getProgram(), n);

    // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should
    // be used in the future)
    my_socket_priv::setAccept(*n, ns);

    return ns;
}

// *Socket Socket::accept(timeout timeout_ms){}
static QoreValue Socket_accept_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1247 "QC_Socket.qpp"
    QoreSocketObject *n = s->accept(timeout_ms, xsink);
    if (!n)
        return QoreValue();

    // ensure that a socket object is returned (and not a subclass)
    QoreObject* ns = new QoreObject(QC_SOCKET, getProgram(), n);

    // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should be used in the future)
    my_socket_priv::setAccept(*n, ns);

    return ns;
}

// nothing Socket::acceptAllCertificates(bool accept_all = True){}
static QoreValue Socket_acceptAllCertificates_Vb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool accept_all = HARD_QORE_VALUE_BOOL(args, 0);
# 3784 "QC_Socket.qpp"
    s->acceptAllCertificates(accept_all);
    return QoreValue();
}

// Socket Socket::acceptSSL(){}
static QoreValue Socket_acceptSSL(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1279 "QC_Socket.qpp"
    QoreSocketObject *n = s->acceptSSL(xsink, -1);
    if (!n) {
        assert(*xsink);
        return QoreValue();
    }

    // ensure that a socket object is returned (and not a subclass)
    QoreObject *ns = new QoreObject(QC_SOCKET, getProgram(), n);

    // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should be
    // used in the future)
    my_socket_priv::setAccept(*n, ns);

    return ns;
}

// *Socket Socket::acceptSSL(timeout timeout_ms){}
static QoreValue Socket_acceptSSL_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1313 "QC_Socket.qpp"
    QoreSocketObject* n = s->acceptSSL(xsink, timeout_ms);
    if (!n)
        return QoreValue();

    // ensure that a socket object is returned (and not a subclass)
    QoreObject* ns = new QoreObject(QC_SOCKET, getProgram(), n);

    // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should be used in the future)
    my_socket_priv::setAccept(*n, ns);

    return ns;
}

// int Socket::bind(string str, softbool reuseaddr = False){}
static QoreValue Socket_bind_Vsvb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    bool reuseaddr = HARD_QORE_VALUE_BOOL(args, 1);
# 1110 "QC_Socket.qpp"
    return s->bind(str->c_str(), reuseaddr);
}

// int Socket::bind(int port, softbool reuseaddr = False){}
static QoreValue Socket_bind_Vivb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 port = HARD_QORE_VALUE_INT(args, 0);
    bool reuseaddr = HARD_QORE_VALUE_BOOL(args, 1);
# 1129 "QC_Socket.qpp"
    return s->bind(port, reuseaddr);
}

// nothing Socket::bindINET(*string iface, *softstring service, softbool reuseaddr = False, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_bindINET_Nsnsvbvivivi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* iface = get_param_value(args, 0).get<const QoreStringNode>();
    const QoreStringNode* service = get_param_value(args, 1).get<const QoreStringNode>();
    bool reuseaddr = HARD_QORE_VALUE_BOOL(args, 2);
    int64 family = HARD_QORE_VALUE_INT(args, 3);
    int64 socktype = HARD_QORE_VALUE_INT(args, 4);
    int64 protocol = HARD_QORE_VALUE_INT(args, 5);
# 1184 "QC_Socket.qpp"
        if ((!iface || iface->empty())
        && (!service || service->empty()))
        return xsink->raiseException("SOCKET-BIND-ERROR", "both interace (first parameter) and service (second "
            "parameter) were either not present or empty strings; at least one of the first 2 parameters must be "
            "present for a successful call to Socket::bindINET()");

    s->bindINET(iface ? iface->c_str() : nullptr, service ? service->c_str() : nullptr, reuseaddr, family,
        socktype, protocol, xsink);
    return QoreValue();
}

// nothing Socket::bindUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_bindUNIX_Vsvivi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 socktype = HARD_QORE_VALUE_INT(args, 1);
    int64 protocol = HARD_QORE_VALUE_INT(args, 2);
# 1150 "QC_Socket.qpp"
    s->bindUNIX(path->c_str(), socktype, protocol, xsink);
    return QoreValue();
}

// bool Socket::captureRemoteCertificates(bool set = True){}
static QoreValue Socket_captureRemoteCertificates_Vb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool set = HARD_QORE_VALUE_BOOL(args, 0);
# 3821 "QC_Socket.qpp"
    return s->captureRemoteCertificates(set);
}

// nothing Socket::clearStats(){}
static QoreValue Socket_clearStats(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3710 "QC_Socket.qpp"
    s->clearStats();
    return QoreValue();
}

// nothing Socket::clearWarningQueue(){}
static QoreValue Socket_clearWarningQueue(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3629 "QC_Socket.qpp"
    s->clearWarningQueue(xsink);
    return QoreValue();
}

// int Socket::close(){}
static QoreValue Socket_close(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3096 "QC_Socket.qpp"
    return s->close();
}

// nothing Socket::connect(string target, timeout timeout_ms = -1){}
static QoreValue Socket_connect_VsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 944 "QC_Socket.qpp"
    s->connect(target->c_str(), timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::connectINET(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectINET_VsvsVtvivivi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* service = HARD_QORE_VALUE_STRING(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
    int64 family = HARD_QORE_VALUE_INT(args, 3);
    int64 socktype = HARD_QORE_VALUE_INT(args, 4);
    int64 protocol = HARD_QORE_VALUE_INT(args, 5);
# 973 "QC_Socket.qpp"
    s->connectINET2(host->c_str(), service->c_str(), family, socktype, protocol, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::connectINETSSL(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectINETSSL_VsvsVtvivivi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* service = HARD_QORE_VALUE_STRING(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
    int64 family = HARD_QORE_VALUE_INT(args, 3);
    int64 socktype = HARD_QORE_VALUE_INT(args, 4);
    int64 protocol = HARD_QORE_VALUE_INT(args, 5);
# 1060 "QC_Socket.qpp"
    s->connectINET2SSL(xsink, host->c_str(), service->c_str(), family, socktype, protocol, timeout_ms);
    return QoreValue();
}

// nothing Socket::connectSSL(string target, timeout timeout_ms = -1){}
static QoreValue Socket_connectSSL_VsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1030 "QC_Socket.qpp"
    s->connectSSL(xsink, target->c_str(), timeout_ms);
    return QoreValue();
}

// nothing Socket::connectUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectUNIX_Vsvivi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 socktype = HARD_QORE_VALUE_INT(args, 1);
    int64 protocol = HARD_QORE_VALUE_INT(args, 2);
# 997 "QC_Socket.qpp"
    s->connectUNIX(path->c_str(), socktype, protocol, xsink);
    return QoreValue();
}

// nothing Socket::connectUNIXSSL(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectUNIXSSL_Vsvivi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 socktype = HARD_QORE_VALUE_INT(args, 1);
    int64 protocol = HARD_QORE_VALUE_INT(args, 2);
# 1085 "QC_Socket.qpp"
    s->connectUNIXSSL(xsink, path->c_str(), socktype, protocol);
    return QoreValue();
}

// Socket::constructor() {}
static void Socket_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 534 "QC_Socket.qpp"
self->setPrivate(CID_SOCKET, new QoreSocketObject);
}

// Socket::copy() {}
static void Socket_copy(QoreObject* self, QoreObject* old, QoreSocketObject* s, ExceptionSink* xsink) {
# 542 "QC_Socket.qpp"
self->setPrivate(CID_SOCKET, new QoreSocketObject);
}

// Socket::destructor() {}
static void Socket_destructor(QoreObject* self, QoreSocketObject* s, ExceptionSink* xsink) {
# 549 "QC_Socket.qpp"
s->invalidate(xsink);
    s->deref(xsink);
}

// bool Socket::getAcceptAllCertificates(){}
static QoreValue Socket_getAcceptAllCertificates(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3803 "QC_Socket.qpp"
    return s->getAcceptAllCertificates();
}

// string Socket::getCharset(){}
static QoreValue Socket_getCharset(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3231 "QC_Socket.qpp"
    return new QoreStringNode(s->getEncoding()->getCode());
}

// int Socket::getConnectionId(){}
static QoreValue Socket_getConnectionId(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3853 "QC_Socket.qpp"
    return s->getConnectionId();
}

// string Socket::getEncoding(){}
static QoreValue Socket_getEncoding(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3216 "QC_Socket.qpp"
    return new QoreStringNode(s->getEncoding()->getCode());
}

// bool Socket::getNoDelay(){}
static QoreValue Socket_getNoDelay(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3535 "QC_Socket.qpp"
    return s->getNoDelay();
}

// hash<auto> Socket::getPeerInfo(bool host_lookup = True){}
static QoreValue Socket_getPeerInfo_Vb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 3556 "QC_Socket.qpp"
    return s->getPeerInfo(xsink, host_lookup);
}

// int Socket::getPort(){}
static QoreValue Socket_getPort(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3137 "QC_Socket.qpp"
    return s->getPort();
}

// int Socket::getRecvTimeout(){}
static QoreValue Socket_getRecvTimeout(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3209 "QC_Socket.qpp"
    return s->getRecvTimeout();
}

// *SSLCertificate Socket::getRemoteCertificate(){}
static QoreValue Socket_getRemoteCertificate(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3837 "QC_Socket.qpp"
    return s->getRemoteCertificate();
}

// *string Socket::getSSLCipherName(){}
static QoreValue Socket_getSSLCipherName(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3309 "QC_Socket.qpp"
    const char *str = s->getSSLCipherName();
    return str ? new QoreStringNode(str) : 0;
}

// *string Socket::getSSLCipherVersion(){}
static QoreValue Socket_getSSLCipherVersion(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3323 "QC_Socket.qpp"
    const char *str = s->getSSLCipherVersion();
    return str ? new QoreStringNode(str) : 0;
}

// int Socket::getSendTimeout(){}
static QoreValue Socket_getSendTimeout(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3197 "QC_Socket.qpp"
    return s->getSendTimeout();
}

// int Socket::getSocket(){}
static QoreValue Socket_getSocket(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3149 "QC_Socket.qpp"
    return s->getSocket();
}

// hash<auto> Socket::getSocketInfo(bool host_lookup = True){}
static QoreValue Socket_getSocketInfo_Vb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 3577 "QC_Socket.qpp"
    return s->getSocketInfo(xsink, host_lookup);
}

// int Socket::getSslVerifyMode(){}
static QoreValue Socket_getSslVerifyMode(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3763 "QC_Socket.qpp"
    return s->getSslVerifyMode();
}

// hash<auto> Socket::getUsageInfo(){}
static QoreValue Socket_getUsageInfo(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3696 "QC_Socket.qpp"
    return s->getUsageInfo();
}

// bool Socket::isDataAvailable(timeout timeout_ms = 0){}
static QoreValue Socket_isDataAvailable_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 3262 "QC_Socket.qpp"
    return s->isDataAvailable(xsink, (int)timeout_ms);
}

// bool Socket::isOpen(){}
static QoreValue Socket_isOpen(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3296 "QC_Socket.qpp"
    return s->isOpen();
}

// bool Socket::isSecure(){}
static QoreValue Socket_isSecure(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3336 "QC_Socket.qpp"
    return s->isSecure();
}

// bool Socket::isWriteFinished(timeout timeout_ms = 0){}
static QoreValue Socket_isWriteFinished_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 3284 "QC_Socket.qpp"
    return s->isWriteFinished(xsink, (int)timeout_ms);
}

// int Socket::listen(int backlog = 20){}
static QoreValue Socket_listen_Vi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 backlog = HARD_QORE_VALUE_INT(args, 0);
# 1341 "QC_Socket.qpp"
    return checkOpenResult(s->listen((int)backlog), "listen", xsink);
}

// bool Socket::pendingHttpChunkedBody(){}
static QoreValue Socket_pendingHttpChunkedBody(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3722 "QC_Socket.qpp"
    return s->pendingHttpChunkedBody();
}

// hash<auto> Socket::readHTTPChunk(timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunk_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2938 "QC_Socket.qpp"
    // when rc = -3 it's a timeout, but rv will be NULL anyway, so we do nothing
    return s->readHttpChunk(timeout_ms, xsink);
}

// hash<auto> Socket::readHTTPChunkedBody(timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBody_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2879 "QC_Socket.qpp"
    // when rc = -3 it's a timeout, but rv will be NULL anyway, so we do nothing
    return s->readHTTPChunkedBody(timeout_ms, xsink);
}

// hash<auto> Socket::readHTTPChunkedBodyBinary(timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyBinary_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2906 "QC_Socket.qpp"
    // when rc = -3 it's a timeout, but rv will be NULL anyway, so we do nothing
    return s->readHTTPChunkedBodyBinary(timeout_ms, xsink);
}

// nothing Socket::readHTTPChunkedBodyBinaryWithCallback(code rcb, timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyBinaryWithCallback_VcVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 3037 "QC_Socket.qpp"
    s->readHTTPChunkedBodyBinaryWithCallback(*rcb, self, timeout_ms, xsink);
    return QoreValue();
}

// hash<auto> Socket::readHTTPChunkedBodyToOutputStream(Qore::OutputStream os, timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyToOutputStream_C12OutputStreamVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "Socket::readHTTPChunkedBodyToOutputStream()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 2966 "QC_Socket.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
    return s->readHTTPChunkedBodyToOutputStream(os, timeout_ms, xsink);
}

// nothing Socket::readHTTPChunkedBodyWithCallback(code rcb, timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyWithCallback_VcVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 3000 "QC_Socket.qpp"
    s->readHTTPChunkedBodyWithCallback(*rcb, self, timeout_ms, xsink);
    return QoreValue();
}

// hash<auto> Socket::readHTTPHeader(timeout timeout_ms = -1, *reference<hash<auto>> info){}
static QoreValue Socket_readHTTPHeader_VtC22_reference_hash_auto__(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
    const ReferenceNode* info = get_param_value(args, 1).get<const ReferenceNode>();
# 2824 "QC_Socket.qpp"
    OptHashRefHelper ohrh(info, xsink);
    return s->readHTTPHeader(xsink, *ohrh, timeout_ms);
}

// string Socket::readHTTPHeaderString(timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPHeaderString_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2853 "QC_Socket.qpp"
    return s->readHTTPHeaderString(xsink, timeout_ms);
}

// hash<SseMessageInfo> Socket::readServerSentEvent(*string content_encoding, timeout timeout_ms = -1){}
static QoreValue Socket_readServerSentEvent_NsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* content_encoding = get_param_value(args, 0).get<const QoreStringNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 3058 "QC_Socket.qpp"
    return s->readServerSentEvent(xsink, content_encoding, timeout_ms);
}

// hash<SseMessageInfo> Socket::readServerSentEvent(timeout timeout_ms = -1){}
static QoreValue Socket_readServerSentEvent_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 3078 "QC_Socket.qpp"
    return s->readServerSentEvent(xsink, nullptr, timeout_ms);
}

// string Socket::recv(softint size = 0, timeout timeout_ms = -1){}
static QoreValue Socket_recv_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1837 "QC_Socket.qpp"
    if (size > 0)
        return s->recv(size, timeout_ms, xsink);

    QoreStringNode* str = s->recv(timeout_ms, xsink);
    return !*xsink && !str ? new QoreStringNode : str;
}

// binary Socket::recvBinary(softint size = 0, timeout timeout_ms = -1){}
static QoreValue Socket_recvBinary_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1869 "QC_Socket.qpp"
    if (size > 0)
        return s->recvBinary(size, timeout_ms, xsink);

    BinaryNode* str = s->recvBinary(timeout_ms, xsink);
    return !*xsink && !str ? new BinaryNode : str;
}

// nothing Socket::recvToOutputStream(Qore::OutputStream os, softint size = -1, timeout timeout_ms = -1){}
static QoreValue Socket_recvToOutputStream_C12OutputStreamviVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "Socket::recvToOutputStream()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    int64 size = HARD_QORE_VALUE_INT(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
# 1901 "QC_Socket.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
    s->recvToOutputStream(os, size, timeout_ms, xsink);
    return QoreValue();
}

// int Socket::recvi1(timeout timeout_ms = -1){}
static QoreValue Socket_recvi1_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1929 "QC_Socket.qpp"
    char b;
    s->recvi1(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvi2(timeout timeout_ms = -1){}
static QoreValue Socket_recvi2_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1958 "QC_Socket.qpp"
    short b;
    s->recvi2(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvi2LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvi2LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2050 "QC_Socket.qpp"
    short b;
    s->recvi2LSB(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvi4(timeout timeout_ms = -1){}
static QoreValue Socket_recvi4_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1992 "QC_Socket.qpp"
    int b;
    s->recvi4(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvi4LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvi4LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2079 "QC_Socket.qpp"
    int b;
    s->recvi4LSB(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvi8(timeout timeout_ms = -1){}
static QoreValue Socket_recvi8_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2021 "QC_Socket.qpp"
    int64 b;
    s->recvi8(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvi8LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvi8LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2108 "QC_Socket.qpp"
    int64 b;
    s->recvi8LSB(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvu1(timeout timeout_ms = -1){}
static QoreValue Socket_recvu1_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2137 "QC_Socket.qpp"
    unsigned char b;
    s->recvu1(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvu2(timeout timeout_ms = -1){}
static QoreValue Socket_recvu2_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2166 "QC_Socket.qpp"
    unsigned short b;
    s->recvu2(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvu2LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvu2LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2224 "QC_Socket.qpp"
    unsigned short b;
    s->recvu2LSB(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvu4(timeout timeout_ms = -1){}
static QoreValue Socket_recvu4_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2195 "QC_Socket.qpp"
    unsigned int b;
    s->recvu4(timeout_ms, &b, xsink);
    return b;
}

// int Socket::recvu4LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvu4LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2253 "QC_Socket.qpp"
    unsigned b;
    s->recvu4LSB(timeout_ms, &b, xsink);
    return b;
}

// int Socket::send(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_send_VyVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1373 "QC_Socket.qpp"
    return s->send(bin, timeout_ms, xsink);
}

// int Socket::send(string str, timeout timeout_ms = -1){}
static QoreValue Socket_send_VsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1406 "QC_Socket.qpp"
    return s->send(*str, timeout_ms, xsink);
}

// nothing Socket::send2(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_send2_VyVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1497 "QC_Socket.qpp"
    s->send(bin, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::send2(string str, timeout timeout_ms = -1){}
static QoreValue Socket_send2_VsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1527 "QC_Socket.qpp"
    s->send(*str, timeout_ms, xsink);
    return QoreValue();
}

// int Socket::sendBinary(string str, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary_VsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1437 "QC_Socket.qpp"
    return s->send(str->c_str(), str->strlen(), timeout_ms, xsink);
}

// int Socket::sendBinary(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary_VyVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1468 "QC_Socket.qpp"
    return s->send(bin, timeout_ms, xsink);
}

// nothing Socket::sendBinary2(string str, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary2_VsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1555 "QC_Socket.qpp"
    s->send(str->c_str(), str->strlen(), timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendBinary2(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary2_VyVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1583 "QC_Socket.qpp"
    s->send(bin, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendFromInputStream(Qore::InputStream input_stream, softint size = -1, timeout timeout_ms = -1){}
static QoreValue Socket_sendFromInputStream_C11InputStreamviVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(input_stream, InputStream, args, 0, CID_INPUTSTREAM, "Socket::sendFromInputStream()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    int64 size = HARD_QORE_VALUE_INT(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
# 1610 "QC_Socket.qpp"
        SimpleRefHolder<InputStream> isHolder(input_stream);
    s->sendFromInputStream(input_stream, size, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendHTTPChunkedBodyFromInputStream(Qore::InputStream input_stream, int max_chunk_size = 4096, timeout timeout_ms = -1, *code tcb){}
static QoreValue Socket_sendHTTPChunkedBodyFromInputStream_C11InputStreamViVtNc(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(input_stream, InputStream, args, 0, CID_INPUTSTREAM, "Socket::sendHTTPChunkedBodyFromInputStream()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    int64 max_chunk_size = HARD_QORE_VALUE_INT(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
    const ResolvedCallReferenceNode* tcb = get_param_value(args, 3).get<const ResolvedCallReferenceNode>();
# 2738 "QC_Socket.qpp"
        SimpleRefHolder<InputStream> isHolder(input_stream);
    if (max_chunk_size < 1) {
        xsink->raiseException("HTTP-CHUNK-ERROR", "the max chunk size must be greater than 0; value given is: " QLLD,
            max_chunk_size);
        return QoreValue();
    }
    s->sendHTTPChunkedBodyFromInputStream(input_stream, max_chunk_size, timeout_ms, tcb, xsink);
    return QoreValue();
}

// nothing Socket::sendHTTPChunkedBodyTrailer(*hash<auto> trailer, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPChunkedBodyTrailer_C11_hash_auto_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* trailer = get_param_value(args, 0).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 2772 "QC_Socket.qpp"
    s->sendHTTPChunkedBodyTrailer(trailer, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash<auto> headers, string body, *reference<hash> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPMessage_VsVsVsC10hash_auto_VsC16_reference_hash_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 2296 "QC_Socket.qpp"
        OptHashRefHelper ohrh(info, xsink);
    s->sendHTTPMessage(xsink, *ohrh, method->c_str(), path->c_str(), http_version->c_str(), headers, *body,
        QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash<auto> headers, *binary body, *reference<hash<auto>> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPMessage_VsVsVsC10hash_auto_NyC22_reference_hash_auto__Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const BinaryNode* body = get_param_value(args, 4).get<const BinaryNode>();
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 2336 "QC_Socket.qpp"
        OptHashRefHelper ohrh(info, xsink);
    s->sendHTTPMessage(xsink, *ohrh, method->c_str(), path->c_str(), http_version->c_str(), headers,
        body ? body->getPtr() : nullptr, body ? body->size() : 0, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPMessageWithCallback(code scb, string method, string path, string http_version, hash<auto> headers, *reference<hash<auto>> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPMessageWithCallback_VcVsVsVsC10hash_auto_C22_reference_hash_auto__Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 3);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 2372 "QC_Socket.qpp"
        OptHashRefHelper ohrh(info, xsink);
    s->sendHTTPMessageWithCallback(xsink, *ohrh, method->c_str(), path->c_str(), http_version->c_str(), headers, *scb,
        QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, string body, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponse_viVsVsC10hash_auto_VsVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 status_code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 4);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
# 2404 "QC_Socket.qpp"
        if (check_response_code(status_code, xsink)) {
        return QoreValue();
    }

    s->sendHTTPResponse(xsink, nullptr, status_code, status_desc->c_str(), http_version->c_str(), headers, *body,
        QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, string body, *reference<hash<auto>> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponse_viVsVsC10hash_auto_VsC22_reference_hash_auto__Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 status_code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 2453 "QC_Socket.qpp"
        if (check_response_code(status_code, xsink)) {
        return QoreValue();
    }

    OptHashRefHelper ohrh(info, xsink);
    s->sendHTTPResponse(xsink, *ohrh, status_code, status_desc->c_str(), http_version->c_str(), headers, *body,
        QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, *binary body, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponse_viVsVsC10hash_auto_NyVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 status_code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const BinaryNode* body = get_param_value(args, 4).get<const BinaryNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
# 2494 "QC_Socket.qpp"
        if (check_response_code(status_code, xsink)) {
        return QoreValue();
    }

    s->sendHTTPResponse(xsink, nullptr, status_code, status_desc->c_str(), http_version->c_str(), headers,
        body ? body->getPtr() : nullptr, body ? body->size() : 0, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, *binary body, *reference<hash<auto>> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponse_viVsVsC10hash_auto_NyC22_reference_hash_auto__Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 status_code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const BinaryNode* body = get_param_value(args, 4).get<const BinaryNode>();
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 2539 "QC_Socket.qpp"
        if (check_response_code(status_code, xsink)) {
        return QoreValue();
    }

    OptHashRefHelper ohrh(info, xsink);
    s->sendHTTPResponse(xsink, *ohrh, status_code, status_desc->c_str(), http_version->c_str(), headers,
        body ? body->getPtr() : nullptr, body ? body->size() : 0, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, Qore::InputStream input_stream, int max_chunk_size = 4096, *reference<hash<auto>> info, timeout timeout_ms = -1, *code tcb){}
static QoreValue Socket_sendHTTPResponse_viVsVsC10hash_auto_C11InputStreamViC22_reference_hash_auto__VtNc(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 status_code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    HARD_QORE_VALUE_OBJ_DATA(input_stream, InputStream, args, 4, CID_INPUTSTREAM, "Socket::sendHTTPResponse()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    int64 max_chunk_size = HARD_QORE_VALUE_INT(args, 5);
    const ReferenceNode* info = get_param_value(args, 6).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 7);
    const ResolvedCallReferenceNode* tcb = get_param_value(args, 8).get<const ResolvedCallReferenceNode>();
# 2593 "QC_Socket.qpp"
        SimpleRefHolder<InputStream> isHolder(input_stream);
    if (check_response_code(status_code, xsink)) {
        return QoreValue();
    }
    if (max_chunk_size < 1) {
        xsink->raiseException("HTTP-CHUNK-ERROR", "the max chunk size must be greater than 0; value given is: " QLLD,
            max_chunk_size);
        return QoreValue();
    }

    OptHashRefHelper ohrh(info, xsink);
    s->sendHTTPResponse(xsink, *ohrh, status_code, status_desc->c_str(), http_version->c_str(), headers, input_stream,
        max_chunk_size, tcb, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponseWithCallback(code scb, softint status_code, string status_desc, string http_version, hash<auto> headers, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponseWithCallback_VcviVsVsC10hash_auto_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    int64 status_code = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 3);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 4);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
# 2646 "QC_Socket.qpp"
        if (check_response_code(status_code, xsink)) {
        return QoreValue();
    }

    s->sendHTTPResponseWithCallback(xsink, nullptr, status_code, status_desc->c_str(), http_version->c_str(), headers,
        *scb, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponseWithCallback(code scb, softint status_code, string status_desc, string http_version, hash<auto> headers, *reference<hash<auto>> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponseWithCallback_VcviVsVsC10hash_auto_C22_reference_hash_auto__Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    int64 status_code = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 3);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 2695 "QC_Socket.qpp"
        if (check_response_code(status_code, xsink)) {
        return QoreValue();
    }

    OptHashRefHelper ohrh(info, xsink);
    s->sendHTTPResponseWithCallback(xsink, *ohrh, status_code, status_desc->c_str(), http_version->c_str(), headers,
        *scb, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// int Socket::sendi1(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi1_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1639 "QC_Socket.qpp"
    return s->sendi1((char)i, timeout_ms, xsink);
}

// int Socket::sendi2(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi2_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1667 "QC_Socket.qpp"
    return s->sendi2((short)i, timeout_ms, xsink);
}

// int Socket::sendi2LSB(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi2LSB_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1751 "QC_Socket.qpp"
    return s->sendi2LSB((short)i, timeout_ms, xsink);
}

// int Socket::sendi4(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi4_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1695 "QC_Socket.qpp"
    return s->sendi4((int)i, timeout_ms, xsink);
}

// int Socket::sendi4LSB(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi4LSB_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1779 "QC_Socket.qpp"
    return s->sendi4LSB((int)i, timeout_ms, xsink);
}

// int Socket::sendi8(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi8_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1723 "QC_Socket.qpp"
    return s->sendi8(i, timeout_ms, xsink);
}

// int Socket::sendi8LSB(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi8LSB_viVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1807 "QC_Socket.qpp"
    return s->sendi8LSB(i, timeout_ms, xsink);
}

// nothing Socket::setCertificate(SSLCertificate cert){}
static QoreValue Socket_setCertificate_C14SSLCertificate(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(cert, QoreSSLCertificate, args, 0, CID_SSLCERTIFICATE, "Socket::setCertificate()", "SSLCertificate", xsink);
    if (*xsink)
        return 0;
# 3361 "QC_Socket.qpp"
    // pass reference from QoreObject::getReferencedPrivateData() to QoreSocketObject::setCertificate()
    s->setCertificate(cert);
    return QoreValue();
}

// nothing Socket::setCertificate(string cert_pem){}
static QoreValue Socket_setCertificate_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cert_pem = HARD_QORE_VALUE_STRING(args, 0);
# 3374 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLCertificate> cert(new QoreSSLCertificate(cert_pem, xsink));
    if (*xsink)
        return QoreValue();

    // QoreSocketObject::setCertificate() takes over ownership of certificate reference
    s->setCertificate(cert.release());
    return QoreValue();
}

// nothing Socket::setCertificate(binary cert_der){}
static QoreValue Socket_setCertificate_Vy(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* cert_der = HARD_QORE_VALUE_BINARY(args, 0);
# 3391 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLCertificate> cert(new QoreSSLCertificate(cert_der, xsink));
    if (*xsink)
        return QoreValue();

    // QoreSocketObject::setCertificate() takes over ownership of certificate reference
    s->setCertificate(cert.release());
    return QoreValue();
}

// nothing Socket::setCertificateAndPrivateKey(SSLCertificate cert, SSLPrivateKey key){}
static QoreValue Socket_setCertificateAndPrivateKey_C14SSLCertificateC13SSLPrivateKey(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(cert, QoreSSLCertificate, args, 0, CID_SSLCERTIFICATE, "Socket::setCertificateAndPrivateKey()", "SSLCertificate", xsink);
    if (*xsink)
        return 0;
    HARD_QORE_VALUE_OBJ_DATA(key, QoreSSLPrivateKey, args, 1, CID_SSLPRIVATEKEY, "Socket::setCertificateAndPrivateKey()", "SSLPrivateKey", xsink);
    if (*xsink)
        return 0;
# 3457 "QC_Socket.qpp"
    // pass reference from QoreObject::getReferencedPrivateData() to QoreSocketObject::setCertificate()
    s->setCertificateAndPrivateKey(cert, key);
    return QoreValue();
}

// nothing Socket::setCharset(string encoding){}
static QoreValue Socket_setCharset_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 3239 "QC_Socket.qpp"
    s->setEncoding(QEM.findCreate(encoding));
    return QoreValue();
}

// nothing Socket::setEncoding(string encoding){}
static QoreValue Socket_setEncoding_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 3223 "QC_Socket.qpp"
    s->setEncoding(QEM.findCreate(encoding));
    return QoreValue();
}

// nothing Socket::setEventQueue(){}
static QoreValue Socket_setEventQueue(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3470 "QC_Socket.qpp"
    s->setEventQueue(xsink, nullptr, QoreValue(), false);
    return QoreValue();
}

// nothing Socket::setEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
static QoreValue Socket_setEventQueue_C5QueueVANb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "Socket::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
    bool with_data = HARD_QORE_VALUE_BOOL(args, 2);
# 3491 "QC_Socket.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (queue->getMax() != -1)
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which " \
            "could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum " \
            "size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

    // pass reference from QoreObject::getReferencedPrivateData() to function
    s->setEventQueue(xsink, q.release(), arg.refSelf(), with_data);
    return QoreValue();
}

// int Socket::setNoDelay(bool nd = True){}
static QoreValue Socket_setNoDelay_Vb(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool nd = HARD_QORE_VALUE_BOOL(args, 0);
# 3521 "QC_Socket.qpp"
    return s->setNoDelay(nd);
}

// nothing Socket::setPrivateKey(SSLPrivateKey key){}
static QoreValue Socket_setPrivateKey_C13SSLPrivateKey(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(key, QoreSSLPrivateKey, args, 0, CID_SSLPRIVATEKEY, "Socket::setPrivateKey()", "SSLPrivateKey", xsink);
    if (*xsink)
        return 0;
# 3408 "QC_Socket.qpp"
    // pass reference from QoreObject::getReferencedPrivateData() to QoreSocketObject::setPrivateKey()
    s->setPrivateKey(key);
    return QoreValue();
}

// nothing Socket::setPrivateKey(string key_pem, *string pass){}
static QoreValue Socket_setPrivateKey_VsNs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key_pem = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* pass = get_param_value(args, 1).get<const QoreStringNode>();
# 3422 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLPrivateKey> key(new QoreSSLPrivateKey(key_pem, pass ? pass->c_str() : 0, xsink));
    if (*xsink)
        return QoreValue();

    s->setPrivateKey(key.release());
    return QoreValue();
}

// nothing Socket::setPrivateKey(binary key_der){}
static QoreValue Socket_setPrivateKey_Vy(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* key_der = HARD_QORE_VALUE_BINARY(args, 0);
# 3438 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLPrivateKey> key(new QoreSSLPrivateKey(key_der, xsink));
    if (*xsink)
        return QoreValue();

    s->setPrivateKey(key.release());
    return QoreValue();
}

// int Socket::setRecvTimeout(timeout timeout_ms){}
static QoreValue Socket_setRecvTimeout_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 3185 "QC_Socket.qpp"
    return s->setRecvTimeout(timeout_ms);
}

// int Socket::setSendTimeout(timeout timeout_ms){}
static QoreValue Socket_setSendTimeout_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 3166 "QC_Socket.qpp"
    return s->setSendTimeout(timeout_ms);
}

// nothing Socket::setSslVerifyMode(int mode){}
static QoreValue Socket_setSslVerifyMode_Vi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 mode = HARD_QORE_VALUE_INT(args, 0);
# 3744 "QC_Socket.qpp"
    s->setSslVerifyMode((int)mode);
    return QoreValue();
}

// nothing Socket::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
static QoreValue Socket_setWarningQueue_ViViC5QueueVAVt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_ms = HARD_QORE_VALUE_INT(args, 0);
    int64 warning_bs = HARD_QORE_VALUE_INT(args, 1);
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "Socket::setWarningQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 3);
    int64 min_ms = HARD_QORE_VALUE_INT(args, 4);
# 3666 "QC_Socket.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (queue->getMax() != -1)
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which "
            "could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum "
            "size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

    // pass reference from QoreObject::getReferencedPrivateData() to function
    s->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg.refSelf(), min_ms);
    return QoreValue();
}

// int Socket::shutdown(){}
static QoreValue Socket_shutdown(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3111 "QC_Socket.qpp"
    return s->shutdown();
}

// nothing Socket::shutdownSSL(){}
static QoreValue Socket_shutdownSSL(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3125 "QC_Socket.qpp"
    s->shutdownSSL(xsink);
    return QoreValue();
}

// AbstractPollOperation Socket::startPollAccept(){}
static QoreValue Socket_startPollAccept(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 665 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketAcceptPollOperation(xsink, s),
        xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("accept"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollConnect(string target){}
static QoreValue Socket_startPollConnect_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 0);
# 562 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketConnectPollOperation(xsink, false, target->c_str(), s),
        xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("connect"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollConnectSsl(string target){}
static QoreValue Socket_startPollConnectSsl_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 0);
# 587 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketConnectPollOperation(xsink, true, target->c_str(), s),
        xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("connect-ssl"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollReadHttpHeader(){}
static QoreValue Socket_startPollReadHttpHeader(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 829 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketReadHttpHeaderPollOperation(xsink, s), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("received"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollRecvBinary(int size){}
static QoreValue Socket_startPollRecvBinary_Vi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
# 714 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketRecvPollOperation(xsink, size, s, false), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("received"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollRecvData(){}
static QoreValue Socket_startPollRecvData(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 737 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketRecvDataPollOperation(xsink, s, false), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("received"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollRecvString(int size){}
static QoreValue Socket_startPollRecvString_Vi(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
# 690 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketRecvPollOperation(xsink, size, s, true), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("send"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollRecvUntilBytesBinary(string pattern){}
static QoreValue Socket_startPollRecvUntilBytesBinary_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
# 785 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketRecvUntilBytesPollOperation(xsink, pattern, s, false), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("received"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollRecvUntilBytesString(string pattern){}
static QoreValue Socket_startPollRecvUntilBytesString_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
# 761 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketRecvUntilBytesPollOperation(xsink, pattern, s, true), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("received"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollSend(string data){}
static QoreValue Socket_startPollSend_Vs(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* data = HARD_QORE_VALUE_STRING(args, 0);
# 609 "QC_Socket.qpp"
    SimpleRefHolder<QoreStringNode> tmp;
    if (data->getEncoding() != s->getEncoding()) {
        tmp = data->convertEncoding(s->getEncoding(), xsink);
        if (*xsink) {
            return QoreValue();
        }
    }

    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketSendPollOperation(xsink,
        tmp ? tmp.release() : data->stringRefSelf(), s), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("send"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollSend(binary data){}
static QoreValue Socket_startPollSend_Vy(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 639 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketSendPollOperation(xsink, data->binRefSelf(), s), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("send"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollUpgradeClientToSSL(){}
static QoreValue Socket_startPollUpgradeClientToSSL(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 868 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketUpgradeClientSslPollOperation(xsink, s), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("upgrade-client-ssl"), xsink);
    }
    return rv.release();
}

// AbstractPollOperation Socket::startPollUpgradeServerToSSL(){}
static QoreValue Socket_startPollUpgradeServerToSSL(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 891 "QC_Socket.qpp"
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketUpgradeServerSslPollOperation(xsink, s), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("upgrade-server-ssl"), xsink);
    }
    return rv.release();
}

// nothing Socket::upgradeClientToSSL(timeout timeout_ms = -1){}
static QoreValue Socket_upgradeClientToSSL_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 3596 "QC_Socket.qpp"
    s->upgradeClientToSSL(xsink, timeout_ms);
    return QoreValue();
}

// nothing Socket::upgradeServerToSSL(timeout timeout_ms = -1){}
static QoreValue Socket_upgradeServerToSSL_Vt(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 3615 "QC_Socket.qpp"
    s->upgradeServerToSSL(xsink, timeout_ms);
    return QoreValue();
}

// *string Socket::verifyPeerCertificate(){}
static QoreValue Socket_verifyPeerCertificate(QoreObject* self, QoreSocketObject* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3348 "QC_Socket.qpp"
    const char *c = getSSLCVCode(s->verifyPeerCertificate());
    return c ? new QoreStringNode(c) : 0;
}

// static hash<SseMessageInfo> Socket::parseServerSentEvent(string str){}
static QoreValue static_Socket_parseServerSentEvent_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 3862 "QC_Socket.qpp"
return QoreSocket::parseServerSentEvent(xsink, str);
}

// static list<hash<SocketPollInfo>> Socket::poll(list<hash<SocketPollInfo>> items, timeout timeout_ms){}
static QoreValue static_Socket_poll_C26list_hash_SocketPollInfo__Vt(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* items = HARD_QORE_VALUE_LIST(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 3898 "QC_Socket.qpp"
return qore_socket_private::poll(items, timeout_ms, xsink);
}

DLLLOCAL void preinitSocketClass() {
    QC_SOCKET = new QoreBuiltinClass("Socket", "::Qore::Socket", QDOM_NETWORK);
    CID_SOCKET = QC_SOCKET->getID();
    QC_SOCKET->setSystem();
}

DLLLOCAL QoreClass* initSocketClass(QoreNamespace& ns) {
    if (!QC_SOCKET)
        preinitSocketClass();

    // set parent class
    assert(QC_ABSTRACTPOLLABLEIOOBJECTBASE);
    QC_SOCKET->addBuiltinVirtualBaseClass(QC_ABSTRACTPOLLABLEIOOBJECTBASE);

    // public members
    QC_SOCKET->addMember("source", Public, stringOrNothingTypeInfo);
    QC_SOCKET->addMember("source_host", Public, stringOrNothingTypeInfo);

    qore_class_private::get(*QC_SOCKET)->unsetPublicMemberFlag();

    // Socket Socket::accept(){}
    QC_SOCKET->addMethod("accept", (q_method_n_t)Socket_accept, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getTypeInfo());

    // *Socket Socket::accept(timeout timeout_ms){}
    QC_SOCKET->addMethod("accept", (q_method_n_t)Socket_accept_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getOrNothingTypeInfo(), 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    // nothing Socket::acceptAllCertificates(bool accept_all = True){}
    QC_SOCKET->addMethod("acceptAllCertificates", (q_method_n_t)Socket_acceptAllCertificates_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "accept_all");

    // Socket Socket::acceptSSL(){}
    QC_SOCKET->addMethod("acceptSSL", (q_method_n_t)Socket_acceptSSL, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getTypeInfo());

    // *Socket Socket::acceptSSL(timeout timeout_ms){}
    QC_SOCKET->addMethod("acceptSSL", (q_method_n_t)Socket_acceptSSL_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getOrNothingTypeInfo(), 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    // int Socket::bind(string str, softbool reuseaddr = False){}
    QC_SOCKET->addMethod("bind", (q_method_n_t)Socket_bind_Vsvb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBoolTypeInfo, QoreSimpleValue().assign(false), "reuseaddr");

    // int Socket::bind(int port, softbool reuseaddr = False){}
    QC_SOCKET->addMethod("bind", (q_method_n_t)Socket_bind_Vivb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "port", softBoolTypeInfo, QoreSimpleValue().assign(false), "reuseaddr");

    // nothing Socket::bindINET(*string iface, *softstring service, softbool reuseaddr = False, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("bindINET", (q_method_n_t)Socket_bindINET_Nsnsvbvivivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "iface", softStringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "service", softBoolTypeInfo, QoreSimpleValue().assign(false), "reuseaddr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)AF_UNSPEC), "family", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SOCK_STREAM), "socktype", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "protocol");

    // nothing Socket::bindUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("bindUNIX", (q_method_n_t)Socket_bindUNIX_Vsvivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "path", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SOCK_STREAM), "socktype", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "protocol");

    // bool Socket::captureRemoteCertificates(bool set = True){}
    QC_SOCKET->addMethod("captureRemoteCertificates", (q_method_n_t)Socket_captureRemoteCertificates_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "set");

    // nothing Socket::clearStats(){}
    QC_SOCKET->addMethod("clearStats", (q_method_n_t)Socket_clearStats, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Socket::clearWarningQueue(){}
    QC_SOCKET->addMethod("clearWarningQueue", (q_method_n_t)Socket_clearWarningQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int Socket::close(){}
    QC_SOCKET->addMethod("close", (q_method_n_t)Socket_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing Socket::connect(string target, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("connect", (q_method_n_t)Socket_connect_VsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "target", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::connectINET(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectINET", (q_method_n_t)Socket_connectINET_VsvsVtvivivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, stringTypeInfo, QORE_PARAM_NO_ARG, "host", softStringTypeInfo, QORE_PARAM_NO_ARG, "service", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms", softBigIntTypeInfo, QoreSimpleValue().assign((int64)AF_UNSPEC), "family", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SOCK_STREAM), "socktype", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "protocol");

    // nothing Socket::connectINETSSL(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectINETSSL", (q_method_n_t)Socket_connectINETSSL_VsvsVtvivivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, stringTypeInfo, QORE_PARAM_NO_ARG, "host", softStringTypeInfo, QORE_PARAM_NO_ARG, "service", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms", softBigIntTypeInfo, QoreSimpleValue().assign((int64)AF_UNSPEC), "family", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SOCK_STREAM), "socktype", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "protocol");

    // nothing Socket::connectSSL(string target, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("connectSSL", (q_method_n_t)Socket_connectSSL_VsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "target", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::connectUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectUNIX", (q_method_n_t)Socket_connectUNIX_Vsvivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "path", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SOCK_STREAM), "socktype", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "protocol");

    // nothing Socket::connectUNIXSSL(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectUNIXSSL", (q_method_n_t)Socket_connectUNIXSSL_Vsvivi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "path", softBigIntTypeInfo, QoreSimpleValue().assign((int64)SOCK_STREAM), "socktype", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "protocol");

    // Socket::constructor() {}
    QC_SOCKET->addConstructor(Socket_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Socket::copy() {}
    QC_SOCKET->setCopy((q_copy_t)Socket_copy);

    // Socket::destructor() {}
    QC_SOCKET->setDestructor((q_destructor_t)Socket_destructor);

    // bool Socket::getAcceptAllCertificates(){}
    QC_SOCKET->addMethod("getAcceptAllCertificates", (q_method_n_t)Socket_getAcceptAllCertificates, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string Socket::getCharset(){}
    QC_SOCKET->addMethod("getCharset", (q_method_n_t)Socket_getCharset, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int Socket::getConnectionId(){}
    QC_SOCKET->addMethod("getConnectionId", (q_method_n_t)Socket_getConnectionId, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string Socket::getEncoding(){}
    QC_SOCKET->addMethod("getEncoding", (q_method_n_t)Socket_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool Socket::getNoDelay(){}
    QC_SOCKET->addMethod("getNoDelay", (q_method_n_t)Socket_getNoDelay, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // hash<auto> Socket::getPeerInfo(bool host_lookup = True){}
    QC_SOCKET->addMethod("getPeerInfo", (q_method_n_t)Socket_getPeerInfo_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "host_lookup");

    // int Socket::getPort(){}
    QC_SOCKET->addMethod("getPort", (q_method_n_t)Socket_getPort, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Socket::getRecvTimeout(){}
    QC_SOCKET->addMethod("getRecvTimeout", (q_method_n_t)Socket_getRecvTimeout, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *SSLCertificate Socket::getRemoteCertificate(){}
    QC_SOCKET->addMethod("getRemoteCertificate", (q_method_n_t)Socket_getRemoteCertificate, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_SSLCERTIFICATE->getOrNothingTypeInfo());

    // *string Socket::getSSLCipherName(){}
    QC_SOCKET->addMethod("getSSLCipherName", (q_method_n_t)Socket_getSSLCipherName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Socket::getSSLCipherVersion(){}
    QC_SOCKET->addMethod("getSSLCipherVersion", (q_method_n_t)Socket_getSSLCipherVersion, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int Socket::getSendTimeout(){}
    QC_SOCKET->addMethod("getSendTimeout", (q_method_n_t)Socket_getSendTimeout, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Socket::getSocket(){}
    QC_SOCKET->addMethod("getSocket", (q_method_n_t)Socket_getSocket, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // hash<auto> Socket::getSocketInfo(bool host_lookup = True){}
    QC_SOCKET->addMethod("getSocketInfo", (q_method_n_t)Socket_getSocketInfo_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "host_lookup");

    // int Socket::getSslVerifyMode(){}
    QC_SOCKET->addMethod("getSslVerifyMode", (q_method_n_t)Socket_getSslVerifyMode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // hash<auto> Socket::getUsageInfo(){}
    QC_SOCKET->addMethod("getUsageInfo", (q_method_n_t)Socket_getUsageInfo, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // bool Socket::isDataAvailable(timeout timeout_ms = 0){}
    QC_SOCKET->addMethod("isDataAvailable", (q_method_n_t)Socket_isDataAvailable_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms");

    // bool Socket::isOpen(){}
    QC_SOCKET->addMethod("isOpen", (q_method_n_t)Socket_isOpen, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool Socket::isSecure(){}
    QC_SOCKET->addMethod("isSecure", (q_method_n_t)Socket_isSecure, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool Socket::isWriteFinished(timeout timeout_ms = 0){}
    QC_SOCKET->addMethod("isWriteFinished", (q_method_n_t)Socket_isWriteFinished_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms");

    // int Socket::listen(int backlog = 20){}
    QC_SOCKET->addMethod("listen", (q_method_n_t)Socket_listen_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)20), "backlog");

    // bool Socket::pendingHttpChunkedBody(){}
    QC_SOCKET->addMethod("pendingHttpChunkedBody", (q_method_n_t)Socket_pendingHttpChunkedBody, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // hash<auto> Socket::readHTTPChunk(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunk", (q_method_n_t)Socket_readHTTPChunk_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // hash<auto> Socket::readHTTPChunkedBody(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBody", (q_method_n_t)Socket_readHTTPChunkedBody_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // hash<auto> Socket::readHTTPChunkedBodyBinary(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyBinary", (q_method_n_t)Socket_readHTTPChunkedBodyBinary_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::readHTTPChunkedBodyBinaryWithCallback(code rcb, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyBinaryWithCallback", (q_method_n_t)Socket_readHTTPChunkedBodyBinaryWithCallback_VcVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, codeTypeInfo, QORE_PARAM_NO_ARG, "rcb", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // hash<auto> Socket::readHTTPChunkedBodyToOutputStream(Qore::OutputStream os, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyToOutputStream", (q_method_n_t)Socket_readHTTPChunkedBodyToOutputStream_C12OutputStreamVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 2, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::readHTTPChunkedBodyWithCallback(code rcb, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyWithCallback", (q_method_n_t)Socket_readHTTPChunkedBodyWithCallback_VcVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, codeTypeInfo, QORE_PARAM_NO_ARG, "rcb", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // hash<auto> Socket::readHTTPHeader(timeout timeout_ms = -1, *reference<hash<auto>> info){}
    QC_SOCKET->addMethod("readHTTPHeader", (q_method_n_t)Socket_readHTTPHeader_VtC22_reference_hash_auto__, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 2, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info");

    // string Socket::readHTTPHeaderString(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPHeaderString", (q_method_n_t)Socket_readHTTPHeaderString_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // hash<SseMessageInfo> Socket::readServerSentEvent(*string content_encoding, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readServerSentEvent", (q_method_n_t)Socket_readServerSentEvent_NsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclSseMessageInfo->getTypeInfo(false), 2, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "content_encoding", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // hash<SseMessageInfo> Socket::readServerSentEvent(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readServerSentEvent", (q_method_n_t)Socket_readServerSentEvent_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclSseMessageInfo->getTypeInfo(false), 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // string Socket::recv(softint size = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recv", (q_method_n_t)Socket_recv_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "size", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // binary Socket::recvBinary(softint size = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvBinary", (q_method_n_t)Socket_recvBinary_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "size", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::recvToOutputStream(Qore::OutputStream os, softint size = -1, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvToOutputStream", (q_method_n_t)Socket_recvToOutputStream_C12OutputStreamviVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "size", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvi1(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi1", (q_method_n_t)Socket_recvi1_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvi2(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi2", (q_method_n_t)Socket_recvi2_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvi2LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi2LSB", (q_method_n_t)Socket_recvi2LSB_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvi4(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi4", (q_method_n_t)Socket_recvi4_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvi4LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi4LSB", (q_method_n_t)Socket_recvi4LSB_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvi8(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi8", (q_method_n_t)Socket_recvi8_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvi8LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi8LSB", (q_method_n_t)Socket_recvi8LSB_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvu1(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu1", (q_method_n_t)Socket_recvu1_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvu2(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu2", (q_method_n_t)Socket_recvu2_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvu2LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu2LSB", (q_method_n_t)Socket_recvu2LSB_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvu4(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu4", (q_method_n_t)Socket_recvu4_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::recvu4LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu4LSB", (q_method_n_t)Socket_recvu4LSB_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::send(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send", (q_method_n_t)Socket_send_VyVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::send(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send", (q_method_n_t)Socket_send_VsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::send2(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send2", (q_method_n_t)Socket_send2_VyVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::send2(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send2", (q_method_n_t)Socket_send2_VsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendBinary(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary", (q_method_n_t)Socket_sendBinary_VsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendBinary(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary", (q_method_n_t)Socket_sendBinary_VyVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendBinary2(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary2", (q_method_n_t)Socket_sendBinary2_VsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendBinary2(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary2", (q_method_n_t)Socket_sendBinary2_VyVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendFromInputStream(Qore::InputStream input_stream, softint size = -1, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendFromInputStream", (q_method_n_t)Socket_sendFromInputStream_C11InputStreamviVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "input_stream", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "size", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPChunkedBodyFromInputStream(Qore::InputStream input_stream, int max_chunk_size = 4096, timeout timeout_ms = -1, *code tcb){}
    QC_SOCKET->addMethod("sendHTTPChunkedBodyFromInputStream", (q_method_n_t)Socket_sendHTTPChunkedBodyFromInputStream_C11InputStreamViVtNc, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 4, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "input_stream", bigIntTypeInfo, QoreSimpleValue().assign((int64)4096), "max_chunk_size", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms", codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "tcb");

    // nothing Socket::sendHTTPChunkedBodyTrailer(*hash<auto> trailer, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPChunkedBodyTrailer", (q_method_n_t)Socket_sendHTTPChunkedBodyTrailer_C11_hash_auto_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "trailer", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash<auto> headers, string body, *reference<hash> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPMessage", (q_method_n_t)Socket_sendHTTPMessage_VsVsVsC10hash_auto_VsC16_reference_hash_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", stringTypeInfo, QORE_PARAM_NO_ARG, "body", qore_get_complex_reference_or_nothing_type(hashTypeInfo), QORE_PARAM_NO_ARG, "info", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash<auto> headers, *binary body, *reference<hash<auto>> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPMessage", (q_method_n_t)Socket_sendHTTPMessage_VsVsVsC10hash_auto_NyC22_reference_hash_auto__Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", binaryOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPMessageWithCallback(code scb, string method, string path, string http_version, hash<auto> headers, *reference<hash<auto>> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPMessageWithCallback", (q_method_n_t)Socket_sendHTTPMessageWithCallback_VcVsVsVsC10hash_auto_C22_reference_hash_auto__Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, codeTypeInfo, QORE_PARAM_NO_ARG, "scb", stringTypeInfo, QORE_PARAM_NO_ARG, "method", stringTypeInfo, QORE_PARAM_NO_ARG, "path", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, string body, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponse", (q_method_n_t)Socket_sendHTTPResponse_viVsVsC10hash_auto_VsVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "status_code", stringTypeInfo, QORE_PARAM_NO_ARG, "status_desc", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", stringTypeInfo, QORE_PARAM_NO_ARG, "body", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, string body, *reference<hash<auto>> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponse", (q_method_n_t)Socket_sendHTTPResponse_viVsVsC10hash_auto_VsC22_reference_hash_auto__Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "status_code", stringTypeInfo, QORE_PARAM_NO_ARG, "status_desc", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", stringTypeInfo, QORE_PARAM_NO_ARG, "body", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, *binary body, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponse", (q_method_n_t)Socket_sendHTTPResponse_viVsVsC10hash_auto_NyVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "status_code", stringTypeInfo, QORE_PARAM_NO_ARG, "status_desc", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", binaryOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, *binary body, *reference<hash<auto>> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponse", (q_method_n_t)Socket_sendHTTPResponse_viVsVsC10hash_auto_NyC22_reference_hash_auto__Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "status_code", stringTypeInfo, QORE_PARAM_NO_ARG, "status_desc", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", binaryOrNothingTypeInfo, QORE_PARAM_NO_ARG, "body", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash<auto> headers, Qore::InputStream input_stream, int max_chunk_size = 4096, *reference<hash<auto>> info, timeout timeout_ms = -1, *code tcb){}
    QC_SOCKET->addMethod("sendHTTPResponse", (q_method_n_t)Socket_sendHTTPResponse_viVsVsC10hash_auto_C11InputStreamViC22_reference_hash_auto__VtNc, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 9, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "status_code", stringTypeInfo, QORE_PARAM_NO_ARG, "status_desc", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "input_stream", bigIntTypeInfo, QoreSimpleValue().assign((int64)4096), "max_chunk_size", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms", codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "tcb");

    // nothing Socket::sendHTTPResponseWithCallback(code scb, softint status_code, string status_desc, string http_version, hash<auto> headers, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponseWithCallback", (q_method_n_t)Socket_sendHTTPResponseWithCallback_VcviVsVsC10hash_auto_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, codeTypeInfo, QORE_PARAM_NO_ARG, "scb", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "status_code", stringTypeInfo, QORE_PARAM_NO_ARG, "status_desc", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::sendHTTPResponseWithCallback(code scb, softint status_code, string status_desc, string http_version, hash<auto> headers, *reference<hash<auto>> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponseWithCallback", (q_method_n_t)Socket_sendHTTPResponseWithCallback_VcviVsVsC10hash_auto_C22_reference_hash_auto__Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, codeTypeInfo, QORE_PARAM_NO_ARG, "scb", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "status_code", stringTypeInfo, QORE_PARAM_NO_ARG, "status_desc", stringTypeInfo, QORE_PARAM_NO_ARG, "http_version", autoHashTypeInfo, QORE_PARAM_NO_ARG, "headers", qore_get_complex_reference_or_nothing_type(autoHashTypeInfo), QORE_PARAM_NO_ARG, "info", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendi1(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi1", (q_method_n_t)Socket_sendi1_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "i", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendi2(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi2", (q_method_n_t)Socket_sendi2_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "i", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendi2LSB(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi2LSB", (q_method_n_t)Socket_sendi2LSB_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "i", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendi4(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi4", (q_method_n_t)Socket_sendi4_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "i", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendi4LSB(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi4LSB", (q_method_n_t)Socket_sendi4LSB_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "i", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendi8(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi8", (q_method_n_t)Socket_sendi8_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "i", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // int Socket::sendi8LSB(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi8LSB", (q_method_n_t)Socket_sendi8LSB_viVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "i", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::setCertificate(SSLCertificate cert){}
    QC_SOCKET->addMethod("setCertificate", (q_method_n_t)Socket_setCertificate_C14SSLCertificate, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_SSLCERTIFICATE->getTypeInfo(), QORE_PARAM_NO_ARG, "cert");

    // nothing Socket::setCertificate(string cert_pem){}
    QC_SOCKET->addMethod("setCertificate", (q_method_n_t)Socket_setCertificate_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "cert_pem");

    // nothing Socket::setCertificate(binary cert_der){}
    QC_SOCKET->addMethod("setCertificate", (q_method_n_t)Socket_setCertificate_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "cert_der");

    // nothing Socket::setCertificateAndPrivateKey(SSLCertificate cert, SSLPrivateKey key){}
    QC_SOCKET->addMethod("setCertificateAndPrivateKey", (q_method_n_t)Socket_setCertificateAndPrivateKey_C14SSLCertificateC13SSLPrivateKey, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_SSLCERTIFICATE->getTypeInfo(), QORE_PARAM_NO_ARG, "cert", QC_SSLPRIVATEKEY->getTypeInfo(), QORE_PARAM_NO_ARG, "key");

    // nothing Socket::setCharset(string encoding){}
    QC_SOCKET->addMethod("setCharset", (q_method_n_t)Socket_setCharset_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing Socket::setEncoding(string encoding){}
    QC_SOCKET->addMethod("setEncoding", (q_method_n_t)Socket_setEncoding_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing Socket::setEventQueue(){}
    QC_SOCKET->addMethod("setEventQueue", (q_method_n_t)Socket_setEventQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Socket::setEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
    QC_SOCKET->addMethod("setEventQueue", (q_method_n_t)Socket_setEventQueue_C5QueueVANb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "with_data");

    // int Socket::setNoDelay(bool nd = True){}
    QC_SOCKET->addMethod("setNoDelay", (q_method_n_t)Socket_setNoDelay_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "nd");

    // nothing Socket::setPrivateKey(SSLPrivateKey key){}
    QC_SOCKET->addMethod("setPrivateKey", (q_method_n_t)Socket_setPrivateKey_C13SSLPrivateKey, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_SSLPRIVATEKEY->getTypeInfo(), QORE_PARAM_NO_ARG, "key");

    // nothing Socket::setPrivateKey(string key_pem, *string pass){}
    QC_SOCKET->addMethod("setPrivateKey", (q_method_n_t)Socket_setPrivateKey_VsNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "key_pem", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "pass");

    // nothing Socket::setPrivateKey(binary key_der){}
    QC_SOCKET->addMethod("setPrivateKey", (q_method_n_t)Socket_setPrivateKey_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "key_der");

    // int Socket::setRecvTimeout(timeout timeout_ms){}
    QC_SOCKET->addMethod("setRecvTimeout", (q_method_n_t)Socket_setRecvTimeout_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    // int Socket::setSendTimeout(timeout timeout_ms){}
    QC_SOCKET->addMethod("setSendTimeout", (q_method_n_t)Socket_setSendTimeout_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    // nothing Socket::setSslVerifyMode(int mode){}
    QC_SOCKET->addMethod("setSslVerifyMode", (q_method_n_t)Socket_setSslVerifyMode_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "mode");

    // nothing Socket::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
    QC_SOCKET->addMethod("setWarningQueue", (q_method_n_t)Socket_setWarningQueue_ViViC5QueueVAVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, QORE_PARAM_NO_ARG, "warning_ms", bigIntTypeInfo, QORE_PARAM_NO_ARG, "warning_bs", QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", timeoutTypeInfo, QoreSimpleValue().assign(DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0)), "min_ms");

    // int Socket::shutdown(){}
    QC_SOCKET->addMethod("shutdown", (q_method_n_t)Socket_shutdown, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing Socket::shutdownSSL(){}
    QC_SOCKET->addMethod("shutdownSSL", (q_method_n_t)Socket_shutdownSSL, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // AbstractPollOperation Socket::startPollAccept(){}
    QC_SOCKET->addMethod("startPollAccept", (q_method_n_t)Socket_startPollAccept, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo());

    // AbstractPollOperation Socket::startPollConnect(string target){}
    QC_SOCKET->addMethod("startPollConnect", (q_method_n_t)Socket_startPollConnect_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "target");

    // AbstractPollOperation Socket::startPollConnectSsl(string target){}
    QC_SOCKET->addMethod("startPollConnectSsl", (q_method_n_t)Socket_startPollConnectSsl_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "target");

    // AbstractPollOperation Socket::startPollReadHttpHeader(){}
    QC_SOCKET->addMethod("startPollReadHttpHeader", (q_method_n_t)Socket_startPollReadHttpHeader, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo());

    // AbstractPollOperation Socket::startPollRecvBinary(int size){}
    QC_SOCKET->addMethod("startPollRecvBinary", (q_method_n_t)Socket_startPollRecvBinary_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "size");

    // AbstractPollOperation Socket::startPollRecvData(){}
    QC_SOCKET->addMethod("startPollRecvData", (q_method_n_t)Socket_startPollRecvData, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo());

    // AbstractPollOperation Socket::startPollRecvString(int size){}
    QC_SOCKET->addMethod("startPollRecvString", (q_method_n_t)Socket_startPollRecvString_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "size");

    // AbstractPollOperation Socket::startPollRecvUntilBytesBinary(string pattern){}
    QC_SOCKET->addMethod("startPollRecvUntilBytesBinary", (q_method_n_t)Socket_startPollRecvUntilBytesBinary_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern");

    // AbstractPollOperation Socket::startPollRecvUntilBytesString(string pattern){}
    QC_SOCKET->addMethod("startPollRecvUntilBytesString", (q_method_n_t)Socket_startPollRecvUntilBytesString_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern");

    // AbstractPollOperation Socket::startPollSend(string data){}
    QC_SOCKET->addMethod("startPollSend", (q_method_n_t)Socket_startPollSend_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "data");

    // AbstractPollOperation Socket::startPollSend(binary data){}
    QC_SOCKET->addMethod("startPollSend", (q_method_n_t)Socket_startPollSend_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo(), 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    // AbstractPollOperation Socket::startPollUpgradeClientToSSL(){}
    QC_SOCKET->addMethod("startPollUpgradeClientToSSL", (q_method_n_t)Socket_startPollUpgradeClientToSSL, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo());

    // AbstractPollOperation Socket::startPollUpgradeServerToSSL(){}
    QC_SOCKET->addMethod("startPollUpgradeServerToSSL", (q_method_n_t)Socket_startPollUpgradeServerToSSL, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTPOLLOPERATION->getTypeInfo());

    // nothing Socket::upgradeClientToSSL(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("upgradeClientToSSL", (q_method_n_t)Socket_upgradeClientToSSL_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // nothing Socket::upgradeServerToSSL(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("upgradeServerToSSL", (q_method_n_t)Socket_upgradeServerToSSL_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms");

    // *string Socket::verifyPeerCertificate(){}
    QC_SOCKET->addMethod("verifyPeerCertificate", (q_method_n_t)Socket_verifyPeerCertificate, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // static hash<SseMessageInfo> Socket::parseServerSentEvent(string str){}
    QC_SOCKET->addStaticMethod("parseServerSentEvent", (q_func_n_t)static_Socket_parseServerSentEvent_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclSseMessageInfo->getTypeInfo(false), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");

    // static list<hash<SocketPollInfo>> Socket::poll(list<hash<SocketPollInfo>> items, timeout timeout_ms){}
    QC_SOCKET->addStaticMethod("poll", (q_func_n_t)static_Socket_poll_C26list_hash_SocketPollInfo__Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(hashdeclSocketPollInfo->getTypeInfo(false)), 2, qore_get_complex_list_type(hashdeclSocketPollInfo->getTypeInfo(false)), QORE_PARAM_NO_ARG, "items", timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");
# 119 "QC_Socket.qpp"
# 125 "QC_Socket.qpp"
    ns.addConstant("SOCK_POLLERR", ((int64)SOCK_POLLERR));
    ns.addConstant("SOCK_POLLIN", ((int64)SOCK_POLLIN));
    ns.addConstant("SOCK_POLLOUT", ((int64)SOCK_POLLOUT));
# 132 "QC_Socket.qpp"
# 147 "QC_Socket.qpp"
# 165 "QC_Socket.qpp"
    ns.addConstant("SSL_VERIFY_CLIENT_ONCE", ((int64)SSL_VERIFY_CLIENT_ONCE));
    ns.addConstant("SSL_VERIFY_FAIL_IF_NO_PEER_CERT", ((int64)SSL_VERIFY_FAIL_IF_NO_PEER_CERT));
    ns.addConstant("SSL_VERIFY_NONE", ((int64)SSL_VERIFY_NONE));
    ns.addConstant("SSL_VERIFY_PEER", ((int64)SSL_VERIFY_PEER));
# 192 "QC_Socket.qpp"
# 202 "QC_Socket.qpp"
# 205 "QC_Socket.qpp"
# 208 "QC_Socket.qpp"
# 211 "QC_Socket.qpp"
# 214 "QC_Socket.qpp"
# 217 "QC_Socket.qpp"
# 220 "QC_Socket.qpp"
# 223 "QC_Socket.qpp"
# 226 "QC_Socket.qpp"
# 229 "QC_Socket.qpp"
# 232 "QC_Socket.qpp"
# 235 "QC_Socket.qpp"
# 238 "QC_Socket.qpp"
# 241 "QC_Socket.qpp"
# 244 "QC_Socket.qpp"
# 247 "QC_Socket.qpp"
# 250 "QC_Socket.qpp"
# 253 "QC_Socket.qpp"
# 256 "QC_Socket.qpp"
# 259 "QC_Socket.qpp"
# 262 "QC_Socket.qpp"
# 265 "QC_Socket.qpp"
# 268 "QC_Socket.qpp"
# 271 "QC_Socket.qpp"
# 274 "QC_Socket.qpp"
# 277 "QC_Socket.qpp"
# 280 "QC_Socket.qpp"
# 283 "QC_Socket.qpp"
# 286 "QC_Socket.qpp"
# 289 "QC_Socket.qpp"
# 292 "QC_Socket.qpp"
# 295 "QC_Socket.qpp"
# 298 "QC_Socket.qpp"
    ns.addConstant("X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH", new QoreStringNode("X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH"));
    ns.addConstant("X509_V_ERR_AKID_SKID_MISMATCH", new QoreStringNode("X509_V_ERR_AKID_SKID_MISMATCH"));
    ns.addConstant("X509_V_ERR_APPLICATION_VERIFICATION", new QoreStringNode("X509_V_ERR_APPLICATION_VERIFICATION"));
    ns.addConstant("X509_V_ERR_CERT_CHAIN_TOO_LONG", new QoreStringNode("X509_V_ERR_CERT_CHAIN_TOO_LONG"));
    ns.addConstant("X509_V_ERR_CERT_HAS_EXPIRED", new QoreStringNode("X509_V_ERR_CERT_HAS_EXPIRED"));
    ns.addConstant("X509_V_ERR_CERT_NOT_YET_VALID", new QoreStringNode("X509_V_ERR_CERT_NOT_YET_VALID"));
    ns.addConstant("X509_V_ERR_CERT_REJECTED", new QoreStringNode("X509_V_ERR_CERT_REJECTED"));
    ns.addConstant("X509_V_ERR_CERT_REVOKED", new QoreStringNode("X509_V_ERR_CERT_REVOKED"));
    ns.addConstant("X509_V_ERR_CERT_SIGNATURE_FAILURE", new QoreStringNode("X509_V_ERR_CERT_SIGNATURE_FAILURE"));
    ns.addConstant("X509_V_ERR_CERT_UNTRUSTED", new QoreStringNode("X509_V_ERR_CERT_UNTRUSTED"));
    ns.addConstant("X509_V_ERR_CRL_HAS_EXPIRED", new QoreStringNode("X509_V_ERR_CRL_HAS_EXPIRED"));
    ns.addConstant("X509_V_ERR_CRL_NOT_YET_VALID", new QoreStringNode("X509_V_ERR_CRL_NOT_YET_VALID"));
    ns.addConstant("X509_V_ERR_CRL_SIGNATURE_FAILURE", new QoreStringNode("X509_V_ERR_CRL_SIGNATURE_FAILURE"));
    ns.addConstant("X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT", new QoreStringNode("X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD"));
    ns.addConstant("X509_V_ERR_INVALID_CA", new QoreStringNode("X509_V_ERR_INVALID_CA"));
    ns.addConstant("X509_V_ERR_INVALID_PURPOSE", new QoreStringNode("X509_V_ERR_INVALID_PURPOSE"));
    ns.addConstant("X509_V_ERR_KEYUSAGE_NO_CERTSIGN", new QoreStringNode("X509_V_ERR_KEYUSAGE_NO_CERTSIGN"));
    ns.addConstant("X509_V_ERR_OUT_OF_MEM", new QoreStringNode("X509_V_ERR_OUT_OF_MEM"));
    ns.addConstant("X509_V_ERR_PATH_LENGTH_EXCEEDED", new QoreStringNode("X509_V_ERR_PATH_LENGTH_EXCEEDED"));
    ns.addConstant("X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN", new QoreStringNode("X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN"));
    ns.addConstant("X509_V_ERR_SUBJECT_ISSUER_MISMATCH", new QoreStringNode("X509_V_ERR_SUBJECT_ISSUER_MISMATCH"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY", new QoreStringNode("X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE", new QoreStringNode("X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE", new QoreStringNode("X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_GET_CRL", new QoreStringNode("X509_V_ERR_UNABLE_TO_GET_CRL"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT", new QoreStringNode("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY", new QoreStringNode("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE", new QoreStringNode("X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE"));
    ns.addConstant("X509_V_OK", new QoreStringNode("X509_V_OK"));
    QoreHashNode* hash_X509_VerificationReasons = new QoreHashNode(autoTypeInfo);
    hash_X509_VerificationReasons->setKeyValue("X509_V_OK", new QoreStringNode("OK"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT", new QoreStringNode("Unable to get issuer certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_GET_CRL", new QoreStringNode("Unable to get certificate CRL"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE", new QoreStringNode("Unable to decrypt certificate's signature. This means that the actual signature value could not be determined rather than it not matching the expected value; this is only meaningful for RSA"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE", new QoreStringNode("Unable to decrypt CRL's signature"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY", new QoreStringNode("Unable to decode issuer public key (SubjectPublicKeyInfo)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_SIGNATURE_FAILURE", new QoreStringNode("Certificate signature failure; the signature of the certificate is invalid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CRL_SIGNATURE_FAILURE", new QoreStringNode("CRL signature failure; the signature of the certificate is invalid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_NOT_YET_VALID", new QoreStringNode("Certificate is not yet valid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_HAS_EXPIRED", new QoreStringNode("Certificate has expired"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CRL_NOT_YET_VALID", new QoreStringNode("CRL is not yet valid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CRL_HAS_EXPIRED", new QoreStringNode("CRL has expired"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD", new QoreStringNode("Format error in certificate's notBefore field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD", new QoreStringNode("Format error in certificate's notAfter field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD", new QoreStringNode("Format error in CRL's lastUpdate field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD", new QoreStringNode("Format error in CRL's nextUpdate field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_OUT_OF_MEM", new QoreStringNode("Out of memory error"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT", new QoreStringNode("Certificate is self-signed and cannot be found in the trusted list"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN", new QoreStringNode("Self signed certificate in certificate chain"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY", new QoreStringNode("Unable to get local issuer certificate. This normally means the list of trusted certificates is not complete"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE", new QoreStringNode("Unable to verify the first certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_CHAIN_TOO_LONG", new QoreStringNode("Certificate chain too long"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_REVOKED", new QoreStringNode("Certificate has been revoked"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_INVALID_CA", new QoreStringNode("Invalid CA certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_PATH_LENGTH_EXCEEDED", new QoreStringNode("The basicConstraints pathlength parameter has been exceeded"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_INVALID_PURPOSE", new QoreStringNode("The certificate cannot be used for the specified purpose"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_UNTRUSTED", new QoreStringNode("Root CA is not marked as trusted for the specified purpose"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_REJECTED", new QoreStringNode("Root CA is marked to reject the specified purpose"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_SUBJECT_ISSUER_MISMATCH", new QoreStringNode("The current candidate issuer certificate was rejected because its subject name did not match the issuer name of the current certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_AKID_SKID_MISMATCH", new QoreStringNode("The current candidate issuer certificate was rejected because its subject key identifier was present and did not match the authority key identifier of the current certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH", new QoreStringNode("Issuer name and serial number of candidate certificate do not match the authority key identifier of the current certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_KEYUSAGE_NO_CERTSIGN", new QoreStringNode("The keyUsage extension does not permit certificate signing"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_APPLICATION_VERIFICATION", new QoreStringNode("Verification failure"), 0);

    ns.addConstant("X509_VerificationReasons", hash_X509_VerificationReasons);
# 336 "QC_Socket.qpp"
# 343 "QC_Socket.qpp"
# 346 "QC_Socket.qpp"
#ifndef AF_LOCAL
#define AF_LOCAL AF_UNIX
#endif
# 353 "QC_Socket.qpp"
# 356 "QC_Socket.qpp"
# 359 "QC_Socket.qpp"
# 362 "QC_Socket.qpp"
    ns.addConstant("AFMap", (get_network_address_family_map()));
    QoreHashNode* hash_AFStrMap = new QoreHashNode(autoTypeInfo);
    hash_AFStrMap->setKeyValue("AF_INET", ((int64)AF_INET), 0);
    hash_AFStrMap->setKeyValue("AF_INET6", ((int64)AF_INET6), 0);
    hash_AFStrMap->setKeyValue("AF_UNIX", ((int64)AF_UNIX), 0);
    hash_AFStrMap->setKeyValue("AF_LOCAL", ((int64)AF_UNIX), 0);
    hash_AFStrMap->setKeyValue("AF_UNSPEC", ((int64)AF_UNSPEC), 0);

    ns.addConstant("AFStrMap", hash_AFStrMap);
    ns.addConstant("AF_INET", ((int64)AF_INET));
    ns.addConstant("AF_INET6", ((int64)AF_INET6));
    ns.addConstant("AF_LOCAL", ((int64)AF_LOCAL));
    ns.addConstant("AF_UNIX", ((int64)AF_UNIX));
    ns.addConstant("AF_UNSPEC", ((int64)AF_UNSPEC));
# 373 "QC_Socket.qpp"
#ifndef AI_ADDRCONFIG
#define AI_ADDRCONFIG 0
#endif
#ifndef AI_ALL
#define AI_ALL 0
#endif
#ifndef AI_CANONNAME
#define AI_CANONNAME 0
#endif
#ifndef AI_NUMERICHOST
#define AI_NUMERICHOST 0
#endif
#ifndef AI_NUMERICSERV
#define AI_NUMERICSERV 0
#endif
#ifndef AI_PASSIVE
#define AI_PASSIVE 0
#endif
#ifndef AI_V4MAPPED
#define AI_V4MAPPED 0
#endif
# 400 "QC_Socket.qpp"
# 403 "QC_Socket.qpp"
# 406 "QC_Socket.qpp"
# 409 "QC_Socket.qpp"
# 412 "QC_Socket.qpp"
# 420 "QC_Socket.qpp"
    ns.addConstant("AI_ADDRCONFIG", ((int64)AI_ADDRCONFIG));
    ns.addConstant("AI_ALL", ((int64)AI_ALL));
    ns.addConstant("AI_CANONNAME", ((int64)AI_CANONNAME));
    ns.addConstant("AI_NUMERICHOST", ((int64)AI_NUMERICHOST));
    ns.addConstant("AI_NUMERICSERV", ((int64)AI_NUMERICSERV));
    ns.addConstant("AI_PASSIVE", ((int64)AI_PASSIVE));
    ns.addConstant("AI_V4MAPPED", ((int64)AI_V4MAPPED));
# 438 "QC_Socket.qpp"
    ns.addConstant("IPPROTO_TCP", ((int64)IPPROTO_TCP));
    ns.addConstant("IPPROTO_UDP", ((int64)IPPROTO_UDP));
# 449 "QC_Socket.qpp"
# 456 "QC_Socket.qpp"
    ns.addConstant("SOCK_DGRAM", ((int64)SOCK_DGRAM));
    ns.addConstant("SOCK_RAW", ((int64)SOCK_RAW));
    ns.addConstant("SOCK_STREAM", ((int64)SOCK_STREAM));

    return QC_SOCKET;
}
/*
AbstractPollOperation Socket::startPollReadServerSentEvent() {
    s->ref();
    ReferenceHolder<SocketPollOperationBase> poller(new SocketReadServerSentEventPollOperation(xsink, s), xsink);
    ReferenceHolder<QoreObject> rv(new QoreObject(QC_SOCKETPOLLOPERATION, getProgram(), *poller), xsink);
    poller->setSelf(*rv);
    poller.release();

    if (!*xsink) {
        rv->setValue("sock", self->objectRefSelf(), xsink);
        rv->setValue("goal", new QoreStringNode("received"), xsink);
    }
    return rv.release();
}
*/
