/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_SocketPollOperation.qpp

    SocketPollOperation class definition

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore/Qore.h"
#include "qore/intern/QC_SocketPollOperation.h"
#include "qore/intern/QC_Socket.h"

static SocketConnectPollOperation* get_connect_poll_operation(ExceptionSink* xsink, bool ssl,
        const QoreHashNode* opts, ReferenceHolder<QoreSocketObject>& holder) {
    QoreValue url = opts ? opts->getKeyValue("url", xsink) : QoreValue();
    if (*xsink) {
        return nullptr;
    }
    if (!url) {
        xsink->raiseException("MISSING-URL", "goal 'connect%s' requires option 'url' giving the target for the "
            "connection (ex: 'www.example.com:%d', '/tmp/socket-file', etc)", ssl ? "-ssl" : "", ssl ? 443 : 80);
        return nullptr;
    }
    if (url.getType() != NT_STRING) {
        xsink->raiseException("INVALID-URL", "expecting type 'string' for option 'url'; got type '%s' instead",
            url.getFullTypeName());
        return nullptr;
    }
    return new SocketConnectPollOperation(xsink, ssl, url.get<QoreStringNode>()->c_str(), holder.release());
}
/* Qore class Qore::SocketPollOperation */

qore_classid_t CID_SOCKETPOLLOPERATION;
QoreClass* QC_SOCKETPOLLOPERATION;

// SocketPollOperation::constructor(Socket socket, string goal, *hash<auto> opts) {}
static void SocketPollOperation_constructor_C6SocketVsC11_hash_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(socket, QoreSocketObject, args, 0, CID_SOCKET, "SocketPollOperation::constructor()", "Socket", xsink);
    if (*xsink)
        return;
    const QoreStringNode* goal = HARD_QORE_VALUE_STRING(args, 1);
    const QoreHashNode* opts = get_param_value(args, 2).get<const QoreHashNode>();
# 80 "QC_SocketPollOperation.qpp"
ReferenceHolder<QoreSocketObject> holder(socket, xsink);
    ReferenceHolder<SocketPollOperationBase> poller(xsink);
    if (*goal == "connect") {
        poller = get_connect_poll_operation(xsink, false, opts, holder);
    } else if (*goal == "connect-ssl") {
        poller = get_connect_poll_operation(xsink, true, opts, holder);
    } else {
        xsink->raiseException("INVALID-GOAL", "goal '%s' not recognized; expecting one of: 'connect', 'connect-ssl'",
            goal->c_str());
        return;
    }
    if (*xsink) {
        return;
    }

    poller->setSelf(self);
    self->setPrivate(CID_SOCKETPOLLOPERATION, poller.release());
    if (!*xsink) {
        self->setValue("sock", obj_socket->objectRefSelf(), xsink);
        self->setValue("goal", goal->stringRefSelf(), xsink);
    }
}

// *hash<SocketPollInfo> SocketPollOperation::continuePoll(){}
static QoreValue SocketPollOperation_continuePoll(QoreObject* self, SocketPollOperationBase* op, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 125 "QC_SocketPollOperation.qpp"
    return op->continuePoll(xsink);
}

// SocketPollOperation::copy() {}
static void SocketPollOperation_copy(QoreObject* self, QoreObject* old, SocketPollOperationBase* op, ExceptionSink* xsink) {
# 115 "QC_SocketPollOperation.qpp"
xsink->raiseException("SOCKETPOLLOPERATION-COPY-ERROR", "objects of this class cannot be copied");
}

// SocketPollOperation::destructor() {}
static void SocketPollOperation_destructor(QoreObject* self, SocketPollOperationBase* op, ExceptionSink* xsink) {
# 107 "QC_SocketPollOperation.qpp"
op->deref(xsink);
}

DLLLOCAL void preinitSocketPollOperationClass() {
    QC_SOCKETPOLLOPERATION = new QoreBuiltinClass("SocketPollOperation", "::Qore::SocketPollOperation", QDOM_DEFAULT);
    CID_SOCKETPOLLOPERATION = QC_SOCKETPOLLOPERATION->getID();
    QC_SOCKETPOLLOPERATION->setSystem();
}

DLLLOCAL QoreClass* initSocketPollOperationClass(QoreNamespace& ns) {
    if (!QC_SOCKETPOLLOPERATION)
        preinitSocketPollOperationClass();

    // set parent class
    assert(QC_SOCKETPOLLOPERATIONBASE);
    QC_SOCKETPOLLOPERATION->addBuiltinVirtualBaseClass(QC_SOCKETPOLLOPERATIONBASE);

    // private members
    QC_SOCKETPOLLOPERATION->addMember("goal", Private, stringTypeInfo);

    // SocketPollOperation::constructor(Socket socket, string goal, *hash<auto> opts) {}
    QC_SOCKETPOLLOPERATION->addConstructor(SocketPollOperation_constructor_C6SocketVsC11_hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, QC_SOCKET->getTypeInfo(), QORE_PARAM_NO_ARG, "socket", stringTypeInfo, QORE_PARAM_NO_ARG, "goal", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "opts");

    // *hash<SocketPollInfo> SocketPollOperation::continuePoll(){}
    QC_SOCKETPOLLOPERATION->addMethod("continuePoll", (q_method_n_t)SocketPollOperation_continuePoll, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclSocketPollInfo->getTypeInfo(true));

    // SocketPollOperation::copy() {}
    QC_SOCKETPOLLOPERATION->setCopy((q_copy_t)SocketPollOperation_copy);

    // SocketPollOperation::destructor() {}
    QC_SOCKETPOLLOPERATION->setDestructor((q_destructor_t)SocketPollOperation_destructor);

    return QC_SOCKETPOLLOPERATION;
}
