/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StreamReader.qpp StreamReader class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/InputStream.h"
#include "qore/intern/StreamReader.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"

DLLEXPORT extern QoreClass* QC_INPUTSTREAM;
/* Qore class Qore::StreamReader */

qore_classid_t CID_STREAMREADER;
QoreClass* QC_STREAMREADER;

// StreamReader::constructor(Qore::InputStream is, *string encoding) {}
static void StreamReader_constructor_C11InputStreamNs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "StreamReader::constructor()", "Qore::InputStream", xsink);
    if (*xsink)
        return;
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 61 "QC_StreamReader.qpp"
SimpleRefHolder<StreamReader> reader(new StreamReader(xsink, is, encoding ? QEM.findCreate(encoding) : QCS_DEFAULT));
    if (*xsink)
        return;
    self->setPrivate(CID_STREAMREADER, reader.release());
    qore_object_private* o = qore_object_private::get(*self);
    const qore_class_private* cls = qore_class_private::get(*QC_STREAMREADER);
    o->setValueIntern(cls, "is", static_cast<QoreObject*>(obj_is->refSelf()), xsink);
}

// string StreamReader::getEncoding(){}
static QoreValue StreamReader_getEncoding(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 74 "QC_StreamReader.qpp"
    return new QoreStringNode(sr->getEncoding()->getCode());
}

// InputStream StreamReader::getInputStream(){}
static QoreValue StreamReader_getInputStream(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 85 "QC_StreamReader.qpp"
    return self->getReferencedMemberNoMethod("is", xsink);
}

// *binary StreamReader::readBinary(int limit = -1){}
static QoreValue StreamReader_readBinary_Vi(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 142 "QC_StreamReader.qpp"
    return sr->readBinary(limit, xsink);
}

// *string StreamReader::readLine(*string eol, bool trim = True){}
static QoreValue StreamReader_readLine_NsVb(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* eol = get_param_value(args, 0).get<const QoreStringNode>();
    bool trim = HARD_QORE_VALUE_BOOL(args, 1);
# 124 "QC_StreamReader.qpp"
    return sr->readLine(eol, trim, xsink);
}

// *string StreamReader::readString(int limit = -1){}
static QoreValue StreamReader_readString_Vi(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 105 "QC_StreamReader.qpp"
    return sr->readString(limit, xsink);
}

// int StreamReader::readi1(){}
static QoreValue StreamReader_readi1(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 158 "QC_StreamReader.qpp"
    return sr->readi1(xsink);
}

// int StreamReader::readi2(){}
static QoreValue StreamReader_readi2(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 174 "QC_StreamReader.qpp"
    return sr->readi2(xsink);
}

// int StreamReader::readi2LSB(){}
static QoreValue StreamReader_readi2LSB(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 222 "QC_StreamReader.qpp"
    return sr->readi2LSB(xsink);
}

// int StreamReader::readi4(){}
static QoreValue StreamReader_readi4(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 190 "QC_StreamReader.qpp"
    return sr->readi4(xsink);
}

// int StreamReader::readi4LSB(){}
static QoreValue StreamReader_readi4LSB(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 238 "QC_StreamReader.qpp"
    return sr->readi4LSB(xsink);
}

// int StreamReader::readi8(){}
static QoreValue StreamReader_readi8(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 206 "QC_StreamReader.qpp"
    return sr->readi8(xsink);
}

// int StreamReader::readi8LSB(){}
static QoreValue StreamReader_readi8LSB(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 254 "QC_StreamReader.qpp"
    return sr->readi8LSB(xsink);
}

// int StreamReader::readu1(){}
static QoreValue StreamReader_readu1(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 270 "QC_StreamReader.qpp"
    return sr->readu1(xsink);
}

// int StreamReader::readu2(){}
static QoreValue StreamReader_readu2(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 286 "QC_StreamReader.qpp"
    return sr->readu2(xsink);
}

// int StreamReader::readu2LSB(){}
static QoreValue StreamReader_readu2LSB(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 318 "QC_StreamReader.qpp"
    return sr->readu2LSB(xsink);
}

// int StreamReader::readu4(){}
static QoreValue StreamReader_readu4(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 302 "QC_StreamReader.qpp"
    return sr->readu4(xsink);
}

// int StreamReader::readu4LSB(){}
static QoreValue StreamReader_readu4LSB(QoreObject* self, StreamReader* sr, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 334 "QC_StreamReader.qpp"
    return sr->readu4LSB(xsink);
}

DLLLOCAL void preinitStreamReaderClass() {
    QC_STREAMREADER = new QoreBuiltinClass("StreamReader", "::Qore::StreamReader", QDOM_DEFAULT);
    CID_STREAMREADER = QC_STREAMREADER->getID();
    QC_STREAMREADER->setSystem();
}

DLLLOCAL QoreClass* initStreamReaderClass(QoreNamespace& ns) {
    if (!QC_STREAMREADER)
        preinitStreamReaderClass();

    // private:internal members
    QC_STREAMREADER->addMember("is", Internal, QC_INPUTSTREAM->getTypeInfo());

    // StreamReader::constructor(Qore::InputStream is, *string encoding) {}
    QC_STREAMREADER->addConstructor(StreamReader_constructor_C11InputStreamNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "is", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // string StreamReader::getEncoding(){}
    QC_STREAMREADER->addMethod("getEncoding", (q_method_n_t)StreamReader_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // InputStream StreamReader::getInputStream(){}
    QC_STREAMREADER->addMethod("getInputStream", (q_method_n_t)StreamReader_getInputStream, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_INPUTSTREAM->getTypeInfo());

    // *binary StreamReader::readBinary(int limit = -1){}
    QC_STREAMREADER->addMethod("readBinary", (q_method_n_t)StreamReader_readBinary_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "limit");

    // *string StreamReader::readLine(*string eol, bool trim = True){}
    QC_STREAMREADER->addMethod("readLine", (q_method_n_t)StreamReader_readLine_NsVb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "eol", boolTypeInfo, QoreSimpleValue().assign(true), "trim");

    // *string StreamReader::readString(int limit = -1){}
    QC_STREAMREADER->addMethod("readString", (q_method_n_t)StreamReader_readString_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "limit");

    // int StreamReader::readi1(){}
    QC_STREAMREADER->addMethod("readi1", (q_method_n_t)StreamReader_readi1, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readi2(){}
    QC_STREAMREADER->addMethod("readi2", (q_method_n_t)StreamReader_readi2, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readi2LSB(){}
    QC_STREAMREADER->addMethod("readi2LSB", (q_method_n_t)StreamReader_readi2LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readi4(){}
    QC_STREAMREADER->addMethod("readi4", (q_method_n_t)StreamReader_readi4, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readi4LSB(){}
    QC_STREAMREADER->addMethod("readi4LSB", (q_method_n_t)StreamReader_readi4LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readi8(){}
    QC_STREAMREADER->addMethod("readi8", (q_method_n_t)StreamReader_readi8, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readi8LSB(){}
    QC_STREAMREADER->addMethod("readi8LSB", (q_method_n_t)StreamReader_readi8LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readu1(){}
    QC_STREAMREADER->addMethod("readu1", (q_method_n_t)StreamReader_readu1, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readu2(){}
    QC_STREAMREADER->addMethod("readu2", (q_method_n_t)StreamReader_readu2, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readu2LSB(){}
    QC_STREAMREADER->addMethod("readu2LSB", (q_method_n_t)StreamReader_readu2LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readu4(){}
    QC_STREAMREADER->addMethod("readu4", (q_method_n_t)StreamReader_readu4, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int StreamReader::readu4LSB(){}
    QC_STREAMREADER->addMethod("readu4LSB", (q_method_n_t)StreamReader_readu4LSB, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_STREAMREADER;
}
