/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ThreadPool.qpp ThreadPool class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ThreadPool.h"

static void tpt_start_thread(ExceptionSink* xsink, ThreadPoolThread* tpt) {
   tpt->worker(xsink);
}

ThreadPoolThread::ThreadPoolThread(ThreadPool& n_tp, ExceptionSink* xsink) : tp(n_tp) {
    id = q_start_thread(xsink, (q_thread_t)tpt_start_thread, this);
    if (id > 0)
        tp.ref();
}

void ThreadPoolThread::worker(ExceptionSink* xsink) {
    SafeLocker sl(m);

    while (!stopflag || task) {
        if (!task) {
            //printd(5, "ThreadPoolThread::worker() id %d about to wait stopflag: %d task: %p\n", id, stopflag, task);
            c.wait(m);
            if (stopflag && !task)
                break;
        }

        assert(task);

        sl.unlock();
        task->run(xsink).discard(xsink);
        sl.lock();
        task->del(xsink);
        task = 0;

        if (stopflag || tp.done(this))
            break;
    }

    //printd(5, "ThreadPoolThread::worker() stopping id %d: %s\n", id, stopCond ? "wait" : "after detach");

    if (stopCond) {
        stopped = true;
        stopCond->signal();
    } else {
        sl.unlock();
        finalize(xsink);
    }
}

void ThreadPoolThread::finalize(ExceptionSink* xsink) {
    tp.deref(xsink);
    delete this;
}

static void tp_start_thread(ExceptionSink* xsink, ThreadPool* tp) {
    tp->worker(xsink);
}

ThreadPool::ThreadPool(ExceptionSink* xsink, int n_max, int n_minidle, int n_maxidle, int n_release_ms) :
    max(n_max), minidle(n_minidle), maxidle(n_maxidle), release_ms(n_release_ms) {
    assert(xsink);
    if (max < 0)
        max = 0;
    if (minidle < 0)
        minidle = 0;
    if (maxidle <= 0)
        maxidle = minidle;
    if (q_start_thread(xsink, (q_thread_t)tp_start_thread, this) == -1) {
        assert(*xsink);
        stopped = true;
    }
}

void ThreadPool::worker(ExceptionSink* xsink) {
    SafeLocker sl(m);

    for (int i = 0; i < minidle; ++i) {
        if (addIdleWorker(xsink)) {
            xsink->handleExceptions();
            break;
        }
    }

    while (!stopflag) {
        if (q.empty()) {
            if (shutdown) {
                stopflag = true;
                break;
            }
            if (release_ms && (int)fh.size() > minidle) {
                if (cond.wait(m, release_ms) && q.empty()) {
                    // timeout occurred: terminate an idle thread
                    ThreadPoolThread* tpt = fh.front();
                    //printd(5, "ThreadPool::worker() this: %p release_ms: %d timeout - stopping idle thread %p (minidle: %d maxidle: %d fh.size(): %ld)\n", this, release_ms, tpt, minidle, maxidle, fh.size());
                    fh.pop_front();
                    tpt->stop();
                    continue;
                }
            } else {
                cond.wait(m);
            }
        }

        if (stopflag)
            break;

        while (!q.empty()) {
            ThreadPoolThread* tpt = getThreadUnlocked(xsink);
            if (!tpt) {
                xsink->handleExceptions();
                break;
            }
            tpt->submit(q.front());
            q.pop_front();
        }

        while ((int)fh.size() < minidle && (!max || ((int)fh.size() + (int)ah.size() < max))) {
            if (addIdleWorker(xsink)) {
                xsink->handleExceptions();
                break;
            }
        }
    }

    // idle threads can be terminated with stop() in all cases
    for (auto& i : fh) {
        i->stop();
    }

    sl.unlock();

    if (!detach) {
        // signal worker threads to terminate and confirm termination inline
        for (auto& i : ah) {
            i->stopWait();
        }
        // wait for termination
        for (auto& i : ah) {
            i->stopConfirm(xsink);
        }
    } else {
        // detach threads and do not wait for termination
        for (auto& i : ah) {
            i->stop();
        }
    }

    sl.lock();

#ifdef DEBUG
    ah.clear();
    fh.clear();
#endif

    stopped = true;
    stopCond.broadcast();

    // issue #2906: we have to copy the task queue here to cancel them outside the lock because
    // the thread pool can be destroyed after we release the lock
    taskq_t q_copy = q;
    q.clear();

    sl.unlock();

    // cancel tasks if any
    for (auto& i : q_copy) {
        i->cancel(xsink);
        i->del(xsink);
    }
}
/* Qore class Qore::Thread::ThreadPool */

qore_classid_t CID_THREADPOOL;
QoreClass* QC_THREADPOOL;

// ThreadPool::constructor(int max = 0, int minidle = 0, int maxidle = 0, timeout release_ms = 5s) {}
static void ThreadPool_constructor_ViViViVt(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 max = HARD_QORE_VALUE_INT(args, 0);
    int64 minidle = HARD_QORE_VALUE_INT(args, 1);
    int64 maxidle = HARD_QORE_VALUE_INT(args, 2);
    int64 release_ms = HARD_QORE_VALUE_INT(args, 3);
# 253 "QC_ThreadPool.qpp"
if (max > 0) {
        if (minidle > max) {
            xsink->raiseException("THREADPOOL-ERROR", "cannot create a ThreadPool object with minidle (%d) > max (%d)", minidle, max);
            return;
        }
        if (maxidle > max) {
            xsink->raiseException("THREADPOOL-ERROR", "cannot create a ThreadPool object with maxidle (%d) > max (%d)", maxidle, max);
            return;
        }
    }
    if (maxidle > 0) {
        if (minidle > maxidle) {
            xsink->raiseException("THREADPOOL-ERROR", "cannot create a ThreadPool object with minidle (%d) > maxidle (%d)", minidle, maxidle);
            return;
        }
    }
    if (release_ms < 0) {
        xsink->raiseException("THREADPOOL-ERROR", "cannot create a ThreadPool object with release_ms < 0 (value passed: %d)", release_ms);
        return;
    }

    ReferenceHolder<ThreadPool> tp(new ThreadPool(xsink, max, minidle, maxidle, release_ms), xsink);
    if (*xsink)
        return;

    self->setPrivate(CID_THREADPOOL, tp.release());
}

// ThreadPool::destructor() {}
static void ThreadPool_destructor(QoreObject* self, ThreadPool* tp, ExceptionSink* xsink) {
# 288 "QC_ThreadPool.qpp"
tp->stop();
    tp->deref();
}

// nothing ThreadPool::stop(){}
static QoreValue ThreadPool_stop(QoreObject* self, ThreadPool* tp, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 308 "QC_ThreadPool.qpp"
    tp->stop();
    return QoreValue();
}

// nothing ThreadPool::stopWait(){}
static QoreValue ThreadPool_stopWait(QoreObject* self, ThreadPool* tp, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 323 "QC_ThreadPool.qpp"
    tp->stopWait(xsink);
    return QoreValue();
}

// nothing ThreadPool::submit(code task, *code cancel){}
static QoreValue ThreadPool_submit_VcNc(QoreObject* self, ThreadPool* tp, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* task = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const ResolvedCallReferenceNode* cancel = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 336 "QC_ThreadPool.qpp"
    tp->submit(task->refRefSelf(), cancel ? cancel->refRefSelf() : 0, xsink);
    return QoreValue();
}

// string ThreadPool::toString(){}
static QoreValue ThreadPool_toString(QoreObject* self, ThreadPool* tp, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 348 "QC_ThreadPool.qpp"
    QoreStringNode* str = new QoreStringNode;
   tp->toString(*str);
   return str;
}

DLLLOCAL void preinitThreadPoolClass() {
    QC_THREADPOOL = new QoreBuiltinClass("ThreadPool", "::Qore::Thread::ThreadPool", QDOM_THREAD_CLASS);
    CID_THREADPOOL = QC_THREADPOOL->getID();
    QC_THREADPOOL->setSystem();
}

DLLLOCAL QoreClass* initThreadPoolClass(QoreNamespace& ns) {
    if (!QC_THREADPOOL)
        preinitThreadPoolClass();

    // ThreadPool::constructor(int max = 0, int minidle = 0, int maxidle = 0, timeout release_ms = 5s) {}
    QC_THREADPOOL->addConstructor(ThreadPool_constructor_ViViViVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 4, bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "max", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "minidle", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "maxidle", timeoutTypeInfo, QoreSimpleValue().assign(DateTimeNode::makeRelative(0, 0, 0, 0, 0, 5, 0)), "release_ms");

    // ThreadPool::destructor() {}
    QC_THREADPOOL->setDestructor((q_destructor_t)ThreadPool_destructor);

    // nothing ThreadPool::stop(){}
    QC_THREADPOOL->addMethod("stop", (q_method_n_t)ThreadPool_stop, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing ThreadPool::stopWait(){}
    QC_THREADPOOL->addMethod("stopWait", (q_method_n_t)ThreadPool_stopWait, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing ThreadPool::submit(code task, *code cancel){}
    QC_THREADPOOL->addMethod("submit", (q_method_n_t)ThreadPool_submit_VcNc, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, codeTypeInfo, QORE_PARAM_NO_ARG, "task", codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "cancel");

    // string ThreadPool::toString(){}
    QC_THREADPOOL->addMethod("toString", (q_method_n_t)ThreadPool_toString, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    return QC_THREADPOOL;
}
