/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_compression.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <bzlib.h>
#include <cerrno>
#include <climits>
#include <zlib.h>

#include "qore/Qore.h"
#include "qore/intern/ql_compression.h"
#include "qore/intern/CompressionTransforms.h"

#ifndef QORE_BZ2_WORK_FACTOR
#define QORE_BZ2_WORK_FACTOR 30
#endif

#ifndef QORE_BZ2_VERBOSITY
#define QORE_BZ2_VERBOSITY 0
#endif

#ifndef BZ2_DEFAULT_COMPRESSION
#define BZ2_DEFAULT_COMPRESSION 3
#endif

#define COMPRESSION_LEVEL_DEFAULT CompressionTransforms::LEVEL_DEFAULT

extern QoreClass* QC_TRANSFORM;

class qore_bz_stream : public bz_stream {
public:
   DLLLOCAL qore_bz_stream() {
      bzalloc = 0;
      bzfree = 0;
      opaque = 0;
   }
};

class qore_bz_compressor : public qore_bz_stream {
   bool ok;

public:
   DLLLOCAL qore_bz_compressor(int level, ExceptionSink* xsink) {
      int rc = BZ2_bzCompressInit(this, level, QORE_BZ2_VERBOSITY, QORE_BZ2_WORK_FACTOR);
      ok = (rc == BZ_OK);
      if (!ok)
	 xsink->raiseException("BZIP2-COMPRESS-ERROR", "code %d returned from BZ2_bzCompressInit()", rc);
   }

   DLLLOCAL ~qore_bz_compressor() {
      if (ok)
	 BZ2_bzCompressEnd(this);
   }
   DLLLOCAL operator bool() const { return ok; }

   DLLLOCAL BinaryNode* compress(const void* ptr, unsigned long len, ExceptionSink* xsink) {
      char* p = (char*)ptr;

      size_t bs = len >> 3;
      if (!bs)
	 bs = 1;

      next_in = p;
      avail_in = len;

      SimpleRefHolder<BinaryNode> b(new BinaryNode);

      if (b->preallocate(bs)) {
	 xsink->outOfMemory();
	 return 0;
      }

      while (true) {
	 int64 done = (int64)total_out_lo32 + (((int64)total_out_hi32) << 32);
	 next_out = ((char*)b->getPtr()) + done;
	 avail_out = (unsigned)(bs - done);

	 int rc = BZ2_bzCompress(this, BZ_FINISH);
	 //printd(5, "bs: %lld, done: %lld, avail_out: %d, rc: %d, total_out_lo32: %d, total_out_hi32 = %d\n", bs, done, avail_out, rc, total_out_lo32, total_out_hi32);
	 if (rc == BZ_STREAM_END)
	    break;
	 if (rc != BZ_FINISH_OK) {
	    xsink->raiseException("BZIP2-COMPRESS-ERROR", "error code %d returned from BZ2_bzCompress()", rc);
	    return 0;
	 }
	 bs *= 2;
         // if we get over size of an int, then we have to abort, since avail_out is an unsigned_int
         if ((bs - done) > UINT_MAX) {
	    xsink->raiseException("BZIP2-COMPRESS-ERROR", "the bzip2 library requested a buffer that is too large to be handled by the API");
	    return 0;
         }

	 if (b->preallocate(bs)) {
	    xsink->raiseException("BZIP2-COMPRESS-ERROR", "the bzip2 library requested a buffer that is too large to be handled by the API");
	    return 0;
	 }
      }
      b->setSize((int64)total_out_lo32 + (((int64)total_out_hi32) << 32));

      return b.release();
   }
};

class qore_bz_decompressor : public qore_bz_stream {
   bool ok;

public:
   DLLLOCAL qore_bz_decompressor(ExceptionSink* xsink) {
      int rc = BZ2_bzDecompressInit(this, QORE_BZ2_VERBOSITY, 0);
      ok = (rc == BZ_OK);
      if (!ok)
	 xsink->raiseException("BZIP2-DECOMPRESS-ERROR", "code %d returned from BZ2_bzDecompressInit()", rc);
   }

   DLLLOCAL ~qore_bz_decompressor() {
      if (ok)
	 BZ2_bzDecompressEnd(this);
   }
   DLLLOCAL operator bool() const { return ok; }

   DLLLOCAL BinaryNode* decompress(const void* ptr, unsigned long len, ExceptionSink* xsink) {
      char* p = (char*)ptr;

      size_t bs = len << 1;

      next_in = p;
      avail_in = len;

      SimpleRefHolder<BinaryNode> b(new BinaryNode);

      if (b->preallocate(bs)) {
	 xsink->outOfMemory();
	 return 0;
      }

      while (true) {
	 int64 done = (int64)total_out_lo32 + (((int64)total_out_hi32) << 32);
	 next_out = ((char*)b->getPtr()) + done;
	 avail_out = (unsigned)(bs - done);

	 int rc = BZ2_bzDecompress(this);
	 //printd(5, "bs: %d, done: %lld, avail_out: %d, rc: %d, total_out_lo32: %d, total_out_hi32 = %d\n", bs, done, avail_out, rc, total_out_lo32, total_out_hi32);
	 if (rc == BZ_STREAM_END)
	    break;
	 if (rc != BZ_OK) {
	    xsink->raiseException("BZIP2-DECOMPRESS-ERROR", "error code %d returned from BZ2_bzDecompress()", rc);
	    return 0;
	 }
	 bs *= 2;
         // if we get over size of an int, then we have to abort, since avail_out is an unsigned_int
         if ((bs - done) > UINT_MAX) {
	    xsink->raiseException("BZIP2-DECOMPRESS-ERROR", "the bzip2 library requested a buffer that is too large to be handled by the API");
	    return 0;
         }

	 if (b->preallocate(bs)) {
	    xsink->raiseException("BZIP2-DECOMPRESS-ERROR", "the bzip2 library requested a buffer that is too large to be handled by the API");
	    return 0;
	 }
      }
      if (avail_in > 0) {
         xsink->raiseException("BZIP2-DECOMPRESS-ERROR", "Unexpected bytes at the end of compressed data");
         return 0;
      }
      b->setSize((int64)total_out_lo32 + (((int64)total_out_hi32) << 32));

      return b.release();
   }

   DLLLOCAL QoreStringNode* decompress_to_string(const void* ptr, unsigned long len, const QoreEncoding* enc, ExceptionSink* xsink) {
      static char np[] = {'\0'};

      SimpleRefHolder<BinaryNode> b(decompress(ptr, len, xsink));
      if (!b)
	 return 0;

      len = b->size();

      // terminate the string
      b->append(np, 1);

      return new QoreStringNode((char*)b->giveBuffer(), len, len + 1, enc);
   }
};

BinaryNode* qore_bzip2(const void* ptr, unsigned long len, int level, ExceptionSink* xsink) {
   qore_bz_compressor c(level, xsink);
   if (!c)
      return 0;

   return c.compress(ptr, len, xsink);
}

BinaryNode* qore_bunzip2_to_binary(const BinaryNode* b, ExceptionSink* xsink) {
   qore_bz_decompressor c(xsink);
   if (!c)
      return 0;

   return c.decompress(b->getPtr(), b->size(), xsink);
}

QoreStringNode* qore_bunzip2_to_string(const BinaryNode* b, const QoreEncoding* enc, ExceptionSink* xsink) {
   qore_bz_decompressor c(xsink);
   if (!c)
      return 0;

   return c.decompress_to_string(b->getPtr(), b->size(), enc, xsink);
}

#ifdef HAVE_GZ_HEADER
class qore_gz_header : public gz_header {
   DLLLOCAL qore_gz_header(bool n_text, char* n_name, char* n_comment) {
      text = n_text;
      time = ::time(0);
      xflags = 0;
      os = 3; // always set to UNIX
      extra = Z_NULL;
      extra_len = 0;
      extra_max = 0;
      name = (Bytef*)n_name;
      name_max = 0;
      comment = (Bytef*)n_comment;
      comm_max = 0;
      hcrc = 0;
      done = 0;
   }
};
#endif

static void do_zlib_exception(int rc, const char* func, ExceptionSink* xsink) {
   QoreStringNode* desc = new QoreStringNode();
   desc->sprintf("%s(): ", func);
   switch (rc) {
      case Z_ERRNO:
	 q_strerror(*desc, errno);
         break;

      case Z_STREAM_ERROR:
         desc->concat("inconsistent stream state");
         break;

      case Z_DATA_ERROR:
         desc->set("unable to process input data; data corrupted");
         break;

      case Z_MEM_ERROR:
         desc->set("insufficient memory to complete operation");
         break;

      case Z_BUF_ERROR:
         desc->set("qore buffer-handling error (report as bug to qore developers)");
         break;

      case Z_VERSION_ERROR:
         desc->set("version mismatch on zlib shared library, check library requirements");
         break;

      default:
         desc->sprintf("error code %d encountered", rc);
         break;
   }
   xsink->raiseException("ZLIB-ERROR", desc);
}

static void do_deflate_end(z_stream* c_stream, ExceptionSink* xsink) {
   int rc = deflateEnd(c_stream);
   if (rc != Z_OK)
      do_zlib_exception(rc, "deflateEnd", xsink);
}

static void do_inflate_end(z_stream* d_stream, ExceptionSink* xsink) {
   int rc = inflateEnd(d_stream);
   if (rc != Z_OK)
      do_zlib_exception(rc, "inflateEnd", xsink);
}

static BinaryNode* qore_deflate_intern(const void* ptr, unsigned long len, int level, bool gzipFormat, ExceptionSink* xsink) {
   z_stream c_stream; // compression stream
   c_stream.zalloc = Z_NULL;
   c_stream.zfree = Z_NULL;
   c_stream.opaque = Z_NULL;
   c_stream.next_in = (Bytef*)ptr;
   c_stream.avail_in = len;

   int rc = deflateInit2(&c_stream, level, Z_DEFLATED, 15 + (gzipFormat ? 16 : 0), 8, Z_DEFAULT_STRATEGY);
   if (rc != Z_OK) {
      do_zlib_exception(rc, "deflateInit2", xsink);
      return 0;
   }

   ON_BLOCK_EXIT(do_deflate_end, &c_stream, xsink);

   // allocate new buffer
   unsigned long bsize = len / 5 + 100;
   void* buf = malloc(bsize);

   c_stream.next_out = (Bytef*)buf;
   c_stream.avail_out = bsize;

   while (c_stream.avail_in) {
      rc = deflate(&c_stream, Z_NO_FLUSH);
      if (rc != Z_OK && rc != Z_BUF_ERROR) {
         free(buf);
         do_zlib_exception(rc, "deflate", xsink);
         return 0;
      }

      if (!c_stream.avail_out) {
         int new_space = ((len / 3) + 100);
         //printd(5, "deflate() Z_BUF_ERROR:1 bsize: %d->%d, new_space: %d avail_out: %d -> %d next_out: %p\n", bsize, bsize + new_space, new_space, c_stream.avail_out, c_stream.avail_out + new_space, c_stream.next_out);

         bsize += new_space;
         c_stream.avail_out += new_space;
         buf = realloc(buf, bsize);
         c_stream.next_out = ((Bytef*)buf) + c_stream.total_out;
      }
      //printd(5, "deflate() Z_BUF_ERROR:1 bsize: %d, avail_out: %d, next_out: %p\n", bsize, c_stream.avail_out, c_stream.next_out);
   }

   while (true) {
      rc = deflate(&c_stream, Z_FINISH);
      if (rc == Z_STREAM_END)
         break;
      if (rc != Z_OK && rc != Z_BUF_ERROR) {
         free(buf);
         do_zlib_exception(rc, "deflate", xsink);
         return 0;
      }
      // resize buffer
      int new_space = 2; //((len / 3) + 100);
      //printd(5, "deflate() Z_BUF_ERROR:2 bsize: %d->%d, new_space: %d avail_out: %d -> %d, next_out: %p\n", bsize, bsize + new_space, new_space, c_stream.avail_out, c_stream.avail_out + new_space, c_stream.next_out);

      bsize += new_space;
      c_stream.avail_out += new_space;
      buf = realloc(buf, bsize);
      c_stream.next_out = ((Bytef*)buf) + c_stream.total_out;
   }

   //printd(5, "deflate() buf: %p, bsize: %d, avail_out: %d, size: %d, next_out: %p\n", buf, bsize, c_stream.avail_out, bsize - c_stream.avail_out, c_stream.next_out);
   return new BinaryNode(buf, bsize - c_stream.avail_out);
}

static char* qore_inflate_intern(const BinaryNode* b, int &len, bool gzipFormat, ExceptionSink* xsink) {
   z_stream d_stream; // decompression stream
   d_stream.zalloc = Z_NULL;
   d_stream.zfree = Z_NULL;
   d_stream.opaque = Z_NULL;
   d_stream.next_in = (Bytef*)b->getPtr();
   d_stream.avail_in = b->size();

   int rc = inflateInit2(&d_stream, 15 + (gzipFormat ? 16 : 0));
   if (rc != Z_OK) {
      do_zlib_exception(rc, "inflateInit", xsink);
      return 0;
   }

   ON_BLOCK_EXIT(do_inflate_end, &d_stream, xsink);

   // allocate new buffer
   unsigned long bsize = b->size() * 2 + 100;
   void* buf = malloc(bsize);

   d_stream.next_out = (Bytef*)buf;
   d_stream.avail_out = bsize;

   while (true) {
      rc = inflate(&d_stream, Z_NO_FLUSH);
      if (rc == Z_STREAM_END)
         break;
      if (rc == Z_BUF_ERROR) {
         if (d_stream.avail_in == 0 && d_stream.avail_out > 0) {
            xsink->raiseException("ZLIB-ERROR", "Unexpected end of compressed data");
            free(buf);
            return 0;
         }
         int new_space = ((b->size() * 3) + 100);
         bsize += new_space;
         d_stream.avail_out += new_space;
         buf = realloc(buf, bsize);
         d_stream.next_out = ((Bytef*)buf) + d_stream.total_out;
      }
      else if (rc != Z_OK) {
         free(buf);
         do_zlib_exception(rc, "inflate", xsink);
         return 0;
      }
   }

   if (d_stream.avail_in > 0) {
      xsink->raiseException("ZLIB-ERROR", "Unexpected bytes at the end of compressed data");
      free(buf);
      return 0;
   }

   // how much data was decompressed
   len = bsize - d_stream.avail_out;
   return static_cast<char *>(buf);
}

BinaryNode* qore_deflate(const void* ptr, unsigned long len, int level, ExceptionSink* xsink) {
   return qore_deflate_intern(ptr, len, level, false, xsink);
}

QoreStringNode* qore_inflate_to_string(const BinaryNode* b, const QoreEncoding* enc, ExceptionSink* xsink) {
   int len;
   char *buf = qore_inflate_intern(b, len, false, xsink);
   return buf ? new QoreStringNode(buf, len, len, enc) : 0;
}

BinaryNode* qore_inflate_to_binary(const BinaryNode* b, ExceptionSink* xsink) {
   int len;
   char *buf = qore_inflate_intern(b, len, false, xsink);
   return buf ? new BinaryNode(buf, len) : 0;
}

BinaryNode* qore_gzip(const void* ptr, unsigned long len, int level, ExceptionSink* xsink) {
   return qore_deflate_intern(ptr, len, level, true, xsink);
}

QoreStringNode* qore_gunzip_to_string(const BinaryNode* bin, const QoreEncoding* enc, ExceptionSink* xsink) {
   int len;
   char *buf = qore_inflate_intern(bin, len, true, xsink);
   return buf ? new QoreStringNode(buf, len, len, enc) : 0;
}

BinaryNode* qore_gunzip_to_binary(const BinaryNode* bin, ExceptionSink* xsink) {
   int len;
   char *buf = qore_inflate_intern(bin, len, true, xsink);
   return buf ? new BinaryNode(buf, len) : 0;
}
# 498 "ql_compression.qpp"
# 528 "ql_compression.qpp"
# 554 "ql_compression.qpp"
# 560 "ql_compression.qpp"
# 579 "ql_compression.qpp"
# 585 "ql_compression.qpp"
# 601 "ql_compression.qpp"
# 607 "ql_compression.qpp"
# 633 "ql_compression.qpp"
# 658 "ql_compression.qpp"
# 664 "ql_compression.qpp"
# 682 "ql_compression.qpp"
# 688 "ql_compression.qpp"
# 704 "ql_compression.qpp"
# 710 "ql_compression.qpp"
# 731 "ql_compression.qpp"
# 754 "ql_compression.qpp"
# 760 "ql_compression.qpp"
# 776 "ql_compression.qpp"
# 782 "ql_compression.qpp"
# 800 "ql_compression.qpp"
# 806 "ql_compression.qpp"
# 830 "ql_compression.qpp"
// binary bunzip2_to_binary(binary bin) {}
static QoreValue f_bunzip2_to_binary_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 772 "ql_compression.qpp"
   return qore_bunzip2_to_binary(bin, xsink);
}

// nothing bunzip2_to_binary() {}
static QoreValue f_bunzip2_to_binary(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 779 "ql_compression.qpp"

    return QoreValue();
}

// string bunzip2_to_string(binary bin, *string encoding) {}
static QoreValue f_bunzip2_to_string_VyNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 795 "ql_compression.qpp"
   const QoreEncoding* qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return qore_bunzip2_to_string(bin, qe, xsink);
}

// nothing bunzip2_to_string() {}
static QoreValue f_bunzip2_to_string(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 803 "ql_compression.qpp"

    return QoreValue();
}

// binary bzip2(binary bin, softint level = BZ2_DEFAULT_COMPRESSION) {}
static QoreValue f_bzip2_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 724 "ql_compression.qpp"
   if (!level || level > 9)
      return xsink->raiseException("BZLIB2-LEVEL-ERROR", "level must be between 1 - 9 (value passed: %d)", level);

   return qore_bzip2(bin->getPtr(), bin->size(), level, xsink);
}

// binary bzip2(string str, softint level = BZ2_DEFAULT_COMPRESSION) {}
static QoreValue f_bzip2_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 747 "ql_compression.qpp"
   if (!level || level > 9)
      return xsink->raiseException("BZLIB2-LEVEL-ERROR", "level must be between 1 - 9 (value passed: %d)", level);

   return qore_bzip2(str->getBuffer(), str->strlen(), level, xsink);
}

// nothing bzip2() {}
static QoreValue f_bzip2(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 757 "ql_compression.qpp"

    return QoreValue();
}

// binary compress(string str, int level = Z_DEFAULT_COMPRESSION) {}
static QoreValue f_compress_VsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 518 "ql_compression.qpp"
   if ((level < 1 && level != -1) || level > 9)
      return xsink->raiseException("ZLIB-LEVEL-ERROR", "level must be between 1 - 9 or -1 (value passed: %d)", level);

   if (!str->strlen())
      return new BinaryNode;

   return qore_deflate(str->getBuffer(), str->strlen(), level, xsink);
}

// binary compress(binary bin, int level = Z_DEFAULT_COMPRESSION) {}
static QoreValue f_compress_VyVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 544 "ql_compression.qpp"
   if ((level < 1 && level != -1) || level > 9)
      return xsink->raiseException("ZLIB-LEVEL-ERROR", "level must be between 1 - 9 or -1 (value passed: %d)", level);

   if (!bin->size())
      return new BinaryNode;

   return qore_deflate(bin->getPtr(), bin->size(), level, xsink);
}

// nothing compress() {}
static QoreValue f_compress(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 557 "ql_compression.qpp"

    return QoreValue();
}

// Transform get_compressor(string alg, int level = COMPRESSION_LEVEL_DEFAULT) {}
static QoreValue f_get_compressor_VsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 822 "ql_compression.qpp"
   SimpleRefHolder<Transform> t(CompressionTransforms::getCompressor(alg, level, xsink));
   if (*xsink) {
      return 0;
   }
   return new QoreObject(QC_TRANSFORM, getProgram(), t.release());
}

// Transform get_decompressor(string alg) {}
static QoreValue f_get_decompressor_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
# 845 "ql_compression.qpp"
   SimpleRefHolder<Transform> t(CompressionTransforms::getDecompressor(alg, xsink));
   if (*xsink) {
      return 0;
   }
   return new QoreObject(QC_TRANSFORM, getProgram(), t.release());
}

// binary gunzip_to_binary(binary bin) {}
static QoreValue f_gunzip_to_binary_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 700 "ql_compression.qpp"
   return qore_gunzip_to_binary(bin, xsink);
}

// nothing gunzip_to_binary() {}
static QoreValue f_gunzip_to_binary(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 707 "ql_compression.qpp"

    return QoreValue();
}

// string gunzip_to_string(binary bin, *string encoding) {}
static QoreValue f_gunzip_to_string_VyNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 677 "ql_compression.qpp"
   const QoreEncoding* qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return qore_gunzip_to_string(bin, qe, xsink);
}

// nothing gunzip_to_string() {}
static QoreValue f_gunzip_to_string(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 685 "ql_compression.qpp"

    return QoreValue();
}

// binary gzip(string str, int level = Z_DEFAULT_COMPRESSION) {}
static QoreValue f_gzip_VsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 623 "ql_compression.qpp"
   if (!level || level > 9)
      return xsink->raiseException("ZLIB-LEVEL-ERROR", "level must be between 0 - 9 (value passed: %d)", level);

   if (!str->strlen())
      return new BinaryNode;

   return qore_gzip(str->getBuffer(), str->strlen(), level, xsink);
}

// binary gzip(binary bin, int level = Z_DEFAULT_COMPRESSION) {}
static QoreValue f_gzip_VyVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 648 "ql_compression.qpp"
   if (!level || level > 9)
      return xsink->raiseException("ZLIB-LEVEL-ERROR", "level must be between 0 - 9 (value passed: %d)", level);

   if (!bin->size())
      return new BinaryNode;

   return qore_gzip(bin->getPtr(), bin->size(), level, xsink);
}

// nothing gzip() {}
static QoreValue f_gzip(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 661 "ql_compression.qpp"

    return QoreValue();
}

// binary uncompress_to_binary(binary bin) {}
static QoreValue f_uncompress_to_binary_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 597 "ql_compression.qpp"
   return qore_inflate_to_binary(bin, xsink);
}

// nothing uncompress_to_binary() {}
static QoreValue f_uncompress_to_binary(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 604 "ql_compression.qpp"

    return QoreValue();
}

// string uncompress_to_string(binary bin, *string encoding) {}
static QoreValue f_uncompress_to_string_VyNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 574 "ql_compression.qpp"
   const QoreEncoding* qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return qore_inflate_to_string(bin, qe, xsink);
}

// nothing uncompress_to_string() {}
static QoreValue f_uncompress_to_string(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 582 "ql_compression.qpp"

    return QoreValue();
}


DLLLOCAL void init_compression_functions(QoreNamespace& ns) {
    // binary bunzip2_to_binary(binary bin) {}
    ns.addBuiltinVariant("bunzip2_to_binary", (q_func_n_t)f_bunzip2_to_binary_Vy, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // nothing bunzip2_to_binary() {}
    ns.addBuiltinVariant("bunzip2_to_binary", (q_func_n_t)f_bunzip2_to_binary, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string bunzip2_to_string(binary bin, *string encoding) {}
    ns.addBuiltinVariant("bunzip2_to_string", (q_func_n_t)f_bunzip2_to_string_VyNs, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // nothing bunzip2_to_string() {}
    ns.addBuiltinVariant("bunzip2_to_string", (q_func_n_t)f_bunzip2_to_string, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // binary bzip2(binary bin, softint level = BZ2_DEFAULT_COMPRESSION) {}
    ns.addBuiltinVariant("bzip2", (q_func_n_t)f_bzip2_Vyvi, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", softBigIntTypeInfo, QoreSimpleValue().assign((int64)BZ2_DEFAULT_COMPRESSION), "level");
    // binary bzip2(string str, softint level = BZ2_DEFAULT_COMPRESSION) {}
    ns.addBuiltinVariant("bzip2", (q_func_n_t)f_bzip2_Vsvi, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)BZ2_DEFAULT_COMPRESSION), "level");
    // nothing bzip2() {}
    ns.addBuiltinVariant("bzip2", (q_func_n_t)f_bzip2, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // binary compress(string str, int level = Z_DEFAULT_COMPRESSION) {}
    ns.addBuiltinVariant("compress", (q_func_n_t)f_compress_VsVi, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", bigIntTypeInfo, QoreSimpleValue().assign((int64)Z_DEFAULT_COMPRESSION), "level");
    // binary compress(binary bin, int level = Z_DEFAULT_COMPRESSION) {}
    ns.addBuiltinVariant("compress", (q_func_n_t)f_compress_VyVi, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", bigIntTypeInfo, QoreSimpleValue().assign((int64)Z_DEFAULT_COMPRESSION), "level");
    // nothing compress() {}
    ns.addBuiltinVariant("compress", (q_func_n_t)f_compress, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // Transform get_compressor(string alg, int level = COMPRESSION_LEVEL_DEFAULT) {}
    ns.addBuiltinVariant("get_compressor", (q_func_n_t)f_get_compressor_VsVi, QCF_NO_FLAGS, QDOM_DEFAULT, QC_TRANSFORM->getTypeInfo(), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "alg", bigIntTypeInfo, QoreSimpleValue().assign((int64)COMPRESSION_LEVEL_DEFAULT), "level");
    // Transform get_decompressor(string alg) {}
    ns.addBuiltinVariant("get_decompressor", (q_func_n_t)f_get_decompressor_Vs, QCF_NO_FLAGS, QDOM_DEFAULT, QC_TRANSFORM->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "alg");
    // binary gunzip_to_binary(binary bin) {}
    ns.addBuiltinVariant("gunzip_to_binary", (q_func_n_t)f_gunzip_to_binary_Vy, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // nothing gunzip_to_binary() {}
    ns.addBuiltinVariant("gunzip_to_binary", (q_func_n_t)f_gunzip_to_binary, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string gunzip_to_string(binary bin, *string encoding) {}
    ns.addBuiltinVariant("gunzip_to_string", (q_func_n_t)f_gunzip_to_string_VyNs, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // nothing gunzip_to_string() {}
    ns.addBuiltinVariant("gunzip_to_string", (q_func_n_t)f_gunzip_to_string, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // binary gzip(string str, int level = Z_DEFAULT_COMPRESSION) {}
    ns.addBuiltinVariant("gzip", (q_func_n_t)f_gzip_VsVi, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", bigIntTypeInfo, QoreSimpleValue().assign((int64)Z_DEFAULT_COMPRESSION), "level");
    // binary gzip(binary bin, int level = Z_DEFAULT_COMPRESSION) {}
    ns.addBuiltinVariant("gzip", (q_func_n_t)f_gzip_VyVi, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", bigIntTypeInfo, QoreSimpleValue().assign((int64)Z_DEFAULT_COMPRESSION), "level");
    // nothing gzip() {}
    ns.addBuiltinVariant("gzip", (q_func_n_t)f_gzip, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // binary uncompress_to_binary(binary bin) {}
    ns.addBuiltinVariant("uncompress_to_binary", (q_func_n_t)f_uncompress_to_binary_Vy, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // nothing uncompress_to_binary() {}
    ns.addBuiltinVariant("uncompress_to_binary", (q_func_n_t)f_uncompress_to_binary, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string uncompress_to_string(binary bin, *string encoding) {}
    ns.addBuiltinVariant("uncompress_to_string", (q_func_n_t)f_uncompress_to_string_VyNs, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // nothing uncompress_to_string() {}
    ns.addBuiltinVariant("uncompress_to_string", (q_func_n_t)f_uncompress_to_string, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);

}

DLLLOCAL void init_compression_constants(QoreNamespace& ns) {
# 460 "ql_compression.qpp"
# 466 "ql_compression.qpp"
    ns.addConstant("BZ2_DEFAULT_COMPRESSION", ((int64)BZ2_DEFAULT_COMPRESSION));
    ns.addConstant("COMPRESSION_LEVEL_DEFAULT", ((int64)COMPRESSION_LEVEL_DEFAULT));
    ns.addConstant("Z_DEFAULT_COMPRESSION", ((int64)Z_DEFAULT_COMPRESSION));
# 473 "ql_compression.qpp"
# 491 "ql_compression.qpp"
    ns.addConstant("COMPRESSION_ALG_BZIP2", new QoreStringNode(CompressionTransforms::ALG_BZIP2));
    ns.addConstant("COMPRESSION_ALG_GZIP", new QoreStringNode(CompressionTransforms::ALG_GZIP));
    ns.addConstant("COMPRESSION_ALG_ZLIB", new QoreStringNode(CompressionTransforms::ALG_ZLIB));
# 498 "ql_compression.qpp"
# 528 "ql_compression.qpp"
# 554 "ql_compression.qpp"
# 560 "ql_compression.qpp"
# 579 "ql_compression.qpp"
# 585 "ql_compression.qpp"
# 601 "ql_compression.qpp"
# 607 "ql_compression.qpp"
# 633 "ql_compression.qpp"
# 658 "ql_compression.qpp"
# 664 "ql_compression.qpp"
# 682 "ql_compression.qpp"
# 688 "ql_compression.qpp"
# 704 "ql_compression.qpp"
# 710 "ql_compression.qpp"
# 731 "ql_compression.qpp"
# 754 "ql_compression.qpp"
# 760 "ql_compression.qpp"
# 776 "ql_compression.qpp"
# 782 "ql_compression.qpp"
# 800 "ql_compression.qpp"
# 806 "ql_compression.qpp"
# 830 "ql_compression.qpp"

}
