/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_misc.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_misc.h"
#include "qore/intern/QoreSignal.h"
#include "qore/intern/QoreNamespaceIntern.h"
#include "qore/intern/QC_Program.h"
#include "qore/intern/ModuleInfo.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreHashNodeIntern.h"

#include <cerrno>
#include <cstring>
#include <ctime>

#ifndef WARN_MODULES
// needed so that the Qore default argument value in sinatures below will match a C++ value
#define WARN_MODULES QP_WARN_MODULES
#endif

static const QoreFunction* get_builtin_func(const QoreStringNode* str, ExceptionSink* xsink) {
   const qore_ns_private* ns;
   const QoreFunction* f = qore_root_ns_private::runtimeFindFunction(*(getRootNS()), str->c_str(), ns);
   if (f)
      return f;

   xsink->raiseException("NO-FUNCTION", "cannot find any builtin function '%s()'", str->c_str());
   return nullptr;
}

static const char* tlist[] = { "OPTION", "ALGORITHM", "FUNCTION" };

// signal constants - if they are not defined define them here
#ifndef SIGABRT
#define SIGABRT 0
#endif
#ifndef SIGALRM
#define SIGALRM 0
#endif
#ifndef SIGBUS
#define SIGBUS 0
#endif
#ifndef SIGCANCEL
#define SIGCANCEL 0
#endif
#ifndef SIGCHLD
#define SIGCHLD 0
#endif
#ifndef SIGCLD
#define SIGCLD 0
#endif
#ifndef SIGCONT
#define SIGCONT 0
#endif
#ifndef SIGEMT
#define SIGEMT 0
#endif
#ifndef SIGFPE
#define SIGFPE 0
#endif
#ifndef SIGFREEZE
#define SIGFREEZE 0
#endif
#ifndef SIGHUP
#define SIGHUP 0
#endif
#ifndef SIGILL
#define SIGILL 0
#endif
#ifndef SIGINFO
#define SIGINFO 0
#endif
#ifndef SIGINT
#define SIGINT 0
#endif
#ifndef SIGIO
#define SIGIO 0
#endif
#ifndef SIGIOT
#define SIGIOT 0
#endif
#ifndef SIGJVM1
#define SIGJVM1 0
#endif
#ifndef SIGJVM2
#define SIGJVM2 0
#endif
#ifndef SIGKILL
#define SIGKILL 0
#endif
#ifndef SIGLOST
#define SIGLOST 0
#endif
#ifndef SIGLWP
#define SIGLWP 0
#endif
#ifndef SIGPIPE
#define SIGPIPE 0
#endif
#ifndef SIGPOLL
#define SIGPOLL 0
#endif
#ifndef SIGPROF
#define SIGPROF 0
#endif
#ifndef SIGPWR
#define SIGPWR 0
#endif
#ifndef SIGQUIT
#define SIGQUIT 0
#endif
#ifndef SIGSEGV
#define SIGSEGV 0
#endif
#ifndef SIGSTKFLT
#define SIGSTKFLT 0
#endif
#ifndef SIGSTOP
#define SIGSTOP 0
#endif
#ifndef SIGSYS
#define SIGSYS 0
#endif
#ifndef SIGTERM
#define SIGTERM 0
#endif
#ifndef SIGTHAW
#define SIGTHAW 0
#endif
#ifndef SIGTRAP
#define SIGTRAP 0
#endif
#ifndef SIGTSTP
#define SIGTSTP 0
#endif
#ifndef SIGTTIN
#define SIGTTIN 0
#endif
#ifndef SIGTTOU
#define SIGTTOU 0
#endif
#ifndef SIGURG
#define SIGURG 0
#endif
#ifndef SIGUSR1
#define SIGUSR1 0
#endif
#ifndef SIGUSR2
#define SIGUSR2 0
#endif
#ifndef SIGVTALRM
#define SIGVTALRM 0
#endif
#ifndef SIGWAITING
#define SIGWAITING 0
#endif
#ifndef SIGWINCH
#define SIGWINCH 0
#endif
#ifndef SIGXCPU
#define SIGXCPU 0
#endif
#ifndef SIGXFSZ
#define SIGXFSZ 0
#endif
#ifndef SIGXRES
#define SIGXRES 0
#endif
TypedHashDecl* init_hashdecl_UrlInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("UrlInfo", "::Qore::UrlInfo");
    hd->addMember("host", stringTypeInfo, QoreValue());
    hd->addMember("password", stringTypeInfo, QoreValue());
    hd->addMember("path", stringTypeInfo, QoreValue());
    hd->addMember("port", bigIntTypeInfo, QoreValue());
    hd->addMember("protocol", stringTypeInfo, QoreValue());
    hd->addMember("query", stringTypeInfo, QoreValue());
    hd->addMember("username", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 532 "ql_misc.qpp"
# 568 "ql_misc.qpp"
// nothing remove_signal_handler(softint signal) {}
static QoreValue f_remove_signal_handler_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 signal = HARD_QORE_VALUE_INT(args, 0);
# 584 "ql_misc.qpp"
#ifdef HAVE_SIGNAL_HANDLING
    if (signal <= 0 || signal > QORE_SIGNAL_MAX) {
        return xsink->raiseException("REMOVE-SIGNAL-HANDLER-ERROR", "%d is not a valid signal", signal);
    }
    QSM.removeHandler(signal, xsink);
#else
    return xsink->raiseException("MISSING-FEATURE-ERROR", "this platform does not support signal handling, therefore "
        "the set_signal_handler() and remove_signal_handler() functions are not available in Qore; for maximum "
        "portability, use the constant Option::HAVE_SIGNAL_HANDLING to check if this function is implemented before "
        "calling");
#endif
    return QoreValue();
}

// nothing set_signal_handler(softint signal, code f) {}
static QoreValue f_set_signal_handler_viVc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 signal = HARD_QORE_VALUE_INT(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 554 "ql_misc.qpp"
#ifdef HAVE_SIGNAL_HANDLING
    if (signal <= 0 || signal > QORE_SIGNAL_MAX)
        return xsink->raiseException("SET-SIGNAL-HANDLER-ERROR", "%d is not a valid signal", signal);

    QSM.setHandler(signal, f, xsink);
#else
    return xsink->raiseException("MISSING-FEATURE-ERROR", "this platform does not support signal handling, therefore "
        "the set_signal_handler() and remove_signal_handler() functions are not available in Qore; for maximum "
        "portability, use the constant Option::HAVE_SIGNAL_HANDLING to check if this function is implemented before "
        "calling");
#endif
    return QoreValue();
}

# 599 "ql_misc.qpp"
# 637 "ql_misc.qpp"
# 666 "ql_misc.qpp"
# 694 "ql_misc.qpp"
# 717 "ql_misc.qpp"
# 750 "ql_misc.qpp"
# 777 "ql_misc.qpp"
# 793 "ql_misc.qpp"
# 804 "ql_misc.qpp"
# 817 "ql_misc.qpp"
# 823 "ql_misc.qpp"
# 839 "ql_misc.qpp"
# 852 "ql_misc.qpp"
# 868 "ql_misc.qpp"
# 874 "ql_misc.qpp"
# 895 "ql_misc.qpp"
# 914 "ql_misc.qpp"
# 920 "ql_misc.qpp"
# 939 "ql_misc.qpp"
# 945 "ql_misc.qpp"
# 957 "ql_misc.qpp"
# 995 "ql_misc.qpp"
# 1001 "ql_misc.qpp"
# 1012 "ql_misc.qpp"
# 1018 "ql_misc.qpp"
# 1036 "ql_misc.qpp"
# 1085 "ql_misc.qpp"
# 1130 "ql_misc.qpp"
# 1173 "ql_misc.qpp"
# 1234 "ql_misc.qpp"
# 1240 "ql_misc.qpp"
# 1274 "ql_misc.qpp"
# 1280 "ql_misc.qpp"
# 1302 "ql_misc.qpp"
# 1322 "ql_misc.qpp"
# 1328 "ql_misc.qpp"
# 1353 "ql_misc.qpp"
# 1376 "ql_misc.qpp"
# 1399 "ql_misc.qpp"
# 1422 "ql_misc.qpp"
# 1441 "ql_misc.qpp"
# 1447 "ql_misc.qpp"
# 1471 "ql_misc.qpp"
# 1494 "ql_misc.qpp"
# 1500 "ql_misc.qpp"
# 1528 "ql_misc.qpp"
# 1551 "ql_misc.qpp"
# 1576 "ql_misc.qpp"
# 1598 "ql_misc.qpp"
# 1625 "ql_misc.qpp"
# 1647 "ql_misc.qpp"
# 1672 "ql_misc.qpp"
# 1683 "ql_misc.qpp"
# 1699 "ql_misc.qpp"
# 1717 "ql_misc.qpp"
# 1723 "ql_misc.qpp"
# 1748 "ql_misc.qpp"
# 1766 "ql_misc.qpp"
# 1791 "ql_misc.qpp"
# 1816 "ql_misc.qpp"
# 1838 "ql_misc.qpp"
# 1844 "ql_misc.qpp"
# 1866 "ql_misc.qpp"
# 1891 "ql_misc.qpp"
# 1916 "ql_misc.qpp"
# 1943 "ql_misc.qpp"
# 1970 "ql_misc.qpp"
# 1992 "ql_misc.qpp"
# 2014 "ql_misc.qpp"
# 2020 "ql_misc.qpp"
# 2045 "ql_misc.qpp"
# 2070 "ql_misc.qpp"
# 2095 "ql_misc.qpp"
# 2120 "ql_misc.qpp"
# 2147 "ql_misc.qpp"
# 2172 "ql_misc.qpp"
# 2197 "ql_misc.qpp"
# 2222 "ql_misc.qpp"
# 2233 "ql_misc.qpp"
# 2254 "ql_misc.qpp"
# 2277 "ql_misc.qpp"
# 2296 "ql_misc.qpp"
# 2318 "ql_misc.qpp"
# 2341 "ql_misc.qpp"
# 2347 "ql_misc.qpp"
# 2365 "ql_misc.qpp"
# 2383 "ql_misc.qpp"
# 2389 "ql_misc.qpp"
# 2412 "ql_misc.qpp"
# 2435 "ql_misc.qpp"
# 2452 "ql_misc.qpp"
# 2458 "ql_misc.qpp"
# 2480 "ql_misc.qpp"
# 2524 "ql_misc.qpp"
# 2530 "ql_misc.qpp"
# 2551 "ql_misc.qpp"
# 2557 "ql_misc.qpp"
# 2584 "ql_misc.qpp"
# 2616 "ql_misc.qpp"
# 2622 "ql_misc.qpp"
# 2641 "ql_misc.qpp"
# 2665 "ql_misc.qpp"
# 2688 "ql_misc.qpp"
# 2694 "ql_misc.qpp"
# 2718 "ql_misc.qpp"
# 2761 "ql_misc.qpp"
# 2796 "ql_misc.qpp"
# 2808 "ql_misc.qpp"
# 2820 "ql_misc.qpp"
# 2832 "ql_misc.qpp"
# 2861 "ql_misc.qpp"
# 2892 "ql_misc.qpp"
# 2937 "ql_misc.qpp"
# 2951 "ql_misc.qpp"
# 2967 "ql_misc.qpp"
# 3015 "ql_misc.qpp"
# 3035 "ql_misc.qpp"
# 3058 "ql_misc.qpp"
# 3074 "ql_misc.qpp"
# 3095 "ql_misc.qpp"
# 3123 "ql_misc.qpp"
# 3150 "ql_misc.qpp"
// string backquote(string cmd, *reference<int> rc) {}
static QoreValue f_backquote_VsC15_reference_int_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cmd = HARD_QORE_VALUE_STRING(args, 0);
    const ReferenceNode* rc = get_param_value(args, 1).get<const ReferenceNode>();
# 1261 "ql_misc.qpp"
    int prc;
    SimpleRefHolder<QoreStringNode> str(backquoteEval(cmd->c_str(), prc, xsink));
    if (!*xsink && rc) {
        QoreTypeSafeReferenceHelper rh(rc, xsink);
        if (!rh)
            return QoreValue();

        rh.assign((int64)prc);
    }
    return str.release();
}

// nothing backquote() {}
static QoreValue f_backquote(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1277 "ql_misc.qpp"

    return QoreValue();
}

// auto call_builtin_function(string name, ...) {}
static QoreValue f_call_builtin_function_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 736 "ql_misc.qpp"
    const QoreFunction* f = get_builtin_func(name, xsink);
    if (!f)
        return QoreValue();

    ReferenceHolder<QoreListNode> vargs(xsink);
    // if there are arguments to pass, create argument list by copying current list
    if (num_args(args) > 1) {
        vargs = args->copyListFrom(1);
    }

    return f->evalDynamic(*vargs, xsink);
}

// auto call_builtin_function_args(string name, *softlist<auto> vargs) {}
static QoreValue f_call_builtin_function_args_VsC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 769 "ql_misc.qpp"
    const QoreFunction* f = get_builtin_func(name, xsink);
    if (!f)
        return QoreValue();

    return f->evalDynamic(vargs, xsink);
}

// auto call_function(string name, ...) {}
static QoreValue f_call_function_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 627 "ql_misc.qpp"
    ReferenceHolder<QoreListNode> vargs(xsink);
    // if there are arguments to pass, create argument list by copying current list
    if (num_args(args) > 1) {
        vargs = args->copyListFrom(1);
    }

    return getProgram()->callFunction(name->c_str(), *vargs, xsink);
}

// auto call_function(code f, ...) {}
static QoreValue f_call_function_VcVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* f = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 656 "ql_misc.qpp"
    ReferenceHolder<QoreListNode> vargs(xsink);
    // if there are arguments to pass, create argument list by copying current list
    if (num_args(args) > 1) {
        vargs = args->copyListFrom(1);
    }

    return f->execValue(*vargs, xsink);
}

// auto call_function_args(string name, *softlist<auto> vargs) {}
static QoreValue f_call_function_args_VsC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 690 "ql_misc.qpp"
    return getProgram()->callFunction(name->c_str(), vargs, xsink);
}

// auto call_function_args(code f, *softlist<auto> vargs) {}
static QoreValue f_call_function_args_VcC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* f = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 713 "ql_misc.qpp"
    return f->execValue(vargs, xsink);
}

// string decode_uri_request(string uri) {}
static QoreValue f_decode_uri_request_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* uri = HARD_QORE_VALUE_STRING(args, 0);
# 2755 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(QCS_UTF8));
   str->concatDecodeUriRequest(*uri, xsink);
   return *xsink ? QoreValue() : str.release();
}

// string decode_url(string url) {}
static QoreValue f_decode_url_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 2682 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(QCS_UTF8));
   str->concatDecodeUrl(*url, xsink);
   return *xsink ? QoreValue() : str.release();
}

// nothing decode_url() {}
static QoreValue f_decode_url(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2691 "ql_misc.qpp"

    return QoreValue();
}

// string encode_uri_request(string url) {}
static QoreValue f_encode_uri_request_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 2790 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(url->getEncoding()));
   str->concatEncodeUriRequest(xsink, url);
   return *xsink ? QoreValue() : str.release();
}

// string encode_url(string url, softbool encode_all = False) {}
static QoreValue f_encode_url_Vsvb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
    bool encode_all = HARD_QORE_VALUE_BOOL(args, 1);
# 2712 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(url->getEncoding()));
   str->concatEncodeUrl(xsink, url, encode_all);
   return *xsink ? QoreValue() : str.release();
}

// bool exists(...) {}
static QoreValue f_exists_VV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 787 "ql_misc.qpp"
    // to emulate the exists operator, we must return True if more than one argument is passed
    // as this will appear to be a list to the exists operator, which is different from NOTHING
    return (num_args(args) <= 1) ? !get_param_value(args, 0).isNothing() : true;
}

// bool existsFunction(string name) {}
static QoreValue f_existsFunction_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 800 "ql_misc.qpp"
    return getProgram()->existsFunction(name->c_str());
}

// bool existsFunction(code c) {}
static QoreValue f_existsFunction_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 813 "ql_misc.qpp"
    return true;
}

// nothing existsFunction() {}
static QoreValue f_existsFunction(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 820 "ql_misc.qpp"

    return QoreValue();
}

// bool exists_function(string name) {}
static QoreValue f_exists_function_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 835 "ql_misc.qpp"
    return getProgram()->existsFunction(name->c_str());
}

// bool exists_function(code c) {}
static QoreValue f_exists_function_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 848 "ql_misc.qpp"
   return true;
}

// *string functionType(string name) {}
static QoreValue f_functionType_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 859 "ql_misc.qpp"
    const qore_ns_private* ns;
    const QoreFunction* f = qore_root_ns_private::runtimeFindFunction(*(getRootNS()), name->c_str(), ns);
    if (!f)
        return QoreValue();

    return new QoreStringNode(f->hasBuiltin() ? "builtin" : "user");
}

// nothing functionType() {}
static QoreValue f_functionType(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 871 "ql_misc.qpp"

    return QoreValue();
}

// *string function_type(string name) {}
static QoreValue f_function_type_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 886 "ql_misc.qpp"
    const qore_ns_private* ns;
    const QoreFunction* f = qore_root_ns_private::runtimeFindFunction(*(getRootNS()), name->c_str(), ns);
    if (!f)
        return QoreValue();

    return new QoreStringNode(f->hasBuiltin() ? "builtin" : "user");
}

// *int getByte(string str, softint offset = 0) {}
static QoreValue f_getByte_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1828 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// nothing getByte() {}
static QoreValue f_getByte(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1841 "ql_misc.qpp"

    return QoreValue();
}

// *int getByte(binary b, softint offset = 0) {}
static QoreValue f_getByte_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1856 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// string getClassName(object obj) {}
static QoreValue f_getClassName_Vo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 1008 "ql_misc.qpp"
    return new QoreStringNode(obj->getClass()->getName());
}

// nothing getClassName() {}
static QoreValue f_getClassName(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1015 "ql_misc.qpp"

    return QoreValue();
}

// list<string> getFeatureList() {}
static QoreValue f_getFeatureList(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1679 "ql_misc.qpp"
   return getProgram()->getFeatureList();
}

// hash<string,hash<auto>> getModuleHash() {}
static QoreValue f_getModuleHash(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1643 "ql_misc.qpp"
   return MM.getModuleHash();
}

// list<hash<auto>> getModuleList() {}
static QoreValue f_getModuleList(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1594 "ql_misc.qpp"
   return MM.getModuleList();
}

// *int getWord32(string str, softint offset = 0) {}
static QoreValue f_getWord32_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1982 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return ntohl(*((unsigned int *)&ptr[offset]));
}

// *int getWord32(binary b, softint offset = 0) {}
static QoreValue f_getWord32_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2004 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return (int64)ntohl(*((unsigned int *)&ptr[offset]));
}

// nothing getWord32() {}
static QoreValue f_getWord32(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2017 "ql_misc.qpp"

    return QoreValue();
}

// *int get_byte(string str, softint offset = 0) {}
static QoreValue f_get_byte_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1781 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// *int get_byte(binary b, softint offset = 0) {}
static QoreValue f_get_byte_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1806 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// code get_call_reference(string identifier) {}
static QoreValue f_get_call_reference_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* identifier = HARD_QORE_VALUE_STRING(args, 0);
# 3174 "ql_misc.qpp"
    return get_call_reference_intern(runtime_get_stack_object(), identifier, xsink);
}

// string get_class_name(object obj) {}
static QoreValue f_get_class_name_Vo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 1032 "ql_misc.qpp"
    return new QoreStringNode(obj->getClass()->getName());
}

// string get_default_encoding() {}
static QoreValue f_get_default_encoding(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 953 "ql_misc.qpp"
    return new QoreStringNode(QCS_DEFAULT->getCode());
}

// string get_ex_pos(hash<auto> ex) {}
static QoreValue f_get_ex_pos_C10hash_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* ex = HARD_QORE_VALUE_HASH(args, 0);
# 2983 "ql_misc.qpp"
    QoreValue n = ex->getKeyValue("file");
    QoreStringNode* str = new QoreStringNode(n.getType() == NT_STRING
        ? n.get<const QoreStringNode>()->c_str()
        : "<unknown>");
    int ln = (int)ex->getKeyValue("line").getAsBigInt();
    str->sprintf(":%d", ln);
    bool openparen = false;
    n = ex->getKeyValue("lang");
    if (n.getType() == NT_STRING) {
        str->sprintf(" (%s", n.get<const QoreStringNode>()->c_str());
        openparen = true;
    }
    n = ex->getKeyValue("source");
    if (n.getType() == NT_STRING) {
        const QoreStringNode* sstr = n.get<const QoreStringNode>();
        if (!sstr->empty()) {
            int offset = (int)ex->getKeyValue("offset").getAsBigInt();
            str->concat(" ");
            if (!openparen) {
                str->concat('(');
                openparen = true;
            }
            str->sprintf("source \"%s\":%d", sstr->c_str(), ln + offset);
        }
    }
    if (openparen) {
        str->concat(')');
    }
    return str;
}

// list<string> get_feature_list() {}
static QoreValue f_get_feature_list(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1695 "ql_misc.qpp"
   return getProgram()->getFeatureList();
}

// hash<auto> get_global_vars() {}
static QoreValue f_get_global_vars(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3070 "ql_misc.qpp"
   return getProgram()->getGlobalVars();
}

// hash<auto> get_local_vars(int frame) {}
static QoreValue f_get_local_vars_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 frame = HARD_QORE_VALUE_INT(args, 0);
# 3031 "ql_misc.qpp"
    return thread_get_local_vars(frame, xsink);
}

// hash<string,hash<auto>> get_module_hash() {}
static QoreValue f_get_module_hash(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1668 "ql_misc.qpp"
   return MM.getModuleHash();
}

// list<hash<auto>> get_module_list() {}
static QoreValue f_get_module_list(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1621 "ql_misc.qpp"
   return MM.getModuleList();
}

// auto get_module_option(string module, string option) {}
static QoreValue f_get_module_option_VsVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* module = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* option = HARD_QORE_VALUE_STRING(args, 1);
# 3110 "ql_misc.qpp"
    TempEncodingHelper mod_str(module, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    TempEncodingHelper opt_str(option, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    return qore_get_module_option(mod_str->c_str(), opt_str->c_str());
}

// int get_parse_options() {}
static QoreValue f_get_parse_options(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2947 "ql_misc.qpp"
   return runtime_get_parse_options();
}

// hash<auto> get_qore_library_info() {}
static QoreValue f_get_qore_library_info(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2912 "ql_misc.qpp"
   QoreHashNode* h = new QoreHashNode(autoTypeInfo);

   h->setKeyValue("PlatformOS", new QoreStringNode(qore_target_os), xsink);
   h->setKeyValue("PlatformCPU", new QoreStringNode(qore_target_arch), xsink);
   h->setKeyValue("VersionString", new QoreStringNode(qore_version_string), xsink);
   h->setKeyValue("VersionMajor", qore_version_major, xsink);
   h->setKeyValue("VersionMinor", qore_version_minor, xsink);
   h->setKeyValue("VersionSub", qore_version_sub, xsink);
   h->setKeyValue("Build", qore_build_number, xsink);

   h->setKeyValue("UserModuleVerDir", new QoreStringNode(qore_user_module_ver_dir), xsink);
   h->setKeyValue("UserModuleDir", new QoreStringNode(qore_user_module_dir), xsink);
   h->setKeyValue("BinaryModuleVerDir", new QoreStringNode(qore_module_ver_dir), xsink);
   h->setKeyValue("BinaryModuleDir", new QoreStringNode(qore_module_dir), xsink);
   h->setKeyValue("ModulePath", new QoreStringNodeMaker("%s:%s:%s:%s", qore_user_module_ver_dir, qore_user_module_dir, qore_module_ver_dir, qore_module_dir), xsink);

   h->setKeyValue("BuildHost", new QoreStringNode(qore_build_host), xsink);
   h->setKeyValue("Compiler", new QoreStringNode(qore_cplusplus_compiler), xsink);
   h->setKeyValue("CFLAGS", new QoreStringNode(qore_cflags), xsink);
   h->setKeyValue("LDFLAGS", new QoreStringNode(qore_ldflags), xsink);

   return h;
}

// hash<string,hash<auto>> get_qore_option_hash() {}
static QoreValue f_get_qore_option_hash(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2877 "ql_misc.qpp"
    QoreHashNode* rv = new QoreHashNode(autoHashTypeInfo);

    for (unsigned j = 0; j < qore_option_list_size; ++j) {
        QoreHashNode* h = new QoreHashNode(autoTypeInfo);
        h->setKeyValue("option", new QoreStringNode(qore_option_list[j].option), xsink);
        h->setKeyValue("constant", new QoreStringNode(qore_option_list[j].constant), xsink);
        h->setKeyValue("type", new QoreStringNode(tlist[qore_option_list[j].type]), xsink);
        h->setKeyValue("value", qore_option_list[j].value, xsink);

        rv->setKeyValue(qore_option_list[j].constant, h, 0);
    }
    return rv;
}

// list<hash<auto>> get_qore_option_list() {}
static QoreValue f_get_qore_option_list(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2846 "ql_misc.qpp"
    QoreListNode* l = new QoreListNode(autoHashTypeInfo);

    for (unsigned j = 0; j < qore_option_list_size; ++j) {
        QoreHashNode* h = new QoreHashNode(autoTypeInfo);
        qore_hash_private* hh = qore_hash_private::get(*h);
        hh->setKeyValueIntern("option", new QoreStringNode(qore_option_list[j].option));
        hh->setKeyValueIntern("constant", new QoreStringNode(qore_option_list[j].constant));
        hh->setKeyValueIntern("type", new QoreStringNode(tlist[qore_option_list[j].type]));
        hh->setKeyValueIntern("value", qore_option_list[j].value);
        l->push(h, xsink);
    }
    return l;
}

// string get_safe_url(string url) {}
static QoreValue f_get_safe_url_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 1180 "ql_misc.qpp"
    QoreURL qurl(xsink, url, QURL_KEEP_BRACKETS);
    if (!qurl.isValid()) {
        assert(*xsink);
        return QoreValue();
    }
    if (!qurl.getPassword()) {
        url->ref();
        return url;
    }

    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(QCS_UTF8));

    const QoreString* str = qurl.getProtocol();
    if (str && !str->empty()) {
        rv->concat(str, xsink);
        rv->concat("://");
    }

    str = qurl.getUserName();
    bool has_user_or_pass = false;
    if (str && !str->empty()) {
        rv->concat(str, xsink);
        has_user_or_pass = true;
    }

    str = qurl.getPassword();
    if (str && !str->empty()) {
        rv->concat(":<masked>");
        if (!has_user_or_pass) {
            has_user_or_pass = true;
        }
    }
    if (has_user_or_pass) {
        rv->concat('@');
    }

    str = qurl.getHost();
    if (str && !str->empty()) {
        rv->concat(str, xsink);
    }

    if (qurl.getPort()) {
        rv->sprintf(":%d", qurl.getPort());
    }

    str = qurl.getPath();
    if (str && !str->empty()) {
        rv->concat(str, xsink);
    }

    return rv.release();
}

// *string get_script_dir() {}
static QoreValue f_get_script_dir(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2816 "ql_misc.qpp"
   return getProgram()->getScriptDir();
}

// *string get_script_name() {}
static QoreValue f_get_script_name(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2828 "ql_misc.qpp"
   return getProgram()->getScriptName();
}

// *string get_script_path() {}
static QoreValue f_get_script_path(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2804 "ql_misc.qpp"
   return getProgram()->getScriptPath();
}

// *int get_word_16(string str, softint offset = 0) {}
static QoreValue f_get_word_16_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1881 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return ntohs(*((unsigned short*)&ptr[offset]));
}

// *int get_word_16(binary b, softint offset = 0) {}
static QoreValue f_get_word_16_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1906 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return ntohs(*((unsigned short*)&ptr[offset]));
}

// *int get_word_16_lsb(string str, softint offset = 0) {}
static QoreValue f_get_word_16_lsb_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2085 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return (int64)LSBi2(*((unsigned short *)&ptr[offset]));
}

// *int get_word_16_lsb(binary b, softint offset = 0) {}
static QoreValue f_get_word_16_lsb_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2110 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return (int64)LSBi2(*((unsigned short *)&ptr[offset]));
}

// *int get_word_32(string str, softint offset = 0) {}
static QoreValue f_get_word_32_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1933 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return ntohl(*((unsigned int*)&ptr[offset]));
}

// *int get_word_32(binary b, softint offset = 0) {}
static QoreValue f_get_word_32_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1960 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return ntohl(*((unsigned int*)&ptr[offset]));
}

// *int get_word_32_lsb(string str, softint offset = 0) {}
static QoreValue f_get_word_32_lsb_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2137 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return (int64)LSBi4(*((unsigned int *)&ptr[offset]));
}

// *int get_word_32_lsb(binary b, softint offset = 0) {}
static QoreValue f_get_word_32_lsb_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2162 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return (int64)LSBi4(*((unsigned int *)&ptr[offset]));
}

// *int get_word_64(string str, softint offset = 0) {}
static QoreValue f_get_word_64_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2035 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= (size - 7) || offset < 0)
      return QoreValue();

   return (int64)MSBi8(*((int64 *)&ptr[offset]));
}

// *int get_word_64(binary b, softint offset = 0) {}
static QoreValue f_get_word_64_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2060 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 7) || offset < 0)
      return QoreValue();

   return (int64)MSBi8(*((int64 *)&ptr[offset]));
}

// *int get_word_64_lsb(string str, softint offset = 0) {}
static QoreValue f_get_word_64_lsb_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2187 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->c_str();
   int size = str->strlen();

   if (offset >= (size - 7) || offset < 0)
      return QoreValue();

   return (int64)LSBi8(*((int64 *)&ptr[offset]));
}

// *int get_word_64_lsb(binary b, softint offset = 0) {}
static QoreValue f_get_word_64_lsb_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 2212 "ql_misc.qpp"
    unsigned char* ptr = (unsigned char* )b->getPtr();
    int size = b->size();

    if (offset >= (size - 7) || offset < 0)
        return QoreValue();

    return (int64)LSBi8(*((int64 *)&ptr[offset]));
}

// bool has_key(hash<auto> h, string key) {}
static QoreValue f_has_key_C10hash_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 1739 "ql_misc.qpp"
    TempEncodingHelper tmp(key, QCS_DEFAULT, xsink);
    if (*xsink)
        return QoreValue();
    bool exists;
    h->getKeyValueExistence(tmp->c_str(), exists);
    return exists;
}

// bool has_key(object obj, string key) {}
static QoreValue f_has_key_VoVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 1759 "ql_misc.qpp"
    TempEncodingHelper tmp(key, QCS_DEFAULT, xsink);
    if (*xsink)
        return QoreValue();
    return obj->hasMember(tmp->c_str(), xsink);
}

// list<auto> hash_values(hash<auto> h) {}
static QoreValue f_hash_values_C10hash_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 1713 "ql_misc.qpp"
    return qore_hash_private::get(*h)->getValues();
}

// nothing hash_values() {}
static QoreValue f_hash_values(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1720 "ql_misc.qpp"

    return QoreValue();
}

// int hextoint(string str) {}
static QoreValue f_hextoint_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 2492 "ql_misc.qpp"
   if (!str->strlen())
      return 0;

   int64 rc = 0;
   int64 pow = 0;
   const char* buf = str->c_str();
   size_t len = str->size();
   if (*buf == '0' && *(buf + 1) == 'x') {
      buf += 2;
      len -= 2;
   }
   else if (*buf == 'x') {
      buf++;
      --len;
   }
   for (const char* p = buf + len - 1; p >= buf; p--) {
      int n = get_nibble(*p, xsink);
      if (xsink->isException())
	 return 0;
      if (!pow) {
	 rc = n;
	 pow = 16;
      }
      else {
	 rc += n * pow;
	 pow *= 16;
      }
   }
   return rc;
}

// nothing hextoint() {}
static QoreValue f_hextoint(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2527 "ql_misc.qpp"

    return QoreValue();
}

// string html_decode(string str) {}
static QoreValue f_html_decode_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 934 "ql_misc.qpp"
    QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
    return rv->concatDecode(xsink, *str, CD_HTML) ? QoreValue() : rv.release();
}

// nothing html_decode() {}
static QoreValue f_html_decode(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 942 "ql_misc.qpp"

    return QoreValue();
}

// string html_encode(string str) {}
static QoreValue f_html_encode_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 909 "ql_misc.qpp"
    QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
    return rv->concatEncode(xsink, *str, CE_HTML) ? QoreValue() : rv.release();
}

// nothing html_encode() {}
static QoreValue f_html_encode(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 917 "ql_misc.qpp"

    return QoreValue();
}

// nothing load_module(string name, int warning_mask = WARN_MODULES) {}
static QoreValue f_load_module_VsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 1);
# 2580 "ql_misc.qpp"
    QMM.runTimeLoadModule(*xsink, *xsink, name->c_str(), getProgram(), nullptr, QMLO_NONE, warning_mask);
    return QoreValue();
}

// nothing load_module() {}
static QoreValue f_load_module(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2619 "ql_misc.qpp"

    return QoreValue();
}

// *hash<ExceptionInfo> load_module_warn(string name, int warning_mask = WARN_MODULES) {}
static QoreValue f_load_module_warn_VsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 1);
# 2607 "ql_misc.qpp"
    ExceptionSink wsink;
    QMM.runTimeLoadModule(*xsink, wsink, name->c_str(), getProgram(), nullptr, QMLO_NONE, warning_mask);
    if (!*xsink && wsink) {
        return wsink.catchException()->makeExceptionObjectAndDelete(xsink);
    }
    return QoreValue();
}

// nothing load_user_module_with_program(string name, Qore::Program pgm) {}
static QoreValue f_load_user_module_with_program_VsC7Program(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    TAKE_HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 1, CID_PROGRAM, "load_user_module_with_program()", "Qore::Program", xsink);
    if (*xsink)
        return 0;
# 2637 "ql_misc.qpp"
    QMM.runTimeLoadModule(*xsink, *xsink, name->c_str(), getProgram(), pgm);
    return QoreValue();
}

// string makeBase64String(string str, softint maxlinelen = -1) {}
static QoreValue f_makeBase64String_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1296 "ql_misc.qpp"
   QoreStringNode* rv = new QoreStringNode;
   rv->concatBase64(str, maxlinelen);
   return rv;
}

// string makeBase64String(binary bin, softint maxlinelen = -1) {}
static QoreValue f_makeBase64String_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1318 "ql_misc.qpp"
   return new QoreStringNode(bin, maxlinelen);
}

// nothing makeBase64String() {}
static QoreValue f_makeBase64String(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1325 "ql_misc.qpp"

    return QoreValue();
}

// string makeHexString(string str) {}
static QoreValue f_makeHexString_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 2359 "ql_misc.qpp"
    QoreStringNode* rv = new QoreStringNode;
    rv->concatHex(str);
    return rv;
}

// string makeHexString(binary bin) {}
static QoreValue f_makeHexString_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 2377 "ql_misc.qpp"
    QoreStringNode* str = new QoreStringNode;
    str->concatHex(bin);
    return str;
}

// nothing makeHexString() {}
static QoreValue f_makeHexString(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2386 "ql_misc.qpp"

    return QoreValue();
}

// string make_base64_string(string str, softint maxlinelen = -1) {}
static QoreValue f_make_base64_string_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1347 "ql_misc.qpp"
   QoreStringNode* rv = new QoreStringNode;
   rv->concatBase64(str, maxlinelen);
   return rv;
}

// string make_base64_string(binary bin, softint maxlinelen = -1) {}
static QoreValue f_make_base64_string_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1372 "ql_misc.qpp"
   return new QoreStringNode(bin, maxlinelen);
}

// string make_base64_url_string(string str) {}
static QoreValue f_make_base64_url_string_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1393 "ql_misc.qpp"
    QoreStringNode* rv = new QoreStringNode;
    rv->concatBase64Url(*str);
    return rv;
}

// string make_base64_url_string(binary bin) {}
static QoreValue f_make_base64_url_string_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 1416 "ql_misc.qpp"
    QoreStringNode* rv = new QoreStringNode;
    rv->concatBase64Url(*bin);
    return rv;
}

// string make_hex_string(string str) {}
static QoreValue f_make_hex_string_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 2406 "ql_misc.qpp"
    QoreStringNode* rv = new QoreStringNode;
    rv->concatHex(str);
    return rv;
}

// string make_hex_string(binary bin) {}
static QoreValue f_make_hex_string_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 2429 "ql_misc.qpp"
    QoreStringNode* str = new QoreStringNode;
    str->concatHex(bin);
    return str;
}

// *hash<auto> parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True) {}
static QoreValue f_parse_VsVsniNsnivb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* code = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* source = get_param_value(args, 3).get<const QoreStringNode>();
    int64 offset = HARD_QORE_VALUE_INT(args, 4);
# 981 "ql_misc.qpp"
    if (warning_mask) {
        ExceptionSink wsink;
        getProgram()->parse(code, label, xsink, &wsink, warning_mask, source, offset);
        if (!wsink.isException())
            return QoreValue();

        QoreException* e = wsink.catchException();
        return e->makeExceptionObjectAndDelete(xsink);
    }

    getProgram()->parse(code, label, xsink, 0, 0, source, offset);
    return QoreValue();
}

// nothing parse() {}
static QoreValue f_parse(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 998 "ql_misc.qpp"

    return QoreValue();
}

// binary parseBase64String(string str) {}
static QoreValue f_parseBase64String_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1437 "ql_misc.qpp"
   return str->parseBase64(xsink);
}

// nothing parseBase64String() {}
static QoreValue f_parseBase64String(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1444 "ql_misc.qpp"

    return QoreValue();
}

// string parseBase64StringToString(string str, *string encoding) {}
static QoreValue f_parseBase64StringToString_VsNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 1489 "ql_misc.qpp"
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return str->parseBase64ToString(qe, xsink);
}

// nothing parseBase64StringToString() {}
static QoreValue f_parseBase64StringToString(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1497 "ql_misc.qpp"

    return QoreValue();
}

// binary parseHexString(string hexstr) {}
static QoreValue f_parseHexString_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* hexstr = HARD_QORE_VALUE_STRING(args, 0);
# 2448 "ql_misc.qpp"
   return hexstr->parseHex(xsink);
}

// nothing parseHexString() {}
static QoreValue f_parseHexString(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2455 "ql_misc.qpp"

    return QoreValue();
}

// *hash<UrlInfo> parseURL(string url, bool keep_brackets = False) {}
static QoreValue f_parseURL_VsVb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
    bool keep_brackets = HARD_QORE_VALUE_BOOL(args, 1);
# 1168 "ql_misc.qpp"
    QoreURL qurl(url, keep_brackets);
    return qurl.isValid() ? qurl.getHash() : QoreValue();
}

// nothing parseURL() {}
static QoreValue f_parseURL(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1237 "ql_misc.qpp"

    return QoreValue();
}

// binary parse_base64_string(string str) {}
static QoreValue f_parse_base64_string_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1467 "ql_misc.qpp"
   return str->parseBase64(xsink);
}

// string parse_base64_string_to_string(string str, *string encoding) {}
static QoreValue f_parse_base64_string_to_string_VsNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 1523 "ql_misc.qpp"
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return str->parseBase64ToString(qe, xsink);
}

// binary parse_base64_url_string(string str) {}
static QoreValue f_parse_base64_url_string_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1547 "ql_misc.qpp"
   return str->parseBase64Url(xsink);
}

// string parse_base64_url_string_to_string(string str, *string encoding) {}
static QoreValue f_parse_base64_url_string_to_string_VsNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 1571 "ql_misc.qpp"
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return str->parseBase64UrlToString(qe, xsink);
}

// binary parse_hex_string(string hexstr) {}
static QoreValue f_parse_hex_string_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* hexstr = HARD_QORE_VALUE_STRING(args, 0);
# 2476 "ql_misc.qpp"
   return hexstr->parseHex(xsink);
}

// hash<UrlInfo> parse_url(string url, bool keep_brackets) {}
static QoreValue f_parse_url_VsVb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
    bool keep_brackets = HARD_QORE_VALUE_BOOL(args, 1);
# 1077 "ql_misc.qpp"
    QoreURL qurl(url, keep_brackets, xsink);
    if (*xsink)
        return QoreValue();
    assert(qurl.isValid());
    return qurl.getHash();
}

// hash<UrlInfo> parse_url(string url, *int options) {}
static QoreValue f_parse_url_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
    int64 options = HARD_QORE_VALUE_INT(args, 1);
# 1125 "ql_misc.qpp"
    QoreURL qurl(xsink, url, options);
    return qurl.isValid() ? qurl.getHash() : QoreValue();
}

// nothing reload_module(string name) {}
static QoreValue f_reload_module_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 2661 "ql_misc.qpp"
    QMM.runTimeLoadModule(*xsink, *xsink, name->c_str(), 0, 0, QMLO_RELOAD);
    return QoreValue();
}

// nothing set_global_var_value(string name, auto value) {}
static QoreValue f_set_global_var_value_VsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 3091 "ql_misc.qpp"
   getProgram()->setGlobalVarValue(name->c_str(), value.refSelf(), xsink);
    return QoreValue();
}

// nothing set_local_var_value(int frame, string var, auto value) {}
static QoreValue f_set_local_var_value_ViVsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 frame = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* var = HARD_QORE_VALUE_STRING(args, 1);
    QoreValue value = get_param_value(args, 2);
# 3053 "ql_misc.qpp"
   if (thread_set_local_var_value(frame, var->c_str(), value, xsink) == 1 && thread_set_closure_var_value(frame, var->c_str(), value, xsink) == 1 && !xsink->isEvent())
      xsink->raiseException("UNKNOWN-VARIABLE", "cannot find local variable '%s' in stack frame %d", var->c_str(), (int)frame);
    return QoreValue();
}

// nothing set_module_option(string module, string option, auto value) {}
static QoreValue f_set_module_option_VsVsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* module = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* option = HARD_QORE_VALUE_STRING(args, 1);
    QoreValue value = get_param_value(args, 2);
# 3137 "ql_misc.qpp"
    TempEncodingHelper mod_str(module, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    TempEncodingHelper opt_str(option, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    qore_set_module_option(mod_str->c_str(), opt_str->c_str(), value.refSelf());
    return QoreValue();
}

// nothing set_return_value(auto val) {}
static QoreValue f_set_return_value_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 2963 "ql_misc.qpp"
    qore_program_private::setReturnValue(*getProgram(), val.refSelf(), xsink);
    return QoreValue();
}

// string splice(string str) {}
static QoreValue f_splice_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2229 "ql_misc.qpp"
    return new QoreStringNode;
}

// string splice(string str, softint start) {}
static QoreValue f_splice_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 2248 "ql_misc.qpp"
    QoreStringNodeHolder pstr(str->copy());
    pstr->splice(start, xsink);
    return *xsink ? QoreValue() : pstr.release();
}

// string splice(string str, softint start, softint len, *string nstr) {}
static QoreValue f_splice_VsviviNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* nstr = get_param_value(args, 3).get<const QoreStringNode>();
# 2271 "ql_misc.qpp"
    QoreStringNodeHolder pstr(str->copy());
    pstr->splice(start, len, QoreValue(nstr), xsink);
    return *xsink ? QoreValue() : pstr.release();
}

// list<auto> splice(list<auto> l, softint start) {}
static QoreValue f_splice_C10list_auto_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 2290 "ql_misc.qpp"
    ReferenceHolder<QoreListNode> lst(l->copy(), xsink);
    ReferenceHolder<> holder(lst->splice(start), xsink);
    return *xsink ? QoreValue() : lst.release();
}

// list<auto> splice(list<auto> l, softint start, softint len) {}
static QoreValue f_splice_C10list_auto_vivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
# 2312 "ql_misc.qpp"
    ReferenceHolder<QoreListNode> lst(l->copy(), xsink);
    ReferenceHolder<> holder(lst->splice(start, len), xsink);
    return *xsink ? QoreValue() : lst.release();
}

// list<auto> splice(list<auto> l, softint start, softint len, softlist<auto> nlist) {}
static QoreValue f_splice_C10list_auto_viviC14softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
    const QoreListNode* nlist = HARD_QORE_VALUE_LIST(args, 3);
# 2335 "ql_misc.qpp"
    ReferenceHolder<QoreListNode> lst(l->copy(), xsink);
    ReferenceHolder<> holder(lst->splice(start, len, nlist, xsink), xsink);
    return *xsink ? QoreValue() : lst.release();
}

// nothing splice() {}
static QoreValue f_splice(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2344 "ql_misc.qpp"

    return QoreValue();
}

// int strtoint(string num, softint base = 10) {}
static QoreValue f_strtoint_Vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* num = HARD_QORE_VALUE_STRING(args, 0);
    int64 base = HARD_QORE_VALUE_INT(args, 1);
# 2543 "ql_misc.qpp"
   errno = 0;
   int64 rv = strtoll(num->c_str(), 0, base);
   if (errno == EINVAL || errno == ERANGE)
      xsink->raiseException("STRTOINT-ERROR", "could not parse '%s' to an integer", num->c_str());
   return rv;
}

// nothing strtoint() {}
static QoreValue f_strtoint(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2554 "ql_misc.qpp"

    return QoreValue();
}


DLLLOCAL void init_misc_functions(QoreNamespace& ns) {
    // nothing remove_signal_handler(softint signal) {}
    ns.addBuiltinVariant("remove_signal_handler", (q_func_n_t)f_remove_signal_handler_vi, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "signal");
    // nothing set_signal_handler(softint signal, code f) {}
    ns.addBuiltinVariant("set_signal_handler", (q_func_n_t)f_set_signal_handler_viVc, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "signal", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // string backquote(string cmd, *reference<int> rc) {}
    ns.addBuiltinVariant("backquote", (q_func_n_t)f_backquote_VsC15_reference_int_, QCF_NO_FLAGS, QDOM_EXTERNAL_PROCESS, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "cmd", qore_get_complex_reference_or_nothing_type(bigIntTypeInfo), QORE_PARAM_NO_ARG, "rc");
    // nothing backquote() {}
    ns.addBuiltinVariant("backquote", (q_func_n_t)f_backquote, QCF_RUNTIME_NOOP, QDOM_EXTERNAL_PROCESS, nothingTypeInfo);
    // auto call_builtin_function(string name, ...) {}
    ns.addBuiltinVariant("call_builtin_function", (q_func_n_t)f_call_builtin_function_VsVV, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // auto call_builtin_function_args(string name, *softlist<auto> vargs) {}
    ns.addBuiltinVariant("call_builtin_function_args", (q_func_n_t)f_call_builtin_function_args_VsC15_softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");
    // auto call_function(string name, ...) {}
    ns.addBuiltinVariant("call_function", (q_func_n_t)f_call_function_VsVV, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // auto call_function(code f, ...) {}
    ns.addBuiltinVariant("call_function", (q_func_n_t)f_call_function_VcVV, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto call_function_args(string name, *softlist<auto> vargs) {}
    ns.addBuiltinVariant("call_function_args", (q_func_n_t)f_call_function_args_VsC15_softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");
    // auto call_function_args(code f, *softlist<auto> vargs) {}
    ns.addBuiltinVariant("call_function_args", (q_func_n_t)f_call_function_args_VcC15_softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, codeTypeInfo, QORE_PARAM_NO_ARG, "f", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");
    // string decode_uri_request(string uri) {}
    ns.addBuiltinVariant("decode_uri_request", (q_func_n_t)f_decode_uri_request_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "uri");
    // string decode_url(string url) {}
    ns.addBuiltinVariant("decode_url", (q_func_n_t)f_decode_url_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "url");
    // nothing decode_url() {}
    ns.addBuiltinVariant("decode_url", (q_func_n_t)f_decode_url, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string encode_uri_request(string url) {}
    ns.addBuiltinVariant("encode_uri_request", (q_func_n_t)f_encode_uri_request_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "url");
    // string encode_url(string url, softbool encode_all = False) {}
    ns.addBuiltinVariant("encode_url", (q_func_n_t)f_encode_url_Vsvb, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "url", softBoolTypeInfo, QoreSimpleValue().assign(false), "encode_all");
    // bool exists(...) {}
    ns.addBuiltinVariant("exists", (q_func_n_t)f_exists_VV, QCF_CONSTANT|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, boolTypeInfo);
    // bool existsFunction(string name) {}
    ns.addBuiltinVariant("existsFunction", (q_func_n_t)f_existsFunction_Vs, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // bool existsFunction(code c) {}
    ns.addBuiltinVariant("existsFunction", (q_func_n_t)f_existsFunction_Vc, QCF_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, boolTypeInfo, 1, codeTypeInfo, QORE_PARAM_NO_ARG, "c");
    // nothing existsFunction() {}
    ns.addBuiltinVariant("existsFunction", (q_func_n_t)f_existsFunction, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // bool exists_function(string name) {}
    ns.addBuiltinVariant("exists_function", (q_func_n_t)f_exists_function_Vs, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // bool exists_function(code c) {}
    ns.addBuiltinVariant("exists_function", (q_func_n_t)f_exists_function_Vc, QCF_NOOP, QDOM_DEFAULT, boolTypeInfo, 1, codeTypeInfo, QORE_PARAM_NO_ARG, "c");
    // *string functionType(string name) {}
    ns.addBuiltinVariant("functionType", (q_func_n_t)f_functionType_Vs, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // nothing functionType() {}
    ns.addBuiltinVariant("functionType", (q_func_n_t)f_functionType, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *string function_type(string name) {}
    ns.addBuiltinVariant("function_type", (q_func_n_t)f_function_type_Vs, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // *int getByte(string str, softint offset = 0) {}
    ns.addBuiltinVariant("getByte", (q_func_n_t)f_getByte_Vsvi, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // nothing getByte() {}
    ns.addBuiltinVariant("getByte", (q_func_n_t)f_getByte, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *int getByte(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("getByte", (q_func_n_t)f_getByte_Vyvi, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // string getClassName(object obj) {}
    ns.addBuiltinVariant("getClassName", (q_func_n_t)f_getClassName_Vo, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");
    // nothing getClassName() {}
    ns.addBuiltinVariant("getClassName", (q_func_n_t)f_getClassName, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // list<string> getFeatureList() {}
    ns.addBuiltinVariant("getFeatureList", (q_func_n_t)f_getFeatureList, QCF_DEPRECATED, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));
    // hash<string,hash<auto>> getModuleHash() {}
    ns.addBuiltinVariant("getModuleHash", (q_func_n_t)f_getModuleHash, QCF_DEPRECATED, QDOM_DEFAULT, qore_get_complex_hash_type(autoHashTypeInfo));
    // list<hash<auto>> getModuleList() {}
    ns.addBuiltinVariant("getModuleList", (q_func_n_t)f_getModuleList, QCF_DEPRECATED, QDOM_DEFAULT, qore_get_complex_list_type(autoHashTypeInfo));
    // *int getWord32(string str, softint offset = 0) {}
    ns.addBuiltinVariant("getWord32", (q_func_n_t)f_getWord32_Vsvi, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int getWord32(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("getWord32", (q_func_n_t)f_getWord32_Vyvi, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // nothing getWord32() {}
    ns.addBuiltinVariant("getWord32", (q_func_n_t)f_getWord32, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *int get_byte(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_byte", (q_func_n_t)f_get_byte_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_byte(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_byte", (q_func_n_t)f_get_byte_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // code get_call_reference(string identifier) {}
    ns.addBuiltinVariant("get_call_reference", (q_func_n_t)f_get_call_reference_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, codeTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "identifier");
    // string get_class_name(object obj) {}
    ns.addBuiltinVariant("get_class_name", (q_func_n_t)f_get_class_name_Vo, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");
    // string get_default_encoding() {}
    ns.addBuiltinVariant("get_default_encoding", (q_func_n_t)f_get_default_encoding, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);
    // string get_ex_pos(hash<auto> ex) {}
    ns.addBuiltinVariant("get_ex_pos", (q_func_n_t)f_get_ex_pos_C10hash_auto_, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "ex");
    // list<string> get_feature_list() {}
    ns.addBuiltinVariant("get_feature_list", (q_func_n_t)f_get_feature_list, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));
    // hash<auto> get_global_vars() {}
    ns.addBuiltinVariant("get_global_vars", (q_func_n_t)f_get_global_vars, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);
    // hash<auto> get_local_vars(int frame) {}
    ns.addBuiltinVariant("get_local_vars", (q_func_n_t)f_get_local_vars_Vi, QCF_RET_VALUE_ONLY, QDOM_DEBUGGER, autoHashTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "frame");
    // hash<string,hash<auto>> get_module_hash() {}
    ns.addBuiltinVariant("get_module_hash", (q_func_n_t)f_get_module_hash, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_hash_type(autoHashTypeInfo));
    // list<hash<auto>> get_module_list() {}
    ns.addBuiltinVariant("get_module_list", (q_func_n_t)f_get_module_list, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(autoHashTypeInfo));
    // auto get_module_option(string module, string option) {}
    ns.addBuiltinVariant("get_module_option", (q_func_n_t)f_get_module_option_VsVs, QCF_CONSTANT, QDOM_PROCESS, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "module", stringTypeInfo, QORE_PARAM_NO_ARG, "option");
    // int get_parse_options() {}
    ns.addBuiltinVariant("get_parse_options", (q_func_n_t)f_get_parse_options, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // hash<auto> get_qore_library_info() {}
    ns.addBuiltinVariant("get_qore_library_info", (q_func_n_t)f_get_qore_library_info, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);
    // hash<string,hash<auto>> get_qore_option_hash() {}
    ns.addBuiltinVariant("get_qore_option_hash", (q_func_n_t)f_get_qore_option_hash, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_hash_type(autoHashTypeInfo));
    // list<hash<auto>> get_qore_option_list() {}
    ns.addBuiltinVariant("get_qore_option_list", (q_func_n_t)f_get_qore_option_list, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(autoHashTypeInfo));
    // string get_safe_url(string url) {}
    ns.addBuiltinVariant("get_safe_url", (q_func_n_t)f_get_safe_url_Vs, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "url");
    // *string get_script_dir() {}
    ns.addBuiltinVariant("get_script_dir", (q_func_n_t)f_get_script_dir, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);
    // *string get_script_name() {}
    ns.addBuiltinVariant("get_script_name", (q_func_n_t)f_get_script_name, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);
    // *string get_script_path() {}
    ns.addBuiltinVariant("get_script_path", (q_func_n_t)f_get_script_path, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);
    // *int get_word_16(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16", (q_func_n_t)f_get_word_16_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_16(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16", (q_func_n_t)f_get_word_16_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_16_lsb(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16_lsb", (q_func_n_t)f_get_word_16_lsb_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_16_lsb(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16_lsb", (q_func_n_t)f_get_word_16_lsb_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_32(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32", (q_func_n_t)f_get_word_32_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_32(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32", (q_func_n_t)f_get_word_32_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_32_lsb(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32_lsb", (q_func_n_t)f_get_word_32_lsb_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_32_lsb(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32_lsb", (q_func_n_t)f_get_word_32_lsb_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_64(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64", (q_func_n_t)f_get_word_64_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_64(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64", (q_func_n_t)f_get_word_64_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_64_lsb(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64_lsb", (q_func_n_t)f_get_word_64_lsb_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // *int get_word_64_lsb(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64_lsb", (q_func_n_t)f_get_word_64_lsb_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // bool has_key(hash<auto> h, string key) {}
    ns.addBuiltinVariant("has_key", (q_func_n_t)f_has_key_C10hash_auto_Vs, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 2, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h", stringTypeInfo, QORE_PARAM_NO_ARG, "key");
    // bool has_key(object obj, string key) {}
    ns.addBuiltinVariant("has_key", (q_func_n_t)f_has_key_VoVs, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 2, objectTypeInfo, QORE_PARAM_NO_ARG, "obj", stringTypeInfo, QORE_PARAM_NO_ARG, "key");
    // list<auto> hash_values(hash<auto> h) {}
    ns.addBuiltinVariant("hash_values", (q_func_n_t)f_hash_values_C10hash_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");
    // nothing hash_values() {}
    ns.addBuiltinVariant("hash_values", (q_func_n_t)f_hash_values, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int hextoint(string str) {}
    ns.addBuiltinVariant("hextoint", (q_func_n_t)f_hextoint_Vs, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing hextoint() {}
    ns.addBuiltinVariant("hextoint", (q_func_n_t)f_hextoint, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string html_decode(string str) {}
    ns.addBuiltinVariant("html_decode", (q_func_n_t)f_html_decode_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing html_decode() {}
    ns.addBuiltinVariant("html_decode", (q_func_n_t)f_html_decode, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string html_encode(string str) {}
    ns.addBuiltinVariant("html_encode", (q_func_n_t)f_html_encode_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing html_encode() {}
    ns.addBuiltinVariant("html_encode", (q_func_n_t)f_html_encode, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // nothing load_module(string name, int warning_mask = WARN_MODULES) {}
    ns.addBuiltinVariant("load_module", (q_func_n_t)f_load_module_VsVi, QCF_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");
    // nothing load_module() {}
    ns.addBuiltinVariant("load_module", (q_func_n_t)f_load_module, QCF_RUNTIME_NOOP, QDOM_MODULES, nothingTypeInfo);
    // *hash<ExceptionInfo> load_module_warn(string name, int warning_mask = WARN_MODULES) {}
    ns.addBuiltinVariant("load_module_warn", (q_func_n_t)f_load_module_warn_VsVi, QCF_NO_FLAGS, QDOM_MODULES, hashdeclExceptionInfo->getTypeInfo(true), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", bigIntTypeInfo, QoreSimpleValue().assign((int64)WARN_MODULES), "warning_mask");
    // nothing load_user_module_with_program(string name, Qore::Program pgm) {}
    ns.addBuiltinVariant("load_user_module_with_program", (q_func_n_t)f_load_user_module_with_program_VsC7Program, QCF_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm");
    // string makeBase64String(string str, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("makeBase64String", (q_func_n_t)f_makeBase64String_Vsvi, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "maxlinelen");
    // string makeBase64String(binary bin, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("makeBase64String", (q_func_n_t)f_makeBase64String_Vyvi, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "maxlinelen");
    // nothing makeBase64String() {}
    ns.addBuiltinVariant("makeBase64String", (q_func_n_t)f_makeBase64String, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string makeHexString(string str) {}
    ns.addBuiltinVariant("makeHexString", (q_func_n_t)f_makeHexString_Vs, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string makeHexString(binary bin) {}
    ns.addBuiltinVariant("makeHexString", (q_func_n_t)f_makeHexString_Vy, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // nothing makeHexString() {}
    ns.addBuiltinVariant("makeHexString", (q_func_n_t)f_makeHexString, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string make_base64_string(string str, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("make_base64_string", (q_func_n_t)f_make_base64_string_Vsvi, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "maxlinelen");
    // string make_base64_string(binary bin, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("make_base64_string", (q_func_n_t)f_make_base64_string_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "maxlinelen");
    // string make_base64_url_string(string str) {}
    ns.addBuiltinVariant("make_base64_url_string", (q_func_n_t)f_make_base64_url_string_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string make_base64_url_string(binary bin) {}
    ns.addBuiltinVariant("make_base64_url_string", (q_func_n_t)f_make_base64_url_string_Vy, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // string make_hex_string(string str) {}
    ns.addBuiltinVariant("make_hex_string", (q_func_n_t)f_make_hex_string_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string make_hex_string(binary bin) {}
    ns.addBuiltinVariant("make_hex_string", (q_func_n_t)f_make_hex_string_Vy, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // *hash<auto> parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True) {}
    ns.addBuiltinVariant("parse", (q_func_n_t)f_parse_VsVsniNsnivb, QCF_NO_FLAGS, QDOM_EMBEDDED_LOGIC|QDOM_IN_MODULE, autoHashOrNothingTypeInfo, 6, stringTypeInfo, QORE_PARAM_NO_ARG, "code", stringTypeInfo, QORE_PARAM_NO_ARG, "label", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "warning_mask", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "source", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "offset", softBoolTypeInfo, QoreSimpleValue().assign(true), "format_label");
    // nothing parse() {}
    ns.addBuiltinVariant("parse", (q_func_n_t)f_parse, QCF_RUNTIME_NOOP, QDOM_EMBEDDED_LOGIC|QDOM_IN_MODULE, nothingTypeInfo);
    // binary parseBase64String(string str) {}
    ns.addBuiltinVariant("parseBase64String", (q_func_n_t)f_parseBase64String_Vs, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing parseBase64String() {}
    ns.addBuiltinVariant("parseBase64String", (q_func_n_t)f_parseBase64String, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string parseBase64StringToString(string str, *string encoding) {}
    ns.addBuiltinVariant("parseBase64StringToString", (q_func_n_t)f_parseBase64StringToString_VsNs, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // nothing parseBase64StringToString() {}
    ns.addBuiltinVariant("parseBase64StringToString", (q_func_n_t)f_parseBase64StringToString, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // binary parseHexString(string hexstr) {}
    ns.addBuiltinVariant("parseHexString", (q_func_n_t)f_parseHexString_Vs, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "hexstr");
    // nothing parseHexString() {}
    ns.addBuiltinVariant("parseHexString", (q_func_n_t)f_parseHexString, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *hash<UrlInfo> parseURL(string url, bool keep_brackets = False) {}
    ns.addBuiltinVariant("parseURL", (q_func_n_t)f_parseURL_VsVb, QCF_CONSTANT, QDOM_DEFAULT, hashdeclUrlInfo->getTypeInfo(true), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "url", boolTypeInfo, QoreSimpleValue().assign(false), "keep_brackets");
    // nothing parseURL() {}
    ns.addBuiltinVariant("parseURL", (q_func_n_t)f_parseURL, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // binary parse_base64_string(string str) {}
    ns.addBuiltinVariant("parse_base64_string", (q_func_n_t)f_parse_base64_string_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string parse_base64_string_to_string(string str, *string encoding) {}
    ns.addBuiltinVariant("parse_base64_string_to_string", (q_func_n_t)f_parse_base64_string_to_string_VsNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary parse_base64_url_string(string str) {}
    ns.addBuiltinVariant("parse_base64_url_string", (q_func_n_t)f_parse_base64_url_string_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string parse_base64_url_string_to_string(string str, *string encoding) {}
    ns.addBuiltinVariant("parse_base64_url_string_to_string", (q_func_n_t)f_parse_base64_url_string_to_string_VsNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary parse_hex_string(string hexstr) {}
    ns.addBuiltinVariant("parse_hex_string", (q_func_n_t)f_parse_hex_string_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "hexstr");
    // hash<UrlInfo> parse_url(string url, bool keep_brackets) {}
    ns.addBuiltinVariant("parse_url", (q_func_n_t)f_parse_url_VsVb, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclUrlInfo->getTypeInfo(false), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "url", boolTypeInfo, QORE_PARAM_NO_ARG, "keep_brackets");
    // hash<UrlInfo> parse_url(string url, *int options) {}
    ns.addBuiltinVariant("parse_url", (q_func_n_t)f_parse_url_VsNi, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclUrlInfo->getTypeInfo(false), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "url", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "options");
    // nothing reload_module(string name) {}
    ns.addBuiltinVariant("reload_module", (q_func_n_t)f_reload_module_Vs, QCF_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // nothing set_global_var_value(string name, auto value) {}
    ns.addBuiltinVariant("set_global_var_value", (q_func_n_t)f_set_global_var_value_VsVA, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", autoTypeInfo, QORE_PARAM_NO_ARG, "value");
    // nothing set_local_var_value(int frame, string var, auto value) {}
    ns.addBuiltinVariant("set_local_var_value", (q_func_n_t)f_set_local_var_value_ViVsVA, QCF_NO_FLAGS, QDOM_DEBUGGER, nothingTypeInfo, 3, bigIntTypeInfo, QORE_PARAM_NO_ARG, "frame", stringTypeInfo, QORE_PARAM_NO_ARG, "var", autoTypeInfo, QORE_PARAM_NO_ARG, "value");
    // nothing set_module_option(string module, string option, auto value) {}
    ns.addBuiltinVariant("set_module_option", (q_func_n_t)f_set_module_option_VsVsVA, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "module", stringTypeInfo, QORE_PARAM_NO_ARG, "option", autoTypeInfo, QORE_PARAM_NO_ARG, "value");
    // nothing set_return_value(auto val) {}
    ns.addBuiltinVariant("set_return_value", (q_func_n_t)f_set_return_value_VA, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");
    // string splice(string str) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_Vs, QCF_NOOP, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string splice(string str, softint start) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_Vsvi, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start");
    // string splice(string str, softint start, softint len, *string nstr) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_VsviviNs, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "nstr");
    // list<auto> splice(list<auto> l, softint start) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_C10list_auto_vi, QCF_NO_FLAGS, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start");
    // list<auto> splice(list<auto> l, softint start, softint len) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_C10list_auto_vivi, QCF_NO_FLAGS, QDOM_DEFAULT, autoListTypeInfo, 3, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len");
    // list<auto> splice(list<auto> l, softint start, softint len, softlist<auto> nlist) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_C10list_auto_viviC14softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, autoListTypeInfo, 4, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len", softAutoListTypeInfo, QORE_PARAM_NO_ARG, "nlist");
    // nothing splice() {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int strtoint(string num, softint base = 10) {}
    ns.addBuiltinVariant("strtoint", (q_func_n_t)f_strtoint_Vsvi, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "num", softBigIntTypeInfo, QoreSimpleValue().assign((int64)10), "base");
    // nothing strtoint() {}
    ns.addBuiltinVariant("strtoint", (q_func_n_t)f_strtoint, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);

}

DLLLOCAL void init_misc_constants(QoreNamespace& ns) {
# 200 "ql_misc.qpp"
# 211 "ql_misc.qpp"
# 214 "ql_misc.qpp"
    ns.addConstant("QURL_DECODE", ((int64)QURL_DECODE));
    ns.addConstant("QURL_DECODE_PATH", ((int64)QURL_DECODE_PATH));
    ns.addConstant("QURL_KEEP_BRACKETS", ((int64)QURL_KEEP_BRACKETS));
    ns.addConstant("QURL_MAINTAIN_CASE", ((int64)QURL_MAINTAIN_CASE));
    ns.addConstant("CE_ALL", ((int64)CE_ALL));
    ns.addConstant("CE_HTML", ((int64)CE_HTML));
    ns.addConstant("CE_NONASCII", ((int64)CE_NONASCII));
    ns.addConstant("CE_XHTML", ((int64)CE_XHTML));
    ns.addConstant("CE_XML", ((int64)CE_XML));
    ns.addConstant("CD_ALL", ((int64)CD_ALL));
    ns.addConstant("CD_HTML", ((int64)CD_HTML));
    ns.addConstant("CD_NUM_REF", ((int64)CD_NUM_REF));
    ns.addConstant("CD_XHTML", ((int64)CD_XHTML));
    ns.addConstant("CD_XML", ((int64)CD_XML));
# 339 "ql_misc.qpp"
# 434 "ql_misc.qpp"
# 483 "ql_misc.qpp"
    QoreHashNode* hash_NameToSignal = new QoreHashNode(autoTypeInfo);
    hash_NameToSignal->setKeyValue("SIGABRT", ((int64)SIGABRT), 0);
    hash_NameToSignal->setKeyValue("SIGALRM", ((int64)SIGALRM), 0);
    hash_NameToSignal->setKeyValue("SIGBUS", ((int64)SIGBUS), 0);
    hash_NameToSignal->setKeyValue("SIGCANCEL", ((int64)SIGCANCEL), 0);
    hash_NameToSignal->setKeyValue("SIGCHLD", ((int64)SIGCHLD), 0);
    hash_NameToSignal->setKeyValue("SIGCLD", ((int64)SIGCLD), 0);
    hash_NameToSignal->setKeyValue("SIGCONT", ((int64)SIGCONT), 0);
    hash_NameToSignal->setKeyValue("SIGEMT", ((int64)SIGEMT), 0);
    hash_NameToSignal->setKeyValue("SIGFPE", ((int64)SIGFPE), 0);
    hash_NameToSignal->setKeyValue("SIGFREEZE", ((int64)SIGFREEZE), 0);
    hash_NameToSignal->setKeyValue("SIGHUP", ((int64)SIGHUP), 0);
    hash_NameToSignal->setKeyValue("SIGILL", ((int64)SIGILL), 0);
    hash_NameToSignal->setKeyValue("SIGINFO", ((int64)SIGINFO), 0);
    hash_NameToSignal->setKeyValue("SIGINT", ((int64)SIGINT), 0);
    hash_NameToSignal->setKeyValue("SIGIO", ((int64)SIGIO), 0);
    hash_NameToSignal->setKeyValue("SIGIOT", ((int64)SIGIOT), 0);
    hash_NameToSignal->setKeyValue("SIGJVM1", ((int64)SIGJVM1), 0);
    hash_NameToSignal->setKeyValue("SIGJVM2", ((int64)SIGJVM2), 0);
    hash_NameToSignal->setKeyValue("SIGKILL", ((int64)SIGKILL), 0);
    hash_NameToSignal->setKeyValue("SIGLOST", ((int64)SIGLOST), 0);
    hash_NameToSignal->setKeyValue("SIGLWP", ((int64)SIGLWP), 0);
    hash_NameToSignal->setKeyValue("SIGPIPE", ((int64)SIGPIPE), 0);
    hash_NameToSignal->setKeyValue("SIGPOLL", ((int64)SIGPOLL), 0);
    hash_NameToSignal->setKeyValue("SIGPROF", ((int64)SIGPROF), 0);
    hash_NameToSignal->setKeyValue("SIGPWR", ((int64)SIGPWR), 0);
    hash_NameToSignal->setKeyValue("SIGQUIT", ((int64)SIGQUIT), 0);
    hash_NameToSignal->setKeyValue("SIGSEGV", ((int64)SIGSEGV), 0);
    hash_NameToSignal->setKeyValue("SIGSTKFLT", ((int64)SIGSTKFLT), 0);
    hash_NameToSignal->setKeyValue("SIGSTOP", ((int64)SIGSTOP), 0);
    hash_NameToSignal->setKeyValue("SIGSYS", ((int64)SIGSYS), 0);
    hash_NameToSignal->setKeyValue("SIGTERM", ((int64)SIGTERM), 0);
    hash_NameToSignal->setKeyValue("SIGTHAW", ((int64)SIGTHAW), 0);
    hash_NameToSignal->setKeyValue("SIGTRAP", ((int64)SIGTRAP), 0);
    hash_NameToSignal->setKeyValue("SIGTSTP", ((int64)SIGTSTP), 0);
    hash_NameToSignal->setKeyValue("SIGTTIN", ((int64)SIGTTIN), 0);
    hash_NameToSignal->setKeyValue("SIGTTOU", ((int64)SIGTTOU), 0);
    hash_NameToSignal->setKeyValue("SIGURG", ((int64)SIGURG), 0);
    hash_NameToSignal->setKeyValue("SIGUSR1", ((int64)SIGUSR1), 0);
    hash_NameToSignal->setKeyValue("SIGUSR2", ((int64)SIGUSR2), 0);
    hash_NameToSignal->setKeyValue("SIGVTALRM", ((int64)SIGVTALRM), 0);
    hash_NameToSignal->setKeyValue("SIGWAITING", ((int64)SIGWAITING), 0);
    hash_NameToSignal->setKeyValue("SIGWINCH", ((int64)SIGWINCH), 0);
    hash_NameToSignal->setKeyValue("SIGXCPU", ((int64)SIGXCPU), 0);
    hash_NameToSignal->setKeyValue("SIGXFSZ", ((int64)SIGXFSZ), 0);
    hash_NameToSignal->setKeyValue("SIGXRES", ((int64)SIGXRES), 0);

    ns.addConstant("NameToSignal", hash_NameToSignal);
    ns.addConstant("SIGABRT", ((int64)SIGABRT));
    ns.addConstant("SIGALRM", ((int64)SIGALRM));
    ns.addConstant("SIGBUS", ((int64)SIGBUS));
    ns.addConstant("SIGCANCEL", ((int64)SIGCANCEL));
    ns.addConstant("SIGCHLD", ((int64)SIGCHLD));
    ns.addConstant("SIGCLD", ((int64)SIGCLD));
    ns.addConstant("SIGCONT", ((int64)SIGCONT));
    ns.addConstant("SIGEMT", ((int64)SIGEMT));
    ns.addConstant("SIGFPE", ((int64)SIGFPE));
    ns.addConstant("SIGFREEZE", ((int64)SIGFREEZE));
    ns.addConstant("SIGHUP", ((int64)SIGHUP));
    ns.addConstant("SIGILL", ((int64)SIGILL));
    ns.addConstant("SIGINFO", ((int64)SIGINFO));
    ns.addConstant("SIGINT", ((int64)SIGINT));
    ns.addConstant("SIGIO", ((int64)SIGIO));
    ns.addConstant("SIGIOT", ((int64)SIGIOT));
    ns.addConstant("SIGJVM1", ((int64)SIGJVM1));
    ns.addConstant("SIGJVM2", ((int64)SIGJVM2));
    ns.addConstant("SIGKILL", ((int64)SIGKILL));
    ns.addConstant("SIGLOST", ((int64)SIGLOST));
    ns.addConstant("SIGLWP", ((int64)SIGLWP));
    ns.addConstant("SIGPIPE", ((int64)SIGPIPE));
    ns.addConstant("SIGPOLL", ((int64)SIGPOLL));
    ns.addConstant("SIGPROF", ((int64)SIGPROF));
    ns.addConstant("SIGPWR", ((int64)SIGPWR));
    ns.addConstant("SIGQUIT", ((int64)SIGQUIT));
    ns.addConstant("SIGSEGV", ((int64)SIGSEGV));
    ns.addConstant("SIGSTKFLT", ((int64)SIGSTKFLT));
    ns.addConstant("SIGSTOP", ((int64)SIGSTOP));
    ns.addConstant("SIGSYS", ((int64)SIGSYS));
    ns.addConstant("SIGTERM", ((int64)SIGTERM));
    ns.addConstant("SIGTHAW", ((int64)SIGTHAW));
    ns.addConstant("SIGTRAP", ((int64)SIGTRAP));
    ns.addConstant("SIGTSTP", ((int64)SIGTSTP));
    ns.addConstant("SIGTTIN", ((int64)SIGTTIN));
    ns.addConstant("SIGTTOU", ((int64)SIGTTOU));
    ns.addConstant("SIGURG", ((int64)SIGURG));
    ns.addConstant("SIGUSR1", ((int64)SIGUSR1));
    ns.addConstant("SIGUSR2", ((int64)SIGUSR2));
    ns.addConstant("SIGVTALRM", ((int64)SIGVTALRM));
    ns.addConstant("SIGWAITING", ((int64)SIGWAITING));
    ns.addConstant("SIGWINCH", ((int64)SIGWINCH));
    ns.addConstant("SIGXCPU", ((int64)SIGXCPU));
    ns.addConstant("SIGXFSZ", ((int64)SIGXFSZ));
    ns.addConstant("SIGXRES", ((int64)SIGXRES));
    QoreHashNode* hash_SignalToName = new QoreHashNode(autoTypeInfo);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGABRT), new QoreStringNode("SIGABRT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGALRM), new QoreStringNode("SIGALRM"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGBUS), new QoreStringNode("SIGBUS"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGCANCEL), new QoreStringNode("SIGCANCEL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGCHLD), new QoreStringNode("SIGCHLD"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGCONT), new QoreStringNode("SIGCONT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGEMT), new QoreStringNode("SIGEMT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGFPE), new QoreStringNode("SIGFPE"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGFREEZE), new QoreStringNode("SIGFREEZE"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGHUP), new QoreStringNode("SIGHUP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGILL), new QoreStringNode("SIGILL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGINFO), new QoreStringNode("SIGINFO"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGINT), new QoreStringNode("SIGINT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGIO), new QoreStringNode("SIGIO"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGIOT), new QoreStringNode("SIGIOT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGJVM1), new QoreStringNode("SIGJVM1"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGJVM2), new QoreStringNode("SIGJVM2"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGKILL), new QoreStringNode("SIGKILL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGLOST), new QoreStringNode("SIGLOST"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGLWP), new QoreStringNode("SIGLWP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPIPE), new QoreStringNode("SIGPIPE"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPOLL), new QoreStringNode("SIGPOLL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPROF), new QoreStringNode("SIGPROF"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPWR), new QoreStringNode("SIGPWR"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGQUIT), new QoreStringNode("SIGQUIT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSEGV), new QoreStringNode("SIGSEGV"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSTKSZ), new QoreStringNode("SIGSTKSZ"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSTOP), new QoreStringNode("SIGSTOP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSYS), new QoreStringNode("SIGSYS"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTERM), new QoreStringNode("SIGTERM"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTHAW), new QoreStringNode("SIGTHAW"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTRAP), new QoreStringNode("SIGTRAP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTSTP), new QoreStringNode("SIGTSTP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTTIN), new QoreStringNode("SIGTTIN"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTTOU), new QoreStringNode("SIGTTOU"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGURG), new QoreStringNode("SIGURG"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGUSR1), new QoreStringNode("SIGUSR1"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGUSR2), new QoreStringNode("SIGUSR2"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGVTALRM), new QoreStringNode("SIGVTALRM"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGWAITING), new QoreStringNode("SIGWAITING"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGWINCH), new QoreStringNode("SIGWINCH"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGXCPU), new QoreStringNode("SIGXCPU"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGXFSZ), new QoreStringNode("SIGXFSZ"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGXRES), new QoreStringNode("SIGXRES"), 0);

    ns.addConstant("SignalToName", hash_SignalToName);
# 532 "ql_misc.qpp"
# 568 "ql_misc.qpp"
# 599 "ql_misc.qpp"
# 637 "ql_misc.qpp"
# 666 "ql_misc.qpp"
# 694 "ql_misc.qpp"
# 717 "ql_misc.qpp"
# 750 "ql_misc.qpp"
# 777 "ql_misc.qpp"
# 793 "ql_misc.qpp"
# 804 "ql_misc.qpp"
# 817 "ql_misc.qpp"
# 823 "ql_misc.qpp"
# 839 "ql_misc.qpp"
# 852 "ql_misc.qpp"
# 868 "ql_misc.qpp"
# 874 "ql_misc.qpp"
# 895 "ql_misc.qpp"
# 914 "ql_misc.qpp"
# 920 "ql_misc.qpp"
# 939 "ql_misc.qpp"
# 945 "ql_misc.qpp"
# 957 "ql_misc.qpp"
# 995 "ql_misc.qpp"
# 1001 "ql_misc.qpp"
# 1012 "ql_misc.qpp"
# 1018 "ql_misc.qpp"
# 1036 "ql_misc.qpp"
# 1085 "ql_misc.qpp"
# 1130 "ql_misc.qpp"
# 1173 "ql_misc.qpp"
# 1234 "ql_misc.qpp"
# 1240 "ql_misc.qpp"
# 1274 "ql_misc.qpp"
# 1280 "ql_misc.qpp"
# 1302 "ql_misc.qpp"
# 1322 "ql_misc.qpp"
# 1328 "ql_misc.qpp"
# 1353 "ql_misc.qpp"
# 1376 "ql_misc.qpp"
# 1399 "ql_misc.qpp"
# 1422 "ql_misc.qpp"
# 1441 "ql_misc.qpp"
# 1447 "ql_misc.qpp"
# 1471 "ql_misc.qpp"
# 1494 "ql_misc.qpp"
# 1500 "ql_misc.qpp"
# 1528 "ql_misc.qpp"
# 1551 "ql_misc.qpp"
# 1576 "ql_misc.qpp"
# 1598 "ql_misc.qpp"
# 1625 "ql_misc.qpp"
# 1647 "ql_misc.qpp"
# 1672 "ql_misc.qpp"
# 1683 "ql_misc.qpp"
# 1699 "ql_misc.qpp"
# 1717 "ql_misc.qpp"
# 1723 "ql_misc.qpp"
# 1748 "ql_misc.qpp"
# 1766 "ql_misc.qpp"
# 1791 "ql_misc.qpp"
# 1816 "ql_misc.qpp"
# 1838 "ql_misc.qpp"
# 1844 "ql_misc.qpp"
# 1866 "ql_misc.qpp"
# 1891 "ql_misc.qpp"
# 1916 "ql_misc.qpp"
# 1943 "ql_misc.qpp"
# 1970 "ql_misc.qpp"
# 1992 "ql_misc.qpp"
# 2014 "ql_misc.qpp"
# 2020 "ql_misc.qpp"
# 2045 "ql_misc.qpp"
# 2070 "ql_misc.qpp"
# 2095 "ql_misc.qpp"
# 2120 "ql_misc.qpp"
# 2147 "ql_misc.qpp"
# 2172 "ql_misc.qpp"
# 2197 "ql_misc.qpp"
# 2222 "ql_misc.qpp"
# 2233 "ql_misc.qpp"
# 2254 "ql_misc.qpp"
# 2277 "ql_misc.qpp"
# 2296 "ql_misc.qpp"
# 2318 "ql_misc.qpp"
# 2341 "ql_misc.qpp"
# 2347 "ql_misc.qpp"
# 2365 "ql_misc.qpp"
# 2383 "ql_misc.qpp"
# 2389 "ql_misc.qpp"
# 2412 "ql_misc.qpp"
# 2435 "ql_misc.qpp"
# 2452 "ql_misc.qpp"
# 2458 "ql_misc.qpp"
# 2480 "ql_misc.qpp"
# 2524 "ql_misc.qpp"
# 2530 "ql_misc.qpp"
# 2551 "ql_misc.qpp"
# 2557 "ql_misc.qpp"
# 2584 "ql_misc.qpp"
# 2616 "ql_misc.qpp"
# 2622 "ql_misc.qpp"
# 2641 "ql_misc.qpp"
# 2665 "ql_misc.qpp"
# 2688 "ql_misc.qpp"
# 2694 "ql_misc.qpp"
# 2718 "ql_misc.qpp"
# 2761 "ql_misc.qpp"
# 2796 "ql_misc.qpp"
# 2808 "ql_misc.qpp"
# 2820 "ql_misc.qpp"
# 2832 "ql_misc.qpp"
# 2861 "ql_misc.qpp"
# 2892 "ql_misc.qpp"
# 2937 "ql_misc.qpp"
# 2951 "ql_misc.qpp"
# 2967 "ql_misc.qpp"
# 3015 "ql_misc.qpp"
# 3035 "ql_misc.qpp"
# 3058 "ql_misc.qpp"
# 3074 "ql_misc.qpp"
# 3095 "ql_misc.qpp"
# 3123 "ql_misc.qpp"
# 3150 "ql_misc.qpp"

}
