/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_object.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_object.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreClassIntern.h"

QoreValue q_call_static_method_args(QoreProgram* pgm, const QoreStringNode* class_name, const QoreStringNode* method, const QoreListNode* args, ExceptionSink* xsink) {
    TempEncodingHelper cls_path(class_name, QCS_DEFAULT, xsink);
    if (!cls_path) {
        assert(*xsink);
        return QoreValue();
    }
    TempEncodingHelper method_name(method, QCS_DEFAULT, xsink);
    if (!method_name) {
        assert(*xsink);
        return QoreValue();
    }
    // we do not need to grab the thread lock, as we are already executing in the current Program object
    const QoreClass* cls = pgm->findClass(cls_path->c_str(), xsink);
    if (!cls) {
        if (!*xsink) {
            xsink->raiseException("UNKNOWN-CLASS", "cannot resolve class '%s'", cls_path->c_str());
        }
        return QoreValue();
    }

    const QoreMethod* m = cls->findLocalStaticMethod(method_name->c_str());
    if (!m) {
        xsink->raiseException("UNKNOWN-METHOD", "cannot resolve static method '%s::%s()'",
            cls_path->c_str(), method_name->c_str());
        return QoreValue();
    }

    return QoreObject::evalStaticMethod(*m, m->getClass(), args, xsink);
}

static QoreValue call_static_method_args_intern(const QoreStringNode* class_name, const QoreStringNode* method, const QoreListNode* args, ExceptionSink* xsink) {
    return q_call_static_method_args(getProgram(), class_name, method, args, xsink);
}

static QoreObject* create_object_intern(const QoreStringNode* class_name, unsigned arg_offset, const QoreListNode* args, ExceptionSink* xsink) {
    TempEncodingHelper tmp(class_name, QCS_DEFAULT, xsink);
    if (!tmp) {
        assert(*xsink);
        return nullptr;
    }

    const QoreClass* qc = qore_program_private::runtimeFindClass(*getProgram(), tmp->c_str(), xsink);
    if (!qc) {
        if (!*xsink)
            xsink->raiseException("CREATE-OBJECT-ERROR", "cannot resolve class '%s' for instantiation", tmp->c_str());
        return nullptr;
    }

    if (getProgram()->getParseOptions64() & qc->getDomain())
        xsink->raiseException("CREATE-OBJECT-ERROR", "current Program sandboxing restrictions do not allow access to the '%s' class", qc->getName());

    if (qore_class_private::runtimeCheckInstantiateClass(*qc, xsink)) {
        return nullptr;
    }

    const QoreListNode* real_args;
    ReferenceHolder<QoreListNode> tmp_args(xsink);
    if (arg_offset) {
        tmp_args = args->size() > arg_offset ? args->copyListFrom(arg_offset) : nullptr;
        real_args = *tmp_args;
    }
    else {
        real_args = args;
    }

    //printd(5, "create_object_intern() '%s' offset: %d size: %d (a: %d)\n", tmp->c_str(), arg_offset, real_args ? real_args->size() : 0, args ? args->size() : 0);
    return qc->execConstructor(real_args, xsink);
}
# 107 "ql_object.qpp"
# 112 "ql_object.qpp"
# 124 "ql_object.qpp"
# 130 "ql_object.qpp"
# 147 "ql_object.qpp"
# 181 "ql_object.qpp"
# 187 "ql_object.qpp"
# 226 "ql_object.qpp"
# 247 "ql_object.qpp"
# 253 "ql_object.qpp"
# 284 "ql_object.qpp"
# 320 "ql_object.qpp"
# 350 "ql_object.qpp"
# 371 "ql_object.qpp"
# 389 "ql_object.qpp"
# 413 "ql_object.qpp"
// auto callObjectMethod(object obj, string method, ...) {}
static QoreValue f_callObjectMethod_VoVsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 169 "ql_object.qpp"
    ReferenceHolder<QoreListNode> call_args(xsink);

    // if there are arguments to pass
    if (num_args(args) > 2) {
        // create argument list by copying current list
        call_args = args->copyListFrom(2);
    }

    return obj->evalMethod(method, *call_args, xsink);
}

// nothing callObjectMethod() {}
static QoreValue f_callObjectMethod(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 184 "ql_object.qpp"

    return QoreValue();
}

// auto callObjectMethodArgs(object obj, string method, *softlist<auto> varg) {}
static QoreValue f_callObjectMethodArgs_VoVsC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* varg = get_param_value(args, 2).get<const QoreListNode>();
# 243 "ql_object.qpp"
    return obj->evalMethod(method, varg, xsink);
}

// nothing callObjectMethodArgs() {}
static QoreValue f_callObjectMethodArgs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 250 "ql_object.qpp"

    return QoreValue();
}

// auto call_object_method(object obj, string method, ...) {}
static QoreValue f_call_object_method_VoVsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 214 "ql_object.qpp"
    ReferenceHolder<QoreListNode> call_args(xsink);

    // if there are arguments to pass
    if (num_args(args) > 2) {
        // create argument list by copying current list
        call_args = args->copyListFrom(2);
    }

    return obj->evalMethod(method, *call_args, xsink);
}

// auto call_object_method_args(object obj, string method, *softlist<auto> varg) {}
static QoreValue f_call_object_method_args_VoVsC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* varg = get_param_value(args, 2).get<const QoreListNode>();
# 280 "ql_object.qpp"
    return obj->evalMethod(method, varg, xsink);
}

// auto call_pseudo(any val, string meth, ...) {}
static QoreValue f_call_pseudo_VaVsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    const QoreStringNode* meth = HARD_QORE_VALUE_STRING(args, 1);
# 364 "ql_object.qpp"
    ReferenceHolder<QoreListNode> nargs(xsink);
    if (args->size() > 2)
        nargs = args->copyListFrom(2);
    return pseudo_classes_eval(val, meth->getBuffer(), *nargs, xsink);
}

// auto call_pseudo_args(any val, string meth, *softlist<auto> argv) {}
static QoreValue f_call_pseudo_args_VaVsC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    const QoreStringNode* meth = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* argv = get_param_value(args, 2).get<const QoreListNode>();
# 385 "ql_object.qpp"
    return pseudo_classes_eval(val, meth->getBuffer(), argv, xsink);
}

// auto call_static_method(string class_name, string method, ...) {}
static QoreValue f_call_static_method_VsVsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 310 "ql_object.qpp"
    ReferenceHolder<QoreListNode> call_args(xsink);
    // if there are arguments to pass
    if (num_args(args) > 2) {
        // create argument list by copying current list
        call_args = args->copyListFrom(2);
    }
    return call_static_method_args_intern(class_name, method, *call_args, xsink);
}

// auto call_static_method_args(string class_name, string method, *softlist<auto> call_args) {}
static QoreValue f_call_static_method_args_VsVsC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* call_args = get_param_value(args, 2).get<const QoreListNode>();
# 346 "ql_object.qpp"
    return call_static_method_args_intern(class_name, method, call_args, xsink);
}

// object create_object(string class_name, ...) {}
static QoreValue f_create_object_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
# 409 "ql_object.qpp"
   return create_object_intern(class_name, 1, args, xsink);
}

// object create_object_args(string class_name, *softlist<auto> argv) {}
static QoreValue f_create_object_args_VsC15_softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 433 "ql_object.qpp"
    return create_object_intern(class_name, 0, argv, xsink);
}

// list getMethodList(object obj) {}
static QoreValue f_getMethodList_Vo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 120 "ql_object.qpp"
    return obj->getClass()->getMethodList();
}

// nothing getMethodList() {}
static QoreValue f_getMethodList(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 127 "ql_object.qpp"

    return QoreValue();
}

// list<string> get_method_list(object obj) {}
static QoreValue f_get_method_list_Vo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 143 "ql_object.qpp"
    return obj->getClass()->getMethodList();
}


DLLLOCAL void init_object_functions(QoreNamespace& ns) {
    // auto callObjectMethod(object obj, string method, ...) {}
    ns.addBuiltinVariant("callObjectMethod", (q_func_n_t)f_callObjectMethod_VoVsVV, QCF_DEPRECATED|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, objectTypeInfo, QORE_PARAM_NO_ARG, "obj", stringTypeInfo, QORE_PARAM_NO_ARG, "method");
    // nothing callObjectMethod() {}
    ns.addBuiltinVariant("callObjectMethod", (q_func_n_t)f_callObjectMethod, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // auto callObjectMethodArgs(object obj, string method, *softlist<auto> varg) {}
    ns.addBuiltinVariant("callObjectMethodArgs", (q_func_n_t)f_callObjectMethodArgs_VoVsC15_softlist_auto_, QCF_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 3, objectTypeInfo, QORE_PARAM_NO_ARG, "obj", stringTypeInfo, QORE_PARAM_NO_ARG, "method", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "varg");
    // nothing callObjectMethodArgs() {}
    ns.addBuiltinVariant("callObjectMethodArgs", (q_func_n_t)f_callObjectMethodArgs, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // auto call_object_method(object obj, string method, ...) {}
    ns.addBuiltinVariant("call_object_method", (q_func_n_t)f_call_object_method_VoVsVV, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, objectTypeInfo, QORE_PARAM_NO_ARG, "obj", stringTypeInfo, QORE_PARAM_NO_ARG, "method");
    // auto call_object_method_args(object obj, string method, *softlist<auto> varg) {}
    ns.addBuiltinVariant("call_object_method_args", (q_func_n_t)f_call_object_method_args_VoVsC15_softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 3, objectTypeInfo, QORE_PARAM_NO_ARG, "obj", stringTypeInfo, QORE_PARAM_NO_ARG, "method", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "varg");
    // auto call_pseudo(any val, string meth, ...) {}
    ns.addBuiltinVariant("call_pseudo", (q_func_n_t)f_call_pseudo_VaVsVV, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, anyTypeInfo, QORE_PARAM_NO_ARG, "val", stringTypeInfo, QORE_PARAM_NO_ARG, "meth");
    // auto call_pseudo_args(any val, string meth, *softlist<auto> argv) {}
    ns.addBuiltinVariant("call_pseudo_args", (q_func_n_t)f_call_pseudo_args_VaVsC15_softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 3, anyTypeInfo, QORE_PARAM_NO_ARG, "val", stringTypeInfo, QORE_PARAM_NO_ARG, "meth", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");
    // auto call_static_method(string class_name, string method, ...) {}
    ns.addBuiltinVariant("call_static_method", (q_func_n_t)f_call_static_method_VsVsVV, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name", stringTypeInfo, QORE_PARAM_NO_ARG, "method");
    // auto call_static_method_args(string class_name, string method, *softlist<auto> call_args) {}
    ns.addBuiltinVariant("call_static_method_args", (q_func_n_t)f_call_static_method_args_VsVsC15_softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name", stringTypeInfo, QORE_PARAM_NO_ARG, "method", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "call_args");
    // object create_object(string class_name, ...) {}
    ns.addBuiltinVariant("create_object", (q_func_n_t)f_create_object_VsVV, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, objectTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name");
    // object create_object_args(string class_name, *softlist<auto> argv) {}
    ns.addBuiltinVariant("create_object_args", (q_func_n_t)f_create_object_args_VsC15_softlist_auto_, QCF_NO_FLAGS, QDOM_DEFAULT, objectTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");
    // list getMethodList(object obj) {}
    ns.addBuiltinVariant("getMethodList", (q_func_n_t)f_getMethodList_Vo, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");
    // nothing getMethodList() {}
    ns.addBuiltinVariant("getMethodList", (q_func_n_t)f_getMethodList, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // list<string> get_method_list(object obj) {}
    ns.addBuiltinVariant("get_method_list", (q_func_n_t)f_get_method_list_Vo, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");

}
