/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_string.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_string.h"
#include "qore/intern/qore_number_private.h"

#include <cctype>
#include <cfloat>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <sys/stat.h>
#include <sys/types.h>

// for use in floating-point number formatting
// from: http://stackoverflow.com/questions/16839658/printf-width-specifier-to-maintain-precision-of-floating-point-value
#ifdef DBL_DECIMAL_DIG
  #define OP_DBL_Digs (DBL_DECIMAL_DIG)
#else
  #ifdef DECIMAL_DIG
    #define OP_DBL_Digs (DECIMAL_DIG)
  #else
    #define OP_DBL_Digs (DBL_DIG + 3)
  #endif
#endif

static int fix_num(QoreString& tmp, const QoreString& fmt, ExceptionSink* xsink) {
    assert(xsink);
    qore_offset_t off = fmt.getByteOffset(1, xsink);
    //printd(5, "fix_num tmp: '%s' fmt: '%s' off: %lld\n", tmp.getBuffer(), fmt.getBuffer(), off);
    if (*xsink)
        return -1;
    if (off < 0)
        return 0;

    // replace all thousands separators with nothing
    QoreString t(fmt.getBuffer(), off, fmt.getEncoding());
    tmp.replaceAll(t.getBuffer(), "");

    if (fmt.size() <= (size_t)off)
        return 0;

    t.clear();
    if (t.concat(fmt, 1, 1, xsink))
        return -1;

    if (t.size() == 1 && t[0] == '.')
        return 0;

    off = tmp.bindex(t.getBuffer(), 0);
    if (off < 0)
        return 0;

    tmp.replace(off, t.size(), ".");
    return 0;
}

static const char* memstr(const char* str, const char* pattern, size_t pl, size_t len) {
    while (true) {
        const char* p = (const char*)memchr(str, pattern[0], len);
        if (!p)
            return nullptr;

        //printd(5, "memstr() pattern: %s str: %p p: %p len: %d pl: %d remaining: %d (%c %c %c)\n", pattern, str, p,
        //  len, pl, len-(p-str), p[1], p[2], p[3]);

        // if there is not enough string left for the pattern, then return
        if ((len - (p - str)) < pl)
            return nullptr;

        bool found = true;
        for (size_t i = 1; i < pl; ++i) {
            if (pattern[i] != p[i]) {
                len -= (p - str + 1);
                str = p + 1;
                found = false;
                break;
            }
        }
        if (!found)
            continue;

        // found
        //printd(5, "memstr() got it! p: %p\n", p);
        return p;
    }
    return nullptr;
}

static void split_add_element(QoreListNode* l, const char* str, unsigned len, const QoreEncoding* enc) {
    if (enc) {
        l->push(new QoreStringNode(str, len, enc), nullptr);
    } else {
        BinaryNode* b = new BinaryNode;
        b->append(str, len);
        l->push(b, nullptr);
    }
}

QoreListNode* split_intern(const char* pattern, size_t pl, const char* str, size_t sl, const QoreEncoding* enc,
        bool with_separator) {
    QoreListNode* l = new QoreListNode(enc ? stringTypeInfo : binaryTypeInfo);
    const char* ostr = str;
    while (const char* p = memstr(str, pattern, pl, sl - (str - ostr))) {
        split_add_element(l, str, p - str + (with_separator ? pl : 0), enc);
        str = p + pl;
    }
    // add last field if there is data remaining
    if (sl - (str - ostr))
        split_add_element(l, str, sl - (str - ostr), enc);

    return l;
}

// count how many consecutive quotes
static size_t count_quotes(const char*& str, size_t& len, const char* quote, size_t quotelen) {
    size_t rv = 0;
    while (len && len >= quotelen) {
        if (!memcmp(quote, str, quotelen)) {
            str += quotelen;
            len -= quotelen;
            ++rv;
            continue;
        }
        break;
    }
    return rv;
}

// count how many consecutive quotes
static size_t count_quotes(const char*& str, size_t& len, const QoreString& quote) {
    return count_quotes(str, len, quote.c_str(), quote.size());
}

QoreListNode* split_with_quote(ExceptionSink* xsink, const QoreString* sep, const QoreString* str,
        const QoreString* quote, bool trim_unquoted, AbstractIteratorHelper* h, const QoreString* eol) {
    // convert pattern encoding to string encoding if necessary
    TempEncodingHelper pat(sep, str->getEncoding(), xsink);
    if (*xsink)
        return nullptr;

    // convert quote encoding to string encoding if necessary
    TempEncodingHelper tquote(quote, str->getEncoding(), xsink);
    if (*xsink)
        return nullptr;

    return split_with_quote(xsink, pat->c_str(), pat->size(), str, tquote->c_str(), tquote->size(), trim_unquoted, h,
        eol);
}

QoreListNode* split_with_quote(ExceptionSink* xsink, const char* pat, size_t patlen, const QoreString* str,
        const char* quote, size_t quotelen, bool trim_unquoted, AbstractIteratorHelper* h, const QoreString* eol) {
    // in case the eol string needs encoding conversion
    TempEncodingHelper teol;

    // temporary string buffer in case additional lines need to be retrieved from a line iterator
    SimpleRefHolder<QoreStringNode> buf;

    //printd(5, "split_with_quote() sep: %s str: %s quote: %s trim_unquoted: %d\n", pat->c_str(), str->c_str(), tquote->c_str(), trim_unquoted);

    if (!quotelen || quotelen > patlen) {
        return split_intern(pat, patlen, str->c_str(), str->strlen(), str->getEncoding());
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(stringTypeInfo), xsink);
    const char* ostr = str->c_str();
    size_t sl = str->strlen();
    const char* tpattern = pat;
    size_t pl = patlen;

    const char* ststr = ostr;

    // remaining byte length
    size_t len = sl;

    //printd(5, "split_with_quote() '%s' len: " QSD "\n", ststr, len);

    while (len) {
        // make sure len is always calculated correctly
        assert(len == (str->size() - (ststr - ostr)));

        size_t num_quotes = count_quotes(ststr, len, quote, quotelen);
        if (num_quotes) {
            // see if we have empty quotes & we're at the end of the field
            if (num_quotes == 2 && (!len || (len >= pl && !memcmp(tpattern, ststr, pl)))) {
                // add an empty string to the list
                l->push(new QoreStringNode(str->getEncoding()), nullptr);

                if (!len) {
                    break;
                }

                ststr += pl;
                len -= pl;
                continue;
            } else if (!(num_quotes & 1)) {
                xsink->raiseException("SPLIT-ERROR", "field with text must begin with an add number of quotes (got "
                    QSD " quotes at the beginning of the field)", num_quotes);
                return nullptr;
            }

            // create the field string
            QoreStringNodeHolder field(new QoreStringNode(str->getEncoding()));

            // process double quotes to single quote
            while (num_quotes >= 2) {
                field->concat(quote);
                assert(!*xsink);
                num_quotes -= 2;
            }

            assert(num_quotes == 1);

            // find next quote character, ignore escaped and doubled quotes
            const char* p;
            while (len) {
                // make sure len is always calculated correctly
                assert(len == (str->size() - (ststr - ostr)));

                p = memstr(ststr, quote, quotelen, len);
                if (!p) {
                    // retrieve another line and add it to the current one if possible
                    if (h) {
                        // advance line index to add the next line to the current one
                        if (!h->next(xsink)) {
                            if (!*xsink) {
                                xsink->raiseException("SPLIT-ERROR", "cannot find closing quote '%s' in field " QSD
                                    " (starting with 1); no more line content in iterator", quote, l->size() + 1);
                            }
                        }
                        if (*xsink) {
                            return nullptr;
                        }
                        // get the line
                        ValueHolder v(h->getValue(xsink), xsink);
                        if (*xsink) {
                            return nullptr;
                        }
                        if (v->getType() != NT_STRING) {
                            xsink->raiseException("SPLIT-ERROR", "expecting iterator value type 'string'; got type " \
                                "'%s' instead", v->getTypeName());
                            return nullptr;
                        }
                        // set up the temporary line buffer if needed
                        if (!buf) {
                            buf = new QoreStringNode(*str);
                        }

                        // get ostr offset so we can recalculate after concatenating
                        size_t offset = ststr - ostr;
                        const QoreStringNode* new_line = v->get<const QoreStringNode>();
                        // add line separator
                        size_t eol_len;
                        if (!eol) {
                            eol_len = 1;
                            buf->concat('\n');
                        } else {
                            if (!teol && !teol.set(eol, str->getEncoding(), xsink)) {
                                assert(*xsink);
                                return nullptr;
                            }
                            buf->concat(teol->c_str());
                            eol_len = teol->size();
                        }
                        buf->concat(new_line, xsink);
                        if (*xsink) {
                            return nullptr;
                        }
                        len += (new_line->size() + eol_len);
                        sl += (new_line->size() + eol_len);
                        str = *buf;
                        ostr = buf->c_str();
                        ststr = ostr + offset;
                        //printd(5, "split_with_quote() got new line %p (" QLLD "): '%s' eol: '%s' (%d) ststr: "
                        //    "'%s'\n", new_line, new_line->size(), new_line->c_str(), teol ? teol->c_str() : "<nl>",
                        //    (int)eol_len, ststr);
                        assert((size_t)(ststr - ostr) == offset);
                        // make sure len is always calculated correctly
                        assert(len == (str->size() - (ststr - ostr)));
                        continue;
                    }

                    xsink->raiseException("SPLIT-ERROR", "cannot find closing quote '%s' in field " QSD " (starting "
                        "with 1)", quote, l->size() + 1);
                    return nullptr;
                }
                //printd(5, "f " QSD " found closing quote in pos " QSD " (len " QSD ") str: '%s' ('%s')\n",
                //    l->size() + 1, p - ststr, len, ststr, field->c_str());

                assert(p != ststr);

                // add text up to the quote to the field
                size_t text_len = p - ststr;
                // if we have an escaped quote, do not copy the escaped quote to the string
                if (*(p - 1) == '\\') {
                    --text_len;
                }

                field->concat(ststr, text_len);
                {
                    QoreString fstr(ststr, text_len);
                    //printd(5, "f " QSD " adding text: '%s' (len " QSD ") ('%s')\n", l->size() + 1, fstr.c_str(), fstr.size(), field->c_str());
                }

                // skip the quote in the source string
                text_len += quotelen;
                ststr += text_len;
                len -= text_len;

                // add escaped quote to field and keep searching
                if (*(p - 1) == '\\') {
                    field->concat(quote);
                    //printd(5, "f " QSD " adding escaped quote: %s ('%s')\n", l->size() + 1, tquote->c_str(), field->c_str());

                    assert(!*xsink);
                    // increment text pointer by escape char
                    ststr += 1;
                    len -= 1;
                    continue;
                }

                // if we have an odd number of quotes, then we have found the end of the field
                num_quotes = count_quotes(ststr, len, quote, quotelen) + 1;

                //printd(5, "f " QSD " found " QSD " quote(s) ('%s')\n", l->size() + 1, num_quotes, field->c_str());

                // process double quotes to single quote
                while (num_quotes >= 2) {
                    field->concat(quote);
                    //printd(5, "f " QSD " adding quote: %s ('%s')\n", l->size() + 1, tquote->c_str(), field->c_str());
                    assert(!*xsink);
                    num_quotes -= 2;
                }

                if (num_quotes) {
                    assert(num_quotes == 1);
                    break;
                }
            }

            // or a separator string comes next
            if (len && (len < pl || memcmp(tpattern, ststr, pl))) {
                xsink->raiseException("SPLIT-ERROR", "separator pattern '%s' does not follow end quote in field "
                    QSD " (starting with 1)", tpattern, l->size() + 1);
                return nullptr;
            }

            // add the field to the list
            l->push(field.release(), nullptr);

            if (!len) {
                break;
            }
            ststr += pl;
            len -= pl;
            continue;
        }

        // no quotes
        assert(!num_quotes);

        // create the field string
        QoreStringNodeHolder field(new QoreStringNode(str->getEncoding()));

        const char* p = memstr(ststr, tpattern, pl, len);
        if (!p) {
            field->concat(ststr, len);
            QoreString fstr(ststr, len);
            //printd(5, "f " QSD " adding whole unquoted text: '%s' (len " QSD ") ('%s')\n", l->size() + 1, fstr.c_str(), fstr.size(), field->c_str());
        } else {
            field->concat(ststr, p - ststr);
            QoreString fstr(ststr, p - ststr);
            //printd(5, "f " QSD " adding unquoted text: '%s' (len " QSD ") ('%s')\n", l->size() + 1, fstr.c_str(), fstr.size(), field->c_str());
        }

        if (trim_unquoted && field->trim(xsink)) {
            return nullptr;
        }

        // add the new field
        l->push(field.release(), nullptr);
        if (!p) {
            break;
        }

        // advance the pointer
        size_t text_len = (p - ststr) + pl;
        ststr += text_len;
        len -= text_len;

        //printd(5, "f " QSD " now setting ststr: '%s' len: " QSD "\n", l->size() + 1, ststr, len);
    }

    return l.release();
}

QoreStringNode* join_intern(const QoreStringNode* p0, const QoreListNode* l, int offset, ExceptionSink* xsink) {
    assert(xsink);
    SimpleRefHolder<QoreStringNode> str(new QoreStringNode(p0->getEncoding()));

    for (unsigned i = offset; i < l->size(); i++) {
        QoreValue p = l->retrieveEntry(i);
        if (!p.isNothing()) {
            QoreStringValueHelper t(p, p0->getEncoding(), xsink);
            if (*xsink) {
                return nullptr;
            }

            str->concat(*t);
        }
        if (i < (l->size() - 1)) {
            str->concat(p0);
        }
    }

    return str.release();
}

QoreStringNode* format_float_intern(const QoreString& fmt, double num, ExceptionSink* xsink) {
    if (fmt.empty()) {
        return q_fix_decimal(new QoreStringNodeMaker("%.9g", num), 0);
    }

    SimpleRefHolder<QoreStringNode> str(q_fix_decimal(new QoreStringNodeMaker("%.*f", OP_DBL_Digs + 6, num), 0));
    if (qore_number_private::formatNumberString(**str, fmt, xsink)) {
        return nullptr;
    }
    return str.release();
}

QoreStringNode* format_float_intern(int prec, const QoreString& dsep, const QoreString& tsep, double num,
        ExceptionSink* xsink) {
    SimpleRefHolder<QoreStringNode> str(q_fix_decimal(new QoreStringNodeMaker("%.*f", OP_DBL_Digs + 6, num), 0));
    if (qore_number_private::formatNumberString(**str, prec, dsep, tsep, xsink)) {
        return nullptr;
    }
    return str.release();
}
# 496 "ql_string.qpp"
# 583 "ql_string.qpp"
# 608 "ql_string.qpp"
# 624 "ql_string.qpp"
# 630 "ql_string.qpp"
# 637 "ql_string.qpp"
# 659 "ql_string.qpp"
# 665 "ql_string.qpp"
# 672 "ql_string.qpp"
# 698 "ql_string.qpp"
# 728 "ql_string.qpp"
# 734 "ql_string.qpp"
# 764 "ql_string.qpp"
# 770 "ql_string.qpp"
# 795 "ql_string.qpp"
# 821 "ql_string.qpp"
# 845 "ql_string.qpp"
# 870 "ql_string.qpp"
# 876 "ql_string.qpp"
# 907 "ql_string.qpp"
# 914 "ql_string.qpp"
# 943 "ql_string.qpp"
# 950 "ql_string.qpp"
# 981 "ql_string.qpp"
# 988 "ql_string.qpp"
# 1017 "ql_string.qpp"
# 1024 "ql_string.qpp"
# 1045 "ql_string.qpp"
# 1052 "ql_string.qpp"
# 1072 "ql_string.qpp"
# 1079 "ql_string.qpp"
# 1085 "ql_string.qpp"
# 1119 "ql_string.qpp"
# 1156 "ql_string.qpp"
# 1177 "ql_string.qpp"
# 1184 "ql_string.qpp"
# 1201 "ql_string.qpp"
# 1207 "ql_string.qpp"
# 1225 "ql_string.qpp"
# 1231 "ql_string.qpp"
# 1248 "ql_string.qpp"
# 1254 "ql_string.qpp"
# 1284 "ql_string.qpp"
# 1290 "ql_string.qpp"
# 1345 "ql_string.qpp"
# 1351 "ql_string.qpp"
# 1383 "ql_string.qpp"
# 1389 "ql_string.qpp"
# 1508 "ql_string.qpp"
# 1514 "ql_string.qpp"
# 1532 "ql_string.qpp"
# 1552 "ql_string.qpp"
# 1558 "ql_string.qpp"
# 1578 "ql_string.qpp"
# 1603 "ql_string.qpp"
# 1609 "ql_string.qpp"
# 1627 "ql_string.qpp"
# 1654 "ql_string.qpp"
# 1660 "ql_string.qpp"
# 1679 "ql_string.qpp"
# 1706 "ql_string.qpp"
# 1727 "ql_string.qpp"
# 1756 "ql_string.qpp"
# 1802 "ql_string.qpp"
# 1839 "ql_string.qpp"
# 1866 "ql_string.qpp"
# 1872 "ql_string.qpp"
# 1890 "ql_string.qpp"
# 1909 "ql_string.qpp"
# 1931 "ql_string.qpp"
# 1938 "ql_string.qpp"
# 1960 "ql_string.qpp"
# 1967 "ql_string.qpp"
# 1991 "ql_string.qpp"
# 2013 "ql_string.qpp"
# 2020 "ql_string.qpp"
# 2044 "ql_string.qpp"
# 2051 "ql_string.qpp"
# 2075 "ql_string.qpp"
# 2082 "ql_string.qpp"
# 2108 "ql_string.qpp"
# 2132 "ql_string.qpp"
# 2139 "ql_string.qpp"
# 2150 "ql_string.qpp"
# 2166 "ql_string.qpp"
# 2180 "ql_string.qpp"
# 2209 "ql_string.qpp"
# 2238 "ql_string.qpp"
// int bindex(softstring str, softstring substr, softint pos = 0) {}
static QoreValue f_bindex_vsvsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 939 "ql_string.qpp"
   return str->bindex(*substr, (qore_offset_t)pos);
}

// int bindex() {}
static QoreValue f_bindex(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 946 "ql_string.qpp"
   return -1;
}

// int brindex(softstring str, softstring substr, softint pos = -1) {}
static QoreValue f_brindex_vsvsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 1013 "ql_string.qpp"
   return str->brindex(*substr, (qore_offset_t)pos);
}

// int brindex() {}
static QoreValue f_brindex(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1020 "ql_string.qpp"
   return -1;
}

// int char_width(softstring str) {}
static QoreValue f_char_width_vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 694 "ql_string.qpp"
    return str->getCharWidth(xsink);
}

// string chomp(string str) {}
static QoreValue f_chomp_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1572 "ql_string.qpp"
   QoreStringNode* rv = str->copy();
   rv->chomp();
   return rv;
}

// *string chomp(reference<string> str) {}
static QoreValue f_chomp_C17reference_string_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
# 1590 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode* >(ref.getUnique());
   if (*xsink)
      return QoreValue();

   rv->chomp();
   return rv->refSelf();
}

// nothing chomp() {}
static QoreValue f_chomp(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1606 "ql_string.qpp"

    return QoreValue();
}

// string chr(softint val, *string encoding) {}
static QoreValue f_chr_viNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 val = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 1065 "ql_string.qpp"
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   QoreStringNode* rv = new QoreStringNode(qe);
   rv->concat((char)val);
   return rv;
}

// string chr(auto arg) {}
static QoreValue f_chr_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1075 "ql_string.qpp"
   return new QoreStringNode('\0');
}

// nothing chr() {}
static QoreValue f_chr(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1082 "ql_string.qpp"

    return QoreValue();
}

// string convert_encoding(string str, string encoding) {}
static QoreValue f_convert_encoding_VsVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 1);
# 1221 "ql_string.qpp"
   return str->convertEncoding(QEM.findCreate(encoding), xsink);
}

// nothing convert_encoding() {}
static QoreValue f_convert_encoding(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1228 "ql_string.qpp"

    return QoreValue();
}

// string f_printf(string fmt, ...) {}
static QoreValue f_f_printf_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2038 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_sprintf(args, 1, 0, xsink));
   return node;
}

// string f_printf() {}
static QoreValue f_f_printf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2047 "ql_string.qpp"
   return null_string();
}

// string f_sprintf(string fmt, ...) {}
static QoreValue f_f_sprintf_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1927 "ql_string.qpp"
   return q_sprintf(args, 1, 0, xsink);
}

// string f_sprintf() {}
static QoreValue f_f_sprintf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1934 "ql_string.qpp"
   return null_string();
}

// string f_vprintf(string fmt, auto varg) {}
static QoreValue f_f_vprintf_VsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2102 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_vsprintf(args, 1, 0, xsink));
   return node;
}

// string f_vsprintf(string fmt, auto varg) {}
static QoreValue f_f_vsprintf_VsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1987 "ql_string.qpp"
   return q_vsprintf(args, 1, 0, xsink);
}

// nothing flush() {}
static QoreValue f_flush(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2146 "ql_string.qpp"
   fflush(stdout);
    return QoreValue();
}

// string force_encoding(string str, string encoding) {}
static QoreValue f_force_encoding_VsVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 1);
# 1244 "ql_string.qpp"
   return new QoreStringNode(str->c_str(), str->strlen(), QEM.findCreate(encoding));
}

// nothing force_encoding() {}
static QoreValue f_force_encoding(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1251 "ql_string.qpp"

    return QoreValue();
}

// string format_number(string fmt, softfloat num) {}
static QoreValue f_format_number_Vsvf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 0);
    double num = HARD_QORE_VALUE_FLOAT(args, 1);
# 1862 "ql_string.qpp"
   return format_float_intern(*fmt, num, xsink);
}

// nothing format_number() {}
static QoreValue f_format_number(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1869 "ql_string.qpp"

    return QoreValue();
}

// string get_encoding(string str) {}
static QoreValue f_get_encoding_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1197 "ql_string.qpp"
   return new QoreStringNode(str->getEncoding()->getCode());
}

// nothing get_encoding() {}
static QoreValue f_get_encoding(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1204 "ql_string.qpp"

    return QoreValue();
}

// int index(softstring str, softstring substr, softint pos = 0) {}
static QoreValue f_index_vsvsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 903 "ql_string.qpp"
   return str->index(*substr, pos, xsink);
}

// int index() {}
static QoreValue f_index(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 910 "ql_string.qpp"
   return -1;
}

// string join(string str, ...) {}
static QoreValue f_join_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1528 "ql_string.qpp"
   return join_intern(str, args, 1, xsink);
}

// string join(string str, list<auto> l) {}
static QoreValue f_join_VsC10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 1);
# 1548 "ql_string.qpp"
   return join_intern(str, l, 0, xsink);
}

// nothing join() {}
static QoreValue f_join(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1555 "ql_string.qpp"

    return QoreValue();
}

// int length(softstring str) {}
static QoreValue f_length_vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 604 "ql_string.qpp"
   return str->length();
}

// int length(binary bin) {}
static QoreValue f_length_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 620 "ql_string.qpp"
   return bin->size();
}

// nothing length() {}
static QoreValue f_length(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 627 "ql_string.qpp"

    return QoreValue();
}

// int length(auto arg) {}
static QoreValue f_length_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 633 "ql_string.qpp"
   return 0;
}

// string ltrim(string str, *string chars) {}
static QoreValue f_ltrim_VsNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1674 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(str->copy());
   return rv->trimLeading(xsink, chars) ? QoreValue() : rv.release();
}

// *string ltrim(reference<string> str, *string chars) {}
static QoreValue f_ltrim_C17reference_string_Ns(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1694 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode*>(ref.getUnique());
   if (*xsink)
      return QoreValue();

   return rv->trimLeading(xsink, chars) ? QoreValue() : rv->refSelf();
}

// int ord(softstring str, softint offset = 0) {}
static QoreValue f_ord_vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1038 "ql_string.qpp"
   if (offset < 0 || (size_t)offset >= str->strlen())
      return -1;

   return (unsigned char)str->c_str()[offset];
}

// int ord() {}
static QoreValue f_ord(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1048 "ql_string.qpp"
   return 0;
}

// bool parse_boolean(string str) {}
static QoreValue f_parse_boolean_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 2162 "ql_string.qpp"
   return q_parse_bool(str->c_str());
}

// bool parse_boolean(any val) {}
static QoreValue f_parse_boolean_Va(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 2176 "ql_string.qpp"
   return val.getAsBool();
}

// float parse_float(string str, string fmt) {}
static QoreValue f_parse_float_VsVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 1);
# 2226 "ql_string.qpp"
   TempEncodingHelper tfmt(fmt, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   QoreString tmp(*str);
   if (fix_num(tmp, **tfmt, xsink))
      return QoreValue();

   return q_strtod(tmp.getBuffer());
}

// int parse_int(string str, string fmt) {}
static QoreValue f_parse_int_VsVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 1);
# 2255 "ql_string.qpp"
   TempEncodingHelper tfmt(fmt, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   QoreString tmp(*str);
   if (fix_num(tmp, **tfmt, xsink))
      return QoreValue();

   return strtoll(tmp.getBuffer(), 0, 10);
}

// number parse_number(string str, string fmt) {}
static QoreValue f_parse_number_VsVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 1);
# 2197 "ql_string.qpp"
   TempEncodingHelper tfmt(fmt, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   QoreString tmp(*str);
   if (fix_num(tmp, **tfmt, xsink))
      return QoreValue();

   return new QoreNumberNode(tmp.getBuffer());
}

// nothing print(...) {}
static QoreValue f_print_VV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1901 "ql_string.qpp"
   if (!args)
      return QoreValue();

   for (unsigned i = 0; i < args->size(); i++)
      print_node(stdout, get_param_value(args, i));
    return QoreValue();
}

// string printf(string fmt, ...) {}
static QoreValue f_printf_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2069 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_sprintf(args, 0, 0, xsink));
   return node;
}

// string printf() {}
static QoreValue f_printf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2078 "ql_string.qpp"
   return null_string();
}

// bool regex(string str, string regex, int options = 0) {}
static QoreValue f_regex_VsVsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 1);
    int64 options = HARD_QORE_VALUE_INT(args, 2);
# 1276 "ql_string.qpp"
   QoreRegex qr(*regex, options, xsink);
   if (*xsink)
      return QoreValue();

   return qr.exec(str, xsink);
}

// nothing regex() {}
static QoreValue f_regex(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1287 "ql_string.qpp"

    return QoreValue();
}

// *list<*string> regex_extract(string str, string regex, int options = 0) {}
static QoreValue f_regex_extract_VsVsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 1);
    int64 options = HARD_QORE_VALUE_INT(args, 2);
# 1375 "ql_string.qpp"
   QoreRegex qr(*regex, options, xsink);
   if (*xsink)
      return QoreValue();

   return qr.extractSubstrings(str, xsink);
}

// nothing regex_extract() {}
static QoreValue f_regex_extract(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1386 "ql_string.qpp"

    return QoreValue();
}

// string regex_subst(string str, string regex, string subst, int options = 0) {}
static QoreValue f_regex_subst_VsVsVsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* subst = HARD_QORE_VALUE_STRING(args, 2);
    int64 options = HARD_QORE_VALUE_INT(args, 3);
# 1327 "ql_string.qpp"
   bool global;
   if (options & QRE_GLOBAL) {
      global = true;
      options &= 0xffffffff;
   }
   else
      global = false;

   QoreRegexSubst qrs(regex, (int)options, xsink);
   if (*xsink)
      return QoreValue();

   if (global)
      qrs.setGlobal();
   return qrs.exec(str, subst, xsink);
}

// nothing regex_subst() {}
static QoreValue f_regex_subst(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1348 "ql_string.qpp"

    return QoreValue();
}

// string replace(string str, string source, string target, int start = 0, int end = -1) {}
static QoreValue f_replace_VsVsVsViVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* source = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 2);
    int64 start = HARD_QORE_VALUE_INT(args, 3);
    int64 end = HARD_QORE_VALUE_INT(args, 4);
# 1416 "ql_string.qpp"
    const QoreEncoding* ccs = str->getEncoding();

    TempEncodingHelper t1(source, ccs, xsink);
    if (*xsink) {
        return QoreValue();
    }

    if (t1->empty() || str->empty()) {
        return str->refSelf();
    }

    TempEncodingHelper t2(target, ccs, xsink);
    if (*xsink) {
        return QoreValue();
    }

    SimpleRefHolder<QoreStringNode> nstr(new QoreStringNode(ccs));

    const char* cstr = str->c_str();
    size_t plen = t1->size();

    // only execute the replace logic if there is a pattern
    if (plen) {
        const char* pattern = t1->c_str();

        bool has_length = false;
        size_t length = 0;

        if (start) {
            if (start < 0) {
                length = str->length();
                has_length = true;
                start = length + start;
            }
            const char* cn = cstr + str->size();
            size_t i = ccs->getByteLen(cstr, cn, start, xsink);
            if (*xsink) {
                return QoreValue();
            }
            if (i == str->size()) {
                return str->refSelf();
            }
            // add skipped beginning bytes to target string
            nstr->concat(cstr, i);
            // reposition source ptr
            cstr += i;
        }

        size_t size = str->size();

        if (end) {
            if (end < 0) {
                if (!has_length) {
                    length = str->length();
                }
                end = length + end;
            }
            const char* cn = str->c_str() + str->size();
            // get the byte length to the character after the requested one
            size_t i = ccs->getByteLen(str->c_str(), cn, end + 1, xsink);
            if (*xsink) {
                return QoreValue();
            }
            if (start > 0 && end < start) {
                return str->refSelf();
            }
            //printd(5, "end: %d\n", (int)i);

            size = i;
        }

        //printd(5, "cstr: %p str: %p size: %d cstr - str: %d\n", cstr, str->c_str(), (int)size, (int)(cstr - str->c_str()));
        assert(static_cast<size_t>(cstr - str->c_str()) < size);
        while (const char* p = (const char*)q_memmem(cstr, size - (cstr - str->c_str()), pattern, plen)) {
            //printd(5, "str: %p p: %p '%s' '%s'->'%s'\n", cstr, p, cstr, pattern, t2->c_str());
            if (p != cstr) {
                nstr->concat(cstr, p - cstr);
            }
            nstr->concat(*t2);
            cstr = p + plen;
        }
    }

    // add last field
    if (*cstr) {
        nstr->concat(cstr);
    }

    return nstr.release();
}

// nothing replace() {}
static QoreValue f_replace(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1511 "ql_string.qpp"

    return QoreValue();
}

// string reverse(softstring str) {}
static QoreValue f_reverse_vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1886 "ql_string.qpp"
   return str->reverse();
}

// int rindex(softstring str, softstring substr, softint pos = -1) {}
static QoreValue f_rindex_vsvsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 977 "ql_string.qpp"
   return str->rindex(*substr, (qore_offset_t)pos, xsink);
}

// int rindex() {}
static QoreValue f_rindex(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 984 "ql_string.qpp"
   return -1;
}

// string rtrim(string str, *string chars) {}
static QoreValue f_rtrim_VsNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1722 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(str->copy());
   return rv->trimTrailing(xsink, chars) ? QoreValue() : rv.release();
}

// *string rtrim(reference<string> str, *string chars) {}
static QoreValue f_rtrim_C17reference_string_Ns(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1744 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode*>(ref.getUnique());
   if (*xsink)
      return QoreValue();

   return rv->trimTrailing(xsink, chars) ? QoreValue() : rv->refSelf();
}

// list<string> split(string sep, string str, bool with_separator = False) {}
static QoreValue f_split_VsVsVb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 1);
    bool with_separator = HARD_QORE_VALUE_BOOL(args, 2);
# 1108 "ql_string.qpp"
    // convert pattern encoding to string if necessary
    TempEncodingHelper temp(sep, str->getEncoding(), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return split_intern(temp->c_str(), temp->strlen(), str->c_str(), str->strlen(), str->getEncoding(),
        with_separator);
    return QoreValue();
}

// list<string> split(string sep, string str, string quote, bool trim_unquoted = False) {}
static QoreValue f_split_VsVsVsVb(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* quote = HARD_QORE_VALUE_STRING(args, 2);
    bool trim_unquoted = HARD_QORE_VALUE_BOOL(args, 3);
# 1152 "ql_string.qpp"
    return split_with_quote(xsink, sep, str, quote, trim_unquoted);
}

// list<binary> split(data sep, binary data) {}
static QoreValue f_split_VDVy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue sep = get_param_value(args, 0);
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 1);
# 1170 "ql_string.qpp"
    const char* bin_ptr;
    size_t bin_len;
    q_get_data(sep, bin_ptr, bin_len);
    return split_intern(bin_ptr, bin_len, (const char*)data->getPtr(), data->size(), nullptr);
}

// list<string> split() {}
static QoreValue f_split(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1180 "ql_string.qpp"
   return new QoreListNode(stringTypeInfo);
}

// string sprintf(string fmt, ...) {}
static QoreValue f_sprintf_VsVV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1956 "ql_string.qpp"
   return q_sprintf(args, 0, 0, xsink);
}

// string sprintf() {}
static QoreValue f_sprintf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1963 "ql_string.qpp"
   return null_string();
}

// int strlen(softstring str) {}
static QoreValue f_strlen_vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 655 "ql_string.qpp"
   return str->strlen();
}

// nothing strlen() {}
static QoreValue f_strlen(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 662 "ql_string.qpp"

    return QoreValue();
}

// int strlen(auto arg) {}
static QoreValue f_strlen_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 668 "ql_string.qpp"
   return 0;
}

// string strmul(softstring str, softint smul, *softint offset) {}
static QoreValue f_strmul_vsvini(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 smul = HARD_QORE_VALUE_INT(args, 1);
    int64 offset = HARD_QORE_VALUE_INT(args, 2);
# 1815 "ql_string.qpp"
   if (offset < 0)
      return xsink->raiseException("STRMUL-ERROR", "Offset argument has to be >= 0; value: " QLLD, offset);

   if (smul < 0)
      return xsink->raiseException("STRMUL-ERROR", "Multiple argument has to be >= 0; value: " QLLD, smul);

   QoreStringNodeHolder ret(new QoreStringNode);
   while (smul > 0) {
      ret->concat(str);
      --smul;
   }

   if (offset) {
      if (offset > (int64)ret->length())
         return xsink->raiseException("STRMUL-ERROR", "Offset (" QLLD ") is greater than string length (" QSD ")", offset, ret->length());
      ret->splice(ret->length() - offset, xsink);
      if (*xsink)
         return QoreValue();
   }

   return ret.release();
}

// string substr(softstring str, softint start) {}
static QoreValue f_substr_vsvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 788 "ql_string.qpp"
   QoreStringNode* rv = str->substr(start, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// string substr(softstring str, softint start, softint len) {}
static QoreValue f_substr_vsvivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
# 814 "ql_string.qpp"
   QoreStringNode* rv = str->substr(start, len, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// binary substr(binary b, softint start) {}
static QoreValue f_substr_Vyvi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 839 "ql_string.qpp"
   BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start);
   return b1;
}

// binary substr(binary b, softint start, softint len) {}
static QoreValue f_substr_Vyvivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
# 864 "ql_string.qpp"
   BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start, len);
   return b1;
}

// nothing substr() {}
static QoreValue f_substr(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 873 "ql_string.qpp"

    return QoreValue();
}

// string tolower(string str) {}
static QoreValue f_tolower_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 720 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_tolower(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// nothing tolower() {}
static QoreValue f_tolower(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 731 "ql_string.qpp"

    return QoreValue();
}

// string toupper(string str) {}
static QoreValue f_toupper_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 756 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_toupper(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// nothing toupper() {}
static QoreValue f_toupper(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 767 "ql_string.qpp"

    return QoreValue();
}

// string trim(string str, *string chars) {}
static QoreValue f_trim_VsNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1622 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(str->copy());
   return rv->trim(xsink, chars) ? QoreValue() : rv.release();
}

// *string trim(reference<string> str, *string chars) {}
static QoreValue f_trim_C17reference_string_Ns(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1642 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode*>(ref.getUnique());
   if (*xsink)
      return QoreValue();

   return rv->trim(xsink, chars) ? QoreValue() : rv->refSelf();
}

// nothing trim() {}
static QoreValue f_trim(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1657 "ql_string.qpp"

    return QoreValue();
}

// string trunc_str(softstring str, softint len, *string encoding) {}
static QoreValue f_trunc_str_vsviNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 len = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* encoding = get_param_value(args, 2).get<const QoreStringNode>();
# 1771 "ql_string.qpp"
   const QoreEncoding* enc = encoding ? QEM.findCreate(encoding) : str->getEncoding();
   if (len <= 0)
      return new QoreStringNode(enc);

   TempEncodingHelper tmp(str, enc, xsink);
   if (!tmp)
      return QoreValue();

   if (tmp->strlen() < (size_t)len) {
      len = tmp->strlen();
      return new QoreStringNode(tmp.giveBuffer(), len, len + 1, enc);
   }

   if (!enc->isMultiByte())
      return new QoreStringNode(tmp->c_str(), len, enc);

   // find position of last character fitting in len bytes
   const char* p = tmp->c_str();
   int64 sl = 0;
   while (true) {
      qore_offset_t size = enc->getCharLen(p, len - sl);
      if (size <= 0 || ((sl + size) > len))
         break;
      sl += size;
      p += size;
   }

   return new QoreStringNode(tmp->c_str(), sl, enc);
}

// string vprintf(string fmt, auto varg) {}
static QoreValue f_vprintf_VsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2126 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_vsprintf(args, 0, 0, xsink));
   return node;
}

// string vprintf() {}
static QoreValue f_vprintf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2135 "ql_string.qpp"
   return null_string();
}

// string vsprintf(string fmt, auto varg) {}
static QoreValue f_vsprintf_VsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2009 "ql_string.qpp"
   return q_vsprintf(args, 0, 0, xsink);
}

// string vsprintf() {}
static QoreValue f_vsprintf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2016 "ql_string.qpp"
   return null_string();
}


DLLLOCAL void init_string_functions(QoreNamespace& ns) {
    // int bindex(softstring str, softstring substr, softint pos = 0) {}
    ns.addBuiltinVariant("bindex", (q_func_n_t)f_bindex_vsvsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softStringTypeInfo, QORE_PARAM_NO_ARG, "substr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "pos");
    // int bindex() {}
    ns.addBuiltinVariant("bindex", (q_func_n_t)f_bindex, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // int brindex(softstring str, softstring substr, softint pos = -1) {}
    ns.addBuiltinVariant("brindex", (q_func_n_t)f_brindex_vsvsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softStringTypeInfo, QORE_PARAM_NO_ARG, "substr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "pos");
    // int brindex() {}
    ns.addBuiltinVariant("brindex", (q_func_n_t)f_brindex, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // int char_width(softstring str) {}
    ns.addBuiltinVariant("char_width", (q_func_n_t)f_char_width_vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, softStringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string chomp(string str) {}
    ns.addBuiltinVariant("chomp", (q_func_n_t)f_chomp_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // *string chomp(reference<string> str) {}
    ns.addBuiltinVariant("chomp", (q_func_n_t)f_chomp_C17reference_string_, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, qore_get_complex_reference_type(stringTypeInfo), QORE_PARAM_NO_ARG, "str");
    // nothing chomp() {}
    ns.addBuiltinVariant("chomp", (q_func_n_t)f_chomp, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string chr(softint val, *string encoding) {}
    ns.addBuiltinVariant("chr", (q_func_n_t)f_chr_viNs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "val", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // string chr(auto arg) {}
    ns.addBuiltinVariant("chr", (q_func_n_t)f_chr_VA, QCF_NOOP, QDOM_DEFAULT, stringTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // nothing chr() {}
    ns.addBuiltinVariant("chr", (q_func_n_t)f_chr, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string convert_encoding(string str, string encoding) {}
    ns.addBuiltinVariant("convert_encoding", (q_func_n_t)f_convert_encoding_VsVs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // nothing convert_encoding() {}
    ns.addBuiltinVariant("convert_encoding", (q_func_n_t)f_convert_encoding, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string f_printf(string fmt, ...) {}
    ns.addBuiltinVariant("f_printf", (q_func_n_t)f_f_printf_VsVV, QCF_USES_EXTRA_ARGS, QDOM_TERMINAL_IO, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");
    // string f_printf() {}
    ns.addBuiltinVariant("f_printf", (q_func_n_t)f_f_printf, QCF_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // string f_sprintf(string fmt, ...) {}
    ns.addBuiltinVariant("f_sprintf", (q_func_n_t)f_f_sprintf_VsVV, QCF_CONSTANT|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");
    // string f_sprintf() {}
    ns.addBuiltinVariant("f_sprintf", (q_func_n_t)f_f_sprintf, QCF_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // string f_vprintf(string fmt, auto varg) {}
    ns.addBuiltinVariant("f_vprintf", (q_func_n_t)f_f_vprintf_VsVA, QCF_NO_FLAGS, QDOM_TERMINAL_IO, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", autoTypeInfo, QORE_PARAM_NO_ARG, "varg");
    // string f_vsprintf(string fmt, auto varg) {}
    ns.addBuiltinVariant("f_vsprintf", (q_func_n_t)f_f_vsprintf_VsVA, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", autoTypeInfo, QORE_PARAM_NO_ARG, "varg");
    // nothing flush() {}
    ns.addBuiltinVariant("flush", (q_func_n_t)f_flush, QCF_NO_FLAGS, QDOM_TERMINAL_IO, nothingTypeInfo);
    // string force_encoding(string str, string encoding) {}
    ns.addBuiltinVariant("force_encoding", (q_func_n_t)f_force_encoding_VsVs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // nothing force_encoding() {}
    ns.addBuiltinVariant("force_encoding", (q_func_n_t)f_force_encoding, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string format_number(string fmt, softfloat num) {}
    ns.addBuiltinVariant("format_number", (q_func_n_t)f_format_number_Vsvf, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", softFloatTypeInfo, QORE_PARAM_NO_ARG, "num");
    // nothing format_number() {}
    ns.addBuiltinVariant("format_number", (q_func_n_t)f_format_number, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string get_encoding(string str) {}
    ns.addBuiltinVariant("get_encoding", (q_func_n_t)f_get_encoding_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing get_encoding() {}
    ns.addBuiltinVariant("get_encoding", (q_func_n_t)f_get_encoding, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int index(softstring str, softstring substr, softint pos = 0) {}
    ns.addBuiltinVariant("index", (q_func_n_t)f_index_vsvsvi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softStringTypeInfo, QORE_PARAM_NO_ARG, "substr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "pos");
    // int index() {}
    ns.addBuiltinVariant("index", (q_func_n_t)f_index, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // string join(string str, ...) {}
    ns.addBuiltinVariant("join", (q_func_n_t)f_join_VsVV, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // string join(string str, list<auto> l) {}
    ns.addBuiltinVariant("join", (q_func_n_t)f_join_VsC10list_auto_, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // nothing join() {}
    ns.addBuiltinVariant("join", (q_func_n_t)f_join, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int length(softstring str) {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length_vs, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, softStringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // int length(binary bin) {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length_Vy, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // nothing length() {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int length(auto arg) {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length_VA, QCF_NOOP, QDOM_DEFAULT, bigIntTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // string ltrim(string str, *string chars) {}
    ns.addBuiltinVariant("ltrim", (q_func_n_t)f_ltrim_VsNs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "chars");
    // *string ltrim(reference<string> str, *string chars) {}
    ns.addBuiltinVariant("ltrim", (q_func_n_t)f_ltrim_C17reference_string_Ns, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, qore_get_complex_reference_type(stringTypeInfo), QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "chars");
    // int ord(softstring str, softint offset = 0) {}
    ns.addBuiltinVariant("ord", (q_func_n_t)f_ord_vsvi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "offset");
    // int ord() {}
    ns.addBuiltinVariant("ord", (q_func_n_t)f_ord, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // bool parse_boolean(string str) {}
    ns.addBuiltinVariant("parse_boolean", (q_func_n_t)f_parse_boolean_Vs, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // bool parse_boolean(any val) {}
    ns.addBuiltinVariant("parse_boolean", (q_func_n_t)f_parse_boolean_Va, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, anyTypeInfo, QORE_PARAM_NO_ARG, "val");
    // float parse_float(string str, string fmt) {}
    ns.addBuiltinVariant("parse_float", (q_func_n_t)f_parse_float_VsVs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, floatTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");
    // int parse_int(string str, string fmt) {}
    ns.addBuiltinVariant("parse_int", (q_func_n_t)f_parse_int_VsVs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");
    // number parse_number(string str, string fmt) {}
    ns.addBuiltinVariant("parse_number", (q_func_n_t)f_parse_number_VsVs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, numberTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");
    // nothing print(...) {}
    ns.addBuiltinVariant("print", (q_func_n_t)f_print_VV, QCF_USES_EXTRA_ARGS, QDOM_TERMINAL_IO, nothingTypeInfo);
    // string printf(string fmt, ...) {}
    ns.addBuiltinVariant("printf", (q_func_n_t)f_printf_VsVV, QCF_USES_EXTRA_ARGS, QDOM_TERMINAL_IO, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");
    // string printf() {}
    ns.addBuiltinVariant("printf", (q_func_n_t)f_printf, QCF_NOOP, QDOM_TERMINAL_IO, stringTypeInfo);
    // bool regex(string str, string regex, int options = 0) {}
    ns.addBuiltinVariant("regex", (q_func_n_t)f_regex_VsVsVi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "regex", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "options");
    // nothing regex() {}
    ns.addBuiltinVariant("regex", (q_func_n_t)f_regex, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *list<*string> regex_extract(string str, string regex, int options = 0) {}
    ns.addBuiltinVariant("regex_extract", (q_func_n_t)f_regex_extract_VsVsVi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_or_nothing_type(stringOrNothingTypeInfo), 3, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "regex", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "options");
    // nothing regex_extract() {}
    ns.addBuiltinVariant("regex_extract", (q_func_n_t)f_regex_extract, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string regex_subst(string str, string regex, string subst, int options = 0) {}
    ns.addBuiltinVariant("regex_subst", (q_func_n_t)f_regex_subst_VsVsVsVi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "regex", stringTypeInfo, QORE_PARAM_NO_ARG, "subst", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "options");
    // nothing regex_subst() {}
    ns.addBuiltinVariant("regex_subst", (q_func_n_t)f_regex_subst, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string replace(string str, string source, string target, int start = 0, int end = -1) {}
    ns.addBuiltinVariant("replace", (q_func_n_t)f_replace_VsVsVsViVi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 5, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "source", stringTypeInfo, QORE_PARAM_NO_ARG, "target", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "start", bigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "end");
    // nothing replace() {}
    ns.addBuiltinVariant("replace", (q_func_n_t)f_replace, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string reverse(softstring str) {}
    ns.addBuiltinVariant("reverse", (q_func_n_t)f_reverse_vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softStringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // int rindex(softstring str, softstring substr, softint pos = -1) {}
    ns.addBuiltinVariant("rindex", (q_func_n_t)f_rindex_vsvsvi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softStringTypeInfo, QORE_PARAM_NO_ARG, "substr", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "pos");
    // int rindex() {}
    ns.addBuiltinVariant("rindex", (q_func_n_t)f_rindex, QCF_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // string rtrim(string str, *string chars) {}
    ns.addBuiltinVariant("rtrim", (q_func_n_t)f_rtrim_VsNs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "chars");
    // *string rtrim(reference<string> str, *string chars) {}
    ns.addBuiltinVariant("rtrim", (q_func_n_t)f_rtrim_C17reference_string_Ns, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, qore_get_complex_reference_type(stringTypeInfo), QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "chars");
    // list<string> split(string sep, string str, bool with_separator = False) {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split_VsVsVb, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 3, stringTypeInfo, QORE_PARAM_NO_ARG, "sep", stringTypeInfo, QORE_PARAM_NO_ARG, "str", boolTypeInfo, QoreSimpleValue().assign(false), "with_separator");
    // list<string> split(string sep, string str, string quote, bool trim_unquoted = False) {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split_VsVsVsVb, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 4, stringTypeInfo, QORE_PARAM_NO_ARG, "sep", stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringTypeInfo, QORE_PARAM_NO_ARG, "quote", boolTypeInfo, QoreSimpleValue().assign(false), "trim_unquoted");
    // list<binary> split(data sep, binary data) {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split_VDVy, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(binaryTypeInfo), 2, dataTypeInfo, QORE_PARAM_NO_ARG, "sep", binaryTypeInfo, QORE_PARAM_NO_ARG, "data");
    // list<string> split() {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split, QCF_RUNTIME_NOOP, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));
    // string sprintf(string fmt, ...) {}
    ns.addBuiltinVariant("sprintf", (q_func_n_t)f_sprintf_VsVV, QCF_CONSTANT|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");
    // string sprintf() {}
    ns.addBuiltinVariant("sprintf", (q_func_n_t)f_sprintf, QCF_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // int strlen(softstring str) {}
    ns.addBuiltinVariant("strlen", (q_func_n_t)f_strlen_vs, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, softStringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing strlen() {}
    ns.addBuiltinVariant("strlen", (q_func_n_t)f_strlen, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int strlen(auto arg) {}
    ns.addBuiltinVariant("strlen", (q_func_n_t)f_strlen_VA, QCF_NOOP, QDOM_DEFAULT, bigIntTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // string strmul(softstring str, softint smul, *softint offset) {}
    ns.addBuiltinVariant("strmul", (q_func_n_t)f_strmul_vsvini, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 3, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "smul", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "offset");
    // string substr(softstring str, softint start) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_vsvi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start");
    // string substr(softstring str, softint start, softint len) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_vsvivi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 3, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len");
    // binary substr(binary b, softint start) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_Vyvi, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start");
    // binary substr(binary b, softint start, softint len) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_Vyvivi, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len");
    // nothing substr() {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr, QCF_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string tolower(string str) {}
    ns.addBuiltinVariant("tolower", (q_func_n_t)f_tolower_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing tolower() {}
    ns.addBuiltinVariant("tolower", (q_func_n_t)f_tolower, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string toupper(string str) {}
    ns.addBuiltinVariant("toupper", (q_func_n_t)f_toupper_Vs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // nothing toupper() {}
    ns.addBuiltinVariant("toupper", (q_func_n_t)f_toupper, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string trim(string str, *string chars) {}
    ns.addBuiltinVariant("trim", (q_func_n_t)f_trim_VsNs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "chars");
    // *string trim(reference<string> str, *string chars) {}
    ns.addBuiltinVariant("trim", (q_func_n_t)f_trim_C17reference_string_Ns, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, qore_get_complex_reference_type(stringTypeInfo), QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "chars");
    // nothing trim() {}
    ns.addBuiltinVariant("trim", (q_func_n_t)f_trim, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string trunc_str(softstring str, softint len, *string encoding) {}
    ns.addBuiltinVariant("trunc_str", (q_func_n_t)f_trunc_str_vsviNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 3, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // string vprintf(string fmt, auto varg) {}
    ns.addBuiltinVariant("vprintf", (q_func_n_t)f_vprintf_VsVA, QCF_NO_FLAGS, QDOM_TERMINAL_IO, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", autoTypeInfo, QORE_PARAM_NO_ARG, "varg");
    // string vprintf() {}
    ns.addBuiltinVariant("vprintf", (q_func_n_t)f_vprintf, QCF_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // string vsprintf(string fmt, auto varg) {}
    ns.addBuiltinVariant("vsprintf", (q_func_n_t)f_vsprintf_VsVA, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", autoTypeInfo, QORE_PARAM_NO_ARG, "varg");
    // string vsprintf() {}
    ns.addBuiltinVariant("vsprintf", (q_func_n_t)f_vsprintf, QCF_NOOP, QDOM_DEFAULT, stringTypeInfo);

}

DLLLOCAL void init_string_constants(QoreNamespace& ns) {
# 470 "ql_string.qpp"
# 478 "ql_string.qpp"
# 481 "ql_string.qpp"
# 484 "ql_string.qpp"
# 487 "ql_string.qpp"
    ns.addConstant("RE_Caseless", ((int64)PCRE2_CASELESS));
    ns.addConstant("RE_DotAll", ((int64)PCRE2_DOTALL));
    ns.addConstant("RE_Extended", ((int64)PCRE2_EXTENDED));
    ns.addConstant("RE_Global", ((int64)QRE_GLOBAL));
    ns.addConstant("RE_MultiLine", ((int64)PCRE2_MULTILINE));
    ns.addConstant("RE_Unicode", ((int64)PCRE2_UCP));
# 496 "ql_string.qpp"
# 583 "ql_string.qpp"
# 608 "ql_string.qpp"
# 624 "ql_string.qpp"
# 630 "ql_string.qpp"
# 637 "ql_string.qpp"
# 659 "ql_string.qpp"
# 665 "ql_string.qpp"
# 672 "ql_string.qpp"
# 698 "ql_string.qpp"
# 728 "ql_string.qpp"
# 734 "ql_string.qpp"
# 764 "ql_string.qpp"
# 770 "ql_string.qpp"
# 795 "ql_string.qpp"
# 821 "ql_string.qpp"
# 845 "ql_string.qpp"
# 870 "ql_string.qpp"
# 876 "ql_string.qpp"
# 907 "ql_string.qpp"
# 914 "ql_string.qpp"
# 943 "ql_string.qpp"
# 950 "ql_string.qpp"
# 981 "ql_string.qpp"
# 988 "ql_string.qpp"
# 1017 "ql_string.qpp"
# 1024 "ql_string.qpp"
# 1045 "ql_string.qpp"
# 1052 "ql_string.qpp"
# 1072 "ql_string.qpp"
# 1079 "ql_string.qpp"
# 1085 "ql_string.qpp"
# 1119 "ql_string.qpp"
# 1156 "ql_string.qpp"
# 1177 "ql_string.qpp"
# 1184 "ql_string.qpp"
# 1201 "ql_string.qpp"
# 1207 "ql_string.qpp"
# 1225 "ql_string.qpp"
# 1231 "ql_string.qpp"
# 1248 "ql_string.qpp"
# 1254 "ql_string.qpp"
# 1284 "ql_string.qpp"
# 1290 "ql_string.qpp"
# 1345 "ql_string.qpp"
# 1351 "ql_string.qpp"
# 1383 "ql_string.qpp"
# 1389 "ql_string.qpp"
# 1508 "ql_string.qpp"
# 1514 "ql_string.qpp"
# 1532 "ql_string.qpp"
# 1552 "ql_string.qpp"
# 1558 "ql_string.qpp"
# 1578 "ql_string.qpp"
# 1603 "ql_string.qpp"
# 1609 "ql_string.qpp"
# 1627 "ql_string.qpp"
# 1654 "ql_string.qpp"
# 1660 "ql_string.qpp"
# 1679 "ql_string.qpp"
# 1706 "ql_string.qpp"
# 1727 "ql_string.qpp"
# 1756 "ql_string.qpp"
# 1802 "ql_string.qpp"
# 1839 "ql_string.qpp"
# 1866 "ql_string.qpp"
# 1872 "ql_string.qpp"
# 1890 "ql_string.qpp"
# 1909 "ql_string.qpp"
# 1931 "ql_string.qpp"
# 1938 "ql_string.qpp"
# 1960 "ql_string.qpp"
# 1967 "ql_string.qpp"
# 1991 "ql_string.qpp"
# 2013 "ql_string.qpp"
# 2020 "ql_string.qpp"
# 2044 "ql_string.qpp"
# 2051 "ql_string.qpp"
# 2075 "ql_string.qpp"
# 2082 "ql_string.qpp"
# 2108 "ql_string.qpp"
# 2132 "ql_string.qpp"
# 2139 "ql_string.qpp"
# 2150 "ql_string.qpp"
# 2166 "ql_string.qpp"
# 2180 "ql_string.qpp"
# 2209 "ql_string.qpp"
# 2238 "ql_string.qpp"

}
