/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_thread.qpp

    POSIX thread library for Qore

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_thread.h"
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QC_AbstractThreadResource.h"
#include "qore/intern/QoreHashNodeIntern.h"
#include "qore/intern/QC_TimeZone.h"

#include <pthread.h>
#include <thread>

extern QoreClass* QC_TIMEZONE;

static void delete_thread_data_intern(const QoreListNode* args, ExceptionSink* xsink) {
    assert(xsink);
    assert(args);
    // get thread data hash
    QoreHashNode *data = getProgram()->getThreadData();

    // iterate through arguments and delete each key
    for (unsigned i = 0; i < args->size(); i++) {
        QoreValue p = args->retrieveEntry(i);
        if (!p.isNothing()) {
            QoreStringValueHelper t(p, QCS_DEFAULT, xsink);
            if (*xsink)
                break;

            data->deleteKey(*t, xsink);
            if (*xsink)
                break;
        }
    }
}

static QoreHashNode* remove_thread_data_intern(const QoreListNode* args, ExceptionSink* xsink) {
    assert(xsink);
    assert(args);
    // get thread data hash
    QoreHashNode* data = getProgram()->getThreadData();
    qore_hash_private* dh = qore_hash_private::get(*data);

    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);

    // iterate through arguments and remove each key
    for (unsigned i = 0; i < args->size(); i++) {
        QoreValue p = args->retrieveEntry(i);
        if (!p.isNothing()) {
            QoreStringValueHelper t(p, QCS_DEFAULT, xsink);
            if (*xsink) {
                return nullptr;
            }

            rv->setKeyValue(*t, dh->takeKeyValueIntern((*t)->c_str()), xsink);
            assert(!*xsink);
        }
    }

    return rv.release();
}
TypedHashDecl* init_hashdecl_CallStackInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("CallStackInfo", "::Qore::CallStackInfo");
    hd->addMember("endline", bigIntTypeInfo, QoreValue());
    hd->addMember("file", stringTypeInfo, QoreValue());
    hd->addMember("function", stringTypeInfo, QoreValue());
    hd->addMember("lang", stringTypeInfo, QoreValue());
    hd->addMember("line", bigIntTypeInfo, QoreValue());
    hd->addMember("offset", bigIntTypeInfo, QoreValue());
    hd->addMember("programid", bigIntOrNothingTypeInfo, QoreValue());
    hd->addMember("source", stringTypeInfo, QoreValue());
    hd->addMember("statementid", bigIntOrNothingTypeInfo, QoreValue());
    hd->addMember("type", stringTypeInfo, QoreValue());
    hd->addMember("typecode", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 133 "ql_thread.qpp"
# 152 "ql_thread.qpp"
# 167 "ql_thread.qpp"
# 183 "ql_thread.qpp"
# 200 "ql_thread.qpp"
# 218 "ql_thread.qpp"
# 224 "ql_thread.qpp"
# 245 "ql_thread.qpp"
# 265 "ql_thread.qpp"
# 283 "ql_thread.qpp"
# 301 "ql_thread.qpp"
# 314 "ql_thread.qpp"
# 331 "ql_thread.qpp"
# 337 "ql_thread.qpp"
# 353 "ql_thread.qpp"
# 363 "ql_thread.qpp"
# 389 "ql_thread.qpp"
# 414 "ql_thread.qpp"
# 424 "ql_thread.qpp"
# 449 "ql_thread.qpp"
# 485 "ql_thread.qpp"
# 523 "ql_thread.qpp"
# 557 "ql_thread.qpp"
# 581 "ql_thread.qpp"
# 606 "ql_thread.qpp"
# 626 "ql_thread.qpp"
# 650 "ql_thread.qpp"
# 673 "ql_thread.qpp"
# 698 "ql_thread.qpp"
# 721 "ql_thread.qpp"
# 745 "ql_thread.qpp"
# 762 "ql_thread.qpp"
# 786 "ql_thread.qpp"
# 812 "ql_thread.qpp"
# 839 "ql_thread.qpp"
# 861 "ql_thread.qpp"
# 889 "ql_thread.qpp"
// bool active_exception() {}
static QoreValue f_active_exception(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 896 "ql_thread.qpp"
    return q_active_exception();
}

// nothing delete_all_thread_data() {}
static QoreValue f_delete_all_thread_data(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 310 "ql_thread.qpp"
    qore_program_private::clearThreadData(*getProgram(), xsink);
    return QoreValue();
}

// nothing delete_thread_data(...) {}
static QoreValue f_delete_thread_data_VV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 240 "ql_thread.qpp"
    if (num_args(args))
        delete_thread_data_intern(args, xsink);
    return QoreValue();
}

// nothing delete_thread_data(list<softstring> l) {}
static QoreValue f_delete_thread_data_C16list_softstring_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 261 "ql_thread.qpp"
    delete_thread_data_intern(l, xsink);
    return QoreValue();
}

// hash<string,list<hash<CallStackInfo>>> getAllThreadCallStacks() {}
static QoreValue f_getAllThreadCallStacks(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 359 "ql_thread.qpp"
    return getAllCallStacks();
}

// hash<string,list<hash<CallStackInfo>>> get_all_thread_call_stacks() {}
static QoreValue f_get_all_thread_call_stacks(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 385 "ql_thread.qpp"
    return getAllCallStacks();
}

// hash<auto> get_all_thread_data() {}
static QoreValue f_get_all_thread_data(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 348 "ql_thread.qpp"
    const QoreHashNode* h = getProgram()->getThreadData();
    return h->copy();
}

// int get_default_thread_stack_size() {}
static QoreValue f_get_default_thread_stack_size(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 777 "ql_thread.qpp"
#ifdef QORE_MANAGE_STACK
    return q_thread_get_stack_size();
#else
    return xsink->raiseException("MISSING-FEATURE-ERROR", "this version of the Qore library was built without "
        "support for stack management; check Qore::Option::HAVE_MANAGE_STACK before calling");
#endif
    return QoreValue();
}

// *hash<CallStackInfo> get_parent_caller_location(int offset = 1) {}
static QoreValue f_get_parent_caller_location_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 offset = HARD_QORE_VALUE_INT(args, 0);
# 420 "ql_thread.qpp"
    return qore_get_parent_caller_location(offset);
}

// int get_stack_size() {}
static QoreValue f_get_stack_size(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 830 "ql_thread.qpp"
#ifdef QORE_HAVE_GET_STACK_SIZE
    return q_thread_get_this_stack_size();
#else
    return xsink->raiseException("MISSING-FEATURE-ERROR", "this version of the Qore library was built without "
        "support for stack management; check Qore::Option::HAVE_GET_STACK_SIZE before calling");
#endif
    return QoreValue();
}

// list<hash<CallStackInfo>> get_thread_call_stack() {}
static QoreValue f_get_thread_call_stack(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 410 "ql_thread.qpp"
    return qore_get_thread_call_stack();
}

// auto get_thread_data(string key) {}
static QoreValue f_get_thread_data_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 323 "ql_thread.qpp"
    TempEncodingHelper tmp(key, QCS_DEFAULT, xsink);
    if (!tmp)
        return QoreValue();
    const QoreHashNode* h = getProgram()->getThreadData();
    return qore_hash_private::get(*h)->getReferencedKeyValueIntern(tmp->c_str());
}

// nothing get_thread_data() {}
static QoreValue f_get_thread_data(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 334 "ql_thread.qpp"

    return QoreValue();
}

// *string get_thread_name() {}
static QoreValue f_get_thread_name(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 879 "ql_thread.qpp"
#ifdef QORE_HAVE_THREAD_NAME
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode);
    q_get_thread_name(**rv);
    return rv->empty() ? QoreValue() : rv.release();
#else
    return QoreValue();
#endif
    return QoreValue();
}

// *TimeZone get_thread_tz() {}
static QoreValue f_get_thread_tz(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 642 "ql_thread.qpp"
    bool set;
    const AbstractQoreZoneInfo* tz = get_thread_tz(set);
    if (!set)
        return QoreValue();
    return new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(tz));
}

// int gettid() {}
static QoreValue f_gettid(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 148 "ql_thread.qpp"
    return q_gettid();
}

// nothing mark_thread_resources() {}
static QoreValue f_mark_thread_resources(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 553 "ql_thread.qpp"
    mark_thread_resources();
    return QoreValue();
}

// int num_threads() {}
static QoreValue f_num_threads(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 163 "ql_thread.qpp"
    return thread_list.getNumThreads();
}

// *hash<auto> remove_thread_data(...) {}
static QoreValue f_remove_thread_data_VV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 279 "ql_thread.qpp"
    return num_args(args) ? remove_thread_data_intern(args, xsink) : nullptr;
}

// hash<auto> remove_thread_data(list<softstring> l) {}
static QoreValue f_remove_thread_data_C16list_softstring_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 297 "ql_thread.qpp"
    return remove_thread_data_intern(l, xsink);
}

// bool remove_thread_resource(AbstractThreadResource resource) {}
static QoreValue f_remove_thread_resource_C22AbstractThreadResource(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(resource, AbstractQoreThreadResource, args, 0, CID_ABSTRACTTHREADRESOURCE, "remove_thread_resource()", "AbstractThreadResource", xsink);
    if (*xsink)
        return 0;
# 693 "ql_thread.qpp"
    ReferenceHolder<AbstractQoreThreadResource> holder(resource, xsink);
    return remove_thread_resource(resource) ? false : true;
}

// bool remove_thread_resource(code resource) {}
static QoreValue f_remove_thread_resource_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* resource = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 741 "ql_thread.qpp"
    return remove_thread_resource(resource, xsink) ? false : true;
}

// nothing save_thread_data(hash<auto> h) {}
static QoreValue f_save_thread_data_C10hash_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 195 "ql_thread.qpp"
    QoreHashNode* data = getProgram()->getThreadData();
    data->merge(h, xsink);
    return QoreValue();
}

// nothing save_thread_data(string key, auto value) {}
static QoreValue f_save_thread_data_VsVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 213 "ql_thread.qpp"
    QoreHashNode* data = getProgram()->getThreadData();
    data->setKeyValue(key, value.refSelf(), xsink);
    return QoreValue();
}

// nothing save_thread_data() {}
static QoreValue f_save_thread_data(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 221 "ql_thread.qpp"

    return QoreValue();
}

// int set_default_thread_stack_size(int size) {}
static QoreValue f_set_default_thread_stack_size_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
# 803 "ql_thread.qpp"
#ifdef QORE_MANAGE_STACK
    return q_thread_set_stack_size((size_t)size, xsink);
#else
    return xsink->raiseException("MISSING-FEATURE-ERROR", "this version of the Qore library was built without "
        "support for stack management; check Qore::Option::HAVE_MANAGE_STACK before calling");
#endif
    return QoreValue();
}

// bool set_thread_init(*code init) {}
static QoreValue f_set_thread_init_Nc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* init = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 577 "ql_thread.qpp"
    return qore_program_private::setThreadInit(*getProgram(), init, xsink);
}

// nothing set_thread_name(string name) {}
static QoreValue f_set_thread_name_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 852 "ql_thread.qpp"
#ifdef QORE_HAVE_THREAD_NAME
    q_set_thread_name(name->c_str());
#else
    xsink->raiseException("MISSING-FEATURE-ERROR", "this version of the Qore library was built without support for "
        "thread names; check Qore::Option::HAVE_THREAD_NAME before calling");
#endif
    return QoreValue();
}

// nothing set_thread_resource(AbstractThreadResource resource) {}
static QoreValue f_set_thread_resource_C22AbstractThreadResource(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(resource, AbstractQoreThreadResource, args, 0, CID_ABSTRACTTHREADRESOURCE, "set_thread_resource()", "AbstractThreadResource", xsink);
    if (*xsink)
        return 0;
# 668 "ql_thread.qpp"
    ReferenceHolder<AbstractQoreThreadResource> holder(resource, xsink);
    set_thread_resource(resource);
    return QoreValue();
}

// nothing set_thread_resource(code resource, auto arg) {}
static QoreValue f_set_thread_resource_VcVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* resource = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    QoreValue arg = get_param_value(args, 1);
# 717 "ql_thread.qpp"
    set_thread_resource(resource, arg);
    return QoreValue();
}

// nothing set_thread_tz(TimeZone zone) {}
static QoreValue f_set_thread_tz_C8TimeZone(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(zone, TimeZoneData, args, 0, CID_TIMEZONE, "set_thread_tz()", "TimeZone", xsink);
    if (*xsink)
        return 0;
# 601 "ql_thread.qpp"
    ReferenceHolder<TimeZoneData> holder(zone, xsink);
    set_thread_tz(zone->get());
    return QoreValue();
}

// nothing set_thread_tz() {}
static QoreValue f_set_thread_tz(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 622 "ql_thread.qpp"
    clear_thread_tz();
    return QoreValue();
}

// list<int> thread_list() {}
static QoreValue f_thread_list(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 179 "ql_thread.qpp"
    return get_thread_list();
}

// nothing thread_yield() {}
static QoreValue f_thread_yield(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 753 "ql_thread.qpp"
#ifdef _Q_WINDOWS
    // cnanot find a working mingw c++ compiler with support for std::this_thread
    SwitchToThread();
#else
    std::this_thread::yield();
#endif
    return QoreValue();
}

// nothing throwThreadResourceExceptions() {}
static QoreValue f_throwThreadResourceExceptions(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 445 "ql_thread.qpp"
    purge_thread_resources(xsink);
    return QoreValue();
}

// nothing throw_thread_resource_exceptions() {}
static QoreValue f_throw_thread_resource_exceptions(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 481 "ql_thread.qpp"
    purge_thread_resources(xsink);
    return QoreValue();
}

// bool throw_thread_resource_exceptions_to_mark() {}
static QoreValue f_throw_thread_resource_exceptions_to_mark(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 519 "ql_thread.qpp"
    return (bool)purge_thread_resources_to_mark(xsink);
}


DLLLOCAL void init_thread_functions(QoreNamespace& ns) {
    // bool active_exception() {}
    ns.addBuiltinVariant("active_exception", (q_func_n_t)f_active_exception, QCF_NO_FLAGS, QDOM_PROCESS, boolTypeInfo);
    // nothing delete_all_thread_data() {}
    ns.addBuiltinVariant("delete_all_thread_data", (q_func_n_t)f_delete_all_thread_data, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // nothing delete_thread_data(...) {}
    ns.addBuiltinVariant("delete_thread_data", (q_func_n_t)f_delete_thread_data_VV, QCF_USES_EXTRA_ARGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // nothing delete_thread_data(list<softstring> l) {}
    ns.addBuiltinVariant("delete_thread_data", (q_func_n_t)f_delete_thread_data_C16list_softstring_, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 1, qore_get_complex_list_type(softStringTypeInfo), QORE_PARAM_NO_ARG, "l");
    // hash<string,list<hash<CallStackInfo>>> getAllThreadCallStacks() {}
    ns.addBuiltinVariant("getAllThreadCallStacks", (q_func_n_t)f_getAllThreadCallStacks, QCF_DEPRECATED, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, qore_get_complex_hash_type(qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false))));
    // hash<string,list<hash<CallStackInfo>>> get_all_thread_call_stacks() {}
    ns.addBuiltinVariant("get_all_thread_call_stacks", (q_func_n_t)f_get_all_thread_call_stacks, QCF_NO_FLAGS, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, qore_get_complex_hash_type(qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false))));
    // hash<auto> get_all_thread_data() {}
    ns.addBuiltinVariant("get_all_thread_data", (q_func_n_t)f_get_all_thread_data, QCF_RET_VALUE_ONLY, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, autoHashTypeInfo);
    // int get_default_thread_stack_size() {}
    ns.addBuiltinVariant("get_default_thread_stack_size", (q_func_n_t)f_get_default_thread_stack_size, QCF_CONSTANT, QDOM_THREAD_INFO, bigIntTypeInfo);
    // *hash<CallStackInfo> get_parent_caller_location(int offset = 1) {}
    ns.addBuiltinVariant("get_parent_caller_location", (q_func_n_t)f_get_parent_caller_location_Vi, QCF_NO_FLAGS, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, hashdeclCallStackInfo->getTypeInfo(true), 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)1), "offset");
    // int get_stack_size() {}
    ns.addBuiltinVariant("get_stack_size", (q_func_n_t)f_get_stack_size, QCF_CONSTANT, QDOM_THREAD_INFO, bigIntTypeInfo);
    // list<hash<CallStackInfo>> get_thread_call_stack() {}
    ns.addBuiltinVariant("get_thread_call_stack", (q_func_n_t)f_get_thread_call_stack, QCF_NO_FLAGS, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false)));
    // auto get_thread_data(string key) {}
    ns.addBuiltinVariant("get_thread_data", (q_func_n_t)f_get_thread_data_Vs, QCF_RET_VALUE_ONLY, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "key");
    // nothing get_thread_data() {}
    ns.addBuiltinVariant("get_thread_data", (q_func_n_t)f_get_thread_data, QCF_RUNTIME_NOOP, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, nothingTypeInfo);
    // *string get_thread_name() {}
    ns.addBuiltinVariant("get_thread_name", (q_func_n_t)f_get_thread_name, QCF_CONSTANT, QDOM_THREAD_CONTROL, stringOrNothingTypeInfo);
    // *TimeZone get_thread_tz() {}
    ns.addBuiltinVariant("get_thread_tz", (q_func_n_t)f_get_thread_tz, QCF_NO_FLAGS, QDOM_LOCALE_CONTROL, QC_TIMEZONE->getOrNothingTypeInfo());
    // int gettid() {}
    ns.addBuiltinVariant("gettid", (q_func_n_t)f_gettid, QCF_RET_VALUE_ONLY, QDOM_THREAD_INFO, bigIntTypeInfo);
    // nothing mark_thread_resources() {}
    ns.addBuiltinVariant("mark_thread_resources", (q_func_n_t)f_mark_thread_resources, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // int num_threads() {}
    ns.addBuiltinVariant("num_threads", (q_func_n_t)f_num_threads, QCF_RET_VALUE_ONLY, QDOM_THREAD_INFO, bigIntTypeInfo);
    // *hash<auto> remove_thread_data(...) {}
    ns.addBuiltinVariant("remove_thread_data", (q_func_n_t)f_remove_thread_data_VV, QCF_USES_EXTRA_ARGS, QDOM_THREAD_CONTROL, autoHashOrNothingTypeInfo);
    // hash<auto> remove_thread_data(list<softstring> l) {}
    ns.addBuiltinVariant("remove_thread_data", (q_func_n_t)f_remove_thread_data_C16list_softstring_, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, autoHashTypeInfo, 1, qore_get_complex_list_type(softStringTypeInfo), QORE_PARAM_NO_ARG, "l");
    // bool remove_thread_resource(AbstractThreadResource resource) {}
    ns.addBuiltinVariant("remove_thread_resource", (q_func_n_t)f_remove_thread_resource_C22AbstractThreadResource, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, QC_ABSTRACTTHREADRESOURCE->getTypeInfo(), QORE_PARAM_NO_ARG, "resource");
    // bool remove_thread_resource(code resource) {}
    ns.addBuiltinVariant("remove_thread_resource", (q_func_n_t)f_remove_thread_resource_Vc, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, codeTypeInfo, QORE_PARAM_NO_ARG, "resource");
    // nothing save_thread_data(hash<auto> h) {}
    ns.addBuiltinVariant("save_thread_data", (q_func_n_t)f_save_thread_data_C10hash_auto_, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");
    // nothing save_thread_data(string key, auto value) {}
    ns.addBuiltinVariant("save_thread_data", (q_func_n_t)f_save_thread_data_VsVA, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "key", autoTypeInfo, QORE_PARAM_NO_ARG, "value");
    // nothing save_thread_data() {}
    ns.addBuiltinVariant("save_thread_data", (q_func_n_t)f_save_thread_data, QCF_RUNTIME_NOOP, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // int set_default_thread_stack_size(int size) {}
    ns.addBuiltinVariant("set_default_thread_stack_size", (q_func_n_t)f_set_default_thread_stack_size_Vi, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, bigIntTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "size");
    // bool set_thread_init(*code init) {}
    ns.addBuiltinVariant("set_thread_init", (q_func_n_t)f_set_thread_init_Nc, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, codeOrNothingTypeInfo, QORE_PARAM_NO_ARG, "init");
    // nothing set_thread_name(string name) {}
    ns.addBuiltinVariant("set_thread_name", (q_func_n_t)f_set_thread_name_Vs, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");
    // nothing set_thread_resource(AbstractThreadResource resource) {}
    ns.addBuiltinVariant("set_thread_resource", (q_func_n_t)f_set_thread_resource_C22AbstractThreadResource, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 1, QC_ABSTRACTTHREADRESOURCE->getTypeInfo(), QORE_PARAM_NO_ARG, "resource");
    // nothing set_thread_resource(code resource, auto arg) {}
    ns.addBuiltinVariant("set_thread_resource", (q_func_n_t)f_set_thread_resource_VcVA, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 2, codeTypeInfo, QORE_PARAM_NO_ARG, "resource", autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // nothing set_thread_tz(TimeZone zone) {}
    ns.addBuiltinVariant("set_thread_tz", (q_func_n_t)f_set_thread_tz_C8TimeZone, QCF_NO_FLAGS, QDOM_LOCALE_CONTROL, nothingTypeInfo, 1, QC_TIMEZONE->getTypeInfo(), QORE_PARAM_NO_ARG, "zone");
    // nothing set_thread_tz() {}
    ns.addBuiltinVariant("set_thread_tz", (q_func_n_t)f_set_thread_tz, QCF_NO_FLAGS, QDOM_LOCALE_CONTROL, nothingTypeInfo);
    // list<int> thread_list() {}
    ns.addBuiltinVariant("thread_list", (q_func_n_t)f_thread_list, QCF_RET_VALUE_ONLY, QDOM_THREAD_INFO, qore_get_complex_list_type(bigIntTypeInfo));
    // nothing thread_yield() {}
    ns.addBuiltinVariant("thread_yield", (q_func_n_t)f_thread_yield, QCF_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo);
    // nothing throwThreadResourceExceptions() {}
    ns.addBuiltinVariant("throwThreadResourceExceptions", (q_func_n_t)f_throwThreadResourceExceptions, QCF_DEPRECATED, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // nothing throw_thread_resource_exceptions() {}
    ns.addBuiltinVariant("throw_thread_resource_exceptions", (q_func_n_t)f_throw_thread_resource_exceptions, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // bool throw_thread_resource_exceptions_to_mark() {}
    ns.addBuiltinVariant("throw_thread_resource_exceptions_to_mark", (q_func_n_t)f_throw_thread_resource_exceptions_to_mark, QCF_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo);

}
