/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_time.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_time.h"
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/qore_date_private.h"
#include "qore/intern/QoreHashNodeIntern.h"

#include <cstdio>
#include <ctime>
#include <sys/time.h>

static int qd_get_digit(unsigned& rv, const char*& p, unsigned factor) {
    if (*p < '0' || *p > '9') {
        throw QoreStandardException("INVALID-DATE", "invalid digit '%c' found in date/time input", *p);
    }

    rv += (*p - '0') * factor;
    ++p;
    return 0;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_2(const char *p) {
    unsigned rv = 0;
    if (qd_get_digit(rv, p, 10))
        return 0;
    if (qd_get_digit(rv, p, 1))
        return 0;
    return rv;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_3(const char *p) {
   unsigned rv = 0;
   if (qd_get_digit(rv, p, 100))
      return 0;
   if (qd_get_digit(rv, p, 10))
      return 0;
   if (qd_get_digit(rv, p, 1))
      return 0;
   return rv;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_4(const char *p) {
   unsigned rv = 0;
   if (qd_get_digit(rv, p, 1000))
      return 0;
   if (qd_get_digit(rv, p, 100))
      return 0;
   if (qd_get_digit(rv, p, 10))
      return 0;
   if (qd_get_digit(rv, p, 1))
      return 0;
   return rv;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_6(const char *p) {
   unsigned rv = 0;
   if (qd_get_digit(rv, p, 100000))
      return 0;
   if (qd_get_digit(rv, p, 10000))
      return 0;
   if (qd_get_digit(rv, p, 1000))
      return 0;
   if (qd_get_digit(rv, p, 100))
      return 0;
   if (qd_get_digit(rv, p, 10))
      return 0;
   if (qd_get_digit(rv, p, 1))
      return 0;
   return rv;
}

QoreHashNode* date_info(const DateTime& d) {
    qore_tm info;
    d.getInfo(info);

    QoreHashNode* h = new QoreHashNode(hashdeclDateTimeInfo, nullptr);
    qore_hash_private* ph = qore_hash_private::get(*h);
    ph->setKeyValueIntern("relative", d.isRelative());
    ph->setKeyValueIntern("year", info.year);
    ph->setKeyValueIntern("month", info.month);
    ph->setKeyValueIntern("day", info.day);
    ph->setKeyValueIntern("hour", info.hour);
    ph->setKeyValueIntern("minute", info.minute);
    ph->setKeyValueIntern("second", info.second);
    ph->setKeyValueIntern("microsecond", info.us);

    if (d.isAbsolute()) {
        ph->setKeyValueIntern("dow", d.getDayOfWeek());
        ph->setKeyValueIntern("doy", d.getDayNumber());
        ph->setKeyValueIntern("utc_secs_east", info.utc_secs_east);
        ph->setKeyValueIntern("dst", info.dst);
        ph->setKeyValueIntern("zone_name", new QoreStringNode(info.zone_name));
        ph->setKeyValueIntern("zone", new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(info.zone)));
    }

    return h;
}

DateTimeNode* make_date_with_mask(const AbstractQoreZoneInfo* tz, const QoreString& dtstr, const QoreString& mask, ExceptionSink* xsink) {
    qore_tm dt;
    dt.clear();
    dt.month = 1;
    dt.day = 1;

    const char* d = dtstr.c_str();
    const char* de = d + dtstr.strlen();

    const char* s = mask.c_str();
    const char* se = s + mask.strlen();

    try {
        while (*s) {
            // break if we run out of data in either the mask or the date string
            if (s >= se)
                break;
            switch (*s) {
                case 'Y':
                    if (s[1] != 'Y') {
                        throw QoreStandardException("INVALID-DATE", "'Y' has to be used as 'YY' or 'YYYY'");
                    }
                    if (s[2] == 'Y' && s[3] == 'Y') {
                        if ((de - d) < 4) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for year mask");
                        }

                        dt.year = parse_int_4(d);
                        s += 3;
                        d += 3;
                    } else {
                        if ((de - d) < 2) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for year mask");
                        }

                        ++s;
                        // obtain the current century
                        {
                            DateTime tmpdt(q_epoch());
                            dt.year = parse_int_2(d) + (tmpdt.getYear() / 100 * 100);
                        }
                        ++d;
                    }
                    break;

                case 'M':
                    if ((de - d) < 2) {
                        throw QoreStandardException("INVALID-DATE", "no more input to process for month mask");
                    }
                    if (s[1] == 'M') {
                        dt.month = parse_int_2(d);
                        s++;
                        d++;
                        break;
                    }
                    // 'M' is not supported because there is no clear way how to get eg. 1 or 11
                    if ((s[1] == 'o' || s[1] == 'O') && (s[2] == 'n' || s[2] == 'N')) {
                        QoreString str(d, 3);
                        dt.month = str.strlen() == 3 ? qore_date_info::getMonthIxFromAbbr(str.getBuffer()) : -1;
                        if (dt.month < 0 || dt.month > 11) {
                            throw QoreStandardException("INVALID-DATE", "Invalid 'Mon' string: '%s'", !str.empty() ? str.getBuffer() : "<none>");
                        }

                        ++dt.month;
                        s += 2;
                        d += 2;
                        break;
                    }
                    break;

                case 'm':
                    if (s[1] == 'm') {
                        if ((de - d) < 2) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for minute mask");
                        }
                        dt.minute = parse_int_2(d);
                        s++;
                        d++;
                        break;
                    }
                    if (s[1] == 's') {
                        if ((de - d) < 2) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for millisecond mask");
                        }
                        dt.us = parse_int_3(d) * 1000;
                        ++s;
                        d += 2;
                        break;
                    }
                    break;

                case 'D':
                case 'd':
                    if (s[1] == 'D' || s[1] == 'd') {
                        if ((de - d) < 2) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for day mask");
                        }
                        dt.day = parse_int_2(d);
                        s++;
                        d++;
                    }
                    break;

                case 'H':
                case 'h':
                    if (s[1] == 'H' || s[1] == 'h') {
                        if ((de - d) < 2) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for hour mask");
                        }
                        dt.hour = parse_int_2(d);
                        s++;
                        d++;
                    }
                    break;

                case 's':
                    if (s[1] == 's' && s[2] == 's') {
                        if (s[3] == 's' && s[4] == 's' && s[5] == 's') {
                            if ((de - d) < 6) {
                                throw QoreStandardException("INVALID-DATE", "no more input to process for second mask");
                            }
                            dt.us = parse_int_6(d);
                            s += 5;
                            d += 5;
                        } else {
                            if ((de - d) < 3) {
                                throw QoreStandardException("INVALID-DATE", "no more input to process for second mask");
                            }
                            dt.us = parse_int_3(d) * 1000;
                            s += 2;
                            d += 2;
                        }
                    }
                    break;

                case 'S':
                    if (s[1] == 'S') {
                        if ((de - d) < 2) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for second mask");
                        }
                        dt.second = parse_int_2(d);
                        s++;
                        d++;
                    }
                    break;

                case 'u':
                    if (s[1] == 's') {
                        if ((de - d) < 6) {
                            throw QoreStandardException("INVALID-DATE", "no more input to process for microsecond mask");
                        }
                        dt.us = parse_int_6(d);
                        ++s;
                        d += 5;
                        break;
                    }
            }
            ++s;
            ++d;
        }
    } catch (AbstractException& e) {
        e.convert(xsink);
    }

    if (*xsink)
        return nullptr;

    SimpleRefHolder<DateTimeNode> rv(DateTimeNode::makeAbsolute(tz, dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second, dt.us, xsink));
    return *xsink ? nullptr : rv.release();
}
TypedHashDecl* init_hashdecl_DateTimeInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("DateTimeInfo", "::Qore::DateTimeInfo");
    hd->addMember("day", bigIntTypeInfo, QoreValue());
    hd->addMember("dow", bigIntTypeInfo, QoreValue());
    hd->addMember("doy", bigIntTypeInfo, QoreValue());
    hd->addMember("dst", boolTypeInfo, QoreValue());
    hd->addMember("hour", bigIntTypeInfo, QoreValue());
    hd->addMember("microsecond", bigIntTypeInfo, QoreValue());
    hd->addMember("minute", bigIntTypeInfo, QoreValue());
    hd->addMember("month", bigIntTypeInfo, QoreValue());
    hd->addMember("relative", boolTypeInfo, QoreValue());
    hd->addMember("second", bigIntTypeInfo, QoreValue());
    hd->addMember("utc_secs_east", bigIntTypeInfo, QoreValue());
    hd->addMember("year", bigIntTypeInfo, QoreValue());
    hd->addMember("zone", QC_TIMEZONE->getTypeInfo(), QoreValue());
    hd->addMember("zone_name", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_IsoWeekInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("IsoWeekInfo", "::Qore::IsoWeekInfo");
    hd->addMember("day", bigIntTypeInfo, QoreValue());
    hd->addMember("week", bigIntTypeInfo, QoreValue());
    hd->addMember("year", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 351 "ql_time.qpp"
# 427 "ql_time.qpp"
# 450 "ql_time.qpp"
# 470 "ql_time.qpp"
# 487 "ql_time.qpp"
# 508 "ql_time.qpp"
# 514 "ql_time.qpp"
# 534 "ql_time.qpp"
# 551 "ql_time.qpp"
# 569 "ql_time.qpp"
# 586 "ql_time.qpp"
# 603 "ql_time.qpp"
# 621 "ql_time.qpp"
# 638 "ql_time.qpp"
# 655 "ql_time.qpp"
# 661 "ql_time.qpp"
# 680 "ql_time.qpp"
# 687 "ql_time.qpp"
# 705 "ql_time.qpp"
# 723 "ql_time.qpp"
# 742 "ql_time.qpp"
# 748 "ql_time.qpp"
# 772 "ql_time.qpp"
# 778 "ql_time.qpp"
# 802 "ql_time.qpp"
# 808 "ql_time.qpp"
# 832 "ql_time.qpp"
# 838 "ql_time.qpp"
# 862 "ql_time.qpp"
# 868 "ql_time.qpp"
# 892 "ql_time.qpp"
# 898 "ql_time.qpp"
# 922 "ql_time.qpp"
# 928 "ql_time.qpp"
# 959 "ql_time.qpp"
# 965 "ql_time.qpp"
# 996 "ql_time.qpp"
# 1013 "ql_time.qpp"
# 1019 "ql_time.qpp"
# 1036 "ql_time.qpp"
# 1042 "ql_time.qpp"
# 1059 "ql_time.qpp"
# 1065 "ql_time.qpp"
# 1082 "ql_time.qpp"
# 1088 "ql_time.qpp"
# 1105 "ql_time.qpp"
# 1111 "ql_time.qpp"
# 1130 "ql_time.qpp"
# 1136 "ql_time.qpp"
# 1155 "ql_time.qpp"
# 1161 "ql_time.qpp"
# 1180 "ql_time.qpp"
# 1199 "ql_time.qpp"
# 1205 "ql_time.qpp"
# 1217 "ql_time.qpp"
# 1223 "ql_time.qpp"
# 1242 "ql_time.qpp"
# 1256 "ql_time.qpp"
# 1262 "ql_time.qpp"
# 1283 "ql_time.qpp"
# 1298 "ql_time.qpp"
# 1304 "ql_time.qpp"
# 1326 "ql_time.qpp"
# 1350 "ql_time.qpp"
# 1356 "ql_time.qpp"
# 1387 "ql_time.qpp"
# 1410 "ql_time.qpp"
# 1416 "ql_time.qpp"
# 1441 "ql_time.qpp"
# 1459 "ql_time.qpp"
# 1482 "ql_time.qpp"
# 1499 "ql_time.qpp"
# 1516 "ql_time.qpp"
# 1533 "ql_time.qpp"
# 1549 "ql_time.qpp"
# 1555 "ql_time.qpp"
# 1571 "ql_time.qpp"
# 1597 "ql_time.qpp"
# 1622 "ql_time.qpp"
# 1646 "ql_time.qpp"
# 1673 "ql_time.qpp"
# 1690 "ql_time.qpp"
# 1707 "ql_time.qpp"
# 1724 "ql_time.qpp"
# 1731 "ql_time.qpp"
# 1748 "ql_time.qpp"
# 1755 "ql_time.qpp"
# 1766 "ql_time.qpp"
# 1794 "ql_time.qpp"
# 1811 "ql_time.qpp"
# 1828 "ql_time.qpp"
# 1837 "ql_time.qpp"
# 1846 "ql_time.qpp"
// int clock_getmicros() {}
static QoreValue f_clock_getmicros(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1529 "ql_time.qpp"
    return q_clock_getmicros();
}

// int clock_getmillis() {}
static QoreValue f_clock_getmillis(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1495 "ql_time.qpp"
    return q_clock_getmillis();
}

// int clock_getnanos() {}
static QoreValue f_clock_getnanos(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1512 "ql_time.qpp"
    return q_clock_getnanos();
}

// date date(date dt) {}
static QoreValue f_date_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1762 "ql_time.qpp"
    return dt->refSelf();
}

// date date(string dtstr) {}
static QoreValue f_date_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* dtstr = HARD_QORE_VALUE_STRING(args, 0);
# 1789 "ql_time.qpp"
    DateTimeNodeValueHelper date(dtstr, xsink);
    return *xsink ? 0 : date.getReferencedValue();
}

// date date(softfloat f) {}
static QoreValue f_date_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 1807 "ql_time.qpp"
    return DateTimeNode::makeRelativeFromSeconds((int64)f, (int)((f - (double)((int)f)) * 1000000));
}

// date date(int i) {}
static QoreValue f_date_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 1824 "ql_time.qpp"
    return DateTimeNode::makeRelativeFromSeconds(i);
}

// date date() {}
static QoreValue f_date(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1833 "ql_time.qpp"
    return zero_date();
}

// date date(null null) {}
static QoreValue f_date_Vu(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1842 "ql_time.qpp"
    return zero_date();
}

// date date(string dtstr, string mask) {}
static QoreValue f_date_VsVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* dtstr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* mask = HARD_QORE_VALUE_STRING(args, 1);
# 1862 "ql_time.qpp"
    return make_date_with_mask(currentTZ(), *dtstr, *mask, xsink);
}

// hash<DateTimeInfo> date_info(date dt) {}
static QoreValue f_date_info_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1686 "ql_time.qpp"
    return date_info(*dt);
}

// hash<DateTimeInfo> date_info() {}
static QoreValue f_date_info(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1699 "ql_time.qpp"
    int us;
    int64 seconds = q_epoch_us(us);
    DateTime d;
    d.setDate(currentTZ(), seconds, us);
    return date_info(d);
}

// date date_ms(softint ms) {}
static QoreValue f_date_ms_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
# 1545 "ql_time.qpp"
    return new DateTimeNode(ms / 1000, (int)(ms % 1000));
}

// nothing date_ms() {}
static QoreValue f_date_ms(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1552 "ql_time.qpp"

    return QoreValue();
}

// date date_us(softint us) {}
static QoreValue f_date_us_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 us = HARD_QORE_VALUE_INT(args, 0);
# 1567 "ql_time.qpp"
    return DateTimeNode::makeAbsoluteLocal(currentTZ(), us / 1000000, (int)(us % 1000000));
}

// date days(softint days) {}
static QoreValue f_days_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 days = HARD_QORE_VALUE_INT(args, 0);
# 828 "ql_time.qpp"
    return new DateTimeNode(0, 0, (int)days, 0, 0, 0, 0, true);
}

// nothing days() {}
static QoreValue f_days(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 835 "ql_time.qpp"

    return QoreValue();
}

// string format_date(string format, date dt) {}
static QoreValue f_format_date_VsVd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* format = HARD_QORE_VALUE_STRING(args, 0);
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 1);
# 502 "ql_time.qpp"
   QoreStringNode *rv = new QoreStringNode;
   dt->format(*rv, format->getBuffer());
   return rv;
}

// nothing format_date() {}
static QoreValue f_format_date(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 511 "ql_time.qpp"

    return QoreValue();
}

// date getDateFromISOWeek(softint year, softint week, softint day = 1) {}
static QoreValue f_getDateFromISOWeek_vivivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 year = HARD_QORE_VALUE_INT(args, 0);
    int64 week = HARD_QORE_VALUE_INT(args, 1);
    int64 day = HARD_QORE_VALUE_INT(args, 2);
# 1455 "ql_time.qpp"
    return DateTimeNode::getDateFromISOWeek(year, week, day, xsink);
}

// int getDayNumber(date dt) {}
static QoreValue f_getDayNumber_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1213 "ql_time.qpp"
    return dt->getDayNumber();
}

// nothing getDayNumber() {}
static QoreValue f_getDayNumber(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1220 "ql_time.qpp"

    return QoreValue();
}

// int getDayOfWeek(date dt) {}
static QoreValue f_getDayOfWeek_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1252 "ql_time.qpp"
    return dt->getDayOfWeek();
}

// nothing getDayOfWeek() {}
static QoreValue f_getDayOfWeek(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1259 "ql_time.qpp"

    return QoreValue();
}

// int getISODayOfWeek(date dt) {}
static QoreValue f_getISODayOfWeek_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1293 "ql_time.qpp"
    int d = dt->getDayOfWeek();
    return (!d ? 7 : d);
}

// nothing getISODayOfWeek() {}
static QoreValue f_getISODayOfWeek(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1301 "ql_time.qpp"

    return QoreValue();
}

// hash<IsoWeekInfo> getISOWeekHash(date dt) {}
static QoreValue f_getISOWeekHash_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1337 "ql_time.qpp"
    int year, week, day;
    dt->getISOWeek(year, week, day);

    QoreHashNode* h = new QoreHashNode(hashdeclIsoWeekInfo, xsink);
    qore_hash_private* ph = qore_hash_private::get(*h);
    ph->setKeyValueIntern("year", year);
    ph->setKeyValueIntern("week", week);
    ph->setKeyValueIntern("day", day);

    return h;
}

// nothing getISOWeekHash() {}
static QoreValue f_getISOWeekHash(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1353 "ql_time.qpp"

    return QoreValue();
}

// string getISOWeekString(date dt) {}
static QoreValue f_getISOWeekString_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1402 "ql_time.qpp"
    int year, week, day;
    dt->getISOWeek(year, week, day);
    QoreStringNode *str = new QoreStringNode();
    str->sprintf("%04d-W%02d-%d", year, week, day);
    return str;
}

// nothing getISOWeekString() {}
static QoreValue f_getISOWeekString(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1413 "ql_time.qpp"

    return QoreValue();
}

// date get_date_from_iso_week(softint year, softint week, softint day = 1) {}
static QoreValue f_get_date_from_iso_week_vivivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 year = HARD_QORE_VALUE_INT(args, 0);
    int64 week = HARD_QORE_VALUE_INT(args, 1);
    int64 day = HARD_QORE_VALUE_INT(args, 2);
# 1478 "ql_time.qpp"
    return DateTimeNode::getDateFromISOWeek(year, week, day, xsink);
}

// int get_day_number(date dt) {}
static QoreValue f_get_day_number_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1238 "ql_time.qpp"
    return dt->getDayNumber();
}

// int get_day_of_week(date dt) {}
static QoreValue f_get_day_of_week_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1279 "ql_time.qpp"
    return dt->getDayOfWeek();
}

// int get_days(date dt) {}
static QoreValue f_get_days_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1055 "ql_time.qpp"
    return dt->getDay();
}

// nothing get_days() {}
static QoreValue f_get_days(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1062 "ql_time.qpp"

    return QoreValue();
}

// int get_duration_microseconds(date dt) {}
static QoreValue f_get_duration_microseconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1642 "ql_time.qpp"
    return dt->getRelativeMicroseconds();
}

// int get_duration_milliseconds(date dt) {}
static QoreValue f_get_duration_milliseconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1618 "ql_time.qpp"
    return dt->getRelativeMilliseconds();
}

// int get_duration_seconds(date dt) {}
static QoreValue f_get_duration_seconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1593 "ql_time.qpp"
    return dt->getRelativeSeconds();
}

// float get_duration_seconds_f(date dt) {}
static QoreValue f_get_duration_seconds_f_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1669 "ql_time.qpp"
    return dt->getRelativeSecondsDouble();
}

// int get_epoch_microseconds(date dt) {}
static QoreValue f_get_epoch_microseconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 719 "ql_time.qpp"
    return dt->getEpochMicrosecondsUTC();
}

// int get_epoch_milliseconds(date dt) {}
static QoreValue f_get_epoch_milliseconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 701 "ql_time.qpp"
    return dt->getEpochMillisecondsUTC();
}

// int get_epoch_seconds(date dt) {}
static QoreValue f_get_epoch_seconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 676 "ql_time.qpp"
    return dt->getEpochSecondsUTC();
}

// nothing get_epoch_seconds() {}
static QoreValue f_get_epoch_seconds(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 684 "ql_time.qpp"

    return QoreValue();
}

// int get_hours(date dt) {}
static QoreValue f_get_hours_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1078 "ql_time.qpp"
    return dt->getHour();
}

// nothing get_hours() {}
static QoreValue f_get_hours(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1085 "ql_time.qpp"

    return QoreValue();
}

// int get_iso_day_of_week(date dt) {}
static QoreValue f_get_iso_day_of_week_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1321 "ql_time.qpp"
    int d = dt->getDayOfWeek();
    return (!d ? 7 : d);
}

// hash<IsoWeekInfo> get_iso_week_hash(date dt) {}
static QoreValue f_get_iso_week_hash_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1374 "ql_time.qpp"
    int year, week, day;
    dt->getISOWeek(year, week, day);

    QoreHashNode* h = new QoreHashNode(hashdeclIsoWeekInfo, xsink);
    qore_hash_private* ph = qore_hash_private::get(*h);
    ph->setKeyValueIntern("year", year);
    ph->setKeyValueIntern("week", week);
    ph->setKeyValueIntern("day", day);

    return h;
}

// string get_iso_week_string(date dt) {}
static QoreValue f_get_iso_week_string_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1433 "ql_time.qpp"
    int year, week, day;
    dt->getISOWeek(year, week, day);
    QoreStringNode *str = new QoreStringNode();
    str->sprintf("%04d-W%02d-%d", year, week, day);
    return str;
}

// int get_microseconds(date dt) {}
static QoreValue f_get_microseconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1176 "ql_time.qpp"
    return dt->getMicrosecond();
}

// date get_midnight(date dt) {}
static QoreValue f_get_midnight_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1193 "ql_time.qpp"
    DateTimeNode* rv = new DateTimeNode(*dt);
    rv->setTime(0, 0, 0);
    return rv;
}

// nothing get_midnight() {}
static QoreValue f_get_midnight(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1202 "ql_time.qpp"

    return QoreValue();
}

// int get_milliseconds(date dt) {}
static QoreValue f_get_milliseconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1151 "ql_time.qpp"
    return dt->getMillisecond();
}

// nothing get_milliseconds() {}
static QoreValue f_get_milliseconds(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1158 "ql_time.qpp"

    return QoreValue();
}

// int get_minutes(date dt) {}
static QoreValue f_get_minutes_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1101 "ql_time.qpp"
    return dt->getMinute();
}

// nothing get_minutes() {}
static QoreValue f_get_minutes(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1108 "ql_time.qpp"

    return QoreValue();
}

// int get_months(date dt) {}
static QoreValue f_get_months_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1032 "ql_time.qpp"
    return dt->getMonth();
}

// nothing get_months() {}
static QoreValue f_get_months(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1039 "ql_time.qpp"

    return QoreValue();
}

// int get_seconds(date dt) {}
static QoreValue f_get_seconds_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1126 "ql_time.qpp"
    return dt->getSecond();
}

// nothing get_seconds() {}
static QoreValue f_get_seconds(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1133 "ql_time.qpp"

    return QoreValue();
}

// int get_years(date dt) {}
static QoreValue f_get_years_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1009 "ql_time.qpp"
    return dt->getYear();
}

// nothing get_years() {}
static QoreValue f_get_years(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1016 "ql_time.qpp"

    return QoreValue();
}

// date gmtime() {}
static QoreValue f_gmtime(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 599 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(NULL, (int64)time(0), 0);
}

// date gmtime(softint secs, softint us = 0) {}
static QoreValue f_gmtime_vivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 secs = HARD_QORE_VALUE_INT(args, 0);
    int64 us = HARD_QORE_VALUE_INT(args, 1);
# 617 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(NULL, secs, (int)us);
}

// date gmtime(date dt) {}
static QoreValue f_gmtime_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 634 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(NULL, dt->getEpochSecondsUTC(), dt->getMicrosecond());
}

// date hours(softint hours) {}
static QoreValue f_hours_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 hours = HARD_QORE_VALUE_INT(args, 0);
# 858 "ql_time.qpp"
    return new DateTimeNode(0, 0, 0, (int)hours, 0, 0, 0, true);
}

// nothing hours() {}
static QoreValue f_hours(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 865 "ql_time.qpp"

    return QoreValue();
}

// bool is_date_absolute(date dt) {}
static QoreValue f_is_date_absolute_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1744 "ql_time.qpp"
    return dt->isAbsolute();
}

// bool is_date_absolute() {}
static QoreValue f_is_date_absolute(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1751 "ql_time.qpp"
   return false;
}

// bool is_date_relative(date dt) {}
static QoreValue f_is_date_relative_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1720 "ql_time.qpp"
    return dt->isRelative();
}

// bool is_date_relative() {}
static QoreValue f_is_date_relative(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1727 "ql_time.qpp"
    return false;
}

// date localtime() {}
static QoreValue f_localtime(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 547 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), (int64)time(0), 0);
}

// date localtime(softint secs, softint us = 0) {}
static QoreValue f_localtime_vivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 secs = HARD_QORE_VALUE_INT(args, 0);
    int64 us = HARD_QORE_VALUE_INT(args, 1);
# 565 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), secs, (int)us);
}

// date localtime(date dt) {}
static QoreValue f_localtime_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 582 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), dt->getEpochSecondsUTC(), dt->getMicrosecond());
}

// date microseconds(softint us) {}
static QoreValue f_microseconds_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 us = HARD_QORE_VALUE_INT(args, 0);
# 985 "ql_time.qpp"
    int64 secs = us / 10000000;
    us -= (secs * 10000000);
    int64 minutes = secs / 60;
    secs -= (minutes * 60);
    int64 hours = minutes / 60;
    minutes -= (hours * 60);

    return DateTimeNode::makeRelative(0, 0, 0, hours, minutes, secs, (int)us);
}

// date milliseconds(softint ms) {}
static QoreValue f_milliseconds_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
# 948 "ql_time.qpp"
    int64 secs = ms / 1000;
    ms -= (secs * 1000);
    int64 minutes = secs / 60;
    secs -= (minutes * 60);
    int64 hours = minutes / 60;
    minutes -= (hours * 60);

    return new DateTimeNode(0, 0, 0, (int)hours, (int)minutes, (int)secs, (int)ms, true);
}

// nothing milliseconds() {}
static QoreValue f_milliseconds(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 962 "ql_time.qpp"

    return QoreValue();
}

// date minutes(softint minutes) {}
static QoreValue f_minutes_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 minutes = HARD_QORE_VALUE_INT(args, 0);
# 888 "ql_time.qpp"
    return new DateTimeNode(0, 0, 0, 0, (int)minutes, 0, 0, true);
}

// nothing minutes() {}
static QoreValue f_minutes(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 895 "ql_time.qpp"

    return QoreValue();
}

// int mktime(date dt) {}
static QoreValue f_mktime_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 738 "ql_time.qpp"
    return dt->getEpochSecondsUTC();
}

// nothing mktime() {}
static QoreValue f_mktime(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 745 "ql_time.qpp"

    return QoreValue();
}

// date months(softint months) {}
static QoreValue f_months_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 months = HARD_QORE_VALUE_INT(args, 0);
# 798 "ql_time.qpp"
    return new DateTimeNode(0, (int)months, 0, 0, 0, 0, 0, true);
}

// nothing months() {}
static QoreValue f_months(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 805 "ql_time.qpp"

    return QoreValue();
}

// date now() {}
static QoreValue f_now(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 530 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), (int64)time(0), 0);
}

// date now_ms() {}
static QoreValue f_now_ms(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 443 "ql_time.qpp"
   int us;
   int64 seconds = q_epoch_us(us);

   return DateTimeNode::makeAbsolute(currentTZ(), seconds, (us / 1000) * 1000);
}

// date now_us() {}
static QoreValue f_now_us(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 466 "ql_time.qpp"
    return DateTimeNode::makeNow();
}

// date now_utc() {}
static QoreValue f_now_utc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 481 "ql_time.qpp"
   int us;
   int64 seconds = q_epoch_us(us);
   return DateTimeNode::makeAbsolute(NULL, seconds, us);
}

// date seconds(softint seconds) {}
static QoreValue f_seconds_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds = HARD_QORE_VALUE_INT(args, 0);
# 918 "ql_time.qpp"
    return new DateTimeNode(0, 0, 0, 0, 0, (int)seconds, 0, true);
}

// nothing seconds() {}
static QoreValue f_seconds(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 925 "ql_time.qpp"

    return QoreValue();
}

// int timegm(date dt) {}
static QoreValue f_timegm_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 651 "ql_time.qpp"
   return dt->getEpochSeconds();
}

// nothing timegm() {}
static QoreValue f_timegm(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 658 "ql_time.qpp"

    return QoreValue();
}

// date years(softint years) {}
static QoreValue f_years_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 years = HARD_QORE_VALUE_INT(args, 0);
# 768 "ql_time.qpp"
    return new DateTimeNode((int)years, 0, 0, 0, 0, 0, 0, true);
}

// nothing years() {}
static QoreValue f_years(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 775 "ql_time.qpp"

    return QoreValue();
}


DLLLOCAL void init_time_functions(QoreNamespace& ns) {
    // int clock_getmicros() {}
    ns.addBuiltinVariant("clock_getmicros", (q_func_n_t)f_clock_getmicros, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // int clock_getmillis() {}
    ns.addBuiltinVariant("clock_getmillis", (q_func_n_t)f_clock_getmillis, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // int clock_getnanos() {}
    ns.addBuiltinVariant("clock_getnanos", (q_func_n_t)f_clock_getnanos, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // date date(date dt) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vd, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // date date(string dtstr) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vs, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "dtstr");
    // date date(softfloat f) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_vf, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // date date(int i) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");
    // date date() {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date date(null null) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vu, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, nullTypeInfo, QORE_PARAM_NO_ARG, "null");
    // date date(string dtstr, string mask) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_VsVs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, dateTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "dtstr", stringTypeInfo, QORE_PARAM_NO_ARG, "mask");
    // hash<DateTimeInfo> date_info(date dt) {}
    ns.addBuiltinVariant("date_info", (q_func_n_t)f_date_info_Vd, QCF_CONSTANT, QDOM_DEFAULT, hashdeclDateTimeInfo->getTypeInfo(false), 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // hash<DateTimeInfo> date_info() {}
    ns.addBuiltinVariant("date_info", (q_func_n_t)f_date_info, QCF_CONSTANT, QDOM_DEFAULT, hashdeclDateTimeInfo->getTypeInfo(false));
    // date date_ms(softint ms) {}
    ns.addBuiltinVariant("date_ms", (q_func_n_t)f_date_ms_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "ms");
    // nothing date_ms() {}
    ns.addBuiltinVariant("date_ms", (q_func_n_t)f_date_ms, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date date_us(softint us) {}
    ns.addBuiltinVariant("date_us", (q_func_n_t)f_date_us_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "us");
    // date days(softint days) {}
    ns.addBuiltinVariant("days", (q_func_n_t)f_days_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "days");
    // nothing days() {}
    ns.addBuiltinVariant("days", (q_func_n_t)f_days, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string format_date(string format, date dt) {}
    ns.addBuiltinVariant("format_date", (q_func_n_t)f_format_date_VsVd, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "format", dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing format_date() {}
    ns.addBuiltinVariant("format_date", (q_func_n_t)f_format_date, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date getDateFromISOWeek(softint year, softint week, softint day = 1) {}
    ns.addBuiltinVariant("getDateFromISOWeek", (q_func_n_t)f_getDateFromISOWeek_vivivi, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, dateTypeInfo, 3, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "year", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "week", softBigIntTypeInfo, QoreSimpleValue().assign((int64)1), "day");
    // int getDayNumber(date dt) {}
    ns.addBuiltinVariant("getDayNumber", (q_func_n_t)f_getDayNumber_Vd, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing getDayNumber() {}
    ns.addBuiltinVariant("getDayNumber", (q_func_n_t)f_getDayNumber, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // int getDayOfWeek(date dt) {}
    ns.addBuiltinVariant("getDayOfWeek", (q_func_n_t)f_getDayOfWeek_Vd, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing getDayOfWeek() {}
    ns.addBuiltinVariant("getDayOfWeek", (q_func_n_t)f_getDayOfWeek, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // int getISODayOfWeek(date dt) {}
    ns.addBuiltinVariant("getISODayOfWeek", (q_func_n_t)f_getISODayOfWeek_Vd, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing getISODayOfWeek() {}
    ns.addBuiltinVariant("getISODayOfWeek", (q_func_n_t)f_getISODayOfWeek, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // hash<IsoWeekInfo> getISOWeekHash(date dt) {}
    ns.addBuiltinVariant("getISOWeekHash", (q_func_n_t)f_getISOWeekHash_Vd, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, hashdeclIsoWeekInfo->getTypeInfo(false), 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing getISOWeekHash() {}
    ns.addBuiltinVariant("getISOWeekHash", (q_func_n_t)f_getISOWeekHash, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string getISOWeekString(date dt) {}
    ns.addBuiltinVariant("getISOWeekString", (q_func_n_t)f_getISOWeekString_Vd, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing getISOWeekString() {}
    ns.addBuiltinVariant("getISOWeekString", (q_func_n_t)f_getISOWeekString, QCF_RUNTIME_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // date get_date_from_iso_week(softint year, softint week, softint day = 1) {}
    ns.addBuiltinVariant("get_date_from_iso_week", (q_func_n_t)f_get_date_from_iso_week_vivivi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, dateTypeInfo, 3, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "year", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "week", softBigIntTypeInfo, QoreSimpleValue().assign((int64)1), "day");
    // int get_day_number(date dt) {}
    ns.addBuiltinVariant("get_day_number", (q_func_n_t)f_get_day_number_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_day_of_week(date dt) {}
    ns.addBuiltinVariant("get_day_of_week", (q_func_n_t)f_get_day_of_week_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_days(date dt) {}
    ns.addBuiltinVariant("get_days", (q_func_n_t)f_get_days_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_days() {}
    ns.addBuiltinVariant("get_days", (q_func_n_t)f_get_days, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_duration_microseconds(date dt) {}
    ns.addBuiltinVariant("get_duration_microseconds", (q_func_n_t)f_get_duration_microseconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_duration_milliseconds(date dt) {}
    ns.addBuiltinVariant("get_duration_milliseconds", (q_func_n_t)f_get_duration_milliseconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_duration_seconds(date dt) {}
    ns.addBuiltinVariant("get_duration_seconds", (q_func_n_t)f_get_duration_seconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // float get_duration_seconds_f(date dt) {}
    ns.addBuiltinVariant("get_duration_seconds_f", (q_func_n_t)f_get_duration_seconds_f_Vd, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_epoch_microseconds(date dt) {}
    ns.addBuiltinVariant("get_epoch_microseconds", (q_func_n_t)f_get_epoch_microseconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_epoch_milliseconds(date dt) {}
    ns.addBuiltinVariant("get_epoch_milliseconds", (q_func_n_t)f_get_epoch_milliseconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_epoch_seconds(date dt) {}
    ns.addBuiltinVariant("get_epoch_seconds", (q_func_n_t)f_get_epoch_seconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_epoch_seconds() {}
    ns.addBuiltinVariant("get_epoch_seconds", (q_func_n_t)f_get_epoch_seconds, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_hours(date dt) {}
    ns.addBuiltinVariant("get_hours", (q_func_n_t)f_get_hours_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_hours() {}
    ns.addBuiltinVariant("get_hours", (q_func_n_t)f_get_hours, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_iso_day_of_week(date dt) {}
    ns.addBuiltinVariant("get_iso_day_of_week", (q_func_n_t)f_get_iso_day_of_week_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // hash<IsoWeekInfo> get_iso_week_hash(date dt) {}
    ns.addBuiltinVariant("get_iso_week_hash", (q_func_n_t)f_get_iso_week_hash_Vd, QCF_CONSTANT, QDOM_DEFAULT, hashdeclIsoWeekInfo->getTypeInfo(false), 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // string get_iso_week_string(date dt) {}
    ns.addBuiltinVariant("get_iso_week_string", (q_func_n_t)f_get_iso_week_string_Vd, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // int get_microseconds(date dt) {}
    ns.addBuiltinVariant("get_microseconds", (q_func_n_t)f_get_microseconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // date get_midnight(date dt) {}
    ns.addBuiltinVariant("get_midnight", (q_func_n_t)f_get_midnight_Vd, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_midnight() {}
    ns.addBuiltinVariant("get_midnight", (q_func_n_t)f_get_midnight, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_milliseconds(date dt) {}
    ns.addBuiltinVariant("get_milliseconds", (q_func_n_t)f_get_milliseconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_milliseconds() {}
    ns.addBuiltinVariant("get_milliseconds", (q_func_n_t)f_get_milliseconds, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_minutes(date dt) {}
    ns.addBuiltinVariant("get_minutes", (q_func_n_t)f_get_minutes_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_minutes() {}
    ns.addBuiltinVariant("get_minutes", (q_func_n_t)f_get_minutes, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_months(date dt) {}
    ns.addBuiltinVariant("get_months", (q_func_n_t)f_get_months_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_months() {}
    ns.addBuiltinVariant("get_months", (q_func_n_t)f_get_months, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_seconds(date dt) {}
    ns.addBuiltinVariant("get_seconds", (q_func_n_t)f_get_seconds_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_seconds() {}
    ns.addBuiltinVariant("get_seconds", (q_func_n_t)f_get_seconds, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_years(date dt) {}
    ns.addBuiltinVariant("get_years", (q_func_n_t)f_get_years_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing get_years() {}
    ns.addBuiltinVariant("get_years", (q_func_n_t)f_get_years, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date gmtime() {}
    ns.addBuiltinVariant("gmtime", (q_func_n_t)f_gmtime, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date gmtime(softint secs, softint us = 0) {}
    ns.addBuiltinVariant("gmtime", (q_func_n_t)f_gmtime_vivi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "secs", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "us");
    // date gmtime(date dt) {}
    ns.addBuiltinVariant("gmtime", (q_func_n_t)f_gmtime_Vd, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // date hours(softint hours) {}
    ns.addBuiltinVariant("hours", (q_func_n_t)f_hours_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "hours");
    // nothing hours() {}
    ns.addBuiltinVariant("hours", (q_func_n_t)f_hours, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // bool is_date_absolute(date dt) {}
    ns.addBuiltinVariant("is_date_absolute", (q_func_n_t)f_is_date_absolute_Vd, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // bool is_date_absolute() {}
    ns.addBuiltinVariant("is_date_absolute", (q_func_n_t)f_is_date_absolute, QCF_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // bool is_date_relative(date dt) {}
    ns.addBuiltinVariant("is_date_relative", (q_func_n_t)f_is_date_relative_Vd, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // bool is_date_relative() {}
    ns.addBuiltinVariant("is_date_relative", (q_func_n_t)f_is_date_relative, QCF_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // date localtime() {}
    ns.addBuiltinVariant("localtime", (q_func_n_t)f_localtime, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date localtime(softint secs, softint us = 0) {}
    ns.addBuiltinVariant("localtime", (q_func_n_t)f_localtime_vivi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "secs", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "us");
    // date localtime(date dt) {}
    ns.addBuiltinVariant("localtime", (q_func_n_t)f_localtime_Vd, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // date microseconds(softint us) {}
    ns.addBuiltinVariant("microseconds", (q_func_n_t)f_microseconds_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "us");
    // date milliseconds(softint ms) {}
    ns.addBuiltinVariant("milliseconds", (q_func_n_t)f_milliseconds_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "ms");
    // nothing milliseconds() {}
    ns.addBuiltinVariant("milliseconds", (q_func_n_t)f_milliseconds, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date minutes(softint minutes) {}
    ns.addBuiltinVariant("minutes", (q_func_n_t)f_minutes_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "minutes");
    // nothing minutes() {}
    ns.addBuiltinVariant("minutes", (q_func_n_t)f_minutes, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int mktime(date dt) {}
    ns.addBuiltinVariant("mktime", (q_func_n_t)f_mktime_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing mktime() {}
    ns.addBuiltinVariant("mktime", (q_func_n_t)f_mktime, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date months(softint months) {}
    ns.addBuiltinVariant("months", (q_func_n_t)f_months_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "months");
    // nothing months() {}
    ns.addBuiltinVariant("months", (q_func_n_t)f_months, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date now() {}
    ns.addBuiltinVariant("now", (q_func_n_t)f_now, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date now_ms() {}
    ns.addBuiltinVariant("now_ms", (q_func_n_t)f_now_ms, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date now_us() {}
    ns.addBuiltinVariant("now_us", (q_func_n_t)f_now_us, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date now_utc() {}
    ns.addBuiltinVariant("now_utc", (q_func_n_t)f_now_utc, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date seconds(softint seconds) {}
    ns.addBuiltinVariant("seconds", (q_func_n_t)f_seconds_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "seconds");
    // nothing seconds() {}
    ns.addBuiltinVariant("seconds", (q_func_n_t)f_seconds, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int timegm(date dt) {}
    ns.addBuiltinVariant("timegm", (q_func_n_t)f_timegm_Vd, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "dt");
    // nothing timegm() {}
    ns.addBuiltinVariant("timegm", (q_func_n_t)f_timegm, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date years(softint years) {}
    ns.addBuiltinVariant("years", (q_func_n_t)f_years_vi, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "years");
    // nothing years() {}
    ns.addBuiltinVariant("years", (q_func_n_t)f_years, QCF_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);

}
