/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_type.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_type.h"
#include "qore/intern/qore_date_private.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreHashNodeIntern.h"
#include "qore/intern/qore_list_private.h"

#include <memory>

static QoreValue auto_cast(const QoreValue v);

static QoreHashNode* auto_cast_hash(const QoreHashNode* h) {
    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), nullptr);

    qore_hash_private* hash_priv = qore_hash_private::get(**rv);

    const QoreTypeInfo* vtype = nullptr;
    bool vcommon = false;

    ConstHashIterator i(h);
    while (i.next()) {
        const char* key = i.getKey();
        const QoreValue val = i.get();
        hash_priv->setKeyValueIntern(key, auto_cast(val));

        const QoreTypeInfo* ctype = val.getTypeInfo();

        if (i.first()) {
            if (ctype && ctype != anyTypeInfo) {
                vtype = ctype;
                vcommon = true;
            }
        } else if (vcommon && !QoreTypeInfo::matchCommonType(vtype, ctype)) {
            vcommon = false;
        }
    }

    // issue #2791: when performing type folding, do not set to type "any" but rather use "auto"
    if (vtype && vtype != anyTypeInfo) {
        hash_priv->complexTypeInfo = qore_get_complex_hash_type(vtype);
    }

    return rv.release();
}

static QoreListNode* auto_cast_list(const QoreListNode* l) {
    ReferenceHolder<QoreListNode> rv(new QoreListNode(autoTypeInfo), nullptr);

    qore_list_private* list_priv = qore_list_private::get(**rv);

    const QoreTypeInfo* vtype = nullptr;
    bool vcommon = false;

    ConstListIterator i(l);
    while (i.next()) {
        const QoreValue val = i.getValue();
        list_priv->pushIntern(auto_cast(val));

        const QoreTypeInfo* ctype = val.getTypeInfo();

        if (!i.index()) {
            if (ctype && ctype != anyTypeInfo) {
                vtype = ctype;
                vcommon = true;
            }
        } else if (vcommon && !QoreTypeInfo::matchCommonType(vtype, ctype)) {
            vcommon = false;
        }
    }

    // issue #2791: when performing type folding, do not set to type "any" but rather use "auto"
    if (vtype && vtype != anyTypeInfo) {
        list_priv->complexTypeInfo = qore_get_complex_list_type(vtype);
    }

    return rv.release();
}

static QoreValue auto_cast(const QoreValue v) {
    switch (v.getType()) {
        case NT_HASH: {
            return auto_cast_hash(v.get<QoreHashNode>());
        }
        case NT_LIST: {
            return auto_cast_list(v.get<QoreListNode>());
        }
        default:
            return v.refSelf();
    }
}
# 173 "ql_type.qpp"
# 177 "ql_type.qpp"
# 194 "ql_type.qpp"
# 229 "ql_type.qpp"
# 244 "ql_type.qpp"
# 253 "ql_type.qpp"
# 268 "ql_type.qpp"
# 277 "ql_type.qpp"
# 292 "ql_type.qpp"
# 301 "ql_type.qpp"
# 324 "ql_type.qpp"
# 333 "ql_type.qpp"
# 344 "ql_type.qpp"
# 353 "ql_type.qpp"
# 369 "ql_type.qpp"
# 376 "ql_type.qpp"
# 400 "ql_type.qpp"
# 416 "ql_type.qpp"
# 441 "ql_type.qpp"
# 474 "ql_type.qpp"
# 491 "ql_type.qpp"
# 498 "ql_type.qpp"
# 519 "ql_type.qpp"
# 538 "ql_type.qpp"
# 558 "ql_type.qpp"
// auto auto_cast(auto val) {}
static QoreValue f_auto_cast_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 573 "ql_type.qpp"
    return auto_cast(val);
}

// binary binary() {}
static QoreValue f_binary(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 340 "ql_type.qpp"
   return new BinaryNode;
}

// binary binary(null x) {}
static QoreValue f_binary_Vu(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 349 "ql_type.qpp"
   return new BinaryNode;
}

// binary binary(softstring str) {}
static QoreValue f_binary_vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 363 "ql_type.qpp"
   BinaryNode *b = new BinaryNode;
   b->append(str->getBuffer(), str->strlen());
   return b;
}

// binary binary(binary bin) {}
static QoreValue f_binary_Vy(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 372 "ql_type.qpp"
   return bin->refSelf();
}

// string binary_to_string(binary b, *string encoding) {}
static QoreValue f_binary_to_string_VyNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 550 "ql_type.qpp"
    const QoreEncoding* qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
    if (!b->size()) {
        return new QoreStringNode(qe);
    }
    return new QoreStringNode((const char*)b->getPtr(), b->size(), qe);
}

// bool boolean(any arg) {}
static QoreValue f_boolean_Va(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 190 "ql_type.qpp"
   return arg.getAsBool();
}

// float float(softfloat f) {}
static QoreValue f_float_vf(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 264 "ql_type.qpp"
   return f;
}

// float float() {}
static QoreValue f_float(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 273 "ql_type.qpp"
   return 0.0;
}

// hash<auto> hash(object obj) {}
static QoreValue f_hash_Vo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 412 "ql_type.qpp"
   return qore_object_private::get(*obj)->getRuntimeMemberHash(xsink);
}

// hash<auto> hash(list<auto> l) {}
static QoreValue f_hash_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 427 "ql_type.qpp"
    ReferenceHolder<QoreHashNode> h(new QoreHashNode(autoTypeInfo), xsink);

    bool valid = true;
    ConstListIterator li(l);
    while (valid && ((valid = li.next()))) {
        QoreStringValueHelper str(li.getValue());
        h->setKeyValue(str->c_str(), ((valid = li.next())) ? li.getReferencedValue() : QoreValue(), xsink);
        if (*xsink)
            return QoreValue();
    }
    return h.release();
}

// hash<auto> hash(list<softstring> keys, list<auto> values) {}
static QoreValue f_hash_C16list_softstring_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* keys = HARD_QORE_VALUE_LIST(args, 0);
    const QoreListNode* values = HARD_QORE_VALUE_LIST(args, 1);
# 454 "ql_type.qpp"
    ReferenceHolder<QoreHashNode> h(new QoreHashNode(autoTypeInfo), xsink);

    ConstListIterator ki(keys);
    ConstListIterator vi(values);

    bool valid = true;
    while (ki.next()) {
        if (valid)
            valid = vi.next();

        QoreStringValueHelper str(ki.getValue());
        h->setKeyValue(str->c_str(), valid ? vi.getReferencedValue() : QoreValue(), xsink);
        if (*xsink)
            return QoreValue();
    }

    return h.release();
}

// hash hash(hash h) {}
static QoreValue f_hash_Vh(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 487 "ql_type.qpp"
    return h->hashRefSelf();
}

// hash<auto> hash() {}
static QoreValue f_hash(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 494 "ql_type.qpp"
    return new QoreHashNode(autoTypeInfo);
}

// int int(string str, int base = 10) {}
static QoreValue f_int_VsVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 base = HARD_QORE_VALUE_INT(args, 1);
# 223 "ql_type.qpp"
    if (base < 0 || base == 1 || base > 36)
        return xsink->raiseException("INVALID-BASE", "base " QLLD " is invalid; base must be 0 or 2 - 36 inclusive", base);
    return strtoll(str->getBuffer(), 0, (int)base);
}

// int int(softint i) {}
static QoreValue f_int_vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 240 "ql_type.qpp"
    return i;
}

// int int() {}
static QoreValue f_int(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 249 "ql_type.qpp"
   return 0;
}

// list<auto> list(...) {}
static QoreValue f_list_VV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 386 "ql_type.qpp"
    QoreListNode *l;
    if (num_args(args) > 1) {
        l = args->listRefSelf();
    } else {
        l = new QoreListNode(autoTypeInfo);
        QoreValue p0 = get_param_value(args, 0);
        if (!p0.isNothing()) {
            l->push(p0.refSelf(), xsink);
        }
    }
    return l;
}

// number number(softnumber n) {}
static QoreValue f_number_vn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = HARD_QORE_VALUE_NUMBER(args, 0);
# 288 "ql_type.qpp"
   return n->refSelf();
}

// number number() {}
static QoreValue f_number(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 297 "ql_type.qpp"
   return ZeroNumber->refSelf();
}

// string string(softstring str, *string enc) {}
static QoreValue f_string_vsNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* enc = get_param_value(args, 1).get<const QoreStringNode>();
# 315 "ql_type.qpp"
   if (enc) {
      const QoreEncoding* e = QEM.findCreate(enc->getBuffer());
      if (e != str->getEncoding())
         return str->convertEncoding(e, xsink);
   }
   return str->refSelf();
}

// string string() {}
static QoreValue f_string(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 329 "ql_type.qpp"
   return null_string();
}

// string type(auto arg) {}
static QoreValue f_type_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 515 "ql_type.qpp"
    return new QoreStringNode(arg.getTypeName());
}

// string typename(auto arg) {}
static QoreValue f_typename_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 534 "ql_type.qpp"
    return new QoreStringNode(arg.getTypeName());
}


DLLLOCAL void init_type_functions(QoreNamespace& ns) {
    // auto auto_cast(auto val) {}
    ns.addBuiltinVariant("auto_cast", (q_func_n_t)f_auto_cast_VA, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");
    // binary binary() {}
    ns.addBuiltinVariant("binary", (q_func_n_t)f_binary, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo);
    // binary binary(null x) {}
    ns.addBuiltinVariant("binary", (q_func_n_t)f_binary_Vu, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 1, nullTypeInfo, QORE_PARAM_NO_ARG, "x");
    // binary binary(softstring str) {}
    ns.addBuiltinVariant("binary", (q_func_n_t)f_binary_vs, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 1, softStringTypeInfo, QORE_PARAM_NO_ARG, "str");
    // binary binary(binary bin) {}
    ns.addBuiltinVariant("binary", (q_func_n_t)f_binary_Vy, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "bin");
    // string binary_to_string(binary b, *string encoding) {}
    ns.addBuiltinVariant("binary_to_string", (q_func_n_t)f_binary_to_string_VyNs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "b", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // bool boolean(any arg) {}
    ns.addBuiltinVariant("boolean", (q_func_n_t)f_boolean_Va, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, anyTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // float float(softfloat f) {}
    ns.addBuiltinVariant("float", (q_func_n_t)f_float_vf, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, softFloatTypeInfo, QORE_PARAM_NO_ARG, "f");
    // float float() {}
    ns.addBuiltinVariant("float", (q_func_n_t)f_float, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo);
    // hash<auto> hash(object obj) {}
    ns.addBuiltinVariant("hash", (q_func_n_t)f_hash_Vo, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");
    // hash<auto> hash(list<auto> l) {}
    ns.addBuiltinVariant("hash", (q_func_n_t)f_hash_C10list_auto_, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // hash<auto> hash(list<softstring> keys, list<auto> values) {}
    ns.addBuiltinVariant("hash", (q_func_n_t)f_hash_C16list_softstring_C10list_auto_, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo, 2, qore_get_complex_list_type(softStringTypeInfo), QORE_PARAM_NO_ARG, "keys", autoListTypeInfo, QORE_PARAM_NO_ARG, "values");
    // hash hash(hash h) {}
    ns.addBuiltinVariant("hash", (q_func_n_t)f_hash_Vh, QCF_CONSTANT, QDOM_DEFAULT, hashTypeInfo, 1, hashTypeInfo, QORE_PARAM_NO_ARG, "h");
    // hash<auto> hash() {}
    ns.addBuiltinVariant("hash", (q_func_n_t)f_hash, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);
    // int int(string str, int base = 10) {}
    ns.addBuiltinVariant("int", (q_func_n_t)f_int_VsVi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "str", bigIntTypeInfo, QoreSimpleValue().assign((int64)10), "base");
    // int int(softint i) {}
    ns.addBuiltinVariant("int", (q_func_n_t)f_int_vi, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "i");
    // int int() {}
    ns.addBuiltinVariant("int", (q_func_n_t)f_int, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // list<auto> list(...) {}
    ns.addBuiltinVariant("list", (q_func_n_t)f_list_VV, QCF_CONSTANT|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoListTypeInfo);
    // number number(softnumber n) {}
    ns.addBuiltinVariant("number", (q_func_n_t)f_number_vn, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo, 1, softNumberTypeInfo, QORE_PARAM_NO_ARG, "n");
    // number number() {}
    ns.addBuiltinVariant("number", (q_func_n_t)f_number, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo);
    // string string(softstring str, *string enc) {}
    ns.addBuiltinVariant("string", (q_func_n_t)f_string_vsNs, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, softStringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "enc");
    // string string() {}
    ns.addBuiltinVariant("string", (q_func_n_t)f_string, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);
    // string type(auto arg) {}
    ns.addBuiltinVariant("type", (q_func_n_t)f_type_VA, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // string typename(auto arg) {}
    ns.addBuiltinVariant("typename", (q_func_n_t)f_typename_VA, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");

}

DLLLOCAL void init_type_constants(QoreNamespace& ns) {
# 124 "ql_type.qpp"
# 130 "ql_type.qpp"
# 133 "ql_type.qpp"
# 136 "ql_type.qpp"
# 139 "ql_type.qpp"
# 142 "ql_type.qpp"
# 145 "ql_type.qpp"
# 148 "ql_type.qpp"
# 151 "ql_type.qpp"
# 154 "ql_type.qpp"
# 157 "ql_type.qpp"
# 160 "ql_type.qpp"
# 163 "ql_type.qpp"
# 166 "ql_type.qpp"
    ns.addConstant("Binary", new QoreStringNode("binary"));
    ns.addConstant("Boolean", new QoreStringNode("bool"));
    ns.addConstant("CallReference", new QoreStringNode("call reference"));
    ns.addConstant("Closure", new QoreStringNode("closure"));
    ns.addConstant("Date", new QoreStringNode("date"));
    ns.addConstant("Float", new QoreStringNode("float"));
    ns.addConstant("Hash", new QoreStringNode("hash"));
    ns.addConstant("Int", new QoreStringNode("integer"));
    ns.addConstant("List", new QoreStringNode("list"));
    ns.addConstant("NothingType", new QoreStringNode("nothing"));
    ns.addConstant("NullType", new QoreStringNode("NULL"));
    ns.addConstant("Number", new QoreStringNode("number"));
    ns.addConstant("Object", new QoreStringNode("object"));
    ns.addConstant("String", new QoreStringNode("string"));
# 173 "ql_type.qpp"
# 177 "ql_type.qpp"
# 194 "ql_type.qpp"
# 229 "ql_type.qpp"
# 244 "ql_type.qpp"
# 253 "ql_type.qpp"
# 268 "ql_type.qpp"
# 277 "ql_type.qpp"
# 292 "ql_type.qpp"
# 301 "ql_type.qpp"
# 324 "ql_type.qpp"
# 333 "ql_type.qpp"
# 344 "ql_type.qpp"
# 353 "ql_type.qpp"
# 369 "ql_type.qpp"
# 376 "ql_type.qpp"
# 400 "ql_type.qpp"
# 416 "ql_type.qpp"
# 441 "ql_type.qpp"
# 474 "ql_type.qpp"
# 491 "ql_type.qpp"
# 498 "ql_type.qpp"
# 519 "ql_type.qpp"
# 538 "ql_type.qpp"
# 558 "ql_type.qpp"

}
