/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AstParser.cpp AstParser class definition */
/*
  Qore AST Parser

  Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <memory>

#include "qore/Qore.h"

#include "AstParserHolder.h"
#include "AstTreeHolder.h"
#include "ast/ASTParseError.h"
#include "ast/ASTTree.h"

#include "QC_AstTree.h"
/* Qore class astparser::AstParser */

qore_classid_t CID_ASTPARSER;
QoreClass* QC_ASTPARSER;

// AstParser::constructor() {}
static void AstParser_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 49 "QC_AstParser.qpp"
self->setPrivate(CID_ASTPARSER, new AstParserHolder);
}

// *list AstParser::getDiagnostics(){}
static QoreValue AstParser_getDiagnostics(QoreObject* self, AstParserHolder* aph, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 93 "QC_AstParser.qpp"
    size_t errorCount = aph->getErrorCount();
    if (errorCount == 0)
        return QoreValue();

    ReferenceHolder<QoreListNode> lst(new QoreListNode, xsink);
    for (size_t i = 0; i < errorCount; i++) {
        ASTParseError* err = aph->getError(i);
        ReferenceHolder<QoreHashNode> start(new QoreHashNode, xsink);
        ReferenceHolder<QoreHashNode> end(new QoreHashNode, xsink);
        ReferenceHolder<QoreHashNode> range(new QoreHashNode, xsink);
        ReferenceHolder<QoreHashNode> diag(new QoreHashNode, xsink);
        if (xsink && *xsink)
            return QoreValue();
        if (!start || !end || !range || !diag)
            return QoreValue();

        start->setKeyValue("line", err->loc.firstLine-1, xsink);
        start->setKeyValue("character", err->loc.firstCol-1, xsink);
        if (xsink && *xsink)
            return QoreValue();
        end->setKeyValue("line", err->loc.lastLine-1, xsink);
        end->setKeyValue("character", err->loc.lastCol-1, xsink);
        if (xsink && *xsink)
            return QoreValue();
        range->setKeyValue("start", start.release(), xsink);
        range->setKeyValue("end", end.release(), xsink);
        if (xsink && *xsink)
            return QoreValue();
        diag->setKeyValue("range", range.release(), xsink);
        diag->setKeyValue("message", new QoreStringNode(err->getError()), xsink);
        diag->setKeyValue("severity", 1, xsink); // 1 == error, 2 == warning, 3 == info, 4 == hint
        if (xsink && *xsink)
            return QoreValue();
        lst->push(diag.release(), nullptr);
    }

    return lst.release();
}

// int AstParser::getErrorCount(){}
static QoreValue AstParser_getErrorCount(QoreObject* self, AstParserHolder* aph, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 85 "QC_AstParser.qpp"
    return aph->getErrorCount();
}

// *AstTree AstParser::parseFile(string filename){}
static QoreValue AstParser_parseFile_Vs(QoreObject* self, AstParserHolder* aph, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* filename = HARD_QORE_VALUE_STRING(args, 0);
# 58 "QC_AstParser.qpp"
    std::unique_ptr<ASTTree> tree(aph->parseFile(filename->getBuffer()));
    if (!tree)
        return QoreValue();

    std::unique_ptr<AstTreeHolder> ath(new AstTreeHolder(tree.release()));
    return new QoreObject(QC_ASTTREE, getProgram(), ath.release());
}

// *AstTree AstParser::parseString(string str){}
static QoreValue AstParser_parseString_Vs(QoreObject* self, AstParserHolder* aph, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 72 "QC_AstParser.qpp"
    std::unique_ptr<ASTTree> tree(aph->parseString(str->getBuffer()));
    if (!tree)
        return QoreValue();

    std::unique_ptr<AstTreeHolder> ath(new AstTreeHolder(tree.release()));
    return new QoreObject(QC_ASTTREE, getProgram(), ath.release());
}

DLLLOCAL void preinitAstParserClass() {
    QC_ASTPARSER = new QoreBuiltinClass("AstParser", "::astparser::AstParser", QDOM_DEFAULT);
    CID_ASTPARSER = QC_ASTPARSER->getID();
    QC_ASTPARSER->setSystem();
}

DLLLOCAL QoreClass* initAstParserClass(QoreNamespace& ns) {
    if (!QC_ASTPARSER)
        preinitAstParserClass();

    QC_ASTPARSER->setFinal();

    // AstParser::constructor() {}
    QC_ASTPARSER->addConstructor(AstParser_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // *list AstParser::getDiagnostics(){}
    QC_ASTPARSER->addMethod("getDiagnostics", (q_method_n_t)AstParser_getDiagnostics, Public, QCF_NO_FLAGS, QDOM_DEFAULT, listOrNothingTypeInfo);

    // int AstParser::getErrorCount(){}
    QC_ASTPARSER->addMethod("getErrorCount", (q_method_n_t)AstParser_getErrorCount, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *AstTree AstParser::parseFile(string filename){}
    QC_ASTPARSER->addMethod("parseFile", (q_method_n_t)AstParser_parseFile_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ASTTREE->getOrNothingTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "filename");

    // *AstTree AstParser::parseString(string str){}
    QC_ASTPARSER->addMethod("parseString", (q_method_n_t)AstParser_parseString_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ASTTREE->getOrNothingTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");

    return QC_ASTPARSER;
}
