/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_Logger.cpp Logger class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_Logger.h"

static QoreHashNode* Logger_serializer(const QoreObject& self, const QoreLogger& l,
        QoreSerializationContext& context, ExceptionSink* xsink) {
    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);
    rv->setKeyValue("additivity", l.additivity, xsink);
    ValueHolder val(const_cast<QoreObject&>(self).getReferencedMemberNoMethod("currentLevel", xsink), xsink);
    if (val) {
        rv->setKeyValue("currentLevel", context.serializeValue(*val, xsink), xsink);
    }
    if (l.name) {
        rv->setKeyValue("name", l.name->stringRefSelf(), xsink);
    }
    val = const_cast<QoreObject&>(self).getReferencedMemberNoMethod("parent", xsink);
    if (val) {
        rv->setKeyValue("parent", context.serializeValue(*val, xsink), xsink);
    }
    return rv.release();
}

static void Logger_deserializer(QoreObject& self, const QoreHashNode* sdata,
        QoreDeserializationContext& context, ExceptionSink* xsink) {
    if (sdata) {
        QoreValue val = sdata->getKeyValue("additivity");
        if (val.getType() != NT_BOOLEAN) {
            xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'additivity' has type '%s'; "
                "expecting 'bool'", val.getTypeName());
            return;
        }
        bool additivity = val.getAsBool();

        const QoreObject* currentLevel = nullptr;
        val = sdata->getKeyValue("currentLevel");
        if (val) {
            if (val.getType() != NT_OBJECT) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'currentLevel' has type '%s'; "
                    "expecting 'object'", val.getTypeName());
                return;
            }
            currentLevel = val.get<const QoreObject>();
            ClassAccess access;
            if (!currentLevel->getClass()->inHierarchy(*QC_LOGGERLEVEL, access)) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'currentLevel' is an object "
                    "of class '%s'; expecting 'LoggerLevel'", val.getTypeName());
                return;
            }
            if (access != Public) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'currentLevel' is an object "
                    "of class '%s' which inherits 'LoggerLevel', but not pulicly", val.getTypeName());
                return;
            }
        }

        const QoreStringNode* name = nullptr;
        val = sdata->getKeyValue("name");
        if (val) {
            if (val.getType() != NT_STRING) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'name' has type '%s'; "
                    "expecting 'string'", val.getTypeName());
                return;
            }
            name = val.get<const QoreStringNode>();
        }

        const QoreObject* parent = nullptr;
        val = sdata->getKeyValue("parent");
        if (val) {
            if (val.getType() != NT_OBJECT) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'parent' has type '%s'; "
                    "expecting 'object'", val.getTypeName());
                return;
            }
            parent = val.get<const QoreObject>();
            ClassAccess access;
            if (!parent->getClass()->inHierarchy(*QC_LOGGER, access)) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'parent' is an object "
                    "of class '%s'; expecting 'Logger'", val.getTypeName());
                return;
            }
            if (access != Public) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Logger serialization key 'parent' is an object "
                    "of class '%s' which inherits 'Logger', but not pulicly", val.getTypeName());
                return;
            }
        }

        ReferenceHolder<QoreLogger> logger(new QoreLogger(&self, additivity, name), xsink);
        if (currentLevel) {
            logger->setLevelIntern(xsink, currentLevel->objectRefSelf());
            if (*xsink) {
                return;
            }
        }
        if (parent) {
            logger->setParentIntern(xsink, parent->objectRefSelf());
            if (*xsink) {
                return;
            }
        }
        self.setPrivate(CID_LOGGER, logger.release());
        return;
    }

    xsink->raiseException("DESERIALIZATION-ERROR", "missing serialization data for the Logger class");
}
/* Qore class Qore::Logger::Logger */

qore_classid_t CID_LOGGER;
QoreClass* QC_LOGGER;

// nothing Logger::addAppender(LoggerAppender appender){}
static QoreValue Logger_addAppender_C14LoggerAppender(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(appender, QoreLoggerAppender, args, 0, CID_LOGGERAPPENDER, "Logger::addAppender()", "LoggerAppender", xsink);
    if (*xsink)
        return 0;
# 362 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerAppender> a(appender, xsink);
    l->addAppender(xsink, obj_appender, a);
    return QoreValue();
}

// nothing Logger::assertLog(bool assertion, string message, ...){}
static QoreValue Logger_assertLog_VbVsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool assertion = HARD_QORE_VALUE_BOOL(args, 0);
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 1);
# 585 "QC_Logger.qpp"
    if (!assertion) {
        l->log(xsink, QoreLoggerLevel::LoggerLevelError, message, args, 2);
    }
    return QoreValue();
}

// nothing Logger::callAppenders(LoggerEvent event){}
static QoreValue Logger_callAppenders_C11LoggerEvent(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(event, QoreLoggerEvent, args, 0, CID_LOGGEREVENT, "Logger::callAppenders()", "LoggerEvent", xsink);
    if (*xsink)
        return 0;
# 352 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerEvent> e(event, xsink);
    l->callAppenders(xsink, obj_event, event);
    return QoreValue();
}

// Logger::constructor() {}
static void Logger_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 148 "QC_Logger.qpp"
self->setPrivate(CID_LOGGER, new QoreLogger(self));
}

// Logger::constructor(string name) {}
static void Logger_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 155 "QC_Logger.qpp"
QoreStringNodeValueHelper n(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }
    self->setPrivate(CID_LOGGER, new QoreLogger(self, true, *n));
}

// Logger::constructor(string name, string level) {}
static void Logger_constructor_VsVs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* level = HARD_QORE_VALUE_STRING(args, 1);
# 167 "QC_Logger.qpp"
QoreStringNodeValueHelper n(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }
    QoreStringNodeValueHelper lstr(level, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }
    ReferenceHolder<QoreObject> lvl(QoreLoggerLevel::getLevel(xsink, *lstr, nullptr), xsink);
    if (*xsink) {
        return;
    }
    ReferenceHolder<QoreLogger> logger(new QoreLogger(self, true, *n), xsink);
    logger->setLevelIntern(xsink, lvl.release());
    if (*xsink) {
        return;
    }
    self->setPrivate(CID_LOGGER, logger.release());
}

// Logger::constructor(string name, int level) {}
static void Logger_constructor_VsVi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    int64 level = HARD_QORE_VALUE_INT(args, 1);
# 192 "QC_Logger.qpp"
QoreStringNodeValueHelper n(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }
    ReferenceHolder<QoreObject> lvl(QoreLoggerLevel::getLevel(xsink, level, nullptr), xsink);
    if (*xsink) {
        return;
    }
    ReferenceHolder<QoreLogger> logger(new QoreLogger(self, true, *n), xsink);
    logger->setLevelIntern(xsink, lvl.release());
    if (*xsink) {
        return;
    }
    self->setPrivate(CID_LOGGER, logger.release());
}

// Logger::constructor(string name, LoggerLevel level) {}
static void Logger_constructor_VsC11LoggerLevel(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    HARD_QORE_VALUE_OBJ_DATA(level, QoreLoggerLevel, args, 1, CID_LOGGERLEVEL, "Logger::constructor()", "LoggerLevel", xsink);
    if (*xsink)
        return;
# 213 "QC_Logger.qpp"
ReferenceHolder<QoreLoggerLevel> holder(level, xsink);
    QoreStringNodeValueHelper n(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }
    ReferenceHolder<QoreLogger> logger(new QoreLogger(self, true, *n), xsink);
    ReferenceHolder<QoreObject> level_holder(obj_level->objectRefSelf(), xsink);
    logger->setLevelIntern(xsink, level_holder, holder);
    self->setPrivate(CID_LOGGER, logger.release());
}

// Logger::constructor(LoggerLevel level) {}
static void Logger_constructor_C11LoggerLevel(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(level, QoreLoggerLevel, args, 0, CID_LOGGERLEVEL, "Logger::constructor()", "LoggerLevel", xsink);
    if (*xsink)
        return;
# 228 "QC_Logger.qpp"
ReferenceHolder<QoreLoggerLevel> holder(level, xsink);
    ReferenceHolder<QoreLogger> logger(new QoreLogger(self), xsink);
    ReferenceHolder<QoreObject> level_holder(obj_level->objectRefSelf(), xsink);
    logger->setLevel(xsink, level_holder, holder);
    self->setPrivate(CID_LOGGER, logger.release());
}

// nothing Logger::debug(string message, ...){}
static QoreValue Logger_debug_VsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 0);
# 514 "QC_Logger.qpp"
    l->log(xsink, QoreLoggerLevel::LoggerLevelDebug, message, args, 1);
    return QoreValue();
}

// nothing Logger::debugVar(string var_name, auto value){}
static QoreValue Logger_debugVar_VsVA(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* var_name = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 610 "QC_Logger.qpp"
    QoreStringNodeHolder msg(new QoreStringNodeMaker("%s: %%y", var_name->c_str()));
    l->log(xsink, QoreLoggerLevel::LoggerLevelDebug, *msg, args, 1);
    return QoreValue();
}

// *LoggerLevel Logger::decLevel(){}
static QoreValue Logger_decLevel(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 321 "QC_Logger.qpp"
    return l->decLevel(xsink);
}

// nothing Logger::detail(string message, ...){}
static QoreValue Logger_detail_VsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 0);
# 525 "QC_Logger.qpp"
    l->log(xsink, QoreLoggerLevel::LoggerLevelDetail, message, args, 1);
    return QoreValue();
}

// nothing Logger::error(string message, ...){}
static QoreValue Logger_error_VsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 0);
# 558 "QC_Logger.qpp"
    l->log(xsink, QoreLoggerLevel::LoggerLevelError, message, args, 1);
    return QoreValue();
}

// nothing Logger::fatal(string message, ...){}
static QoreValue Logger_fatal_VsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 0);
# 569 "QC_Logger.qpp"
    l->log(xsink, QoreLoggerLevel::LoggerLevelFatal, message, args, 1);
    return QoreValue();
}

// bool Logger::getAdditivity(){}
static QoreValue Logger_getAdditivity(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 344 "QC_Logger.qpp"
    return l->getAdditivity();
}

// list<LoggerAppender> Logger::getAppenders(){}
static QoreValue Logger_getAppenders(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 388 "QC_Logger.qpp"
    return l->getAppenders(xsink);
}

// *LoggerLevel Logger::getLevel(bool effective = True){}
static QoreValue Logger_getLevel_Vb(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool effective = HARD_QORE_VALUE_BOOL(args, 0);
# 267 "QC_Logger.qpp"
    return l->getLevel(xsink, effective);
}

// string Logger::getName(){}
static QoreValue Logger_getName(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 239 "QC_Logger.qpp"
    return l->getName();
}

// *Logger Logger::getParent(){}
static QoreValue Logger_getParent(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 246 "QC_Logger.qpp"
    return l->getParent(xsink);
}

// *LoggerLevel Logger::incLevel(){}
static QoreValue Logger_incLevel(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 329 "QC_Logger.qpp"
    return l->incLevel(xsink);
}

// nothing Logger::info(string message, ...){}
static QoreValue Logger_info_VsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 0);
# 536 "QC_Logger.qpp"
    l->log(xsink, QoreLoggerLevel::LoggerLevelInfo, message, args, 1);
    return QoreValue();
}

// bool Logger::isDebugEnabled(){}
static QoreValue Logger_isDebugEnabled(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 662 "QC_Logger.qpp"
    return l->isEnabledFor(xsink, QoreLoggerLevel::LoggerLevelDebug);
}

// bool Logger::isDetailEnabled(){}
static QoreValue Logger_isDetailEnabled(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 669 "QC_Logger.qpp"
    return l->isEnabledFor(xsink, QoreLoggerLevel::LoggerLevelDetail);
}

// bool Logger::isEnabledFor(LoggerLevel level){}
static QoreValue Logger_isEnabledFor_C11LoggerLevel(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(level, QoreLoggerLevel, args, 0, CID_LOGGERLEVEL, "Logger::isEnabledFor()", "LoggerLevel", xsink);
    if (*xsink)
        return 0;
# 619 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerLevel> lvl(level, xsink);
    return l->isEnabledFor(xsink, level);
}

// bool Logger::isEnabledFor(int level){}
static QoreValue Logger_isEnabledFor_Vi(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 level = HARD_QORE_VALUE_INT(args, 0);
# 628 "QC_Logger.qpp"
    const QoreLoggerLevel* lvl = QoreLoggerLevel::getLoggerLevel(xsink, level);
    if (*xsink) {
        return QoreValue();
    }
    return l->isEnabledFor(xsink, lvl);
}

// bool Logger::isEnabledFor(string level){}
static QoreValue Logger_isEnabledFor_Vs(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* level = HARD_QORE_VALUE_STRING(args, 0);
# 640 "QC_Logger.qpp"
    QoreStringNodeValueHelper lstr(level, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    const QoreLoggerLevel* lvl = QoreLoggerLevel::getLoggerLevel(xsink, level);
    if (*xsink) {
        return QoreValue();
    }
    return l->isEnabledFor(xsink, lvl);
}

// bool Logger::isErrorEnabled(){}
static QoreValue Logger_isErrorEnabled(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 690 "QC_Logger.qpp"
    return l->isEnabledFor(xsink, QoreLoggerLevel::LoggerLevelError);
}

// bool Logger::isFatalEnabled(){}
static QoreValue Logger_isFatalEnabled(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 697 "QC_Logger.qpp"
    return l->isEnabledFor(xsink, QoreLoggerLevel::LoggerLevelFatal);
}

// bool Logger::isInfoEnabled(){}
static QoreValue Logger_isInfoEnabled(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 676 "QC_Logger.qpp"
    return l->isEnabledFor(xsink, QoreLoggerLevel::LoggerLevelInfo);
}

// bool Logger::isTraceEnabled(){}
static QoreValue Logger_isTraceEnabled(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 655 "QC_Logger.qpp"
    return l->isEnabledFor(xsink, QoreLoggerLevel::LoggerLevelTrace);
}

// bool Logger::isWarnEnabled(){}
static QoreValue Logger_isWarnEnabled(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 683 "QC_Logger.qpp"
    return l->isEnabledFor(xsink, QoreLoggerLevel::LoggerLevelWarn);
}

// nothing Logger::log(LoggerLevel level, string message, ...){}
static QoreValue Logger_log_C11LoggerLevelVsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(level, QoreLoggerLevel, args, 0, CID_LOGGERLEVEL, "Logger::log()", "LoggerLevel", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 1);
# 399 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerLevel> lvl(level, xsink);
    l->log(xsink, level, message, args, 2);
    return QoreValue();
}

// nothing Logger::log(int level, string message, ...){}
static QoreValue Logger_log_ViVsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 level = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 1);
# 411 "QC_Logger.qpp"
    const QoreLoggerLevel* lvl = QoreLoggerLevel::getLoggerLevel(xsink, level);
    if (*xsink) {
        return QoreValue();
    }
    l->log(xsink, lvl, message, args, 2);
    return QoreValue();
}

// nothing Logger::log(string level, string message, ...){}
static QoreValue Logger_log_VsVsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* level = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 1);
# 426 "QC_Logger.qpp"
    QoreStringNodeValueHelper lstr(level, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    const QoreLoggerLevel* lvl = QoreLoggerLevel::getLoggerLevel(xsink, *lstr);
    if (*xsink) {
        return QoreValue();
    }
    l->log(xsink, lvl, message, args, 2);
    return QoreValue();
}

// nothing Logger::logArgs(LoggerLevel level, string message, *softlist<auto> margs){}
static QoreValue Logger_logArgs_C11LoggerLevelVsC15_softlist_auto_(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(level, QoreLoggerLevel, args, 0, CID_LOGGERLEVEL, "Logger::logArgs()", "LoggerLevel", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* margs = get_param_value(args, 2).get<const QoreListNode>();
# 446 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerLevel> lvl(level, xsink);
    l->log(xsink, level, message, margs, 0);
    return QoreValue();
}

// nothing Logger::logArgs(int level, string message, *softlist<auto> margs){}
static QoreValue Logger_logArgs_ViVsC15_softlist_auto_(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 level = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* margs = get_param_value(args, 2).get<const QoreListNode>();
# 459 "QC_Logger.qpp"
    const QoreLoggerLevel* lvl = QoreLoggerLevel::getLoggerLevel(xsink, level);
    if (*xsink) {
        return QoreValue();
    }
    l->log(xsink, lvl, message, margs, 0);
    return QoreValue();
}

// nothing Logger::logArgs(string level, string message, *softlist<auto> margs){}
static QoreValue Logger_logArgs_VsVsC15_softlist_auto_(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* level = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* margs = get_param_value(args, 2).get<const QoreListNode>();
# 475 "QC_Logger.qpp"
    QoreStringNodeValueHelper lstr(level, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    const QoreLoggerLevel* lvl = QoreLoggerLevel::getLoggerLevel(xsink, *lstr);
    if (*xsink) {
        return QoreValue();
    }
    l->log(xsink, lvl, message, margs, 0);
    return QoreValue();
}

// nothing Logger::logEvent(LoggerEvent event){}
static QoreValue Logger_logEvent_C11LoggerEvent(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(event, QoreLoggerEvent, args, 0, CID_LOGGEREVENT, "Logger::logEvent()", "LoggerEvent", xsink);
    if (*xsink)
        return 0;
# 491 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerEvent> e(event, xsink);
    l->logEvent(xsink, obj_event, event);
    return QoreValue();
}

// nothing Logger::removeAllAppenders(){}
static QoreValue Logger_removeAllAppenders(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 379 "QC_Logger.qpp"
    l->removeAllAppenders(xsink);
    return QoreValue();
}

// nothing Logger::removeAppender(LoggerAppender appender){}
static QoreValue Logger_removeAppender_C14LoggerAppender(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(appender, QoreLoggerAppender, args, 0, CID_LOGGERAPPENDER, "Logger::removeAppender()", "LoggerAppender", xsink);
    if (*xsink)
        return 0;
# 371 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerAppender> a(appender, xsink);
    l->removeAppender(xsink, obj_appender);
    return QoreValue();
}

// nothing Logger::setAdditivity(bool enable){}
static QoreValue Logger_setAdditivity_Vb(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool enable = HARD_QORE_VALUE_BOOL(args, 0);
# 337 "QC_Logger.qpp"
    l->setAdditivity(enable);
    return QoreValue();
}

// nothing Logger::setLevel(*LoggerLevel level){}
static QoreValue Logger_setLevel_C12_LoggerLevel(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(level, QoreLoggerLevel, args, 0, CID_LOGGERLEVEL, xsink);
    if (*xsink)
        return 0;
# 275 "QC_Logger.qpp"
    ReferenceHolder<QoreLoggerLevel> lvl(level, xsink);
    ReferenceHolder<QoreObject> level_holder(obj_level ? obj_level->objectRefSelf() : nullptr, xsink);
    l->setLevel(xsink, level_holder, lvl);
    return QoreValue();
}

// nothing Logger::setLevel(string level){}
static QoreValue Logger_setLevel_Vs(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* level = HARD_QORE_VALUE_STRING(args, 0);
# 285 "QC_Logger.qpp"
    QoreStringNodeValueHelper lstr(level, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }
    ReferenceHolder<QoreObject> lvl(QoreLoggerLevel::getLevel(xsink, *lstr, nullptr), xsink);
    if (*xsink) {
        return QoreValue();
    }
    ReferenceHolder<QoreLoggerLevel> llvl(lvl->getReferencedPrivateData<QoreLoggerLevel>(CID_LOGGERLEVEL, xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }
    l->setLevel(xsink, lvl, llvl);
    return QoreValue();
}

// nothing Logger::setLevel(int level){}
static QoreValue Logger_setLevel_Vi(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 level = HARD_QORE_VALUE_INT(args, 0);
# 305 "QC_Logger.qpp"
    ReferenceHolder<QoreObject> lvl(QoreLoggerLevel::getLevel(xsink, level, nullptr), xsink);
    if (*xsink) {
        return QoreValue();
    }
    ReferenceHolder<QoreLoggerLevel> llvl(lvl->getReferencedPrivateData<QoreLoggerLevel>(CID_LOGGERLEVEL, xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }
    l->setLevel(xsink, lvl, llvl);
    return QoreValue();
}

// nothing Logger::setParent(*Logger parent){}
static QoreValue Logger_setParent_C7_Logger(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(parent, QoreLogger, args, 0, CID_LOGGER, xsink);
    if (*xsink)
        return 0;
# 256 "QC_Logger.qpp"
    ReferenceHolder<QoreLogger> holder(parent, xsink);
    ReferenceHolder<QoreObject> p(obj_parent ? obj_parent->objectRefSelf() : nullptr, xsink);
    l->setParent(xsink, p, holder);
    return QoreValue();
}

// nothing Logger::trace(string message, ...){}
static QoreValue Logger_trace_VsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 0);
# 503 "QC_Logger.qpp"
    l->log(xsink, QoreLoggerLevel::LoggerLevelTrace, message, args, 1);
    return QoreValue();
}

// nothing Logger::traceVar(string var_name, auto value){}
static QoreValue Logger_traceVar_VsVA(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* var_name = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 598 "QC_Logger.qpp"
    QoreStringNodeHolder msg(new QoreStringNodeMaker("%s: %%y", var_name->c_str()));
    l->log(xsink, QoreLoggerLevel::LoggerLevelTrace, *msg, args, 1);
    return QoreValue();
}

// nothing Logger::warn(string message, ...){}
static QoreValue Logger_warn_VsVV(QoreObject* self, QoreLogger* l, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 0);
# 547 "QC_Logger.qpp"
    l->log(xsink, QoreLoggerLevel::LoggerLevelWarn, message, args, 1);
    return QoreValue();
}

// static private hash<CallStackInfo> Logger::getLocation(){}
static QoreValue static_Logger_getLocation(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 704 "QC_Logger.qpp"
QoreLogger::getLocation();
    return QoreValue();
}

DLLLOCAL void preinitLoggerClass() {
    QC_LOGGER = new QoreBuiltinClass("Logger", "::Qore::Logger::Logger", QDOM_DEFAULT);
    CID_LOGGER = QC_LOGGER->getID();
    QC_LOGGER->setSystem();
}

DLLLOCAL QoreClass* initLoggerClass(QoreNamespace& ns) {
    if (!QC_LOGGER)
        preinitLoggerClass();

    // set serializer function
    QC_LOGGER->setSerializer(reinterpret_cast<q_serializer_t>(Logger_serializer));

    // set deserializer function
    QC_LOGGER->setDeserializer(Logger_deserializer);

    // set parent class
    assert(QC_LOGGERINTERFACE);
    QC_LOGGER->addBuiltinVirtualBaseClass(QC_LOGGERINTERFACE);

    // private:internal members
    QC_LOGGER->addMember("parent", Internal, QC_LOGGER->getOrNothingTypeInfo());
    QC_LOGGER->addMember("currentLevel", Internal, QC_LOGGERLEVEL->getTypeInfo());

    // nothing Logger::addAppender(LoggerAppender appender){}
    QC_LOGGER->addMethod("addAppender", (q_method_n_t)Logger_addAppender_C14LoggerAppender, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGERAPPENDER->getTypeInfo(), QORE_PARAM_NO_ARG, "appender");

    // nothing Logger::assertLog(bool assertion, string message, ...){}
    QC_LOGGER->addMethod("assertLog", (q_method_n_t)Logger_assertLog_VbVsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 2, boolTypeInfo, QORE_PARAM_NO_ARG, "assertion", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::callAppenders(LoggerEvent event){}
    QC_LOGGER->addMethod("callAppenders", (q_method_n_t)Logger_callAppenders_C11LoggerEvent, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    // Logger::constructor() {}
    QC_LOGGER->addConstructor(Logger_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Logger::constructor(string name) {}
    QC_LOGGER->addConstructor(Logger_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // Logger::constructor(string name, string level) {}
    QC_LOGGER->addConstructor(Logger_constructor_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", stringTypeInfo, QORE_PARAM_NO_ARG, "level");

    // Logger::constructor(string name, int level) {}
    QC_LOGGER->addConstructor(Logger_constructor_VsVi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", bigIntTypeInfo, QORE_PARAM_NO_ARG, "level");

    // Logger::constructor(string name, LoggerLevel level) {}
    QC_LOGGER->addConstructor(Logger_constructor_VsC11LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level");

    // Logger::constructor(LoggerLevel level) {}
    QC_LOGGER->addConstructor(Logger_constructor_C11LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level");

    // nothing Logger::debug(string message, ...){}
    QC_LOGGER->addMethod("debug", (q_method_n_t)Logger_debug_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::debugVar(string var_name, auto value){}
    QC_LOGGER->addMethod("debugVar", (q_method_n_t)Logger_debugVar_VsVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "var_name", autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // *LoggerLevel Logger::decLevel(){}
    QC_LOGGER->addMethod("decLevel", (q_method_n_t)Logger_decLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getOrNothingTypeInfo());

    // nothing Logger::detail(string message, ...){}
    QC_LOGGER->addMethod("detail", (q_method_n_t)Logger_detail_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::error(string message, ...){}
    QC_LOGGER->addMethod("error", (q_method_n_t)Logger_error_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::fatal(string message, ...){}
    QC_LOGGER->addMethod("fatal", (q_method_n_t)Logger_fatal_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // bool Logger::getAdditivity(){}
    QC_LOGGER->addMethod("getAdditivity", (q_method_n_t)Logger_getAdditivity, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // list<LoggerAppender> Logger::getAppenders(){}
    QC_LOGGER->addMethod("getAppenders", (q_method_n_t)Logger_getAppenders, Public, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(QC_LOGGERAPPENDER->getTypeInfo()));

    // *LoggerLevel Logger::getLevel(bool effective = True){}
    QC_LOGGER->addMethod("getLevel", (q_method_n_t)Logger_getLevel_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getOrNothingTypeInfo(), 1, boolTypeInfo, QoreSimpleValue().assign(true), "effective");

    // string Logger::getName(){}
    QC_LOGGER->addMethod("getName", (q_method_n_t)Logger_getName, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // *Logger Logger::getParent(){}
    QC_LOGGER->addMethod("getParent", (q_method_n_t)Logger_getParent, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGER->getOrNothingTypeInfo());

    // *LoggerLevel Logger::incLevel(){}
    QC_LOGGER->addMethod("incLevel", (q_method_n_t)Logger_incLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getOrNothingTypeInfo());

    // nothing Logger::info(string message, ...){}
    QC_LOGGER->addMethod("info", (q_method_n_t)Logger_info_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // bool Logger::isDebugEnabled(){}
    QC_LOGGER->addMethod("isDebugEnabled", (q_method_n_t)Logger_isDebugEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool Logger::isDetailEnabled(){}
    QC_LOGGER->addMethod("isDetailEnabled", (q_method_n_t)Logger_isDetailEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool Logger::isEnabledFor(LoggerLevel level){}
    QC_LOGGER->addMethod("isEnabledFor", (q_method_n_t)Logger_isEnabledFor_C11LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level");

    // bool Logger::isEnabledFor(int level){}
    QC_LOGGER->addMethod("isEnabledFor", (q_method_n_t)Logger_isEnabledFor_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level");

    // bool Logger::isEnabledFor(string level){}
    QC_LOGGER->addMethod("isEnabledFor", (q_method_n_t)Logger_isEnabledFor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "level");

    // bool Logger::isErrorEnabled(){}
    QC_LOGGER->addMethod("isErrorEnabled", (q_method_n_t)Logger_isErrorEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool Logger::isFatalEnabled(){}
    QC_LOGGER->addMethod("isFatalEnabled", (q_method_n_t)Logger_isFatalEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool Logger::isInfoEnabled(){}
    QC_LOGGER->addMethod("isInfoEnabled", (q_method_n_t)Logger_isInfoEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool Logger::isTraceEnabled(){}
    QC_LOGGER->addMethod("isTraceEnabled", (q_method_n_t)Logger_isTraceEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool Logger::isWarnEnabled(){}
    QC_LOGGER->addMethod("isWarnEnabled", (q_method_n_t)Logger_isWarnEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing Logger::log(LoggerLevel level, string message, ...){}
    QC_LOGGER->addMethod("log", (q_method_n_t)Logger_log_C11LoggerLevelVsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::log(int level, string message, ...){}
    QC_LOGGER->addMethod("log", (q_method_n_t)Logger_log_ViVsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::log(string level, string message, ...){}
    QC_LOGGER->addMethod("log", (q_method_n_t)Logger_log_VsVsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::logArgs(LoggerLevel level, string message, *softlist<auto> margs){}
    QC_LOGGER->addMethod("logArgs", (q_method_n_t)Logger_logArgs_C11LoggerLevelVsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "margs");

    // nothing Logger::logArgs(int level, string message, *softlist<auto> margs){}
    QC_LOGGER->addMethod("logArgs", (q_method_n_t)Logger_logArgs_ViVsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "margs");

    // nothing Logger::logArgs(string level, string message, *softlist<auto> margs){}
    QC_LOGGER->addMethod("logArgs", (q_method_n_t)Logger_logArgs_VsVsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "margs");

    // nothing Logger::logEvent(LoggerEvent event){}
    QC_LOGGER->addMethod("logEvent", (q_method_n_t)Logger_logEvent_C11LoggerEvent, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    // nothing Logger::removeAllAppenders(){}
    QC_LOGGER->addMethod("removeAllAppenders", (q_method_n_t)Logger_removeAllAppenders, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Logger::removeAppender(LoggerAppender appender){}
    QC_LOGGER->addMethod("removeAppender", (q_method_n_t)Logger_removeAppender_C14LoggerAppender, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGERAPPENDER->getTypeInfo(), QORE_PARAM_NO_ARG, "appender");

    // nothing Logger::setAdditivity(bool enable){}
    QC_LOGGER->addMethod("setAdditivity", (q_method_n_t)Logger_setAdditivity_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, QORE_PARAM_NO_ARG, "enable");

    // nothing Logger::setLevel(*LoggerLevel level){}
    QC_LOGGER->addMethod("setLevel", (q_method_n_t)Logger_setLevel_C12_LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGERLEVEL->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "level");

    // nothing Logger::setLevel(string level){}
    QC_LOGGER->addMethod("setLevel", (q_method_n_t)Logger_setLevel_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "level");

    // nothing Logger::setLevel(int level){}
    QC_LOGGER->addMethod("setLevel", (q_method_n_t)Logger_setLevel_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level");

    // nothing Logger::setParent(*Logger parent){}
    QC_LOGGER->addMethod("setParent", (q_method_n_t)Logger_setParent_C7_Logger, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGER->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "parent");

    // nothing Logger::trace(string message, ...){}
    QC_LOGGER->addMethod("trace", (q_method_n_t)Logger_trace_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing Logger::traceVar(string var_name, auto value){}
    QC_LOGGER->addMethod("traceVar", (q_method_n_t)Logger_traceVar_VsVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "var_name", autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // nothing Logger::warn(string message, ...){}
    QC_LOGGER->addMethod("warn", (q_method_n_t)Logger_warn_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // static private hash<CallStackInfo> Logger::getLocation(){}
    QC_LOGGER->addStaticMethod("getLocation", (q_func_n_t)static_Logger_getLocation, Private, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclCallStackInfo->getTypeInfo(false));

    return QC_LOGGER;
}
