/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerLayoutPattern.cpp LoggerLayoutPattern class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerLayoutPattern.h"
#include "QC_LoggerEvent.h"
/* Qore class Qore::Logger::LoggerLayoutPattern */

qore_classid_t CID_LOGGERLAYOUTPATTERN;
QoreClass* QC_LOGGERLAYOUTPATTERN;

// LoggerLayoutPattern::constructor(string pattern = "%r [%t] %p %c - %m%n") {}
static void LoggerLayoutPattern_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
# 75 "QC_LoggerLayoutPattern.qpp"
ReferenceHolder<QoreLoggerLayoutPattern> llp(new QoreLoggerLayoutPattern(self), xsink);
    if (*xsink) {
        return;
    }
    if (llp->setPattern(pattern, xsink)) {
        assert(*xsink);
        return;
    }
    self->setPrivate(CID_LOGGERLAYOUTPATTERN, llp.release());
}

// string LoggerLayoutPattern::format(LoggerEvent event){}
static QoreValue LoggerLayoutPattern_format_C11LoggerEvent(QoreObject* self, QoreLoggerLayoutPattern* llp, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(event, QoreLoggerEvent, args, 0, CID_LOGGEREVENT, "LoggerLayoutPattern::format()", "LoggerEvent", xsink);
    if (*xsink)
        return 0;
# 91 "QC_LoggerLayoutPattern.qpp"
    ReferenceHolder<AbstractPrivateData> holder(event, xsink);
    return llp->format(xsink, obj_event, obj_event, event);
}

// private *string LoggerLayoutPattern::resolveField(auto event, string key, *string option){}
static QoreValue LoggerLayoutPattern_resolveField_VAVsNs(QoreObject* self, QoreLoggerLayoutPattern* llp, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue event = get_param_value(args, 0);
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* option = get_param_value(args, 2).get<const QoreStringNode>();
# 111 "QC_LoggerLayoutPattern.qpp"
    bool ok = false;
    QoreObject* obj_event = event.getType() == NT_OBJECT ? event.get<QoreObject>() : nullptr;
    if (obj_event) {
        ok = obj_event->validInstanceOf(*QC_LOGGEREVENT);
    }
    if (!ok) {
        xsink->raiseException("UNSUPPORTED-EVENT-TYPE", new QoreStringNodeMaker("\"event\" is not a LoggerEvent "
            "object; got \"%s\" instead\n", event.getFullTypeName()));
        return QoreValue();
    }
    // only set the private data object if the class is not overridden

    ReferenceHolder<QoreLoggerEvent> ev(xsink);
    if (QC_LOGGEREVENT->isEqual(*obj_event->getClass())) {
        ev = obj_event->tryGetReferencedPrivateData<QoreLoggerEvent>(CID_LOGGEREVENT, xsink);
        if (*xsink) {
            return QoreValue();
        }
    }
    return llp->resolveField(const_cast<QoreObject*>(obj_event), *ev, key, option, xsink);
}

// static string LoggerLayoutPattern::getLineDelimiter(){}
static QoreValue static_LoggerLayoutPattern_getLineDelimiter(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 99 "QC_LoggerLayoutPattern.qpp"
return QoreLoggerLayoutPattern::LineDelimeter->stringRefSelf();
}

DLLLOCAL void preinitLoggerLayoutPatternClass() {
    QC_LOGGERLAYOUTPATTERN = new QoreBuiltinClass("LoggerLayoutPattern", "::Qore::Logger::LoggerLayoutPattern", QDOM_DEFAULT);
    CID_LOGGERLAYOUTPATTERN = QC_LOGGERLAYOUTPATTERN->getID();
    QC_LOGGERLAYOUTPATTERN->setSystem();
}

DLLLOCAL QoreClass* initLoggerLayoutPatternClass(QoreNamespace& ns) {
    if (!QC_LOGGERLAYOUTPATTERN)
        preinitLoggerLayoutPatternClass();

    // set parent class
    assert(QC_LOGGERLAYOUT);
    QC_LOGGERLAYOUTPATTERN->addBuiltinVirtualBaseClass(QC_LOGGERLAYOUT);

    // set parent class
    assert(QC_LOGGERPATTERN);
    QC_LOGGERLAYOUTPATTERN->addBuiltinVirtualBaseClass(QC_LOGGERPATTERN);

    // LoggerLayoutPattern::constructor(string pattern = "%r [%t] %p %c - %m%n") {}
    QC_LOGGERLAYOUTPATTERN->addConstructor(LoggerLayoutPattern_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QoreSimpleValue().assign(new QoreStringNode("%r [%t] %p %c - %m%n")), "pattern");

    // string LoggerLayoutPattern::format(LoggerEvent event){}
    QC_LOGGERLAYOUTPATTERN->addMethod("format", (q_method_n_t)LoggerLayoutPattern_format_C11LoggerEvent, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    // private *string LoggerLayoutPattern::resolveField(auto event, string key, *string option){}
    QC_LOGGERLAYOUTPATTERN->addMethod("resolveField", (q_method_n_t)LoggerLayoutPattern_resolveField_VAVsNs, Private, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 3, autoTypeInfo, QORE_PARAM_NO_ARG, "event", stringTypeInfo, QORE_PARAM_NO_ARG, "key", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "option");

    // static string LoggerLayoutPattern::getLineDelimiter(){}
    QC_LOGGERLAYOUTPATTERN->addStaticMethod("getLineDelimiter", (q_func_n_t)static_LoggerLayoutPattern_getLineDelimiter, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    return QC_LOGGERLAYOUTPATTERN;
}
