/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerLevel.cpp LoggerLevel class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerLevel.h"

static QoreHashNode* LoggerLevel_serializer(const QoreObject& self, const QoreLoggerLevel& level,
        QoreSerializationContext& context, ExceptionSink* xsink) {
    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);

    rv->setKeyValue("code", level.getValue(), xsink);
    rv->setKeyValue("str", level.getStr(), xsink);

    return rv.release();
}

static void LoggerLevel_deserializer(QoreObject& self, const QoreHashNode* sdata,
        QoreDeserializationContext& context, ExceptionSink* xsink) {
    if (sdata) {
        QoreValue val = sdata->getKeyValue("code");
        if (val.getType() != NT_INT) {
            xsink->raiseException("DESERIALIZATION-ERROR", "LoggerLevel serialization key 'code' has type '%s'; "
                "expecting 'int'", val.getTypeName());
            return;
        }
        int64 code = val.getAsBigInt();

        val = sdata->getKeyValue("str");
        if (val.getType() != NT_STRING) {
            xsink->raiseException("DESERIALIZATION-ERROR", "LoggerLevel serialization key 'str' has type '%s'; "
                "expecting 'string'", val.getTypeName());
            return;
        }

        self.setPrivate(CID_LOGGERLEVEL, new QoreLoggerLevel(code, val.get<const QoreStringNode>()));
        return;
    }

    xsink->raiseException("DESERIALIZATION-ERROR", "missing serialization data for the LoggerLevel class");
}
/* Qore class Qore::Logger::LoggerLevel */

qore_classid_t CID_LOGGERLEVEL;
QoreClass* QC_LOGGERLEVEL;

// LoggerLevel::constructor(int code, string level) {}
static void LoggerLevel_constructor_ViVs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* level = HARD_QORE_VALUE_STRING(args, 1);
# 80 "QC_LoggerLevel.qpp"
self->setPrivate(CID_LOGGERLEVEL, new QoreLoggerLevel(code, level));
}

// string LoggerLevel::getStr(){}
static QoreValue LoggerLevel_getStr(QoreObject* self, QoreLoggerLevel* level, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 94 "QC_LoggerLevel.qpp"
    return level->getStr();
}

// int LoggerLevel::getValue(){}
static QoreValue LoggerLevel_getValue(QoreObject* self, QoreLoggerLevel* level, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 87 "QC_LoggerLevel.qpp"
    return level->getValue();
}

// bool LoggerLevel::isEqual(LoggerLevel other){}
static QoreValue LoggerLevel_isEqual_C11LoggerLevel(QoreObject* self, QoreLoggerLevel* level, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(other, QoreLoggerLevel, args, 0, CID_LOGGERLEVEL, "LoggerLevel::isEqual()", "LoggerLevel", xsink);
    if (*xsink)
        return 0;
# 112 "QC_LoggerLevel.qpp"
    ReferenceHolder<AbstractPrivateData> holder(other, xsink);
    return level->isEqual(other);
}

// bool LoggerLevel::isGreaterOrEqual(LoggerLevel other){}
static QoreValue LoggerLevel_isGreaterOrEqual_C11LoggerLevel(QoreObject* self, QoreLoggerLevel* level, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(other, QoreLoggerLevel, args, 0, CID_LOGGERLEVEL, "LoggerLevel::isGreaterOrEqual()", "LoggerLevel", xsink);
    if (*xsink)
        return 0;
# 103 "QC_LoggerLevel.qpp"
    ReferenceHolder<AbstractPrivateData> holder(other, xsink);
    return level->isGreaterOrEqual(other);
}

// static LoggerLevel LoggerLevel::getLevel(int level, *LoggerLevel default_level){}
static QoreValue static_LoggerLevel_getLevel_ViC12_LoggerLevel(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 level = HARD_QORE_VALUE_INT(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(default_level, QoreLoggerLevel, args, 1, CID_LOGGERLEVEL, xsink);
    if (*xsink)
        return 0;
# 126 "QC_LoggerLevel.qpp"
ReferenceHolder<AbstractPrivateData> holder(default_level, xsink);
    return QoreLoggerLevel::getLevel(xsink, level, obj_default_level);
}

// static LoggerLevel LoggerLevel::getLevel(string level_str, *LoggerLevel default_level){}
static QoreValue static_LoggerLevel_getLevel_VsC12_LoggerLevel(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* level_str = HARD_QORE_VALUE_STRING(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(default_level, QoreLoggerLevel, args, 1, CID_LOGGERLEVEL, xsink);
    if (*xsink)
        return 0;
# 140 "QC_LoggerLevel.qpp"
ReferenceHolder<AbstractPrivateData> holder(default_level, xsink);
    return QoreLoggerLevel::getLevel(xsink, level_str, obj_default_level);
}

// static LoggerLevel LoggerLevel::getLevelAll(){}
static QoreValue static_LoggerLevel_getLevelAll(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 226 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelAll();
}

// static LoggerLevel LoggerLevel::getLevelDebug(){}
static QoreValue static_LoggerLevel_getLevelDebug(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 212 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelDebug();
}

// static LoggerLevel LoggerLevel::getLevelDetail(){}
static QoreValue static_LoggerLevel_getLevelDetail(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 205 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelDetail();
}

// static LoggerLevel LoggerLevel::getLevelError(){}
static QoreValue static_LoggerLevel_getLevelError(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 184 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelError();
}

// static LoggerLevel LoggerLevel::getLevelFatal(){}
static QoreValue static_LoggerLevel_getLevelFatal(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 177 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelFatal();
}

// static LoggerLevel LoggerLevel::getLevelInfo(){}
static QoreValue static_LoggerLevel_getLevelInfo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 198 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelInfo();
}

// static LoggerLevel LoggerLevel::getLevelOff(){}
static QoreValue static_LoggerLevel_getLevelOff(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 170 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelOff();
}

// static LoggerLevel LoggerLevel::getLevelTrace(){}
static QoreValue static_LoggerLevel_getLevelTrace(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 219 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelTrace();
}

// static LoggerLevel LoggerLevel::getLevelWarn(){}
static QoreValue static_LoggerLevel_getLevelWarn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 191 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getLevelWarn();
}

// static *LoggerLevel LoggerLevel::getNextHigherLevel(int level){}
static QoreValue static_LoggerLevel_getNextHigherLevel_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 level = HARD_QORE_VALUE_INT(args, 0);
# 163 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getNextHigherLevel(level);
}

// static *LoggerLevel LoggerLevel::getNextLowerLevel(int level){}
static QoreValue static_LoggerLevel_getNextLowerLevel_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 level = HARD_QORE_VALUE_INT(args, 0);
# 152 "QC_LoggerLevel.qpp"
return QoreLoggerLevel::getNextLowerLevel(level);
}

DLLLOCAL void preinitLoggerLevelClass() {
    QC_LOGGERLEVEL = new QoreBuiltinClass("LoggerLevel", "::Qore::Logger::LoggerLevel", QDOM_DEFAULT);
    CID_LOGGERLEVEL = QC_LOGGERLEVEL->getID();
    QC_LOGGERLEVEL->setSystem();
}

DLLLOCAL QoreClass* initLoggerLevelClass(QoreNamespace& ns) {
    if (!QC_LOGGERLEVEL)
        preinitLoggerLevelClass();

    // set serializer function
    QC_LOGGERLEVEL->setSerializer(reinterpret_cast<q_serializer_t>(LoggerLevel_serializer));

    // set deserializer function
    QC_LOGGERLEVEL->setDeserializer(LoggerLevel_deserializer);

    // set parent class
    assert(QC_SERIALIZABLE);
    QC_LOGGERLEVEL->addBuiltinVirtualBaseClass(QC_SERIALIZABLE);

    // LoggerLevel::constructor(int code, string level) {}
    QC_LOGGERLEVEL->addConstructor(LoggerLevel_constructor_ViVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "code", stringTypeInfo, QORE_PARAM_NO_ARG, "level");

    // string LoggerLevel::getStr(){}
    QC_LOGGERLEVEL->addMethod("getStr", (q_method_n_t)LoggerLevel_getStr, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // int LoggerLevel::getValue(){}
    QC_LOGGERLEVEL->addMethod("getValue", (q_method_n_t)LoggerLevel_getValue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // bool LoggerLevel::isEqual(LoggerLevel other){}
    QC_LOGGERLEVEL->addMethod("isEqual", (q_method_n_t)LoggerLevel_isEqual_C11LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "other");

    // bool LoggerLevel::isGreaterOrEqual(LoggerLevel other){}
    QC_LOGGERLEVEL->addMethod("isGreaterOrEqual", (q_method_n_t)LoggerLevel_isGreaterOrEqual_C11LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "other");

    // static LoggerLevel LoggerLevel::getLevel(int level, *LoggerLevel default_level){}
    QC_LOGGERLEVEL->addStaticMethod("getLevel", (q_func_n_t)static_LoggerLevel_getLevel_ViC12_LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo(), 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level", QC_LOGGERLEVEL->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "default_level");

    // static LoggerLevel LoggerLevel::getLevel(string level_str, *LoggerLevel default_level){}
    QC_LOGGERLEVEL->addStaticMethod("getLevel", (q_func_n_t)static_LoggerLevel_getLevel_VsC12_LoggerLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo(), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "level_str", QC_LOGGERLEVEL->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "default_level");

    // static LoggerLevel LoggerLevel::getLevelAll(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelAll", (q_func_n_t)static_LoggerLevel_getLevelAll, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelDebug(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelDebug", (q_func_n_t)static_LoggerLevel_getLevelDebug, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelDetail(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelDetail", (q_func_n_t)static_LoggerLevel_getLevelDetail, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelError(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelError", (q_func_n_t)static_LoggerLevel_getLevelError, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelFatal(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelFatal", (q_func_n_t)static_LoggerLevel_getLevelFatal, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelInfo(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelInfo", (q_func_n_t)static_LoggerLevel_getLevelInfo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelOff(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelOff", (q_func_n_t)static_LoggerLevel_getLevelOff, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelTrace(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelTrace", (q_func_n_t)static_LoggerLevel_getLevelTrace, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static LoggerLevel LoggerLevel::getLevelWarn(){}
    QC_LOGGERLEVEL->addStaticMethod("getLevelWarn", (q_func_n_t)static_LoggerLevel_getLevelWarn, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // static *LoggerLevel LoggerLevel::getNextHigherLevel(int level){}
    QC_LOGGERLEVEL->addStaticMethod("getNextHigherLevel", (q_func_n_t)static_LoggerLevel_getNextHigherLevel_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getOrNothingTypeInfo(), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level");

    // static *LoggerLevel LoggerLevel::getNextLowerLevel(int level){}
    QC_LOGGERLEVEL->addStaticMethod("getNextLowerLevel", (q_func_n_t)static_LoggerLevel_getNextLowerLevel_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getOrNothingTypeInfo(), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level");

    return QC_LOGGERLEVEL;
}
