/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractClass.cpp AbstractClass class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include <QC_AbstractClass.h>
#include <QC_Class.h>
#include <QC_PseudoClass.h>
#include <QC_AbstractMethod.h>
#include <QC_NormalMethod.h>
#include <QC_StaticMethod.h>
#include <QC_ConstructorMethod.h>
#include <QC_DestructorMethod.h>
#include <QC_CopyMethod.h>
#include <QC_PseudoMethod.h>
#include <QC_AbstractReflectionFunction.h>
#include <QC_AbstractMethodVariant.h>
#include <QC_NormalMethodVariant.h>
#include <QC_StaticMethodVariant.h>
#include <QC_ConstructorMethodVariant.h>
#include <QC_DestructorMethodVariant.h>
#include <QC_CopyMethodVariant.h>
#include <QC_AbstractClassMember.h>
#include <QC_NormalMember.h>
#include <QC_StaticMember.h>
#include <QC_ClassConstant.h>
#include <QC_Type.h>
#include <QC_Namespace.h>

QoreReflectionClass::QoreReflectionClass(ExceptionSink* xsink, const char* name, QoreProgram* pgm) : AbstractReflectionObject(pgm) {
    cls = pgm->findClass(name, xsink);
    if (!cls) {
        if (!*xsink) {
            xsink->raiseException("UNKNOWN-CLASS", "cannot resolve class '%s'", name);
        }
        return;
    }
}

QoreReflectionClass::QoreReflectionClass(QoreProgram* pgm, const QoreClass* cls) : AbstractReflectionObject(pgm), cls(cls) {
}

QoreObject* get_class_object(QoreProgram* pgm, const QoreClass* cls) {
    return new QoreObject(cls->isPseudoClass() ? QC_PSEUDOCLASS : QC_CLASS, pgm, new QoreReflectionClass(pgm, cls));
}

QoreObject* get_method_object(ReferenceHolder<QoreReflectionMethod>& m, ExceptionSink* xsink) {
    QoreProgram* pgm = m->pgm;
    switch (m->mtype) {
        case MT_Normal: return new QoreObject(QC_NORMALMETHOD, pgm, m.release());
        case MT_Static: return new QoreObject(QC_STATICMETHOD, pgm, m.release());
        case MT_Constructor: return new QoreObject(QC_CONSTRUCTORMETHOD, pgm, m.release());
        case MT_Destructor: return new QoreObject(QC_DESTRUCTORMETHOD, pgm, m.release());
        case MT_Copy: return new QoreObject(QC_COPYMETHOD, pgm, m.release());
        case MT_Pseudo: return new QoreObject(QC_PSEUDOMETHOD, pgm, m.release());
        default:
            break;
    }

    assert(false);
    return nullptr;
}

QoreObject* find_variant(const QoreReflectionClass* c, const char* name, unsigned which, const QoreListNode* args, size_t offset, ExceptionSink* xsink) {
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return nullptr;
    }

    // the class ptr must be valid
    // first find the method
    const QoreMethod* m = nullptr;
    method_type_e mtype = MT_None;
    if (which & (FV_NORMAL | FV_CONSTRUCTOR | FV_DESTRUCTOR | FV_COPY | FV_PSEUDO)) {
        m = c->cls->findLocalMethod(name);
        if (m) {
            mtype = m->getMethodType();
            switch (mtype) {
                case MT_Normal:
                    if (!(which & FV_NORMAL)) {
                        m = nullptr;
                    }
                    break;
                case MT_Constructor:
                    if (!(which & FV_CONSTRUCTOR)) {
                        m = nullptr;
                    }
                    break;
                case MT_Destructor:
                    if (!(which & FV_DESTRUCTOR)) {
                        m = nullptr;
                    }
                    break;
                case MT_Copy:
                    if (!(which & FV_COPY)) {
                        m = nullptr;
                    }
                    break;
                case MT_Pseudo:
                    if (!(which & FV_PSEUDO)) {
                        m = nullptr;
                    }
                    break;
                default:
                    assert(false);
                    break;
            }
        }
    }
    if (!m && (which & FV_STATIC)) {
        m = c->cls->findLocalStaticMethod(name);
        if (m) {
            mtype = MT_Static;
        }
    }
    if (!m) {
        const char* ms;
        switch (which) {
            case FV_ALL: ms = "any"; break;
            case FV_STATIC: ms = "static"; break;
            case FV_NORMAL: ms = "normal (non-static, non-special)"; break;
            case FV_CONSTRUCTOR: ms = "constructor"; break;
            case FV_DESTRUCTOR: ms = "destructor"; break;
            case FV_COPY: ms = "copy"; break;
            case FV_PSEUDO: ms = "pseudo"; break;
            default: assert(false); break;
        }

        if (which == FV_DESTRUCTOR || which == FV_COPY) {
            xsink->raiseException("METHOD-ERROR", "cannot resolve %s method in class '%s'", ms,
                c->cls->getName());
        } else {
            xsink->raiseException("METHOD-ERROR", "cannot resolve %s method '%s' in class '%s'", ms, name,
                c->cls->getName());
        }
        return nullptr;
    }

    const QoreExternalFunction* func = m->getFunction();
    return find_function_variant(c->pgm, func, args, offset, mtype, xsink);
}

QoreObject* find_variant(const QoreReflectionClass* c, const QoreStringNode* name, unsigned which, const QoreListNode* args, size_t offset, ExceptionSink* xsink) {
    TempEncodingHelper nname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return nullptr;
    }

    return find_variant(c, nname->c_str(), which, args, offset, xsink);
}
TypedHashDecl* init_hashdecl_ClassAccessInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("ClassAccessInfo", "::Qore::ClassAccessInfo");
    hd->addMember("access", bigIntTypeInfo, QoreValue());
    hd->addMember("access_string", stringTypeInfo, QoreValue());
    hd->addMember("cls", QC_ABSTRACTCLASS->getTypeInfo(), QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::Reflection::AbstractClass */

qore_classid_t CID_ABSTRACTCLASS;
QoreClass* QC_ABSTRACTCLASS;

// AbstractClass::constructor() {}
static void AbstractClass_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 305 "QC_AbstractClass.qpp"
xsink->raiseException("ABSTRACTCLASS-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

// int AbstractClass::getDomain(){}
static QoreValue AbstractClass_getDomain(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 692 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->getDomain();
}

// list<string> AbstractClass::getDomainStringList(){}
static QoreValue AbstractClass_getDomainStringList(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 717 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return domain_bitfield_to_string_list(c->cls->getDomain(), xsink);
}

// binary AbstractClass::getHash(){}
static QoreValue AbstractClass_getHash(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 604 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->getBinaryHash();
}

// int AbstractClass::getId(){}
static QoreValue AbstractClass_getId(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 583 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return (int64)c->cls->getID();
}

// string AbstractClass::getLanguage(){}
static QoreValue AbstractClass_getLanguage(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 343 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }
    // the class ptr must be valid
    return new QoreStringNode(c->cls->getLanguage());
}

// AbstractMethod AbstractClass::getMethod(string name){}
static QoreValue AbstractClass_getMethod_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 809 "QC_AbstractClass.qpp"
    TempEncodingHelper nname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->cls, nname->c_str(), xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return get_method_object(m, xsink);
}

// list<AbstractMethod> AbstractClass::getMethods(){}
static QoreValue AbstractClass_getMethods(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 839 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_ABSTRACTMETHOD->getTypeInfo()), xsink);

    {
        QoreMethodIterator i(*c->cls);
        while (i.next()) {
            ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->pgm, i.getMethod()), xsink);
            l->push(get_method_object(m, xsink), xsink);
            if (*xsink) {
                return QoreValue();
            }
        }
    }

    {
        QoreStaticMethodIterator i(*c->cls);
        while (i.next()) {
            ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->pgm, i.getMethod()), xsink);
            l->push(get_method_object(m, xsink), xsink);
            if (*xsink) {
                return QoreValue();
            }
        }
    }

    return l.release();
}

// list<string> AbstractClass::getModifierList(){}
static QoreValue AbstractClass_getModifierList(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 435 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), xsink);
    if (c->cls->isAbstract()) {
        rv->push(new QoreStringNode("abstract"), xsink);
    }
    if (c->cls->isModulePublic()) {
        rv->push(new QoreStringNode("public"), xsink);
    }
    if (c->cls->isFinal()) {
        rv->push(new QoreStringNode("final"), xsink);
    }

    return rv.release();
}

// int AbstractClass::getModifiers(){}
static QoreValue AbstractClass_getModifiers(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 398 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    int64 rc = 0;
    if (c->cls->isAbstract()) {
        rc |= MC_ABSTRACT;
    }
    if (c->cls->isModulePublic()) {
        rc |= MC_PUBLIC;
    }
    if (c->cls->isFinal()) {
        rc |= MC_FINAL;
    }

    return rc;
}

// string AbstractClass::getName(){}
static QoreValue AbstractClass_getName(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 321 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }
    // the class ptr must be valid
    return new QoreStringNode(c->cls->getName());
}

// list<hash<ClassAccessInfo>> AbstractClass::getParentClasses(){}
static QoreValue AbstractClass_getParentClasses(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 912 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(hashdeclClassAccessInfo->getTypeInfo()), xsink);

    QoreParentClassIterator i(*c->cls);
    while (i.next()) {
        ReferenceHolder<QoreHashNode> h(new QoreHashNode(hashdeclClassAccessInfo, nullptr), xsink);
        h->setKeyValue("cls", get_class_object(c->pgm, &i.getParentClass()), xsink);
        h->setKeyValue("access", get_access(i.getAccess()), xsink);
        h->setKeyValue("access_string", new QoreStringNode(get_access_string(i.getAccess())), xsink);

        l->push(h.release(), xsink);
    }

    return l.release();
}

// AbstractMethodVariant AbstractClass::getVariant(string name, ...){}
static QoreValue AbstractClass_getVariant_VsVV(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 753 "QC_AbstractClass.qpp"
    return find_variant(c, name, FV_ALL, args, 1, xsink);
}

// AbstractMethodVariant AbstractClass::getVariantArgs(string name, *softlist<Type> argv){}
static QoreValue AbstractClass_getVariantArgs_VsC15_softlist_Type_(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 782 "QC_AbstractClass.qpp"
    return find_variant(c, name, FV_ALL, argv, 0, xsink);
}

// list<AbstractMethodVariant> AbstractClass::getVariants(){}
static QoreValue AbstractClass_getVariants(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 883 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(QC_ABSTRACTMETHODVARIANT->getTypeInfo()), xsink);

    QoreMethodIterator i(*c->cls);
    while (i.next()) {
        const QoreMethod* m = i.getMethod();
        append_variant_objects(**rv, c->pgm, m->getFunction(), m->getMethodType(), xsink);
    }

    return rv.release();
}

// bool AbstractClass::hasMemberGate(){}
static QoreValue AbstractClass_hasMemberGate(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 625 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->hasMemberGate();
}

// bool AbstractClass::hasMemberNotification(){}
static QoreValue AbstractClass_hasMemberNotification(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 667 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->hasMemberNotification();
}

// bool AbstractClass::hasMethodGate(){}
static QoreValue AbstractClass_hasMethodGate(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 646 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->hasMethodGate();
}

// bool AbstractClass::isAbstract(){}
static QoreValue AbstractClass_isAbstract(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 503 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->isAbstract();
}

// bool AbstractClass::isBuiltin(){}
static QoreValue AbstractClass_isBuiltin(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 465 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->isSystem();
}

// bool AbstractClass::isEqual(AbstractClass cls){}
static QoreValue AbstractClass_isEqual_C13AbstractClass(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(cls, QoreReflectionClass, args, 0, CID_ABSTRACTCLASS, "AbstractClass::isEqual()", "AbstractClass", xsink);
    if (*xsink)
        return 0;
# 365 "QC_AbstractClass.qpp"
    ReferenceHolder<QoreReflectionClass> holder(cls, xsink);

    // get read access to program object
    QoreExternalProgramContextHelper tch1(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch2(xsink, cls->pgm);
    if (*xsink) {
        return QoreValue();
    }

    return c->cls->isEqual(*cls->cls);
}

// bool AbstractClass::isInjected(){}
static QoreValue AbstractClass_isInjected(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 522 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->isInjected();
}

// bool AbstractClass::isModulePublic(){}
static QoreValue AbstractClass_isModulePublic(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 541 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->isModulePublic();
}

// bool AbstractClass::isPseudoClass(){}
static QoreValue AbstractClass_isPseudoClass(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 560 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return c->cls->isPseudoClass();
}

// bool AbstractClass::isUser(){}
static QoreValue AbstractClass_isUser(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 484 "QC_AbstractClass.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return !c->cls->isSystem();
}

DLLLOCAL void preinitAbstractClassClass() {
    QC_ABSTRACTCLASS = new QoreBuiltinClass("AbstractClass", "::Qore::Reflection::AbstractClass", QDOM_REFLECTION);
    CID_ABSTRACTCLASS = QC_ABSTRACTCLASS->getID();
    QC_ABSTRACTCLASS->setSystem();
}

DLLLOCAL QoreClass* initAbstractClassClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTCLASS)
        preinitAbstractClassClass();

    // AbstractClass::constructor() {}
    QC_ABSTRACTCLASS->addConstructor(AbstractClass_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // int AbstractClass::getDomain(){}
    QC_ABSTRACTCLASS->addMethod("getDomain", (q_method_n_t)AbstractClass_getDomain, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // list<string> AbstractClass::getDomainStringList(){}
    QC_ABSTRACTCLASS->addMethod("getDomainStringList", (q_method_n_t)AbstractClass_getDomainStringList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // binary AbstractClass::getHash(){}
    QC_ABSTRACTCLASS->addMethod("getHash", (q_method_n_t)AbstractClass_getHash, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo);

    // int AbstractClass::getId(){}
    QC_ABSTRACTCLASS->addMethod("getId", (q_method_n_t)AbstractClass_getId, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // string AbstractClass::getLanguage(){}
    QC_ABSTRACTCLASS->addMethod("getLanguage", (q_method_n_t)AbstractClass_getLanguage, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // AbstractMethod AbstractClass::getMethod(string name){}
    QC_ABSTRACTCLASS->addMethod("getMethod", (q_method_n_t)AbstractClass_getMethod_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_ABSTRACTMETHOD->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<AbstractMethod> AbstractClass::getMethods(){}
    QC_ABSTRACTCLASS->addMethod("getMethods", (q_method_n_t)AbstractClass_getMethods, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_ABSTRACTMETHOD->getTypeInfo()));

    // list<string> AbstractClass::getModifierList(){}
    QC_ABSTRACTCLASS->addMethod("getModifierList", (q_method_n_t)AbstractClass_getModifierList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int AbstractClass::getModifiers(){}
    QC_ABSTRACTCLASS->addMethod("getModifiers", (q_method_n_t)AbstractClass_getModifiers, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // string AbstractClass::getName(){}
    QC_ABSTRACTCLASS->addMethod("getName", (q_method_n_t)AbstractClass_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // list<hash<ClassAccessInfo>> AbstractClass::getParentClasses(){}
    QC_ABSTRACTCLASS->addMethod("getParentClasses", (q_method_n_t)AbstractClass_getParentClasses, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(hashdeclClassAccessInfo->getTypeInfo(false)));

    // AbstractMethodVariant AbstractClass::getVariant(string name, ...){}
    QC_ABSTRACTCLASS->addMethod("getVariant", (q_method_n_t)AbstractClass_getVariant_VsVV, Public, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, QC_ABSTRACTMETHODVARIANT->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // AbstractMethodVariant AbstractClass::getVariantArgs(string name, *softlist<Type> argv){}
    QC_ABSTRACTCLASS->addMethod("getVariantArgs", (q_method_n_t)AbstractClass_getVariantArgs_VsC15_softlist_Type_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_ABSTRACTMETHODVARIANT->getTypeInfo(), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", qore_get_complex_softlist_or_nothing_type(QC_TYPE->getTypeInfo()), QORE_PARAM_NO_ARG, "argv");

    // list<AbstractMethodVariant> AbstractClass::getVariants(){}
    QC_ABSTRACTCLASS->addMethod("getVariants", (q_method_n_t)AbstractClass_getVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_ABSTRACTMETHODVARIANT->getTypeInfo()));

    // bool AbstractClass::hasMemberGate(){}
    QC_ABSTRACTCLASS->addMethod("hasMemberGate", (q_method_n_t)AbstractClass_hasMemberGate, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::hasMemberNotification(){}
    QC_ABSTRACTCLASS->addMethod("hasMemberNotification", (q_method_n_t)AbstractClass_hasMemberNotification, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::hasMethodGate(){}
    QC_ABSTRACTCLASS->addMethod("hasMethodGate", (q_method_n_t)AbstractClass_hasMethodGate, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::isAbstract(){}
    QC_ABSTRACTCLASS->addMethod("isAbstract", (q_method_n_t)AbstractClass_isAbstract, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::isBuiltin(){}
    QC_ABSTRACTCLASS->addMethod("isBuiltin", (q_method_n_t)AbstractClass_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::isEqual(AbstractClass cls){}
    QC_ABSTRACTCLASS->addMethod("isEqual", (q_method_n_t)AbstractClass_isEqual_C13AbstractClass, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, QC_ABSTRACTCLASS->getTypeInfo(), QORE_PARAM_NO_ARG, "cls");

    // bool AbstractClass::isInjected(){}
    QC_ABSTRACTCLASS->addMethod("isInjected", (q_method_n_t)AbstractClass_isInjected, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::isModulePublic(){}
    QC_ABSTRACTCLASS->addMethod("isModulePublic", (q_method_n_t)AbstractClass_isModulePublic, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::isPseudoClass(){}
    QC_ABSTRACTCLASS->addMethod("isPseudoClass", (q_method_n_t)AbstractClass_isPseudoClass, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClass::isUser(){}
    QC_ABSTRACTCLASS->addMethod("isUser", (q_method_n_t)AbstractClass_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);
# 180 "QC_AbstractClass.qpp"
# 191 "QC_AbstractClass.qpp"
    ns.addConstant("AC_PRIVATE", ((int64)MC_PRIVATE));
    ns.addConstant("AC_PRIVATEINTERNAL", ((int64)MC_PRIVATEINTERNAL));
    ns.addConstant("AC_PUBLIC", ((int64)MC_PUBLIC));
# 202 "QC_AbstractClass.qpp"
# 213 "QC_AbstractClass.qpp"
# 218 "QC_AbstractClass.qpp"
# 223 "QC_AbstractClass.qpp"
# 226 "QC_AbstractClass.qpp"
# 229 "QC_AbstractClass.qpp"
# 232 "QC_AbstractClass.qpp"
# 235 "QC_AbstractClass.qpp"
    ns.addConstant("MC_ABSTRACT", ((int64)MC_ABSTRACT));
    ns.addConstant("MC_DEPRECATED", ((int64)MC_DEPRECATED));
    ns.addConstant("MC_FINAL", ((int64)MC_FINAL));
    ns.addConstant("MC_PRIVATE", ((int64)MC_PRIVATE));
    ns.addConstant("MC_PRIVATEINTERNAL", ((int64)MC_PRIVATEINTERNAL));
    ns.addConstant("MC_PUBLIC", ((int64)MC_PUBLIC));
    ns.addConstant("MC_STATIC", ((int64)MC_STATIC));
    ns.addConstant("MC_SYNCHRONIZED", ((int64)MC_SYNCHRONIZED));
    ns.addConstant("MC_TRANSIENT", ((int64)MC_TRANSIENT));
# 242 "QC_AbstractClass.qpp"
# 254 "QC_AbstractClass.qpp"
# 259 "QC_AbstractClass.qpp"
# 264 "QC_AbstractClass.qpp"
# 269 "QC_AbstractClass.qpp"
    ns.addConstant("CF_CONSTANT", ((int64)QCF_CONSTANT_INTERN));
    ns.addConstant("CF_DEPRECATED", ((int64)QCF_DEPRECATED));
    ns.addConstant("CF_NOOP", ((int64)QCF_NOOP));
    ns.addConstant("CF_RET_VALUE_ONLY", ((int64)QCF_RET_VALUE_ONLY));
    ns.addConstant("CF_RUNTIME_NOOP", ((int64)QCF_RUNTIME_NOOP));
    ns.addConstant("CF_USES_EXTRA_ARGS", ((int64)QCF_USES_EXTRA_ARGS));

    return QC_ABSTRACTCLASS;
}
