/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractVariant.cpp AbstractVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_AbstractVariant.h"
#include "QC_AbstractReflectionFunction.h"
#include "QC_Type.h"
#include "QC_Class.h"

QoreListNode* get_code_flag_list(int64 c) {
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), nullptr);

    if (c & QCF_NOOP) {
        rv->push(new QoreStringNode("NOOP"), nullptr);
    }

    if (c & QCF_USES_EXTRA_ARGS) {
        rv->push(new QoreStringNode("USES_EXTRA_ARGS"), nullptr);
    }

    if (c & QCF_CONSTANT_INTERN) {
        rv->push(new QoreStringNode("CONSTANT"), nullptr);
    }

    if (c & QCF_DEPRECATED) {
        rv->push(new QoreStringNode("DEPRECATED"), nullptr);
    }

    if (c & QCF_RET_VALUE_ONLY) {
        rv->push(new QoreStringNode("RET_VALUE_ONLY"), nullptr);
    }

    if (c & QCF_RUNTIME_NOOP) {
        rv->push(new QoreStringNode("RUNTIME_NOOP"), nullptr);
    }

    return rv.release();
}
/* Qore class Qore::Reflection::AbstractVariant */

qore_classid_t CID_ABSTRACTVARIANT;
QoreClass* QC_ABSTRACTVARIANT;

// AbstractVariant::constructor() {}
static void AbstractVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 81 "QC_AbstractVariant.qpp"
xsink->raiseException("ABSTRACTVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

// bool AbstractVariant::empty(){}
static QoreValue AbstractVariant_empty(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 441 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    // we have to check for abstract methods here and return True, because QoreExternalVariant::hasBody()
    // returns True for builtin abstract variants
    if (v->variant->getClass() && static_cast<const QoreExternalMethodVariant*>(v->variant)->isAbstract()) {
        return true;
    }

    // the variant ptr must be valid
    return !v->variant->hasBody();
}

// list<string> AbstractVariant::getCodeFlagList(){}
static QoreValue AbstractVariant_getCodeFlagList(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 145 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return get_code_flag_list(v->variant->getCodeFlags());
}

// int AbstractVariant::getCodeFlags(){}
static QoreValue AbstractVariant_getCodeFlags(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 116 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->variant->getCodeFlags();
}

// list<auto> AbstractVariant::getDefaultArgs(){}
static QoreValue AbstractVariant_getDefaultArgs(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 703 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(autoTypeInfo), xsink);

    for (auto& i : v->variant->getDefaultArgList()) {
        rv->push(i.refSelf(), xsink);
    }

    return rv.release();
}

// int AbstractVariant::getDomain(){}
static QoreValue AbstractVariant_getDomain(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 295 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->variant->getDomain();
}

// list<string> AbstractVariant::getDomainStringList(){}
static QoreValue AbstractVariant_getDomainStringList(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 318 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return domain_bitfield_to_string_list(v->variant->getDomain(), xsink);
}

// AbstractReflectionFunction AbstractVariant::getFunction(){}
static QoreValue AbstractVariant_getFunction(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 272 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->getFunctionObject(xsink);
}

// list<string> AbstractVariant::getModifierList(){}
static QoreValue AbstractVariant_getModifierList(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 228 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), xsink);
    if (v->variant->getClass()) {
        const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
        if (mv->isAbstract()) {
            rv->push(new QoreStringNode("abstract"), xsink);
        }
        if (mv->isFinal()) {
            rv->push(new QoreStringNode("final"), xsink);
        }
        rv->push(new QoreStringNode(mv->getAccessString()), xsink);
        if (mv->isStatic()) {
            rv->push(new QoreStringNode("static"), xsink);
        }
    } else {
        if (v->variant->isModulePublic()) {
            rv->push(new QoreStringNode("public"), xsink);
        }
    }

    if (v->variant->isSynchronized()) {
        rv->push(new QoreStringNode("synchronized"), xsink);
    }

    return rv.release();
}

// int AbstractVariant::getModifiers(){}
static QoreValue AbstractVariant_getModifiers(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 175 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    int64 rc = 0;
    if (v->variant->getClass()) {
        const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
        if (mv->isAbstract()) {
            rc |= MC_ABSTRACT;
        }
        if (mv->isFinal()) {
            rc |= MC_FINAL;
        }
        rc |= get_access(mv->getAccess());
        if (mv->isStatic()) {
            rc |= MC_STATIC;
        }
    } else {
        if (v->variant->isModulePublic()) {
            rc |= MC_PUBLIC;
        }
    }

    if (v->variant->isSynchronized()) {
        rc |= MC_SYNCHRONIZED;
    }

    return rc;
}

// list<string> AbstractVariant::getParamNames(){}
static QoreValue AbstractVariant_getParamNames(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 730 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), xsink);

    for (auto& i : v->variant->getParamNames()) {
        rv->push(i.empty() ? QoreValue() : QoreValue(new QoreStringNode(i)), xsink);
    }

    return rv.release();
}

// string AbstractVariant::getParamString(){}
static QoreValue AbstractVariant_getParamString(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 535 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return new QoreStringNode(v->variant->getSignatureText());
}

// list<Type> AbstractVariant::getParamTypes(){}
static QoreValue AbstractVariant_getParamTypes(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 676 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(QC_TYPE->getTypeInfo()), xsink);

    for (auto& i : v->variant->getParamTypeList()) {
        rv->push(new QoreObject(QC_TYPE, v->pgm, new QoreType(i)), xsink);
    }

    return rv.release();
}

// Type AbstractVariant::getReturnType(){}
static QoreValue AbstractVariant_getReturnType(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 655 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return new QoreObject(QC_TYPE, v->pgm, new QoreType(v->variant->getReturnTypeInfo()));
}

// string AbstractVariant::getReturnTypeString(){}
static QoreValue AbstractVariant_getReturnTypeString(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 557 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return new QoreStringNode(type_get_name(v->variant->getReturnTypeInfo()));
}

// hash<SourceLocationInfo> AbstractVariant::getSourceLocation(){}
static QoreValue AbstractVariant_getSourceLocation(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 757 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->variant->getSourceLocation()->getHash();
}

// bool AbstractVariant::hasBody(){}
static QoreValue AbstractVariant_hasBody(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 410 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // we have to check for abstract methods here and return False, because QoreExternalVariant::hasBody()
    // returns True for builtin abstract variants
    if (v->variant->getClass() && static_cast<const QoreExternalMethodVariant*>(v->variant)->isAbstract()) {
        return false;
    }

    // the variant ptr must be valid
    return v->variant->hasBody();
}

// bool AbstractVariant::isBuiltin(){}
static QoreValue AbstractVariant_isBuiltin(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 341 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->variant->isBuiltin();
}

// bool AbstractVariant::isEqual(AbstractVariant variant){}
static QoreValue AbstractVariant_isEqual_C15AbstractVariant(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(variant, QoreReflectionVariant, args, 0, CID_ABSTRACTVARIANT, "AbstractVariant::isEqual()", "AbstractVariant", xsink);
    if (*xsink)
        return 0;
# 98 "QC_AbstractVariant.qpp"
    ReferenceHolder<QoreReflectionVariant> holder(variant, xsink);

    return variant->variant == v->variant;
}

// bool AbstractVariant::isInjected(){}
static QoreValue AbstractVariant_isInjected(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 385 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return v->f->isInjected();
}

// bool AbstractVariant::isMethodVariant(){}
static QoreValue AbstractVariant_isMethodVariant(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 469 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    const QoreClass* cls = v->variant->getClass();
    return (bool)(cls && !cls->isPseudoClass());
}

// bool AbstractVariant::isPseudoMethodVariant(){}
static QoreValue AbstractVariant_isPseudoMethodVariant(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 491 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    const QoreClass* cls = v->variant->getClass();
    return (bool)(cls && cls->isPseudoClass());
}

// bool AbstractVariant::isUser(){}
static QoreValue AbstractVariant_isUser(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 364 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return !v->variant->isBuiltin();
}

// int AbstractVariant::numParams(){}
static QoreValue AbstractVariant_numParams(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 513 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->variant->numParams();
}

// string AbstractVariant::toString(){}
static QoreValue AbstractVariant_toString(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 578 "QC_AbstractVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode);

    // do not output type for constructor, destructor, and copy methods
    bool no_type = false;

    const QoreClass* cls = v->variant->getClass();
    if (cls) {
        const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
        if (mv->isFinal()) {
            rv->concat("final ");
        }

        if (mv->isStatic()) {
            rv->concat("static ");
        }

        if (mv->isAbstract()) {
            rv->concat("abstract ");
        }

        rv->concat(mv->getAccessString());
        rv->concat(' ');

        method_type_e mt = mv->getMethod()->getMethodType();
        if (mt == MT_Constructor || mt == MT_Destructor || mt == MT_Copy) {
            no_type = true;
        }
    } else {
        // it is a function variant
        if (v->variant->isModulePublic()) {
            rv->concat("public ");
        }
    }

    // check for synchronized variants
    if (v->variant->isSynchronized()) {
        rv->concat("synchronized ");
    }

    // add return type
    if (!no_type) {
        rv->concat(type_get_name(v->variant->getReturnTypeInfo()));
        rv->concat(' ');
    }

    if (cls) {
        rv->concat(cls->getName());
        rv->concat("::");
    }

    rv->concat(v->f->getName());
    // add parameters
    rv->concat('(');
    rv->concat(v->variant->getSignatureText());
    rv->concat(')');

    return rv.release();
}

DLLLOCAL void preinitAbstractVariantClass() {
    QC_ABSTRACTVARIANT = new QoreBuiltinClass("AbstractVariant", "::Qore::Reflection::AbstractVariant", QDOM_REFLECTION);
    CID_ABSTRACTVARIANT = QC_ABSTRACTVARIANT->getID();
    QC_ABSTRACTVARIANT->setSystem();
}

DLLLOCAL QoreClass* initAbstractVariantClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTVARIANT)
        preinitAbstractVariantClass();

    // AbstractVariant::constructor() {}
    QC_ABSTRACTVARIANT->addConstructor(AbstractVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // bool AbstractVariant::empty(){}
    QC_ABSTRACTVARIANT->addMethod("empty", (q_method_n_t)AbstractVariant_empty, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // list<string> AbstractVariant::getCodeFlagList(){}
    QC_ABSTRACTVARIANT->addMethod("getCodeFlagList", (q_method_n_t)AbstractVariant_getCodeFlagList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int AbstractVariant::getCodeFlags(){}
    QC_ABSTRACTVARIANT->addMethod("getCodeFlags", (q_method_n_t)AbstractVariant_getCodeFlags, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // list<auto> AbstractVariant::getDefaultArgs(){}
    QC_ABSTRACTVARIANT->addMethod("getDefaultArgs", (q_method_n_t)AbstractVariant_getDefaultArgs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo);

    // int AbstractVariant::getDomain(){}
    QC_ABSTRACTVARIANT->addMethod("getDomain", (q_method_n_t)AbstractVariant_getDomain, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // list<string> AbstractVariant::getDomainStringList(){}
    QC_ABSTRACTVARIANT->addMethod("getDomainStringList", (q_method_n_t)AbstractVariant_getDomainStringList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // AbstractReflectionFunction AbstractVariant::getFunction(){}
    QC_ABSTRACTVARIANT->addMethod("getFunction", (q_method_n_t)AbstractVariant_getFunction, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_ABSTRACTREFLECTIONFUNCTION->getTypeInfo());

    // list<string> AbstractVariant::getModifierList(){}
    QC_ABSTRACTVARIANT->addMethod("getModifierList", (q_method_n_t)AbstractVariant_getModifierList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int AbstractVariant::getModifiers(){}
    QC_ABSTRACTVARIANT->addMethod("getModifiers", (q_method_n_t)AbstractVariant_getModifiers, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // list<string> AbstractVariant::getParamNames(){}
    QC_ABSTRACTVARIANT->addMethod("getParamNames", (q_method_n_t)AbstractVariant_getParamNames, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // string AbstractVariant::getParamString(){}
    QC_ABSTRACTVARIANT->addMethod("getParamString", (q_method_n_t)AbstractVariant_getParamString, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // list<Type> AbstractVariant::getParamTypes(){}
    QC_ABSTRACTVARIANT->addMethod("getParamTypes", (q_method_n_t)AbstractVariant_getParamTypes, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_TYPE->getTypeInfo()));

    // Type AbstractVariant::getReturnType(){}
    QC_ABSTRACTVARIANT->addMethod("getReturnType", (q_method_n_t)AbstractVariant_getReturnType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // string AbstractVariant::getReturnTypeString(){}
    QC_ABSTRACTVARIANT->addMethod("getReturnTypeString", (q_method_n_t)AbstractVariant_getReturnTypeString, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // hash<SourceLocationInfo> AbstractVariant::getSourceLocation(){}
    QC_ABSTRACTVARIANT->addMethod("getSourceLocation", (q_method_n_t)AbstractVariant_getSourceLocation, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSourceLocationInfo->getTypeInfo(false));

    // bool AbstractVariant::hasBody(){}
    QC_ABSTRACTVARIANT->addMethod("hasBody", (q_method_n_t)AbstractVariant_hasBody, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractVariant::isBuiltin(){}
    QC_ABSTRACTVARIANT->addMethod("isBuiltin", (q_method_n_t)AbstractVariant_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractVariant::isEqual(AbstractVariant variant){}
    QC_ABSTRACTVARIANT->addMethod("isEqual", (q_method_n_t)AbstractVariant_isEqual_C15AbstractVariant, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_ABSTRACTVARIANT->getTypeInfo(), QORE_PARAM_NO_ARG, "variant");

    // bool AbstractVariant::isInjected(){}
    QC_ABSTRACTVARIANT->addMethod("isInjected", (q_method_n_t)AbstractVariant_isInjected, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractVariant::isMethodVariant(){}
    QC_ABSTRACTVARIANT->addMethod("isMethodVariant", (q_method_n_t)AbstractVariant_isMethodVariant, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractVariant::isPseudoMethodVariant(){}
    QC_ABSTRACTVARIANT->addMethod("isPseudoMethodVariant", (q_method_n_t)AbstractVariant_isPseudoMethodVariant, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractVariant::isUser(){}
    QC_ABSTRACTVARIANT->addMethod("isUser", (q_method_n_t)AbstractVariant_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // int AbstractVariant::numParams(){}
    QC_ABSTRACTVARIANT->addMethod("numParams", (q_method_n_t)AbstractVariant_numParams, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // string AbstractVariant::toString(){}
    QC_ABSTRACTVARIANT->addMethod("toString", (q_method_n_t)AbstractVariant_toString, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    return QC_ABSTRACTVARIANT;
}
