/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_Class.cpp Class class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include <QC_Class.h>
#include <QC_AbstractMethod.h>
#include <QC_NormalMethod.h>
#include <QC_StaticMethod.h>
#include <QC_ConstructorMethod.h>
#include <QC_DestructorMethod.h>
#include <QC_CopyMethod.h>
#include <QC_AbstractReflectionFunction.h>
#include <QC_AbstractMethodVariant.h>
#include <QC_NormalMethodVariant.h>
#include <QC_StaticMethodVariant.h>
#include <QC_ConstructorMethodVariant.h>
#include <QC_DestructorMethodVariant.h>
#include <QC_CopyMethodVariant.h>
#include <QC_AbstractClassMember.h>
#include <QC_NormalMember.h>
#include <QC_StaticMember.h>
#include <QC_ClassConstant.h>
#include <QC_Type.h>
#include <QC_Namespace.h>

int check_constructor(const QoreClass* cls, ExceptionSink* xsink) {
    if (getProgram()->getParseOptions64() & cls->getDomain()) {
        xsink->raiseException("CREATE-OBJECT-ERROR", "Program sandboxing restrictions do not allow access to the " \
            "'%s' class", cls->getName());
        return -1;
    }

    return 0;
}

static QoreHashNode* find_method_intern(const QoreStringNode* name, QoreReflectionClass* c, method_type_e type,
    ExceptionSink* xsink) {
    TempEncodingHelper nname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return nullptr;
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return nullptr;
    }

    QoreReflectionHierarchyMethod* method;
    ReferenceHolder<QoreReflectionMethod> m(method = new QoreReflectionHierarchyMethod(c->cls, nname->c_str(), xsink,
        type), xsink);
    if (*xsink) {
        return nullptr;
    }

    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(hashdeclMethodAccessInfo, xsink), xsink);
    rv->setKeyValue("method", get_method_object(m, xsink), xsink);
    rv->setKeyValue("access", get_access(method->getClassAccess()), xsink);
    rv->setKeyValue("access_string", new QoreStringNode(get_access_string(method->getClassAccess())), xsink);

    return rv.release();
}

static QoreValue find_all_classes(const QoreString& pattern, int re_opts, QoreProgram* pgm, ExceptionSink* xsink) {
    class_vec_t class_vec = pgm->findAllClassesRegex(pattern, re_opts, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_CLASS->getTypeInfo()), xsink);
    for (auto& i : class_vec) {
        l->push(new QoreObject(QC_CLASS, pgm, new QoreReflectionClass(pgm, i)), xsink);
    }

    return l.release();
}
TypedHashDecl* init_hashdecl_MethodAccessInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("MethodAccessInfo", "::Qore::MethodAccessInfo");
    hd->addMember("access", bigIntTypeInfo, QoreValue());
    hd->addMember("access_string", stringTypeInfo, QoreValue());
    hd->addMember("method", QC_ABSTRACTMETHOD->getTypeInfo(), QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::Reflection::Class */

qore_classid_t CID_CLASS;
QoreClass* QC_CLASS;

// Class::constructor(string path) {}
static void Class_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 143 "QC_Class.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionClass> cls(new QoreReflectionClass(xsink, npath->c_str()), xsink);
    if (*xsink) {
        return;
    }
    self->setPrivate(CID_CLASS, cls.release());
}

// hash<MethodAccessInfo> Class::findMethod(string name){}
static QoreValue Class_findMethod_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 232 "QC_Class.qpp"
    return find_method_intern(name, c, MT_None, xsink);
}

// hash<MethodAccessInfo> Class::findNormalMethod(string name){}
static QoreValue Class_findNormalMethod_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 266 "QC_Class.qpp"
    return find_method_intern(name, c, MT_Normal, xsink);
}

// hash<MethodAccessInfo> Class::findStaticMethod(string name){}
static QoreValue Class_findStaticMethod_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 300 "QC_Class.qpp"
    return find_method_intern(name, c, MT_Static, xsink);
}

// list<Class> Class::getClassHierarchy(){}
static QoreValue Class_getClassHierarchy(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1119 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_CLASS->getTypeInfo()), xsink);

    QoreClassHierarchyIterator i(*c->cls);
    while (i.next()) {
        l->push(new QoreObject(QC_CLASS, c->pgm, new QoreReflectionClass(c->pgm, &i.get())), xsink);
    }

    return l.release();
}

// ClassConstant Class::getConstant(string name){}
static QoreValue Class_getConstant_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1320 "QC_Class.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    const QoreExternalConstant* ce = c->cls->findConstant(tname->c_str());
    if (!ce) {
        xsink->raiseException("UNKNOWN-CONSTANT", "constant '%s' is not defined in class '%s'", tname->c_str(),
            c->cls->getName());
        return QoreValue();
    }

    return new QoreObject(QC_CLASSCONSTANT, c->pgm, new QoreReflectionClassConstant(c->pgm, ce, c->cls));
}

// list<ClassConstant> Class::getConstants(){}
static QoreValue Class_getConstants(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1347 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_CLASSCONSTANT->getTypeInfo()), xsink);
    QoreClassConstantIterator i(*c->cls);
    while (i.next()) {
        l->push(new QoreObject(QC_CLASSCONSTANT, c->pgm, new QoreReflectionClassConstant(c->pgm, &i.get(), c->cls)),
            xsink);
    }

    return l.release();
}

// ConstructorMethod Class::getConstructorMethod(){}
static QoreValue Class_getConstructorMethod(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 689 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->cls, "constructor", xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }

    assert(m->mtype == MT_Constructor);

    QoreProgram* cls_pgm = m->pgm;
    return new QoreObject(QC_CONSTRUCTORMETHOD, cls_pgm, m.release());
}

// ConstructorMethodVariant Class::getConstructorVariant(...){}
static QoreValue Class_getConstructorVariant_VV(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 511 "QC_Class.qpp"
    return find_variant(c, "constructor", FV_CONSTRUCTOR, args, 0, xsink);
}

// ConstructorMethodVariant Class::getConstructorVariantArgs(*softlist<Type> argv){}
static QoreValue Class_getConstructorVariantArgs_C15_softlist_Type_(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 538 "QC_Class.qpp"
    return find_variant(c, "constructor", FV_CONSTRUCTOR, argv, 0, xsink);
}

// list<ConstructorMethodVariant> Class::getConstructorVariants(){}
static QoreValue Class_getConstructorVariants(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 891 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(QC_CONSTRUCTORMETHODVARIANT->getTypeInfo()), xsink);

    const QoreMethod* con = c->cls->getConstructor();

    if (con) {
        append_variant_objects(**rv, c->pgm, con->getFunction(), MT_Constructor, xsink);
    }

    return rv.release();
}

// CopyMethod Class::getCopyMethod(){}
static QoreValue Class_getCopyMethod(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 761 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->cls, "copy", xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }

    assert(m->mtype == MT_Copy);

    QoreProgram* cls_pgm = m->pgm;
    return new QoreObject(QC_COPYMETHOD, cls_pgm, m.release());
}

// CopyMethodVariant Class::getCopyVariant(){}
static QoreValue Class_getCopyVariant(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 580 "QC_Class.qpp"
    return find_variant(c, "copy", FV_COPY, nullptr, 0, xsink);
}

// DestructorMethod Class::getDestructorMethod(){}
static QoreValue Class_getDestructorMethod(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 725 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->cls, "destructor", xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }

    assert(m->mtype == MT_Destructor);

    QoreProgram* cls_pgm = m->pgm;
    return new QoreObject(QC_DESTRUCTORMETHOD, cls_pgm, m.release());
}

// DestructorMethodVariant Class::getDestructorVariant(){}
static QoreValue Class_getDestructorVariant(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 559 "QC_Class.qpp"
    return find_variant(c, "destructor", FV_DESTRUCTOR, nullptr, 0, xsink);
}

// *CopyMethodVariant Class::getIfCopyVariant(){}
static QoreValue Class_getIfCopyVariant(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 957 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    const QoreMethod* copy = c->cls->getCopyMethod();

    if (copy) {
        const QoreExternalFunction* func = copy->getFunction();
        const QoreExternalVariant* v = func->getFirstVariant();
        if (v) {
            return get_variant_object(c->pgm, func, MT_Copy, v, xsink);
        }
    }

    return QoreValue();
}

// *DestructorMethodVariant Class::getIfDestructorVariant(){}
static QoreValue Class_getIfDestructorVariant(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 922 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    const QoreMethod* destr = c->cls->getDestructor();

    if (destr) {
        const QoreExternalFunction* func = destr->getFunction();
        const QoreExternalVariant* v = func->getFirstVariant();
        if (v) {
            return get_variant_object(c->pgm, func, MT_Destructor, v, xsink);
        }
    }

    return QoreValue();
}

// int Class::getInheritanceAccess(Class base_class){}
static QoreValue Class_getInheritanceAccess_C5Class(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(base_class, QoreReflectionClass, args, 0, CID_CLASS, "Class::getInheritanceAccess()", "Class", xsink);
    if (*xsink)
        return 0;
# 1424 "QC_Class.qpp"
    ReferenceHolder<QoreReflectionClass> base_cls_holder(base_class, xsink);

    // get read access to program object
    QoreExternalProgramContextHelper tch1(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch2(xsink, base_class->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ClassAccess rv;
    if (!c->cls->inHierarchy(*base_class->cls, rv)) {
        return 0;
    }
    return get_access(rv);
}

// AbstractClassMember Class::getMember(string name){}
static QoreValue Class_getMember_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1154 "QC_Class.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    {
        const QoreExternalMemberVarBase* mi = c->cls->findLocalMember(tname->c_str());
        if (mi) {
            return new QoreObject(QC_NORMALMEMBER, c->pgm, new QoreReflectionClassMember(c->pgm, c->cls,
                tname->c_str(), mi, false));
        }
    }

    {
        const QoreExternalStaticMember* vi = c->cls->findLocalStaticMember(tname->c_str());
        if (vi) {
            return new QoreObject(QC_STATICMEMBER, c->pgm, new QoreReflectionClassMember(c->pgm, c->cls,
                tname->c_str(), vi, true));
        }
    }

    xsink->raiseException("UNKNOWN-MEMBER", "member '%s' is not defined in class '%s'", tname->c_str(),
        c->cls->getName());
    return QoreValue();
}

// list<AbstractClassMember> Class::getMembers(){}
static QoreValue Class_getMembers(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1282 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_ABSTRACTCLASSMEMBER->getTypeInfo()), xsink);

    QoreClassMemberIterator mi(*c->cls);
    while (mi.next()) {
        l->push(new QoreObject(QC_NORMALMEMBER, c->pgm, new QoreReflectionClassMember(c->pgm, c->cls, mi.getName(),
            &mi.getMember(), false)), xsink);
    }

    QoreClassStaticMemberIterator smi(*c->cls);
    while (smi.next()) {
        l->push(new QoreObject(QC_STATICMEMBER, c->pgm, new QoreReflectionClassMember(c->pgm, c->cls, smi.getName(),
            &smi.getMember(), false)), xsink);
    }

    return l.release();
}

// *string Class::getModuleName(){}
static QoreValue Class_getModuleName(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 189 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    const char* mod = c->cls->getModuleName();
    return mod ? new QoreStringNode(mod) : QoreValue();
}

// Namespace Class::getNamespace(){}
static QoreValue Class_getNamespace(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1375 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return new QoreObject(QC_NAMESPACE, c->pgm, new QoreReflectionNamespace(c->pgm, c->cls->getNamespace()));
}

// NormalMember Class::getNormalMember(string name){}
static QoreValue Class_getNormalMember_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1205 "QC_Class.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    const QoreExternalMemberVarBase* mi = c->cls->findLocalMember(tname->c_str());
    if (mi) {
        return new QoreObject(QC_NORMALMEMBER, c->pgm, new QoreReflectionClassMember(c->pgm, c->cls, tname->c_str(),
            mi, false));
    }

    xsink->raiseException("UNKNOWN-NORMAL-MEMBER", "normal (non-static) member '%s' is not defined in class '%s'",
        tname->c_str(), c->cls->getName());
    return QoreValue();
}

// NormalMethod Class::getNormalMethod(string name){}
static QoreValue Class_getNormalMethod_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 606 "QC_Class.qpp"
    TempEncodingHelper nname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->cls, nname->c_str(), xsink, MT_Normal),
        xsink);
    if (*xsink) {
        return QoreValue();
    }

    QoreProgram* cls_pgm = m->pgm;
    return new QoreObject(QC_NORMALMETHOD, cls_pgm, m.release());
}

// list<NormalMethod> Class::getNormalMethods(){}
static QoreValue Class_getNormalMethods(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 789 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_ABSTRACTMETHOD->getTypeInfo()), xsink);

    QoreMethodIterator i(*c->cls);
    while (i.next()) {
        const QoreMethod* m = i.getMethod();
        if (m->getMethodType() != MT_Normal) {
            continue;
        }

        ReferenceHolder<QoreReflectionMethod> meth(new QoreReflectionMethod(c->pgm, i.getMethod()), xsink);
        l->push(get_method_object(meth, xsink), xsink);
        if (*xsink) {
            return QoreValue();
        }
    }

    return l.release();
}

// NormalMethodVariant Class::getNormalVariant(string name, ...){}
static QoreValue Class_getNormalVariant_VsVV(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 395 "QC_Class.qpp"
    return find_variant(c, name, FV_NORMAL, args, 1, xsink);
}

// NormalMethodVariant Class::getNormalVariantArgs(string name, *softlist<Type> argv){}
static QoreValue Class_getNormalVariantArgs_VsC15_softlist_Type_(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 425 "QC_Class.qpp"
    return find_variant(c, name, FV_NORMAL, argv, 0, xsink);
}

// list<NormalMethodVariant> Class::getNormalVariants(){}
static QoreValue Class_getNormalVariants(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 825 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(QC_NORMALMETHODVARIANT->getTypeInfo()), xsink);

    QoreMethodIterator i(*c->cls);
    while (i.next()) {
        const QoreMethod* m = i.getMethod();
        method_type_e mtype = m->getMethodType();
        if (mtype != MT_Normal) {
            continue;
        }
        append_variant_objects(**rv, c->pgm, m->getFunction(), mtype, xsink);
    }

    return rv.release();
}

// Type Class::getOrNothingType(){}
static QoreValue Class_getOrNothingType(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1097 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return new QoreObject(QC_TYPE, c->pgm, new QoreType(c->cls->getOrNothingTypeInfo()));
}

// string Class::getPathName(){}
static QoreValue Class_getPathName(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 168 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode);
    rv->concat(c->cls->getNamespacePath());
    return rv.release();
}

// StaticMember Class::getStaticMember(string name){}
static QoreValue Class_getStaticMember_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1246 "QC_Class.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    const QoreExternalStaticMember* vi = c->cls->findLocalStaticMember(tname->c_str());
    if (vi) {
        return new QoreObject(QC_STATICMEMBER, c->pgm, new QoreReflectionClassMember(c->pgm, c->cls, tname->c_str(),
            vi, true));
    }

    xsink->raiseException("UNKNOWN-STATIC-MEMBER", "static member '%s' is not defined in class '%s'", tname->c_str(),
        c->cls->getName());
    return QoreValue();
}

// StaticMethod Class::getStaticMethod(string name){}
static QoreValue Class_getStaticMethod_Vs(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 650 "QC_Class.qpp"
    TempEncodingHelper nname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(c->cls, nname->c_str(), xsink, MT_Static), xsink);
    if (*xsink) {
        return QoreValue();
    }

    QoreProgram* cls_pgm = m->pgm;
    return new QoreObject(QC_STATICMETHOD, cls_pgm, m.release());
}

// StaticMethodVariant Class::getStaticVariant(string name, ...){}
static QoreValue Class_getStaticVariant_VsVV(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 455 "QC_Class.qpp"
    return find_variant(c, name, FV_STATIC, args, 1, xsink);
}

// StaticMethodVariant Class::getStaticVariantArgs(string name, *softlist<Type> argv){}
static QoreValue Class_getStaticVariantArgs_VsC15_softlist_Type_(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 485 "QC_Class.qpp"
    return find_variant(c, name, FV_STATIC, argv, 0, xsink);
}

// list<StaticMethodVariant> Class::getStaticVariants(){}
static QoreValue Class_getStaticVariants(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 858 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(QC_STATICMETHODVARIANT->getTypeInfo()), xsink);

    QoreMethodIterator i(*c->cls);
    while (i.next()) {
        const QoreMethod* m = i.getMethod();
        method_type_e mtype = m->getMethodType();
        if (mtype != MT_Static) {
            continue;
        }
        append_variant_objects(**rv, c->pgm, m->getFunction(), mtype, xsink);
    }

    return rv.release();
}

// Type Class::getType(){}
static QoreValue Class_getType(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1074 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return new QoreObject(QC_TYPE, c->pgm, new QoreType(c->cls->getTypeInfo()));
}

// bool Class::hasConstructorMethod(){}
static QoreValue Class_hasConstructorMethod(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 314 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return (bool)c->cls->getConstructor();
}

// bool Class::hasCopyMethod(){}
static QoreValue Class_hasCopyMethod(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 356 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return (bool)c->cls->getCopyMethod();
}

// bool Class::hasDestructorMethod(){}
static QoreValue Class_hasDestructorMethod(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 335 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return (bool)c->cls->getDestructor();
}

// int Class::instanceOf(object obj){}
static QoreValue Class_instanceOf_Vo(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 1400 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    return get_access(obj->getClassAccess(*c->cls));
}

// object Class::newObject(...){}
static QoreValue Class_newObject_VV(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1001 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    if (check_constructor(c->cls, xsink)) {
        return QoreValue();
    }

    if (c->cls->runtimeCheckInstantiateClass(xsink)) {
        return QoreValue();
    }

    return c->cls->execConstructor(args, xsink);
}

// object Class::newObjectArgs(*softlist argv){}
static QoreValue Class_newObjectArgs_nl(QoreObject* self, QoreReflectionClass* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 1043 "QC_Class.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    if (check_constructor(c->cls, xsink)) {
        return QoreValue();
    }

    if (c->cls->runtimeCheckInstantiateClass(xsink)) {
        return QoreValue();
    }

    return c->cls->execConstructor(argv, xsink);
}

// static list<Class> Class::findAllRegex(string pattern, *int re_opts){}
static QoreValue static_Class_findAllRegex_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 1);
# 1529 "QC_Class.qpp"
return find_all_classes(*pattern, re_opts, getProgram(), xsink);
}

// static list<Class> Class::findAllRegex(Program pgm, string pattern, *int re_opts){}
static QoreValue static_Class_findAllRegex_C7ProgramVsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Class::findAllRegex()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 1);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 2);
# 1546 "QC_Class.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    return find_all_classes(*pattern, re_opts, pgm, xsink);
}

// static Class Class::forName(string path){}
static QoreValue static_Class_forName_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1459 "QC_Class.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionClass> cls(new QoreReflectionClass(xsink, npath->c_str()), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return new QoreObject(QC_CLASS, getProgram(), cls.release());
}

// static Class Class::forName(Program pgm, string path){}
static QoreValue static_Class_forName_C7ProgramVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Class::forName()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
# 1487 "QC_Class.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionClass> cls(new QoreReflectionClass(xsink, npath->c_str(), pgm), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return new QoreObject(QC_CLASS, pgm, cls.release());
}

// static Class Class::getClass(object obj){}
static QoreValue static_Class_getClass_Vo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 1512 "QC_Class.qpp"
QoreProgram* pgm = obj->getProgram();
    return new QoreObject(QC_CLASS, pgm, new QoreReflectionClass(pgm, obj->getClass()));
}

DLLLOCAL void preinitClassClass() {
    QC_CLASS = new QoreBuiltinClass("Class", "::Qore::Reflection::Class", QDOM_REFLECTION);
    CID_CLASS = QC_CLASS->getID();
    QC_CLASS->setSystem();
}

DLLLOCAL QoreClass* initClassClass(QoreNamespace& ns) {
    if (!QC_CLASS)
        preinitClassClass();

    // set parent class
    assert(QC_ABSTRACTCLASS);
    QC_CLASS->addBuiltinVirtualBaseClass(QC_ABSTRACTCLASS);

    // Class::constructor(string path) {}
    QC_CLASS->addConstructor(Class_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // hash<MethodAccessInfo> Class::findMethod(string name){}
    QC_CLASS->addMethod("findMethod", (q_method_n_t)Class_findMethod_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclMethodAccessInfo->getTypeInfo(false), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // hash<MethodAccessInfo> Class::findNormalMethod(string name){}
    QC_CLASS->addMethod("findNormalMethod", (q_method_n_t)Class_findNormalMethod_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclMethodAccessInfo->getTypeInfo(false), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // hash<MethodAccessInfo> Class::findStaticMethod(string name){}
    QC_CLASS->addMethod("findStaticMethod", (q_method_n_t)Class_findStaticMethod_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclMethodAccessInfo->getTypeInfo(false), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<Class> Class::getClassHierarchy(){}
    QC_CLASS->addMethod("getClassHierarchy", (q_method_n_t)Class_getClassHierarchy, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CLASS->getTypeInfo()));

    // ClassConstant Class::getConstant(string name){}
    QC_CLASS->addMethod("getConstant", (q_method_n_t)Class_getConstant_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CLASSCONSTANT->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<ClassConstant> Class::getConstants(){}
    QC_CLASS->addMethod("getConstants", (q_method_n_t)Class_getConstants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CLASSCONSTANT->getTypeInfo()));

    // ConstructorMethod Class::getConstructorMethod(){}
    QC_CLASS->addMethod("getConstructorMethod", (q_method_n_t)Class_getConstructorMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CONSTRUCTORMETHOD->getTypeInfo());

    // ConstructorMethodVariant Class::getConstructorVariant(...){}
    QC_CLASS->addMethod("getConstructorVariant", (q_method_n_t)Class_getConstructorVariant_VV, Public, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, QC_CONSTRUCTORMETHODVARIANT->getTypeInfo());

    // ConstructorMethodVariant Class::getConstructorVariantArgs(*softlist<Type> argv){}
    QC_CLASS->addMethod("getConstructorVariantArgs", (q_method_n_t)Class_getConstructorVariantArgs_C15_softlist_Type_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CONSTRUCTORMETHODVARIANT->getTypeInfo(), 1, qore_get_complex_softlist_or_nothing_type(QC_TYPE->getTypeInfo()), QORE_PARAM_NO_ARG, "argv");

    // list<ConstructorMethodVariant> Class::getConstructorVariants(){}
    QC_CLASS->addMethod("getConstructorVariants", (q_method_n_t)Class_getConstructorVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CONSTRUCTORMETHODVARIANT->getTypeInfo()));

    // CopyMethod Class::getCopyMethod(){}
    QC_CLASS->addMethod("getCopyMethod", (q_method_n_t)Class_getCopyMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_COPYMETHOD->getTypeInfo());

    // CopyMethodVariant Class::getCopyVariant(){}
    QC_CLASS->addMethod("getCopyVariant", (q_method_n_t)Class_getCopyVariant, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_COPYMETHODVARIANT->getTypeInfo());

    // DestructorMethod Class::getDestructorMethod(){}
    QC_CLASS->addMethod("getDestructorMethod", (q_method_n_t)Class_getDestructorMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_DESTRUCTORMETHOD->getTypeInfo());

    // DestructorMethodVariant Class::getDestructorVariant(){}
    QC_CLASS->addMethod("getDestructorVariant", (q_method_n_t)Class_getDestructorVariant, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_DESTRUCTORMETHODVARIANT->getTypeInfo());

    // *CopyMethodVariant Class::getIfCopyVariant(){}
    QC_CLASS->addMethod("getIfCopyVariant", (q_method_n_t)Class_getIfCopyVariant, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_COPYMETHODVARIANT->getOrNothingTypeInfo());

    // *DestructorMethodVariant Class::getIfDestructorVariant(){}
    QC_CLASS->addMethod("getIfDestructorVariant", (q_method_n_t)Class_getIfDestructorVariant, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_DESTRUCTORMETHODVARIANT->getOrNothingTypeInfo());

    // int Class::getInheritanceAccess(Class base_class){}
    QC_CLASS->addMethod("getInheritanceAccess", (q_method_n_t)Class_getInheritanceAccess_C5Class, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, QC_CLASS->getTypeInfo(), QORE_PARAM_NO_ARG, "base_class");

    // AbstractClassMember Class::getMember(string name){}
    QC_CLASS->addMethod("getMember", (q_method_n_t)Class_getMember_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_ABSTRACTCLASSMEMBER->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<AbstractClassMember> Class::getMembers(){}
    QC_CLASS->addMethod("getMembers", (q_method_n_t)Class_getMembers, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_ABSTRACTCLASSMEMBER->getTypeInfo()));

    // *string Class::getModuleName(){}
    QC_CLASS->addMethod("getModuleName", (q_method_n_t)Class_getModuleName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // Namespace Class::getNamespace(){}
    QC_CLASS->addMethod("getNamespace", (q_method_n_t)Class_getNamespace, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_NAMESPACE->getTypeInfo());

    // NormalMember Class::getNormalMember(string name){}
    QC_CLASS->addMethod("getNormalMember", (q_method_n_t)Class_getNormalMember_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_NORMALMEMBER->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // NormalMethod Class::getNormalMethod(string name){}
    QC_CLASS->addMethod("getNormalMethod", (q_method_n_t)Class_getNormalMethod_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_NORMALMETHOD->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<NormalMethod> Class::getNormalMethods(){}
    QC_CLASS->addMethod("getNormalMethods", (q_method_n_t)Class_getNormalMethods, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_NORMALMETHOD->getTypeInfo()));

    // NormalMethodVariant Class::getNormalVariant(string name, ...){}
    QC_CLASS->addMethod("getNormalVariant", (q_method_n_t)Class_getNormalVariant_VsVV, Public, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, QC_NORMALMETHODVARIANT->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // NormalMethodVariant Class::getNormalVariantArgs(string name, *softlist<Type> argv){}
    QC_CLASS->addMethod("getNormalVariantArgs", (q_method_n_t)Class_getNormalVariantArgs_VsC15_softlist_Type_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_NORMALMETHODVARIANT->getTypeInfo(), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", qore_get_complex_softlist_or_nothing_type(QC_TYPE->getTypeInfo()), QORE_PARAM_NO_ARG, "argv");

    // list<NormalMethodVariant> Class::getNormalVariants(){}
    QC_CLASS->addMethod("getNormalVariants", (q_method_n_t)Class_getNormalVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_NORMALMETHODVARIANT->getTypeInfo()));

    // Type Class::getOrNothingType(){}
    QC_CLASS->addMethod("getOrNothingType", (q_method_n_t)Class_getOrNothingType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // string Class::getPathName(){}
    QC_CLASS->addMethod("getPathName", (q_method_n_t)Class_getPathName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // StaticMember Class::getStaticMember(string name){}
    QC_CLASS->addMethod("getStaticMember", (q_method_n_t)Class_getStaticMember_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_STATICMEMBER->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // StaticMethod Class::getStaticMethod(string name){}
    QC_CLASS->addMethod("getStaticMethod", (q_method_n_t)Class_getStaticMethod_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_STATICMETHOD->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // StaticMethodVariant Class::getStaticVariant(string name, ...){}
    QC_CLASS->addMethod("getStaticVariant", (q_method_n_t)Class_getStaticVariant_VsVV, Public, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, QC_STATICMETHODVARIANT->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // StaticMethodVariant Class::getStaticVariantArgs(string name, *softlist<Type> argv){}
    QC_CLASS->addMethod("getStaticVariantArgs", (q_method_n_t)Class_getStaticVariantArgs_VsC15_softlist_Type_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_STATICMETHODVARIANT->getTypeInfo(), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", qore_get_complex_softlist_or_nothing_type(QC_TYPE->getTypeInfo()), QORE_PARAM_NO_ARG, "argv");

    // list<StaticMethodVariant> Class::getStaticVariants(){}
    QC_CLASS->addMethod("getStaticVariants", (q_method_n_t)Class_getStaticVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_STATICMETHODVARIANT->getTypeInfo()));

    // Type Class::getType(){}
    QC_CLASS->addMethod("getType", (q_method_n_t)Class_getType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // bool Class::hasConstructorMethod(){}
    QC_CLASS->addMethod("hasConstructorMethod", (q_method_n_t)Class_hasConstructorMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool Class::hasCopyMethod(){}
    QC_CLASS->addMethod("hasCopyMethod", (q_method_n_t)Class_hasCopyMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool Class::hasDestructorMethod(){}
    QC_CLASS->addMethod("hasDestructorMethod", (q_method_n_t)Class_hasDestructorMethod, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // int Class::instanceOf(object obj){}
    QC_CLASS->addMethod("instanceOf", (q_method_n_t)Class_instanceOf_Vo, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");

    // object Class::newObject(...){}
    QC_CLASS->addMethod("newObject", (q_method_n_t)Class_newObject_VV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, objectTypeInfo);

    // object Class::newObjectArgs(*softlist argv){}
    QC_CLASS->addMethod("newObjectArgs", (q_method_n_t)Class_newObjectArgs_nl, Public, QCF_NO_FLAGS, QDOM_DEFAULT, objectTypeInfo, 1, softListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // static list<Class> Class::findAllRegex(string pattern, *int re_opts){}
    QC_CLASS->addStaticMethod("findAllRegex", (q_func_n_t)static_Class_findAllRegex_VsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CLASS->getTypeInfo()), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static list<Class> Class::findAllRegex(Program pgm, string pattern, *int re_opts){}
    QC_CLASS->addStaticMethod("findAllRegex", (q_func_n_t)static_Class_findAllRegex_C7ProgramVsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CLASS->getTypeInfo()), 3, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static Class Class::forName(string path){}
    QC_CLASS->addStaticMethod("forName", (q_func_n_t)static_Class_forName_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CLASS->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static Class Class::forName(Program pgm, string path){}
    QC_CLASS->addStaticMethod("forName", (q_func_n_t)static_Class_forName_C7ProgramVs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CLASS->getTypeInfo(), 2, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static Class Class::getClass(object obj){}
    QC_CLASS->addStaticMethod("getClass", (q_func_n_t)static_Class_getClass_Vo, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_CLASS->getTypeInfo(), 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");

    return QC_CLASS;
}
