/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_Constant.cpp Constant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_Constant.h"
#include "QC_Namespace.h"

QoreReflectionNamespaceConstant::QoreReflectionNamespaceConstant(const char* name, ExceptionSink* xsink) {
    ce = pgm->findNamespaceConstant(name, ns);
    if (!ce) {
        xsink->raiseException("UNKNOWN-CONSTANT", "cannot resolve namespace constant '%s'", name);
    }
}

QoreReflectionNamespaceConstant::QoreReflectionNamespaceConstant(QoreProgram* pgm, const char* name, ExceptionSink* xsink) {
    ce = pgm->findNamespaceConstant(name, ns);
    if (!ce) {
        xsink->raiseException("UNKNOWN-CONSTANT", "cannot resolve namespace constant '%s'", name);
    }
}

static QoreValue find_all_constants(const QoreString& pattern, int re_opts, QoreProgram* pgm, ExceptionSink* xsink) {
    const_vec_t const_vec = pgm->findAllNamespaceConstantsRegex(pattern, re_opts, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_CONSTANT->getTypeInfo()), xsink);
    for (auto& i : const_vec) {
        l->push(new QoreObject(QC_CONSTANT, pgm, new QoreReflectionNamespaceConstant(pgm, i.first, i.second)), xsink);
    }

    return l.release();
}
/* Qore class Qore::Reflection::Constant */

qore_classid_t CID_CONSTANT;
QoreClass* QC_CONSTANT;

// Constant::constructor(string path) {}
static void Constant_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 86 "QC_Constant.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionNamespaceConstant> con(new QoreReflectionNamespaceConstant(npath->c_str(), xsink), xsink);
    if (*xsink) {
        return;
    }
    self->setPrivate(CID_CONSTANT, con.release());
}

// Namespace Constant::getNamespace(){}
static QoreValue Constant_getNamespace(QoreObject* self, QoreReflectionNamespaceConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 107 "QC_Constant.qpp"
    return new QoreObject(QC_NAMESPACE, c->pgm, new QoreReflectionNamespace(c->pgm, c->ns));
}

// static list<Constant> Constant::findAllRegex(string pattern, *int re_opts){}
static QoreValue static_Constant_findAllRegex_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 1);
# 185 "QC_Constant.qpp"
return find_all_constants(*pattern, re_opts, getProgram(), xsink);
}

// static list<Constant> Constant::findAllRegex(Program pgm, string pattern, *int re_opts){}
static QoreValue static_Constant_findAllRegex_C7ProgramVsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Constant::findAllRegex()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 1);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 2);
# 204 "QC_Constant.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    return find_all_constants(*pattern, re_opts, pgm, xsink);
}

// static Constant Constant::forName(string path){}
static QoreValue static_Constant_forName_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 126 "QC_Constant.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionNamespaceConstant> cls(new QoreReflectionNamespaceConstant(npath->c_str(), xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return new QoreObject(QC_CONSTANT, getProgram(), cls.release());
}

// static Constant Constant::forName(Program pgm, string path){}
static QoreValue static_Constant_forName_C7ProgramVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Constant::forName()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
# 156 "QC_Constant.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionNamespaceConstant> cls(new QoreReflectionNamespaceConstant(pgm, npath->c_str(), xsink), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return new QoreObject(QC_CONSTANT, pgm, cls.release());
}

DLLLOCAL void preinitConstantClass() {
    QC_CONSTANT = new QoreBuiltinClass("Constant", "::Qore::Reflection::Constant", QDOM_REFLECTION);
    CID_CONSTANT = QC_CONSTANT->getID();
    QC_CONSTANT->setSystem();
}

DLLLOCAL QoreClass* initConstantClass(QoreNamespace& ns) {
    if (!QC_CONSTANT)
        preinitConstantClass();

    // set parent class
    assert(QC_ABSTRACTCONSTANT);
    QC_CONSTANT->addBuiltinVirtualBaseClass(QC_ABSTRACTCONSTANT);

    // Constant::constructor(string path) {}
    QC_CONSTANT->addConstructor(Constant_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // Namespace Constant::getNamespace(){}
    QC_CONSTANT->addMethod("getNamespace", (q_method_n_t)Constant_getNamespace, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_NAMESPACE->getTypeInfo());

    // static list<Constant> Constant::findAllRegex(string pattern, *int re_opts){}
    QC_CONSTANT->addStaticMethod("findAllRegex", (q_func_n_t)static_Constant_findAllRegex_VsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CONSTANT->getTypeInfo()), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static list<Constant> Constant::findAllRegex(Program pgm, string pattern, *int re_opts){}
    QC_CONSTANT->addStaticMethod("findAllRegex", (q_func_n_t)static_Constant_findAllRegex_C7ProgramVsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CONSTANT->getTypeInfo()), 3, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static Constant Constant::forName(string path){}
    QC_CONSTANT->addStaticMethod("forName", (q_func_n_t)static_Constant_forName_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CONSTANT->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static Constant Constant::forName(Program pgm, string path){}
    QC_CONSTANT->addStaticMethod("forName", (q_func_n_t)static_Constant_forName_C7ProgramVs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CONSTANT->getTypeInfo(), 2, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    return QC_CONSTANT;
}
