/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ConstructorMethod.cpp ConstructorMethod class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_ConstructorMethod.h"
#include "QC_Class.h"
/* Qore class Qore::Reflection::ConstructorMethod */

qore_classid_t CID_CONSTRUCTORMETHOD;
QoreClass* QC_CONSTRUCTORMETHOD;

// auto ConstructorMethod::call(...){}
static QoreValue ConstructorMethod_call_VV(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 116 "QC_ConstructorMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    const QoreClass* cls = m->m->getClass();
    if (check_constructor(cls, xsink)) {
        return QoreValue();
    }

    return cls->execConstructor(args, xsink);
}

// auto ConstructorMethod::callArgs(*softlist<auto> argv){}
static QoreValue ConstructorMethod_callArgs_C15_softlist_auto_(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 147 "QC_ConstructorMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    const QoreClass* cls = m->m->getClass();
    if (check_constructor(cls, xsink)) {
        return QoreValue();
    }

    return cls->execConstructor(argv, xsink);
}

// ConstructorMethod::constructor(Class cls) {}
static void ConstructorMethod_constructor_C5Class(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(cls, QoreReflectionClass, args, 0, CID_CLASS, "ConstructorMethod::constructor()", "Class", xsink);
    if (*xsink)
        return;
# 54 "QC_ConstructorMethod.qpp"
ReferenceHolder<QoreReflectionClass> holder(cls, xsink);

    // get exclusive access to program object
    QoreExternalProgramContextHelper pch(xsink, cls->pgm);
    if (*xsink) {
        return;
    }

    // the class pointer must be valid here
    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(cls->cls, "constructor", xsink), xsink);
    if (*xsink) {
        return;
    }

    assert(m->mtype == MT_Constructor);

    self->setPrivate(CID_CONSTRUCTORMETHOD, m.release());
}

// ConstructorMethod::constructor(string class_name) {}
static void ConstructorMethod_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
# 85 "QC_ConstructorMethod.qpp"
TempEncodingHelper npath(class_name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(npath->c_str(), "constructor", xsink), xsink);
    if (*xsink) {
        return;
    }

    assert(m->mtype == MT_Constructor);

    self->setPrivate(CID_CONSTRUCTORMETHOD, m.release());
}

DLLLOCAL void preinitConstructorMethodClass() {
    QC_CONSTRUCTORMETHOD = new QoreBuiltinClass("ConstructorMethod", "::Qore::Reflection::ConstructorMethod", QDOM_REFLECTION);
    CID_CONSTRUCTORMETHOD = QC_CONSTRUCTORMETHOD->getID();
    QC_CONSTRUCTORMETHOD->setSystem();
}

DLLLOCAL QoreClass* initConstructorMethodClass(QoreNamespace& ns) {
    if (!QC_CONSTRUCTORMETHOD)
        preinitConstructorMethodClass();

    // set parent class
    assert(QC_ABSTRACTMETHOD);
    QC_CONSTRUCTORMETHOD->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHOD);

    // auto ConstructorMethod::call(...){}
    QC_CONSTRUCTORMETHOD->addMethod("call", (q_method_n_t)ConstructorMethod_call_VV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);

    // auto ConstructorMethod::callArgs(*softlist<auto> argv){}
    QC_CONSTRUCTORMETHOD->addMethod("callArgs", (q_method_n_t)ConstructorMethod_callArgs_C15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // ConstructorMethod::constructor(Class cls) {}
    QC_CONSTRUCTORMETHOD->addConstructor(ConstructorMethod_constructor_C5Class, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_CLASS->getTypeInfo(), QORE_PARAM_NO_ARG, "cls");

    // ConstructorMethod::constructor(string class_name) {}
    QC_CONSTRUCTORMETHOD->addConstructor(ConstructorMethod_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name");

    return QC_CONSTRUCTORMETHOD;
}
