/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_Function.cpp Function class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_Function.h"
#include "QC_FunctionVariant.h"
#include "QC_AbstractVariant.h"
#include "QC_Type.h"

static QoreReflectionFunction* get_function(ExceptionSink* xsink, const QoreString& path, QoreProgram* pgm = getProgram()) {
    TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return nullptr;
    }

    const QoreExternalFunction* f = pgm->findFunction(npath->c_str());
    if (!f) {
        xsink->raiseException("UNKNOWN-FUNCTION", "cannot find function '%s'", npath->c_str());
        return nullptr;
    }

    return new QoreReflectionFunction(pgm, f);
}

static QoreObject* get_function_object(ExceptionSink* xsink, const QoreString& path, QoreProgram* pgm = getProgram()) {
    QoreReflectionFunction* f = get_function(xsink, path, pgm);
    if (!f) {
        assert(*xsink);
        return nullptr;
    }

    return new QoreObject(QC_FUNCTION, pgm, f);
}

static QoreValue find_all_funcs(const QoreString& pattern, int re_opts, QoreProgram* pgm, ExceptionSink* xsink) {
    func_vec_t func_vec = pgm->findAllFunctionsRegex(pattern, re_opts, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_FUNCTION->getTypeInfo()), xsink);
    for (auto& i : func_vec) {
        l->push(new QoreObject(QC_FUNCTION, pgm, new QoreReflectionFunction(pgm, i)), xsink);
    }

    return l.release();
}
/* Qore class Qore::Reflection::Function */

qore_classid_t CID_FUNCTION;
QoreClass* QC_FUNCTION;

// auto Function::call(...){}
static QoreValue Function_call_VV(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 200 "QC_Function.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return f->f->evalFunction(nullptr, args, f->pgm, xsink);
}

// auto Function::callArgs(*softlist<auto> argv){}
static QoreValue Function_callArgs_C15_softlist_auto_(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 225 "QC_Function.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return f->f->evalFunction(nullptr, argv, f->pgm, xsink);
}

// Function::constructor(string path) {}
static void Function_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 96 "QC_Function.qpp"
QoreReflectionFunction* f = get_function(xsink, *path);
    if (f) {
        self->setPrivate(CID_FUNCTION, f);
    }
}

// FunctionVariant Function::getVariant(...){}
static QoreValue Function_getVariant_VV(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 145 "QC_Function.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return find_function_variant(f->pgm, f->f, args, 0, MT_None, xsink);
}

// FunctionVariant Function::getVariantArgs(softlist<Type> argv){}
static QoreValue Function_getVariantArgs_C14softlist_Type_(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = HARD_QORE_VALUE_LIST(args, 0);
# 175 "QC_Function.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    return find_function_variant(f->pgm, f->f, argv, 0, MT_None, xsink);
}

// list<FunctionVariant> Function::getVariants(){}
static QoreValue Function_getVariants(QoreObject* self, QoreReflectionFunction* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 113 "QC_Function.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, f->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the function ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(QC_FUNCTIONVARIANT->getTypeInfo()), xsink);
    append_variant_objects(**rv, f->pgm, f->f, MT_None, xsink);
    return rv.release();
}

// static list<Function> Function::findAllRegex(string pattern, *int re_opts){}
static QoreValue static_Function_findAllRegex_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 1);
# 284 "QC_Function.qpp"
return find_all_funcs(*pattern, re_opts, getProgram(), xsink);
}

// static list<Function> Function::findAllRegex(Program pgm, string pattern, *int re_opts){}
static QoreValue static_Function_findAllRegex_C7ProgramVsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Function::findAllRegex()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 1);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 2);
# 301 "QC_Function.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    return find_all_funcs(*pattern, re_opts, pgm, xsink);
}

// static Function Function::forName(string path){}
static QoreValue static_Function_forName_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 249 "QC_Function.qpp"
return get_function_object(xsink, *path);
}

// static Function Function::forName(Program pgm, string path){}
static QoreValue static_Function_forName_C7ProgramVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Function::forName()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
# 267 "QC_Function.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    return get_function_object(xsink, *path, pgm);
}

DLLLOCAL void preinitFunctionClass() {
    QC_FUNCTION = new QoreBuiltinClass("Function", "::Qore::Reflection::Function", QDOM_REFLECTION);
    CID_FUNCTION = QC_FUNCTION->getID();
    QC_FUNCTION->setSystem();
}

DLLLOCAL QoreClass* initFunctionClass(QoreNamespace& ns) {
    if (!QC_FUNCTION)
        preinitFunctionClass();

    // set parent class
    assert(QC_ABSTRACTREFLECTIONFUNCTION);
    QC_FUNCTION->addBuiltinVirtualBaseClass(QC_ABSTRACTREFLECTIONFUNCTION);

    // auto Function::call(...){}
    QC_FUNCTION->addMethod("call", (q_method_n_t)Function_call_VV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);

    // auto Function::callArgs(*softlist<auto> argv){}
    QC_FUNCTION->addMethod("callArgs", (q_method_n_t)Function_callArgs_C15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // Function::constructor(string path) {}
    QC_FUNCTION->addConstructor(Function_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // FunctionVariant Function::getVariant(...){}
    QC_FUNCTION->addMethod("getVariant", (q_method_n_t)Function_getVariant_VV, Public, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, QC_FUNCTIONVARIANT->getTypeInfo());

    // FunctionVariant Function::getVariantArgs(softlist<Type> argv){}
    QC_FUNCTION->addMethod("getVariantArgs", (q_method_n_t)Function_getVariantArgs_C14softlist_Type_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_FUNCTIONVARIANT->getTypeInfo(), 1, qore_get_complex_softlist_type(QC_TYPE->getTypeInfo()), QORE_PARAM_NO_ARG, "argv");

    // list<FunctionVariant> Function::getVariants(){}
    QC_FUNCTION->addMethod("getVariants", (q_method_n_t)Function_getVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_FUNCTIONVARIANT->getTypeInfo()));

    // static list<Function> Function::findAllRegex(string pattern, *int re_opts){}
    QC_FUNCTION->addStaticMethod("findAllRegex", (q_func_n_t)static_Function_findAllRegex_VsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_FUNCTION->getTypeInfo()), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static list<Function> Function::findAllRegex(Program pgm, string pattern, *int re_opts){}
    QC_FUNCTION->addStaticMethod("findAllRegex", (q_func_n_t)static_Function_findAllRegex_C7ProgramVsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_FUNCTION->getTypeInfo()), 3, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static Function Function::forName(string path){}
    QC_FUNCTION->addStaticMethod("forName", (q_func_n_t)static_Function_forName_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_FUNCTION->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static Function Function::forName(Program pgm, string path){}
    QC_FUNCTION->addStaticMethod("forName", (q_func_n_t)static_Function_forName_C7ProgramVs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_FUNCTION->getTypeInfo(), 2, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    return QC_FUNCTION;
}
