/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_GlobalVar.cpp global variable class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_GlobalVar.h"
#include "QC_Class.h"
#include "QC_Type.h"
#include "QC_Namespace.h"

QoreReflectionGlobalVar::QoreReflectionGlobalVar(ExceptionSink* xsink, const char* path, QoreProgram* pgm)
    : AbstractReflectionObject(pgm) {
    var = pgm->findGlobalVar(path, ns);
    if (!var) {
        xsink->raiseException("UNKNOWN-GLOBAL-VAR", "cannot resolve global variable '%s'", path);
        return;
    }
}

QoreObject* QoreReflectionGlobalVar::getVarObject() const {
    return new QoreObject(QC_GLOBALVAR, pgm, const_cast<QoreReflectionGlobalVar*>(this));
}

static QoreValue find_all_gvars(const QoreString& pattern, int re_opts, QoreProgram* pgm, ExceptionSink* xsink) {
    gvar_vec_t gvar_vec = pgm->findAllGlobalVarsRegex(pattern, re_opts, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_GLOBALVAR->getTypeInfo()), xsink);
    for (auto& i : gvar_vec) {
        l->push(new QoreObject(QC_GLOBALVAR, pgm, new QoreReflectionGlobalVar(pgm, i.first, i.second)), xsink);
    }

    return l.release();
}
/* Qore class Qore::Reflection::GlobalVar */

qore_classid_t CID_GLOBALVAR;
QoreClass* QC_GLOBALVAR;

// GlobalVar::constructor(string path) {}
static void GlobalVar_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 84 "QC_GlobalVar.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionGlobalVar> var(new QoreReflectionGlobalVar(xsink, npath->c_str()), xsink);
    if (*xsink) {
        return;
    }
    self->setPrivate(CID_GLOBALVAR, var.release());
}

// string GlobalVar::getName(){}
static QoreValue GlobalVar_getName(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 126 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return new QoreStringNode(v->var->getName());
}

// Namespace GlobalVar::getNamespace(){}
static QoreValue GlobalVar_getNamespace(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 254 "QC_GlobalVar.qpp"
    return new QoreObject(QC_NAMESPACE, v->pgm, new QoreReflectionNamespace(v->pgm, v->ns));
}

// hash<SourceLocationInfo> GlobalVar::getSourceLocation(){}
static QoreValue GlobalVar_getSourceLocation(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 214 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return v->var->getSourceLocation()->getHash();
}

// Type GlobalVar::getType(){}
static QoreValue GlobalVar_getType(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 235 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return new QoreObject(QC_TYPE, v->pgm, new QoreType(v->var->getTypeInfo()));
}

// auto GlobalVar::getValue(){}
static QoreValue GlobalVar_getValue(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 268 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return v->var->getReferencedValue();
}

// bool GlobalVar::isBuiltin(){}
static QoreValue GlobalVar_isBuiltin(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 149 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return v->var->isBuiltin();
}

// bool GlobalVar::isEqual(GlobalVar var){}
static QoreValue GlobalVar_isEqual_C9GlobalVar(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(var, QoreReflectionGlobalVar, args, 0, CID_GLOBALVAR, "GlobalVar::isEqual()", "GlobalVar", xsink);
    if (*xsink)
        return 0;
# 110 "QC_GlobalVar.qpp"
    ReferenceHolder<QoreReflectionGlobalVar> holder(var, xsink);

    return v->var == var->var;
}

// bool GlobalVar::isModulePublic(){}
static QoreValue GlobalVar_isModulePublic(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 193 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return v->var->isModulePublic();
}

// bool GlobalVar::isUser(){}
static QoreValue GlobalVar_isUser(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 172 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return !v->var->isBuiltin();
}

// nothing GlobalVar::setValue(auto val){}
static QoreValue GlobalVar_setValue_VA(QoreObject* self, QoreReflectionGlobalVar* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 290 "QC_GlobalVar.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    v->var->setValue(val, xsink);
    return QoreValue();
}

// static list<GlobalVar> GlobalVar::findAllRegex(string pattern, *int re_opts){}
static QoreValue static_GlobalVar_findAllRegex_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 1);
# 373 "QC_GlobalVar.qpp"
return find_all_gvars(*pattern, re_opts, getProgram(), xsink);
}

// static list<GlobalVar> GlobalVar::findAllRegex(Program pgm, string pattern, *int re_opts){}
static QoreValue static_GlobalVar_findAllRegex_C7ProgramVsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "GlobalVar::findAllRegex()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 1);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 2);
# 390 "QC_GlobalVar.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    return find_all_gvars(*pattern, re_opts, pgm, xsink);
}

// static GlobalVar GlobalVar::forName(string path){}
static QoreValue static_GlobalVar_forName_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 315 "QC_GlobalVar.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionGlobalVar> var(new QoreReflectionGlobalVar(xsink, npath->c_str()), xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return var.release()->getVarObject();
}

// static GlobalVar GlobalVar::forName(Program pgm, string path){}
static QoreValue static_GlobalVar_forName_C7ProgramVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "GlobalVar::forName()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
# 345 "QC_GlobalVar.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionGlobalVar> var(new QoreReflectionGlobalVar(xsink, npath->c_str(), pgm), xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the global variable ptr must be valid
    return var.release()->getVarObject();
}

DLLLOCAL void preinitGlobalVarClass() {
    QC_GLOBALVAR = new QoreBuiltinClass("GlobalVar", "::Qore::Reflection::GlobalVar", QDOM_REFLECTION);
    CID_GLOBALVAR = QC_GLOBALVAR->getID();
    QC_GLOBALVAR->setSystem();
}

DLLLOCAL QoreClass* initGlobalVarClass(QoreNamespace& ns) {
    if (!QC_GLOBALVAR)
        preinitGlobalVarClass();

    // GlobalVar::constructor(string path) {}
    QC_GLOBALVAR->addConstructor(GlobalVar_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // string GlobalVar::getName(){}
    QC_GLOBALVAR->addMethod("getName", (q_method_n_t)GlobalVar_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // Namespace GlobalVar::getNamespace(){}
    QC_GLOBALVAR->addMethod("getNamespace", (q_method_n_t)GlobalVar_getNamespace, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_NAMESPACE->getTypeInfo());

    // hash<SourceLocationInfo> GlobalVar::getSourceLocation(){}
    QC_GLOBALVAR->addMethod("getSourceLocation", (q_method_n_t)GlobalVar_getSourceLocation, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSourceLocationInfo->getTypeInfo(false));

    // Type GlobalVar::getType(){}
    QC_GLOBALVAR->addMethod("getType", (q_method_n_t)GlobalVar_getType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // auto GlobalVar::getValue(){}
    QC_GLOBALVAR->addMethod("getValue", (q_method_n_t)GlobalVar_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // bool GlobalVar::isBuiltin(){}
    QC_GLOBALVAR->addMethod("isBuiltin", (q_method_n_t)GlobalVar_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool GlobalVar::isEqual(GlobalVar var){}
    QC_GLOBALVAR->addMethod("isEqual", (q_method_n_t)GlobalVar_isEqual_C9GlobalVar, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_GLOBALVAR->getTypeInfo(), QORE_PARAM_NO_ARG, "var");

    // bool GlobalVar::isModulePublic(){}
    QC_GLOBALVAR->addMethod("isModulePublic", (q_method_n_t)GlobalVar_isModulePublic, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool GlobalVar::isUser(){}
    QC_GLOBALVAR->addMethod("isUser", (q_method_n_t)GlobalVar_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // nothing GlobalVar::setValue(auto val){}
    QC_GLOBALVAR->addMethod("setValue", (q_method_n_t)GlobalVar_setValue_VA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // static list<GlobalVar> GlobalVar::findAllRegex(string pattern, *int re_opts){}
    QC_GLOBALVAR->addStaticMethod("findAllRegex", (q_func_n_t)static_GlobalVar_findAllRegex_VsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_GLOBALVAR->getTypeInfo()), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static list<GlobalVar> GlobalVar::findAllRegex(Program pgm, string pattern, *int re_opts){}
    QC_GLOBALVAR->addStaticMethod("findAllRegex", (q_func_n_t)static_GlobalVar_findAllRegex_C7ProgramVsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_GLOBALVAR->getTypeInfo()), 3, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static GlobalVar GlobalVar::forName(string path){}
    QC_GLOBALVAR->addStaticMethod("forName", (q_func_n_t)static_GlobalVar_forName_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_GLOBALVAR->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static GlobalVar GlobalVar::forName(Program pgm, string path){}
    QC_GLOBALVAR->addStaticMethod("forName", (q_func_n_t)static_GlobalVar_forName_C7ProgramVs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_GLOBALVAR->getTypeInfo(), 2, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    return QC_GLOBALVAR;
}
