/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_Namespace.cpp Namespace namespace definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include <QC_Namespace.h>
#include <QC_Class.h>
#include <QC_Type.h>
#include "QC_AbstractReflectionFunction.h"
#include "QC_Function.h"
#include "QC_FunctionVariant.h"
#include "QC_GlobalVar.h"
#include "QC_TypedHash.h"
#include "QC_Constant.h"

QoreReflectionNamespace::QoreReflectionNamespace(ExceptionSink* xsink, const QoreString& name, QoreProgram* pgm)
    : AbstractReflectionObject(pgm) {
    ns = pgm->findNamespace(name);

    if (!ns) {
        xsink->raiseException("UNKNOWN-NAMESPACE", "cannot resolve namespace '%s'", name.c_str());
        return;
    }
}

QoreReflectionNamespace::QoreReflectionNamespace(QoreProgram* pgm, const QoreNamespace* ns) : AbstractReflectionObject(pgm), ns(ns) {
}

static QoreValue find_all_namespaces(const QoreString& pattern, int re_opts, QoreProgram* pgm, ExceptionSink* xsink) {
    ns_vec_t ns_vec = pgm->findAllNamespacesRegex(pattern, re_opts, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_NAMESPACE->getTypeInfo()), xsink);
    for (auto& i : ns_vec) {
        l->push(new QoreObject(QC_NAMESPACE, pgm, new QoreReflectionNamespace(pgm, i)), xsink);
    }

    return l.release();
}
/* Qore class Qore::Reflection::Namespace */

qore_classid_t CID_NAMESPACE;
QoreClass* QC_NAMESPACE;

// Namespace::constructor(string path) {}
static void Namespace_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 88 "QC_Namespace.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionNamespace> ns(new QoreReflectionNamespace(xsink, **npath), xsink);
    if (*xsink) {
        return;
    }

    self->setPrivate(CID_NAMESPACE, ns.release());
}

// Class Namespace::getClass(string name){}
static QoreValue Namespace_getClass_Vs(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 505 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid

    const QoreClass* rv = n->ns->findLocalClass(tname->c_str());
    if (!rv) {
        xsink->raiseException("UNKNOWN-CLASS", "class '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return new QoreObject(QC_CLASS, n->pgm, new QoreReflectionClass(n->pgm, rv));
}

// list<Class> Namespace::getClasses(){}
static QoreValue Namespace_getClasses(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 532 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_CLASS->getTypeInfo()), xsink);

    QoreNamespaceClassIterator i(*n->ns);
    while (i.next()) {
        l->push(new QoreObject(QC_CLASS, n->pgm, new QoreReflectionClass(n->pgm, &i.get())), xsink);
    }

    return l.release();
}

// Constant Namespace::getConstant(string name){}
static QoreValue Namespace_getConstant_Vs(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 394 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const QoreExternalConstant* ce = n->ns->findLocalConstant(tname->c_str());
    if (!ce) {
        xsink->raiseException("UNKNOWN-CONSTANT", "constant '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return new QoreObject(QC_CONSTANT, n->pgm, new QoreReflectionNamespaceConstant(n->pgm, ce, n->ns));
}

// list<Constant> Namespace::getConstants(){}
static QoreValue Namespace_getConstants(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 420 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_CONSTANT->getTypeInfo()), xsink);

    QoreNamespaceConstantIterator i(*n->ns);
    while (i.next()) {
        l->push(new QoreObject(QC_CONSTANT, n->pgm, new QoreReflectionNamespaceConstant(n->pgm, &i.get(), n->ns)), xsink);
    }

    return l.release();
}

// Function Namespace::getFunction(string name){}
static QoreValue Namespace_getFunction_Vs(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 564 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const QoreExternalFunction* rv = n->ns->findLocalFunction(tname->c_str());
    if (!rv) {
        xsink->raiseException("UNKNOWN-FUNCTION", "function '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return new QoreObject(QC_FUNCTION, n->pgm, new QoreReflectionFunction(n->pgm, rv));
}

// list<Function> Namespace::getFunctions(){}
static QoreValue Namespace_getFunctions(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 596 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_FUNCTION->getTypeInfo()), xsink);

    QoreNamespaceFunctionIterator i(*n->ns);
    while (i.next()) {
        l->push(new QoreObject(QC_FUNCTION, n->pgm, new QoreReflectionFunction(n->pgm, &i.get())), xsink);
    }

    return l.release();
}

// GlobalVar Namespace::getGlobalVar(string name){}
static QoreValue Namespace_getGlobalVar_Vs(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 744 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const QoreExternalGlobalVar* rv = n->ns->findLocalGlobalVar(tname->c_str());
    if (!rv) {
        xsink->raiseException("UNKNOWN-GLOBAL-VAR", "global variable '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return new QoreObject(QC_GLOBALVAR, n->pgm, new QoreReflectionGlobalVar(n->pgm, rv, n->ns));
}

// list<GlobalVar> Namespace::getGlobalVars(){}
static QoreValue Namespace_getGlobalVars(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 770 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_GLOBALVAR->getTypeInfo()), xsink);
    QoreNamespaceGlobalVarIterator i(*n->ns);
    while (i.next()) {
        l->push(new QoreObject(QC_GLOBALVAR, n->pgm, new QoreReflectionGlobalVar(n->pgm, &i.get(), n->ns)), xsink);
    }

    return l.release();
}

// list<string> Namespace::getModifierList(){}
static QoreValue Namespace_getModifierList(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 242 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), xsink);
    if (n->ns->isModulePublic()) {
        rv->push(new QoreStringNode("public"), xsink);
    }

    return rv.release();
}

// int Namespace::getModifiers(){}
static QoreValue Namespace_getModifiers(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 213 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    int64 rc = 0;
    if (n->ns->isModulePublic()) {
        rc |= MC_PUBLIC;
    }

    return rc;
}

// *string Namespace::getModuleName(){}
static QoreValue Namespace_getModuleName(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 135 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const char* mod = n->ns->getModuleName();
    return mod ? new QoreStringNode(mod) : QoreValue();
}

// string Namespace::getName(){}
static QoreValue Namespace_getName(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 114 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    return new QoreStringNode(n->ns->getName());
}

// Namespace Namespace::getNamespace(string name){}
static QoreValue Namespace_getNamespace_Vs(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 450 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const QoreNamespace* rv = n->ns->findLocalNamespace(tname->c_str());
    if (!rv) {
        xsink->raiseException("UNKNOWN-NAMESPACE", "namespace '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return new QoreObject(QC_NAMESPACE, n->pgm, new QoreReflectionNamespace(n->pgm, rv));
}

// list<Namespace> Namespace::getNamespaces(){}
static QoreValue Namespace_getNamespaces(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 476 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_NAMESPACE->getTypeInfo()), xsink);
    QoreNamespaceNamespaceIterator i(*n->ns);
    while (i.next()) {
        l->push(new QoreObject(QC_NAMESPACE, n->pgm, new QoreReflectionNamespace(n->pgm, &i.get())), xsink);
    }

    return l.release();
}

// *Namespace Namespace::getParentNamespace(){}
static QoreValue Namespace_getParentNamespace(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 367 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const QoreNamespace* parent = n->ns->getParent();
    if (parent) {
        return new QoreObject(QC_NAMESPACE, n->pgm, new QoreReflectionNamespace(n->pgm, parent));
    }
    return QoreValue();
}

// string Namespace::getPathName(){}
static QoreValue Namespace_getPathName(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 159 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    return new QoreStringNode(n->ns->getPath());
}

// TypedHash Namespace::getTypedHash(string name){}
static QoreValue Namespace_getTypedHash_Vs(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 799 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const TypedHashDecl* rv = n->ns->findLocalTypedHash(tname->c_str());
    if (!rv) {
        xsink->raiseException("UNKNOWN-TYPED-HASH", "typed hash (hashdecl) '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return new QoreObject(QC_TYPEDHASH, n->pgm, new QoreReflectionTypedHash(n->pgm, rv, n->ns));
}

// list<TypedHash> Namespace::getTypedHashes(){}
static QoreValue Namespace_getTypedHashes(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 825 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_TYPEDHASH->getTypeInfo()), xsink);
    QoreNamespaceTypedHashIterator i(*n->ns);
    while (i.next()) {
        l->push(new QoreObject(QC_TYPEDHASH, n->pgm, new QoreReflectionTypedHash(n->pgm, &i.get(), n->ns)), xsink);
    }

    return l.release();
}

// FunctionVariant Namespace::getVariant(string name, ...){}
static QoreValue Namespace_getVariant_VsVV(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 635 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const QoreExternalFunction* func = n->ns->findLocalFunction(tname->c_str());
    if (!func) {
        xsink->raiseException("UNKNOWN-FUNCTION", "function '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return find_function_variant(n->pgm, func, args, 1, MT_None, xsink);
}

// FunctionVariant Namespace::getVariantArgs(string name, softlist<Type> argv){}
static QoreValue Namespace_getVariantArgs_VsC14softlist_Type_(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* argv = HARD_QORE_VALUE_LIST(args, 1);
# 678 "QC_Namespace.qpp"
    TempEncodingHelper tname(name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    const QoreExternalFunction* func = n->ns->findLocalFunction(tname->c_str());
    if (!func) {
        xsink->raiseException("UNKNOWN-FUNCTION", "function '%s' is not defined in namespace '%s'", tname->c_str(), n->ns->getName());
        return QoreValue();
    }

    return find_function_variant(n->pgm, func, argv, 0, MT_None, xsink);
}

// list<FunctionVariant> Namespace::getVariants(){}
static QoreValue Namespace_getVariants(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 710 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    ReferenceHolder<QoreListNode> l(new QoreListNode(QC_FUNCTIONVARIANT->getTypeInfo()), xsink);

    QoreNamespaceFunctionIterator i(*n->ns);
    while (i.next()) {
        const QoreExternalFunction& f = i.get();
        append_variant_objects(**l, n->pgm, &f, MT_None, xsink);
        if (*xsink) {
            return QoreValue();
        }
    }

    return l.release();
}

// bool Namespace::isBuiltin(){}
static QoreValue Namespace_isBuiltin(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 266 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    return n->ns->isBuiltin();
}

// bool Namespace::isEqual(Namespace ns){}
static QoreValue Namespace_isEqual_C9Namespace(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(ns, QoreReflectionNamespace, args, 0, CID_NAMESPACE, "Namespace::isEqual()", "Namespace", xsink);
    if (*xsink)
        return 0;
# 182 "QC_Namespace.qpp"
    ReferenceHolder<QoreReflectionNamespace> holder(ns, xsink);

    // get read access to program object
    QoreExternalProgramContextHelper tnh1(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // get read access to program object
    QoreExternalProgramContextHelper tnh2(xsink, ns->pgm);
    if (*xsink) {
        return QoreValue();
    }

    return ns->ns == n->ns;
}

// bool Namespace::isImported(){}
static QoreValue Namespace_isImported(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 325 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    return n->ns->isImported();
}

// bool Namespace::isModulePublic(){}
static QoreValue Namespace_isModulePublic(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 304 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    return n->ns->isModulePublic();
}

// bool Namespace::isRoot(){}
static QoreValue Namespace_isRoot(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 346 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    return n->ns->isRoot();
}

// bool Namespace::isUser(){}
static QoreValue Namespace_isUser(QoreObject* self, QoreReflectionNamespace* n, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 285 "QC_Namespace.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, n->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the namespace ptr must be valid
    return !n->ns->isBuiltin();
}

// static list<Namespace> Namespace::findAllRegex(string pattern, *int re_opts){}
static QoreValue static_Namespace_findAllRegex_VsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 1);
# 910 "QC_Namespace.qpp"
return find_all_namespaces(*pattern, re_opts, getProgram(), xsink);
}

// static list<Namespace> Namespace::findAllRegex(Program pgm, string pattern, *int re_opts){}
static QoreValue static_Namespace_findAllRegex_C7ProgramVsNi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Namespace::findAllRegex()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 1);
    int64 re_opts = HARD_QORE_VALUE_INT(args, 2);
# 927 "QC_Namespace.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    return find_all_namespaces(*pattern, re_opts, pgm, xsink);
}

// static Namespace Namespace::forName(string path){}
static QoreValue static_Namespace_forName_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 855 "QC_Namespace.qpp"
TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionNamespace> n(new QoreReflectionNamespace(xsink, **npath), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return new QoreObject(QC_NAMESPACE, getProgram(), n.release());
}

// static Namespace Namespace::forName(Program pgm, string path){}
static QoreValue static_Namespace_forName_C7ProgramVs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Namespace::forName()", "Program", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
# 883 "QC_Namespace.qpp"
ReferenceHolder<QoreProgram> pgm_holder(pgm, xsink);
    TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    ReferenceHolder<QoreReflectionNamespace> n(new QoreReflectionNamespace(xsink, **npath, pgm), xsink);
    if (*xsink) {
        return QoreValue();
    }

    return new QoreObject(QC_NAMESPACE, pgm, n.release());
}

DLLLOCAL void preinitNamespaceClass() {
    QC_NAMESPACE = new QoreBuiltinClass("Namespace", "::Qore::Reflection::Namespace", QDOM_REFLECTION);
    CID_NAMESPACE = QC_NAMESPACE->getID();
    QC_NAMESPACE->setSystem();
}

DLLLOCAL QoreClass* initNamespaceClass(QoreNamespace& ns) {
    if (!QC_NAMESPACE)
        preinitNamespaceClass();

    // Namespace::constructor(string path) {}
    QC_NAMESPACE->addConstructor(Namespace_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // Class Namespace::getClass(string name){}
    QC_NAMESPACE->addMethod("getClass", (q_method_n_t)Namespace_getClass_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CLASS->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<Class> Namespace::getClasses(){}
    QC_NAMESPACE->addMethod("getClasses", (q_method_n_t)Namespace_getClasses, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CLASS->getTypeInfo()));

    // Constant Namespace::getConstant(string name){}
    QC_NAMESPACE->addMethod("getConstant", (q_method_n_t)Namespace_getConstant_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_CONSTANT->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<Constant> Namespace::getConstants(){}
    QC_NAMESPACE->addMethod("getConstants", (q_method_n_t)Namespace_getConstants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_CONSTANT->getTypeInfo()));

    // Function Namespace::getFunction(string name){}
    QC_NAMESPACE->addMethod("getFunction", (q_method_n_t)Namespace_getFunction_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_FUNCTION->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<Function> Namespace::getFunctions(){}
    QC_NAMESPACE->addMethod("getFunctions", (q_method_n_t)Namespace_getFunctions, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_FUNCTION->getTypeInfo()));

    // GlobalVar Namespace::getGlobalVar(string name){}
    QC_NAMESPACE->addMethod("getGlobalVar", (q_method_n_t)Namespace_getGlobalVar_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_GLOBALVAR->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<GlobalVar> Namespace::getGlobalVars(){}
    QC_NAMESPACE->addMethod("getGlobalVars", (q_method_n_t)Namespace_getGlobalVars, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_GLOBALVAR->getTypeInfo()));

    // list<string> Namespace::getModifierList(){}
    QC_NAMESPACE->addMethod("getModifierList", (q_method_n_t)Namespace_getModifierList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int Namespace::getModifiers(){}
    QC_NAMESPACE->addMethod("getModifiers", (q_method_n_t)Namespace_getModifiers, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // *string Namespace::getModuleName(){}
    QC_NAMESPACE->addMethod("getModuleName", (q_method_n_t)Namespace_getModuleName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string Namespace::getName(){}
    QC_NAMESPACE->addMethod("getName", (q_method_n_t)Namespace_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // Namespace Namespace::getNamespace(string name){}
    QC_NAMESPACE->addMethod("getNamespace", (q_method_n_t)Namespace_getNamespace_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_NAMESPACE->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<Namespace> Namespace::getNamespaces(){}
    QC_NAMESPACE->addMethod("getNamespaces", (q_method_n_t)Namespace_getNamespaces, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_NAMESPACE->getTypeInfo()));

    // *Namespace Namespace::getParentNamespace(){}
    QC_NAMESPACE->addMethod("getParentNamespace", (q_method_n_t)Namespace_getParentNamespace, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_NAMESPACE->getOrNothingTypeInfo());

    // string Namespace::getPathName(){}
    QC_NAMESPACE->addMethod("getPathName", (q_method_n_t)Namespace_getPathName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // TypedHash Namespace::getTypedHash(string name){}
    QC_NAMESPACE->addMethod("getTypedHash", (q_method_n_t)Namespace_getTypedHash_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPEDHASH->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // list<TypedHash> Namespace::getTypedHashes(){}
    QC_NAMESPACE->addMethod("getTypedHashes", (q_method_n_t)Namespace_getTypedHashes, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_TYPEDHASH->getTypeInfo()));

    // FunctionVariant Namespace::getVariant(string name, ...){}
    QC_NAMESPACE->addMethod("getVariant", (q_method_n_t)Namespace_getVariant_VsVV, Public, QCF_RET_VALUE_ONLY|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, QC_FUNCTIONVARIANT->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // FunctionVariant Namespace::getVariantArgs(string name, softlist<Type> argv){}
    QC_NAMESPACE->addMethod("getVariantArgs", (q_method_n_t)Namespace_getVariantArgs_VsC14softlist_Type_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_FUNCTIONVARIANT->getTypeInfo(), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "name", qore_get_complex_softlist_type(QC_TYPE->getTypeInfo()), QORE_PARAM_NO_ARG, "argv");

    // list<FunctionVariant> Namespace::getVariants(){}
    QC_NAMESPACE->addMethod("getVariants", (q_method_n_t)Namespace_getVariants, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_FUNCTIONVARIANT->getTypeInfo()));

    // bool Namespace::isBuiltin(){}
    QC_NAMESPACE->addMethod("isBuiltin", (q_method_n_t)Namespace_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool Namespace::isEqual(Namespace ns){}
    QC_NAMESPACE->addMethod("isEqual", (q_method_n_t)Namespace_isEqual_C9Namespace, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, QC_NAMESPACE->getTypeInfo(), QORE_PARAM_NO_ARG, "ns");

    // bool Namespace::isImported(){}
    QC_NAMESPACE->addMethod("isImported", (q_method_n_t)Namespace_isImported, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool Namespace::isModulePublic(){}
    QC_NAMESPACE->addMethod("isModulePublic", (q_method_n_t)Namespace_isModulePublic, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool Namespace::isRoot(){}
    QC_NAMESPACE->addMethod("isRoot", (q_method_n_t)Namespace_isRoot, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool Namespace::isUser(){}
    QC_NAMESPACE->addMethod("isUser", (q_method_n_t)Namespace_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // static list<Namespace> Namespace::findAllRegex(string pattern, *int re_opts){}
    QC_NAMESPACE->addStaticMethod("findAllRegex", (q_func_n_t)static_Namespace_findAllRegex_VsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_NAMESPACE->getTypeInfo()), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static list<Namespace> Namespace::findAllRegex(Program pgm, string pattern, *int re_opts){}
    QC_NAMESPACE->addStaticMethod("findAllRegex", (q_func_n_t)static_Namespace_findAllRegex_C7ProgramVsNi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(QC_NAMESPACE->getTypeInfo()), 3, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "pattern", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "re_opts");

    // static Namespace Namespace::forName(string path){}
    QC_NAMESPACE->addStaticMethod("forName", (q_func_n_t)static_Namespace_forName_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_NAMESPACE->getTypeInfo(), 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // static Namespace Namespace::forName(Program pgm, string path){}
    QC_NAMESPACE->addStaticMethod("forName", (q_func_n_t)static_Namespace_forName_C7ProgramVs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_NAMESPACE->getTypeInfo(), 2, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    return QC_NAMESPACE;
}
