/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_NormalMethodVariant.cpp NormalMethodVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_NormalMethodVariant.h"
#include "QC_AbstractMethod.h"

int check_variant_call(const QoreExternalVariant* v, QoreObject* obj, const QoreMethod& m, ExceptionSink* xsink) {
    if (check_call(obj, m, xsink)) {
        return -1;
    }

    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v);
    // we have to explicitly check for abstract methods here and throw an exception
    if (mv->isAbstract()) {
        xsink->raiseException("ABSTRACT-VARIANT-ERROR", "cannot call abstract variant %s::%s(%s) directly",
            mv->getClass()->getName(),
            m.getName(), mv->getSignatureText());
        return -1;
    }

    return 0;
}
/* Qore class Qore::Reflection::NormalMethodVariant */

qore_classid_t CID_NORMALMETHODVARIANT;
QoreClass* QC_NORMALMETHODVARIANT;

// auto NormalMethodVariant::call(object obj, ...){}
static QoreValue NormalMethodVariant_call_VoVV(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 87 "QC_NormalMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // we have to explictly test object compatibility here, otherwise QoreObject::evalMethod() will throw
    // a confusing exception
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreMethod* m = mv->getMethod();
    if (check_variant_call(v->variant, obj, *m, xsink)) {
        return QoreValue();
    }

    ReferenceHolder<QoreListNode> call_args(xsink);

    // if there are arguments to pass
    if (num_args(args) > 1) {
        // create argument list by copying current list
        call_args = args->copyListFrom(1);
    }

    // the method ptr must be valid
    return obj->evalMethodVariant(*m, mv, *call_args, xsink);
}

// auto NormalMethodVariant::callArgs(object obj, *softlist<auto> argv){}
static QoreValue NormalMethodVariant_callArgs_VoC15_softlist_auto_(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 131 "QC_NormalMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // we have to explictly test object compatibility here, otherwise QoreObject::evalMethod() will throw
    // a confusing exception
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreMethod* m = mv->getMethod();
    if (check_variant_call(v->variant, obj, *m, xsink)) {
        return QoreValue();
    }

    // the method ptr must be valid
    return obj->evalMethodVariant(*m, mv, argv, xsink);
}

// NormalMethodVariant::constructor() {}
static void NormalMethodVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 66 "QC_NormalMethodVariant.qpp"
xsink->raiseException("NORMALMETHODVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

DLLLOCAL void preinitNormalMethodVariantClass() {
    QC_NORMALMETHODVARIANT = new QoreBuiltinClass("NormalMethodVariant", "::Qore::Reflection::NormalMethodVariant", QDOM_REFLECTION);
    CID_NORMALMETHODVARIANT = QC_NORMALMETHODVARIANT->getID();
    QC_NORMALMETHODVARIANT->setSystem();
}

DLLLOCAL QoreClass* initNormalMethodVariantClass(QoreNamespace& ns) {
    if (!QC_NORMALMETHODVARIANT)
        preinitNormalMethodVariantClass();

    // set parent class
    assert(QC_ABSTRACTMETHODVARIANT);
    QC_NORMALMETHODVARIANT->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHODVARIANT);

    // auto NormalMethodVariant::call(object obj, ...){}
    QC_NORMALMETHODVARIANT->addMethod("call", (q_method_n_t)NormalMethodVariant_call_VoVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");

    // auto NormalMethodVariant::callArgs(object obj, *softlist<auto> argv){}
    QC_NORMALMETHODVARIANT->addMethod("callArgs", (q_method_n_t)NormalMethodVariant_callArgs_VoC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, objectTypeInfo, QORE_PARAM_NO_ARG, "obj", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // NormalMethodVariant::constructor() {}
    QC_NORMALMETHODVARIANT->addConstructor(NormalMethodVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    return QC_NORMALMETHODVARIANT;
}
