/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StaticMethod.cpp StaticMethod class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_StaticMethod.h"
#include "QC_Class.h"
/* Qore class Qore::Reflection::StaticMethod */

qore_classid_t CID_STATICMETHOD;
QoreClass* QC_STATICMETHOD;

// auto StaticMethod::call(...){}
static QoreValue StaticMethod_call_VV(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 133 "QC_StaticMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    return QoreObject::evalStaticMethod(*m->m, m->m->getClass(), args, xsink);
}

// auto StaticMethod::callArgs(*softlist<auto> argv){}
static QoreValue StaticMethod_callArgs_C15_softlist_auto_(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 158 "QC_StaticMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the method ptr must be valid
    return QoreObject::evalStaticMethod(*m->m, m->m->getClass(), argv, xsink);
}

// StaticMethod::constructor(Class cls, string method) {}
static void StaticMethod_constructor_C5ClassVs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(cls, QoreReflectionClass, args, 0, CID_CLASS, "StaticMethod::constructor()", "Class", xsink);
    if (*xsink)
        return;
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 55 "QC_StaticMethod.qpp"
ReferenceHolder<QoreReflectionClass> holder(cls, xsink);

    TempEncodingHelper nmethod(method, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    // get exclusive access to program object
    QoreExternalProgramContextHelper pch(xsink, cls->pgm);
    if (*xsink) {
        return;
    }

    // the class pointer must be valid here
    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(cls->cls, nmethod->c_str(), xsink), xsink);
    if (*xsink) {
        return;
    }

    if (m->mtype != MT_Static) {
        xsink->raiseException("METHOD-ERROR", "method '%s()' in class '%s' has type '%s'; expecting 'static'", nmethod->c_str(), cls->cls->getName(), m->getType());
        return;
    }

    self->setPrivate(CID_STATICMETHOD, m.release());
}

// StaticMethod::constructor(string class_name, string method) {}
static void StaticMethod_constructor_VsVs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 95 "QC_StaticMethod.qpp"
TempEncodingHelper npath(class_name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    TempEncodingHelper nmethod(method, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(npath->c_str(), nmethod->c_str(), xsink), xsink);
    if (*xsink) {
        return;
    }

    if (m->mtype != MT_Static) {
        xsink->raiseException("METHOD-ERROR", "method '%s()' in class '%s' has type '%s'; expecting 'static'", nmethod->c_str(), npath->c_str(), m->getType());
        return;
    }

    self->setPrivate(CID_STATICMETHOD, m.release());
}

DLLLOCAL void preinitStaticMethodClass() {
    QC_STATICMETHOD = new QoreBuiltinClass("StaticMethod", "::Qore::Reflection::StaticMethod", QDOM_REFLECTION);
    CID_STATICMETHOD = QC_STATICMETHOD->getID();
    QC_STATICMETHOD->setSystem();
}

DLLLOCAL QoreClass* initStaticMethodClass(QoreNamespace& ns) {
    if (!QC_STATICMETHOD)
        preinitStaticMethodClass();

    // set parent class
    assert(QC_ABSTRACTMETHOD);
    QC_STATICMETHOD->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHOD);

    // auto StaticMethod::call(...){}
    QC_STATICMETHOD->addMethod("call", (q_method_n_t)StaticMethod_call_VV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);

    // auto StaticMethod::callArgs(*softlist<auto> argv){}
    QC_STATICMETHOD->addMethod("callArgs", (q_method_n_t)StaticMethod_callArgs_C15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // StaticMethod::constructor(Class cls, string method) {}
    QC_STATICMETHOD->addConstructor(StaticMethod_constructor_C5ClassVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, QC_CLASS->getTypeInfo(), QORE_PARAM_NO_ARG, "cls", stringTypeInfo, QORE_PARAM_NO_ARG, "method");

    // StaticMethod::constructor(string class_name, string method) {}
    QC_STATICMETHOD->addConstructor(StaticMethod_constructor_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name", stringTypeInfo, QORE_PARAM_NO_ARG, "method");

    return QC_STATICMETHOD;
}
